@@ -14,8 +14,9 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifdef STANDALONE_MAIN
@@ -31,7 +32,7 @@
 
 #include "config.h"
 
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 #include <gd.h>
 #endif
 #include <stdlib.h>
@@ -51,7 +52,7 @@ static const int corr_tables[4][8] = {
 	{ 0,  4,  8, 12, -16, -12,  -8, -4 },
 };
 
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 /* With in a compressed 4x4 block, the data is stored 4 component values at a
    time, compressed into 2 bytes, the first 5 bits are a starting value,
    then 2 bits encoding which correction/delta table to use and
@@ -62,7 +63,7 @@ ax203_decode_component_values(char *src, char *dest)
 	int i, table, corr;
 	dest[0] = src[0] & ~0x07;
        	table = (src[0] >> 1) & 3;
-       	
+
 	for (i = 1; i < 4; i++) {
 		switch (i) {
 		case 1:
@@ -98,7 +99,7 @@ ax203_decode_block_yuv_delta(char *src, int **dest, int dest_x, int dest_y)
 	/* The Y components are stored 4 at a time in a pattern like this:
 	   1 2  1 2
 	   3 4  3 4
-	   
+
 	   1 2  1 2
 	   3 4  3 4 */
 	for (y = 0; y < 4; y += 2) {
@@ -220,7 +221,7 @@ ax203_encode_signed_component_values(int8_t *src, char *dest)
 		}
 		/* If we did not break out the above loop the min / max
 		   correction in the current table is enough */
-		if (j == 4) 
+		if (j == 4)
 			break;
 	}
 	table = i;
@@ -270,7 +271,7 @@ ax203_encode_unsigned_component_values(uint8_t *src, char *dest)
 		}
 		/* If we did not break out the above loop the min / max
 		   correction in the current table is enough */
-		if (j == 4) 
+		if (j == 4)
 			break;
 	}
 	table = i;
@@ -351,7 +352,7 @@ ax203_encode_block_yuv_delta(int **src, int src_x, int src_y, char *dest)
 	/* The Y components are stored 4 at a time in a pattern like this:
 	   1 2  1 2
 	   3 4  3 4
-	   
+
 	   1 2  1 2
 	   3 4  3 4 */
 	for (y = 0; y < 4; y += 2) {
@@ -436,7 +437,7 @@ main(int argc, char *argv[])
 		ax203_decode(buf, im->tpixels, width, height);
 		gdImagePng (im, fout);
 	} else {
-		fprintf (stderr, "%s: unkown option: %s\n", argv[0], argv[1]);
+		fprintf (stderr, "%s: unknown option: %s\n", argv[0], argv[1]);
 		ret = 1;
 		goto exit;
 	}
