@@ -4,7 +4,7 @@
  * *** NOTE: This is a modified version to deal with the ax203 "JPEG" fmt
  * *** This version can not decompress regular JPEG files, see
  * *** README.ax203-compression for details
- * 
+ *
  * Copyright (c) 2006, Luc Saillard <luc@saillard.org>
  *
  * ax203 modifications:
@@ -39,6 +39,8 @@
  *
  */
 
+#define _DEFAULT_SOURCE
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -124,16 +126,16 @@ static const unsigned char zigzag[64] =
 #define fill_nbits(reservoir,nbits_in_reservoir,stream,nbits_wanted) do { \
    while (nbits_in_reservoir<nbits_wanted) \
     { \
-      unsigned char c; \
+      unsigned char __c; \
       if (stream >= priv->stream_end) { \
 	snprintf(priv->error_string, sizeof(priv->error_string), \
 	  "fill_nbits error: need %u more bits\n", \
 	  nbits_wanted - nbits_in_reservoir); \
 	longjmp(priv->jump_state, -EIO); \
       } \
-      c = *stream++; \
+      __c = *stream++; \
       reservoir <<= 8; \
-      reservoir |= c; \
+      reservoir |= __c; \
       nbits_in_reservoir+=8; \
     } \
 }  while(0);
@@ -288,7 +290,6 @@ static int build_huffman_table(struct jdec_private *priv, const unsigned char *b
   unsigned int i, j, code, code_size, val, nbits;
   unsigned char huffsize[257], *hz;
   unsigned int huffcode[257], *hc;
-  int next_free_entry;
   int slowtable_used[16-HUFFMAN_HASH_NBITS];
 
   /*
@@ -327,7 +328,6 @@ static int build_huffman_table(struct jdec_private *priv, const unsigned char *b
   /*
    * Build the lookup table, and the slowtable if needed.
    */
-  next_free_entry = -1;
   for (i=0; huffsize[i]; i++)
    {
      val = vals[i];
@@ -544,15 +544,15 @@ static void decode_MCU_1x1_3planes(struct jdec_private *priv, int block_nr)
   priv->nbits_in_reservoir = 0;
   priv->reservoir = 0;
 
-  // Cb
+  /* Cb */
   process_Huffman_data_unit(priv, cCb, block_nr);
   IDCT(&priv->component_infos[cCb], priv->Cb, 8);
 
-  // Cr
+  /* Cr */
   process_Huffman_data_unit(priv, cCr, block_nr);
   IDCT(&priv->component_infos[cCr], priv->Cr, 8);
 
-  // Y
+  /* Y */
   process_Huffman_data_unit(priv, cY, block_nr);
   IDCT(&priv->component_infos[cY], priv->Y, 8);
 
@@ -573,15 +573,15 @@ static void decode_MCU_2x2_3planes(struct jdec_private *priv, int block_nr)
   priv->nbits_in_reservoir = 0;
   priv->reservoir = 0;
 
-  // Cb
+  /* Cb */
   process_Huffman_data_unit(priv, cCb, 0);
   IDCT(&priv->component_infos[cCb], priv->Cb, 8);
 
-  // Cr
+  /* Cr */
   process_Huffman_data_unit(priv, cCr, 0);
   IDCT(&priv->component_infos[cCr], priv->Cr, 8);
 
-  // Y
+  /* Y */
   process_Huffman_data_unit(priv, cY, block_nr);
   IDCT(&priv->component_infos[cY], priv->Y, 16);
   process_Huffman_data_unit(priv, cY, -1);
@@ -815,7 +815,7 @@ int tinyjpeg_parse_header(struct jdec_private *priv, const unsigned char *buf, u
   stream += 16;
 
   /* Skip MCU info blocks (we don't need them) */
-  stream += (priv->width  / (8 * priv->component_infos[cY].Hfactor)) * 
+  stream += (priv->width  / (8 * priv->component_infos[cY].Hfactor)) *
             (priv->height / (8 * priv->component_infos[cY].Vfactor)) * 8;
 
   /* Parse DQT table */
@@ -940,7 +940,7 @@ void tinyjpeg_get_size(struct jdec_private *priv, unsigned int *width, unsigned
 int tinyjpeg_get_components(struct jdec_private *priv, unsigned char **components)
 {
   int i;
-  for (i=0; priv->components[i] && i<COMPONENTS; i++)
+  for (i=0; i<COMPONENTS && priv->components[i]; i++)
     components[i] = priv->components[i];
   return 0;
 }
