@@ -20,7 +20,7 @@
  *
  ****************************************************************************/
 
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 
 #include "config.h"
 
@@ -233,24 +233,24 @@ static const struct canonFlashModeStateStruct flashModeStateArray[] = {
 };
 
 static const struct canonExposureBiasStateStruct exposureBiasStateArray[] = {
-	{0x10,"+2"},
-	{0x0d,"+1 2/3"},
-	{0x0c,"+1 1/2"},
-	{0x0b,"+1 1/3"},
-	{0x08,"+1"},
-	{0x05,"+2/3"},
-	{0x04,"+1/2"},
-	{0x03,"+1/3"},
-	{0x00,"0"},
-	{0xfd,"-1/3"},
-	{0xfc,"-1/2"},
-	{0xfb,"-2/3"},
-	{0xf8,"-1"},
-	{0xf5,"-1 1/3"},
-	{0xf4,"-1 1/2"},
-	{0xf3,"-1 2/3"},
-	{0xf0,"-2"},
-	{0, NULL},
+	{0x10,0x08,"+2"},
+	{0x0d,0x0b,"+1 2/3"},
+	{0x0c,0x0c,"+1 1/2"},
+	{0x0b,0x0d,"+1 1/3"},
+	{0x08,0x10,"+1"},
+	{0x05,0x13,"+2/3"},
+	{0x04,0x14,"+1/2"},
+	{0x03,0x15,"+1/3"},
+	{0x00,0x18,"0"},
+	{0xfd,0x1b,"-1/3"},
+	{0xfc,0x1c,"-1/2"},
+	{0xfb,0x1d,"-2/3"},
+	{0xf8,0x20,"-1"},
+	{0xf5,0x23,"-1 1/3"},
+	{0xf4,0x24,"-1 1/2"},
+	{0xf3,0x25,"-1 2/3"},
+	{0xf0,0x28,"-2"},
+	{0,0,NULL},
 };
 
 
@@ -358,7 +358,7 @@ int camera_id (CameraText *id)
  *
  */
 static int
-camera_manual (Camera __unused__ *camera, CameraText *manual, 
+camera_manual (Camera __unused__ *camera, CameraText *manual,
 	       GPContext __unused__ *context)
 {
 	GP_DEBUG ("camera_manual()");
@@ -404,14 +404,7 @@ camera_abilities (CameraAbilitiesList *list)
 	for (i = 0; models[i].id_str; i++) {
 		memset (&a, 0, sizeof (a));
 
-		/* For now, flag EOS 20D as experimental. */
-		if ((UPLOAD_BOOL || (models[i].usb_capture_support == CAP_EXP)
-		     || models[i].model == CANON_CLASS_6) && 
-		    (models[i].usb_vendor && models[i].usb_product)) {
-			a.status = GP_DRIVER_STATUS_EXPERIMENTAL;
-		} else {
-			a.status = GP_DRIVER_STATUS_PRODUCTION;
-		}
+		a.status = GP_DRIVER_STATUS_PRODUCTION;
 
 		strcpy (a.model, models[i].id_str);
 		a.port = 0;
@@ -539,9 +532,9 @@ camera_exit (Camera *camera, GPContext *context)
 {
 	int res = 0;
 
-	if (camera->port->type == GP_PORT_USB) 
+	if (camera->port->type == GP_PORT_USB)
 		canon_usb_unlock_keys (camera, context);
-	
+
 	/* Turn off remote control if enabled (remote control only
 	 * applies to USB cameras, but in case of serial connection
 	 * remote_control should never be seen set here) */
@@ -697,7 +690,7 @@ update_disk_cache (Camera *camera, GPContext *context)
  ****************************************************************************/
 
 static int
-file_list_func (CameraFilesystem __unused__ *fs, const char *folder, 
+file_list_func (CameraFilesystem __unused__ *fs, const char *folder,
 		CameraList *list, void *data,
 		GPContext *context)
 {
@@ -708,12 +701,12 @@ file_list_func (CameraFilesystem __unused__ *fs, const char *folder,
 	if (!check_readiness (camera, context))
 		return GP_ERROR;
 
-	return canon_int_list_directory (camera, folder, list, 
+	return canon_int_list_directory (camera, folder, list,
 					 CANON_LIST_FILES, context);
 }
 
 static int
-folder_list_func (CameraFilesystem __unused__ *fs, const char *folder, 
+folder_list_func (CameraFilesystem __unused__ *fs, const char *folder,
 		  CameraList *list, void *data,
 		  GPContext *context)
 {
@@ -724,13 +717,13 @@ folder_list_func (CameraFilesystem __unused__ *fs, const char *folder,
 	if (!check_readiness (camera, context))
 		return GP_ERROR;
 
-	return canon_int_list_directory (camera, folder, list, 
+	return canon_int_list_directory (camera, folder, list,
 					 CANON_LIST_FOLDERS, context);
 }
 
 static int
 get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
-	       CameraFileType type, CameraFile *file, void *user_data, 
+	       CameraFileType type, CameraFile *file, void *user_data,
 	       GPContext *context)
 {
 	Camera *camera = user_data;
@@ -1130,12 +1123,12 @@ storage_info_func (
 	(*sinfos)->access = GP_STORAGEINFO_AC_READONLY_WITH_DELETE;
 	return GP_OK;
 }
-	
+
 
 /****************************************************************************/
 
 static int
-camera_about (Camera __unused__ *camera, CameraText *about, 
+camera_about (Camera __unused__ *camera, CameraText *about,
 	      GPContext __unused__ *context)
 {
 	GP_DEBUG ("camera_about()");
@@ -1158,7 +1151,7 @@ camera_about (Camera __unused__ *camera, CameraText *about,
 /****************************************************************************/
 
 static int
-delete_file_func (CameraFilesystem __unused__ *fs, const char *folder, 
+delete_file_func (CameraFilesystem __unused__ *fs, const char *folder,
 		  const char *filename, void *data,
 		  GPContext *context)
 {
@@ -1212,10 +1205,10 @@ delete_file_func (CameraFilesystem __unused__ *fs, const char *folder,
 #ifdef CANON_EXPERIMENTAL_UPLOAD
 /*
  * get from the filesystem the name of the highest numbered picture or directory
- * 
+ *
  */
 static int
-get_last_file (Camera *camera, char *directory, char *destname, 
+get_last_file (Camera *camera, char *directory, char *destname,
 	       GPContext* context, char getdirectory, char*result)
 {
 	CameraFilesystem * fs = camera->fs;
@@ -1225,9 +1218,9 @@ get_last_file (Camera *camera, char *directory, char *destname,
 	GP_DEBUG ("get_last_file()");
 
 	if(directory[1] == ':') {
-	    /* gp_file_list_folder() needs absolute filenames 
+	    /* gp_file_list_folder() needs absolute filenames
 	       starting with '/' */
-	    GP_DEBUG ("get_last_file(): replacing '%c:%c' by '/'", 
+	    GP_DEBUG ("get_last_file(): replacing '%c:%c' by '/'",
 		      directory[0], directory[2]);
 	    sprintf(dir2, "/%s", &directory[3]);
 	    directory = dir2;
@@ -1245,7 +1238,7 @@ get_last_file (Camera *camera, char *directory, char *destname,
 	for(t=0;t<list->count;t++)
 	{
 	    char* name = list->entry[t].name;
-	    if(getdirectory) 
+	    if(getdirectory)
 		GP_DEBUG ("get_last_file(): folder: %s", name);
 	    else
 		GP_DEBUG ("get_last_file(): file: %s", name);
@@ -1258,7 +1251,7 @@ get_last_file (Camera *camera, char *directory, char *destname,
 		continue;
 
 	    /* ...and only for files similar to AUT_[0-9]{4} */
-	    if(!getdirectory && (!isdigit(name[6]) || 
+	    if(!getdirectory && (!isdigit(name[6]) ||
 			         !isdigit(name[7])))
 		continue;
 
@@ -1285,9 +1278,9 @@ get_last_dir (Camera *camera, char *directory, char *destname, GPContext* contex
 }
 
 /*
- * convert a filename to 8.3 format by truncating 
+ * convert a filename to 8.3 format by truncating
  * the basename to 8 and the extension to 3 chars
- * 
+ *
  */
 static void
 convert_filename_to_8_3(const char* filename, char* dest)
@@ -1300,7 +1293,7 @@ convert_filename_to_8_3(const char* filename, char* dest)
     }
     else {
 	int l = c-filename;
-	if(l>8) 
+	if(l>8)
 	    l=8;
 	memcpy(dest, filename, l);
 	dest[l]=0;
@@ -1464,7 +1457,7 @@ put_file_func (CameraFilesystem __unused__ *fs, const char __unused__ *folder, c
 
 	gp_camera_get_abilities (camera, &a);
 	/* Special case for A50 and Pro 70 */
-	if ((camera->pl->speed > 57600) && 
+	if ((camera->pl->speed > 57600) &&
 	    ((camera->pl->md->model == CANON_CLASS_1)
 	     || (camera->pl->md->model == CANON_CLASS_2))) {
 		gp_context_error (context,
@@ -1557,7 +1550,7 @@ put_file_func (CameraFilesystem __unused__ *fs, const char __unused__ *folder, c
 
 /* Wait for an event */
 static int
-camera_wait_for_event (Camera *camera, int timeout, CameraEventType *eventtype, void **eventdata, GPContext *context) 
+camera_wait_for_event (Camera *camera, int timeout, CameraEventType *eventtype, void **eventdata, GPContext *context)
 {
 	return canon_int_wait_for_event (camera, timeout, eventtype, eventdata, context);
 }
@@ -1609,9 +1602,9 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 		}
 		i++;
 	}
-	
+
 	/* Set to "Compatibility mode" if not currently set */
-	if (menuval == -1) 
+	if (menuval == -1)
 		gp_widget_set_value (t, _("Compatibility Mode"));
 
 	gp_widget_append (section, t);
@@ -1622,7 +1615,7 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	/* Necessary for release params: ISO, aperture, etc. */
 	if (!camera->pl->remote_control) {
 		res = canon_int_start_remote_control (camera, context);
-		if (res != GP_OK) 
+		if (res != GP_OK)
 			return -1;
 	}
 
@@ -1649,8 +1642,8 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 		}
 		i++;
 	}
-	
-	/* Set an unknown ISO value if the 
+
+	/* Set an unknown ISO value if the
 	 * camera is set to something weird */
 	if (menuval == -1) {
 		gp_widget_add_choice (t, _("Unknown"));
@@ -1703,8 +1696,8 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 		}
 		i++;
 	}
-	
-	/* Set an unknown shutter value if the 
+
+	/* Set an unknown shutter value if the
 	 * camera is set to something weird */
 	if (menuval == -1) {
 		gp_widget_add_choice (t, _("Unknown"));
@@ -1743,8 +1736,8 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 		}
 		i++;
 	}
-	
-	/* Set an unknown aperture value if the 
+
+	/* Set an unknown aperture value if the
 	 * camera is set to something weird */
 	if (menuval == -1) {
 		gp_widget_add_choice (t, _("Unknown"));
@@ -1764,14 +1757,14 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	while (exposureBiasStateArray[i].label) {
 		gp_widget_add_choice (t, _(exposureBiasStateArray[i].label));
 		if (camera->pl->release_params[EXPOSUREBIAS_INDEX] ==
-		    exposureBiasStateArray[i].value) {
+		    ((camera->pl->md->id_str && !strstr(camera->pl->md->id_str,"EOS") && !strstr(camera->pl->md->id_str,"Rebel")) ? exposureBiasStateArray[i].valuePS : exposureBiasStateArray[i].valueEOS)) {
 			gp_widget_set_value (t, _(exposureBiasStateArray[i].label));
 			menuval = i;
 		}
 		i++;
 	}
-	
-	/* Set an unknown exp bias value if the 
+
+	/* Set an unknown exp bias value if the
 	 * camera is set to something weird */
 	if (menuval == -1) {
 		gp_widget_add_choice (t, _("Unknown"));
@@ -1801,7 +1794,7 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 		}
 		i++;
 	}
-	
+
 	/* Set an unknown imageFormat value if the
 	 * camera is set to something weird */
 	if (menuval == -1) {
@@ -1829,8 +1822,8 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 		}
 		i++;
 	}
-	
-	/* Set an unknown focus mode value if the 
+
+	/* Set an unknown focus mode value if the
 	 * camera is set to something weird */
 	if (menuval == -1) {
 		gp_widget_add_choice (t, _("Unknown"));
@@ -1857,8 +1850,8 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 		}
 		i++;
 	}
-	
-	/* Set an unknown flash value if the 
+
+	/* Set an unknown flash value if the
 	 * camera is set to something weird */
 	if (menuval == -1) {
 		gp_widget_add_choice (t, _("Unknown"));
@@ -1885,8 +1878,8 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 		}
 		i++;
 	}
-	
-	/* Set an unknown beep value if the 
+
+	/* Set an unknown beep value if the
 	 * camera is set to something weird */
 	if (menuval == -1) {
 		gp_widget_add_choice (t, _("Unknown"));
@@ -1994,6 +1987,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("Owner Name"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2007,6 +2001,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("Capture Size Class"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 
 		i = 0;
@@ -2018,8 +2013,8 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 			}
 			i++;
 		}
-		
-		if (!captureSizeArray[i].label) 
+
+		if (!captureSizeArray[i].label)
 			gp_context_status (context, _("Invalid capture size class setting"));
 
 	}
@@ -2033,6 +2028,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("ISO Speed"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2062,6 +2058,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("Shooting mode"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2070,7 +2067,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 			/* Map the menu option setting to the camera binary value */
 			i = 0;
 			while (shootingModeStateArray[i].label) {
-				if (strcmp (shootingModeStateArray[i].label, wvalue) == 0) {
+				if (strcmp (_(shootingModeStateArray[i].label), wvalue) == 0) {
 					shooting_mode = shootingModeStateArray[i].value;
 					break;
 				}
@@ -2091,6 +2088,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("Shutter Speed"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2119,6 +2117,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("Aperture"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2127,7 +2126,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 			/* Map the menu option setting to the camera binary value */
 			i = 0;
 			while (apertureStateArray[i].label) {
-				if (strcmp (apertureStateArray[i].label, wvalue) == 0) {
+				if (strcmp (_(apertureStateArray[i].label), wvalue) == 0) {
 					aperture = apertureStateArray[i].value;
 					break;
 				}
@@ -2147,6 +2146,8 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 	gp_widget_get_child_by_label (window, _("Exposure Compensation"), &w);
 	if (gp_widget_changed (w)) {
 		unsigned char expbias;
+
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2155,8 +2156,8 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 			/* Map the menu option setting to the camera binary value */
 			i = 0;
 			while (exposureBiasStateArray[i].label) {
-				if (strcmp (exposureBiasStateArray[i].label, wvalue) == 0) {
-					expbias = exposureBiasStateArray[i].value;
+				if (strcmp (_(exposureBiasStateArray[i].label), wvalue) == 0) {
+					expbias = ((camera->pl->md->id_str && !strstr(camera->pl->md->id_str,"EOS") && !strstr(camera->pl->md->id_str,"Rebel")) ? exposureBiasStateArray[i].valuePS : exposureBiasStateArray[i].valueEOS);
 					break;
 				}
 				i++;
@@ -2175,6 +2176,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("Image Format"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2183,9 +2185,9 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 			/* Map the menu option setting to the camera binary value */
 			i = 0;
 			while (imageFormatStateArray[i].label) {
-				if (strcmp (imageFormatStateArray[i].label, wvalue) == 0)
+				if (strcmp (_(imageFormatStateArray[i].label), wvalue) == 0)
 					break;
-				
+
 				i++;
 			}
 
@@ -2203,6 +2205,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("Focus Mode"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2211,7 +2214,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 			/* Map the menu option setting to the camera binary value */
 			i = 0;
 			while (focusModeStateArray[i].label) {
-				if (strcmp (focusModeStateArray[i].label, wvalue) == 0) {
+				if (strcmp (_(focusModeStateArray[i].label), wvalue) == 0) {
 					focus_mode = focusModeStateArray[i].value;
 					break;
 				}
@@ -2232,6 +2235,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 	/* Beep */
 	gp_widget_get_child_by_label (window, _("Beep"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2254,7 +2258,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 					gp_context_status (context, _("Beep mode changed"));
 				else
 					gp_context_status (context, _("Could not change beep mode"));
-			}		
+			}
 		}
 	}
 
@@ -2265,6 +2269,8 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 		gp_widget_get_child_by_label (window, _("Zoom"), &w);
 		if (gp_widget_changed (w)) {
 			float zoom;
+
+	        	gp_widget_set_changed (w, 0);
 			gp_widget_get_value (w, &zoom);
 			if (!check_readiness (camera, context)) {
 				gp_context_status (context, _("Camera unavailable"));
@@ -2273,13 +2279,14 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 						gp_context_status (context, _("Zoom level changed"));
 					else
 						gp_context_status (context, _("Could not change zoom level"));
-				}		
+				}
 			}
 	}
 
 	/* Aperture */
 	gp_widget_get_child_by_label (window, _("Aperture"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2309,6 +2316,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 	/* Flash mode */
 	gp_widget_get_child_by_label (window, _("Flash Mode"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2331,13 +2339,14 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 					gp_context_status (context, _("Flash mode changed"));
 				else
 					gp_context_status (context, _("Could not change flash mode"));
-			}		
+			}
 		}
 	}
 
 
 	gp_widget_get_child_by_label (window, _("Synchronize camera date and time with PC"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		if (!check_readiness (camera, context)) {
 			gp_context_status (context, _("Camera unavailable"));
@@ -2353,6 +2362,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 	gp_widget_get_child_by_label (window, _("List all files"), &w);
 	if (gp_widget_changed (w)) {
 		/* XXXXX mark CameraFS as dirty */
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &val);
 		camera->pl->list_all_files = val;
 		sprintf (str, "%d", val);
@@ -2365,6 +2375,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 #ifdef CANON_EXPERIMENTAL_UPLOAD
 	gp_widget_get_child_by_label (window, _("Keep filename on upload"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &camera->pl->upload_keep_filename);
 		GP_DEBUG ("New config value for \"Keep filename on upload\": %i",
 			  camera->pl->upload_keep_filename);
@@ -2377,9 +2388,9 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 }
 
 static int
-get_info_func (CameraFilesystem __unused__ *fs, const char *folder, 
+get_info_func (CameraFilesystem __unused__ *fs, const char *folder,
 	       const char *filename,
-	       CameraFileInfo * info, 
+	       CameraFileInfo * info,
 	       void __unused__ *data, GPContext __unused__ *context)
 {
 	Camera *camera = data;
@@ -2441,7 +2452,7 @@ make_dir_func (CameraFilesystem __unused__ *fs, const char *folder,
 	if (canonpath == NULL)
 		return GP_ERROR_BAD_PARAMETERS;
 
-	r = canon_int_directory_operations (camera, canonpath, 
+	r = canon_int_directory_operations (camera, canonpath,
 					    DIR_CREATE, context);
 	if (r != GP_OK)
 		return (r);
@@ -2483,7 +2494,7 @@ remove_dir_func (CameraFilesystem __unused__ *fs, const char *folder,
 	if (canonpath == NULL)
 		return GP_ERROR_BAD_PARAMETERS;
 
-	r = canon_int_directory_operations (camera, canonpath, 
+	r = canon_int_directory_operations (camera, canonpath,
 					    DIR_REMOVE, context);
 	if (r != GP_OK)
 		return (r);
@@ -2538,10 +2549,9 @@ camera_init (Camera *camera, GPContext *context)
 
 	/* Set up the CameraFilesystem */
 	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
-	camera->pl = malloc (sizeof (CameraPrivateLibrary));
+	camera->pl = calloc (1, sizeof (CameraPrivateLibrary));
 	if (!camera->pl)
-		return (GP_ERROR_NO_MEMORY);
-	memset (camera->pl, 0, sizeof (CameraPrivateLibrary));
+		return GP_ERROR_NO_MEMORY;
 	camera->pl->first_init = 1;
 	camera->pl->seq_tx = 1;
 	camera->pl->seq_rx = 1;
