@@ -12,16 +12,15 @@
  *
  ****************************************************************************/
 
+#define _DEFAULT_SOURCE
+
 #include "config.h"
 
 #include <stdio.h>
-#include <unistd.h>
-#include <termios.h>
 #include <fcntl.h>
 #include <string.h>
 #include <sys/time.h>
 #include <errno.h>
-#include <sys/ioctl.h>
 #include <stdlib.h>
 #include <ctype.h>
 
@@ -131,6 +130,7 @@ canon_serial_change_speed (GPPort *gdev, int speed)
  *          0 on CTS low.
  *
  */
+#if 0
 static int
 canon_serial_get_cts (GPPort *gdev)
 {
@@ -139,6 +139,7 @@ canon_serial_get_cts (GPPort *gdev)
 	gp_port_get_pin (gdev, PIN_CTS, &level);
 	return (level);
 }
+#endif
 
 /**
  * canon_serial_init
@@ -336,11 +337,7 @@ canon_serial_recv_frame (Camera *camera, int *len)
 		*p++ = c;
 	}
 
-	/* If you don't want to see the data dumped, change the frontend to
-	 * set a lower debug level
-	 */
-	gp_log (GP_LOG_DATA, "canon", "RECV (without CANON_FBEG and CANON_FEND bytes)");
-	gp_log_data ("canon", (char *)buffer, p - buffer);
+	GP_LOG_DATA ((char *)buffer, p - buffer, "RECV (without CANON_FBEG and CANON_FEND bytes)");
 
 	if (len)
 		*len = p - buffer;
@@ -352,8 +349,8 @@ canon_serial_recv_frame (Camera *camera, int *len)
 /**
  * canon_serial_send_packet
  * @camera: Camera object to work with
- * @type: 
- * @seq: 
+ * @type:
+ * @seq:
  * @pkt: data to send to camera
  * @len: length of data
  *
@@ -369,7 +366,7 @@ canon_serial_send_packet (Camera *camera, unsigned char type, unsigned char seq,
 			  unsigned char *pkt, int len)
 {
 	unsigned char *hdr = pkt - PKT_HDR_LEN;
-	unsigned short crc;
+	int crc;
 
 	hdr[PKT_TYPE] = type;
 	hdr[PKT_SEQ] = seq;
@@ -390,6 +387,8 @@ canon_serial_send_packet (Camera *camera, unsigned char type, unsigned char seq,
 	if (type == PKT_EOT || type == PKT_ACK || type == PKT_NACK)
 		len = 2;	/* @@@ hack */
 	crc = canon_psa50_gen_crc (hdr, len + PKT_HDR_LEN);
+	if (crc == -1)
+		return GP_ERROR;
 	pkt[len] = crc & 0xff;
 	pkt[len + 1] = crc >> 8;
 
@@ -570,7 +569,7 @@ canon_serial_send_msg (Camera *camera, unsigned char mtype, unsigned char dir, v
 		len = va_arg (*ap, int);
 
 		if (pos + len - pkt > MAX_MSG_SIZE && camera->pl->uploading != 1) {
-			GP_DEBUG ("FATAL ERROR: message too big (%i)", pos + len - pkt);
+			GP_DEBUG ("FATAL ERROR: message too big (%i)", (int)(pos + len - pkt));
 			return -1;
 		}
 		memcpy (pos, str, len);
@@ -692,7 +691,7 @@ canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, u
 		if (type == PKT_ACK && mtype == 0x3 && dir == 0x21) {
 			GP_DEBUG("ignoring ACK received while waiting for MSG");
 			return frag;
-		} 
+		}
 */
 		if (len < MSG_HDR_LEN || frag[MSG_02] != 2) {
 			GP_DEBUG ("ERROR: message format error");
@@ -722,10 +721,8 @@ canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, u
 			if (msg_pos + len > msg_size || !msg) {
 				msg_size *= 2;
 				msg = realloc (msg, msg_size);
-				if (!msg) {
-					perror ("realloc");
-					exit (1);
-				}
+				if (!msg)
+					return NULL;
 			}
 			memcpy (msg + msg_pos, frag, len);
 			msg_pos += len;
@@ -892,6 +889,7 @@ canon_serial_dialogue (Camera *camera, GPContext *context, unsigned char mtype,
  * Returns: %GP_OK
  *
  */
+#if 0
 static int
 canon_serial_end (Camera *camera)
 {
@@ -899,6 +897,7 @@ canon_serial_end (Camera *camera)
 	canon_serial_send (camera, (unsigned char *)"\xC0\x00\x04\x01\x00\x00\x00\x24\xC6\xC1", 8, USLEEP2);
 	return GP_OK;
 }
+#endif
 
 /**
  * canon_serial_off:
@@ -960,11 +959,10 @@ canon_serial_error_type (Camera *camera)
  *
  */
 int
-canon_serial_put_file (Camera *camera, CameraFile *file, char *destname, char *destpath,
+canon_serial_put_file (Camera *camera, CameraFile *file, const char *name, const char *destname, const char *destpath,
 		       GPContext *context)
 {
 	unsigned char *msg;
-	char filename[64];
 	char buf[4096];
 	int offset = 0;
 	char offset2[4];
@@ -972,18 +970,12 @@ canon_serial_put_file (Camera *camera, CameraFile *file, char *destname, char *d
 	char block_len2[4];
 	unsigned int sent = 0;
 	int i, j = 0;
-	unsigned int len, hdr_len;
+	unsigned int len;
 	unsigned long int size;
-	const char *data, *name;
+	const char *data;
 	unsigned int id;
 
 	camera->pl->uploading = 1;
-	gp_file_get_name (file, &name);
-	for (i = 0; name[i]; i++)
-		filename[i] = toupper (name[i]);
-	filename[i] = '\0';
-
-	hdr_len = HDR_FIXED_LEN + strlen (name) + strlen (destpath);
 
 	gp_file_get_data_and_size (file, &data, &size);
 
@@ -1142,11 +1134,9 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 		return GP_ERROR;
 	}
 
-	/* don't use GP_DEBUG since we log this with GP_LOG_DATA */
-	gp_log (GP_LOG_DATA, "canon",
-		"canon_serial_get_dirents: "
-		"dirent packet received from canon_serial_dialogue:");
-	gp_log_data ("canon", (char *)p, *dirents_length);
+	GP_LOG_DATA ((char *)p, *dirents_length,
+				 "canon_serial_get_dirents: "
+				 "dirent packet received from canon_serial_dialogue:");
 
 	/* the first five bytes is only for the RS232 implementation
 	 * of this command, we do not need to copy them so therefore
@@ -1183,11 +1173,9 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 			return GP_ERROR;
 		}
 
-		/* don't use GP_DEBUG since we log this with GP_LOG_DATA */
-		gp_log (GP_LOG_DATA, "canon",
-			"canon_serial_get_dirents: "
-			"dirent packet received from canon_serial_recv_msg:");
-		gp_log_data ("canon", (char *)p, *dirents_length);
+		GP_LOG_DATA ((char *)p, *dirents_length,
+					 "canon_serial_get_dirents: "
+					 "dirent packet received from canon_serial_recv_msg:");
 
 		/* the first five bytes is only for the RS232 implementation,
 		 * don't count them when checking dirent size
@@ -1380,11 +1368,11 @@ canon_serial_ready (Camera *camera, GPContext *context)
 	 * don't actually use id_str, but serial_id_string.
 	 */
 	for (i = 0; models[i].id_str != NULL; i++) {
-		if ((models[i].serial_id_string != NULL) && 
+		if ((models[i].serial_id_string != NULL) &&
 		    !strcmp (models[i].serial_id_string, cam_id_str)) {
 			GP_DEBUG ("canon_serial_ready: Serial ID string matches '%s'",
 				  models[i].serial_id_string);
-			gp_context_status (context, _("Detected a \"%s\" aka \"%s\""), 
+			gp_context_status (context, _("Detected a \"%s\" aka \"%s\""),
 					   models[i].id_str, models[i].serial_id_string);
 			camera->pl->md = (struct canonCamModelData *) &models[i];
 			break;
@@ -1490,7 +1478,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
  * @context: context for error reporting
  *
  * This is just the serial specific part extracted from the older
- * canon_get_thumbnail() routine. 
+ * canon_get_thumbnail() routine.
  *
  * Returns: gphoto2 error code
  *
