@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include <config.h>
@@ -80,10 +80,7 @@ camera_abilities (CameraAbilitiesList *list)
        		a.speed[0] = 0;
        		a.usb_vendor = models[i].idVendor;
        		a.usb_product= models[i].idProduct;
-       		if (a.status == GP_DRIVER_STATUS_EXPERIMENTAL)
-			a.operations = GP_OPERATION_NONE;
-		else
-			a.operations = GP_OPERATION_NONE;
+		a.operations = GP_OPERATION_NONE;
        		a.folder_operations = GP_FOLDER_OPERATION_DELETE_ALL;
 ;
 		a.file_operations   = GP_FILE_OPERATION_PREVIEW
@@ -154,7 +151,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
         Camera *camera = data;
 	int n;
 	GP_DEBUG ("List files in %s\n", folder);
-    	n = camera->pl->num_pics;	
+    	n = camera->pl->num_pics;
 	gp_list_populate(list, "cs%03i.jpeg", n);
 	return GP_OK;
 }
@@ -175,7 +172,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 
 	/* Get the entry number of the photo on the camera */
     	k = gp_filesystem_number (camera->fs, "/", filename, context);
-    	
+
 	if (GP_FILE_TYPE_EXIF==type) return GP_ERROR_FILE_EXISTS;
 
 	if (GP_FILE_TYPE_RAW!=type && GP_FILE_TYPE_NORMAL !=type && GP_FILE_TYPE_PREVIEW!=type)
@@ -195,31 +192,29 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		jpeg_format = JPEG_QCIF_FORMAT;
 		break;
 	default:  GP_DEBUG ( "Unknown resolution setting %i\n", res);
-		return GP_ERROR;		
+		return GP_ERROR;
 	}
 
-	data = malloc (w*h);
-	if(!data)
-		return GP_ERROR_NO_MEMORY;
+	data = NULL;
 
 	GP_DEBUG("Fetch entry %i\n", k);
-	b = clicksmart_read_pic_data (camera->pl, camera->port, data, k);
+	b = clicksmart_read_pic_data (camera->pl, camera->port, &data, k);
 
 	if (GP_FILE_TYPE_RAW == type) {	/* type is GP_FILE_TYPE_RAW */
 		gp_file_set_mime_type (file, GP_MIME_RAW);
 	        gp_file_set_data_and_size (file, (char *)data, b);
 		/* Reset camera when done, for more graceful exit. */
 		if (k +1 == camera->pl->num_pics) {
-	    		clicksmart_reset (camera->port);	
+	    		clicksmart_reset (camera->port);
 		}
 		return GP_OK;
 	}
-		
-	GP_DEBUG ("size = %i\n", b);	
+
+	GP_DEBUG ("size = %i\n", b);
 
 	/* It looks as though o_size = b */
 	/* It seems that qIndex is byte7, which is always 3, so I use that. */
-	
+
 	file_size = b + 589 + 1024 * 10;
 
 	jpeg_out = malloc(file_size);
