@@ -14,11 +14,11 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 
 #include <config.h>
 
@@ -197,7 +197,8 @@ get_file_func(CameraFilesystem *fs, const char *folder, const char *filename,
 {
 	int status = GP_OK;
 	Camera *camera = user_data;
-	int w, h, b;
+	unsigned int b;
+	unsigned int w, h;
 	int k, next;
 	unsigned char comp_ratio;
 	unsigned char lighting;
@@ -246,7 +247,11 @@ get_file_func(CameraFilesystem *fs, const char *folder, const char *filename,
 		camera->pl->last_fetched_entry = k;
 		return GP_OK;
 	}
-	data = malloc (w*h);
+	if (b < w*h) {
+		GP_DEBUG("need %d bytes, supposed to read only %d", w*h, b);
+		return GP_ERROR;
+	}
+	data = malloc (b);
 	if(!data) return GP_ERROR_NO_MEMORY;
 
 	GP_DEBUG("Fetch entry %i\n", k);
@@ -289,12 +294,14 @@ get_file_func(CameraFilesystem *fs, const char *folder, const char *filename,
 	p_data = malloc(w * h);
 	if (!p_data) {
 		status =  GP_ERROR_NO_MEMORY;
+		free (ppm);
 		goto end;
 	}
 	if(comp_ratio) {
 		digi_decompress (p_data, data, w, h);
 	} else
 		memcpy(p_data, data, w * h);
+	GP_DEBUG("w %d, h %d, size %d", w, h, size);
 	gp_ahd_decode(p_data, w , h , ptr, BAYER_TILE_BGGR);
 	free(p_data);
 	digi_postprocess(w, h, ptr);
@@ -354,25 +361,28 @@ camera_capture_preview(Camera *camera, CameraFile *file, GPContext *context)
 						| get_size[0x43]<<24;
 	GP_DEBUG("b = 0x%x\n", b);
 	raw_data = malloc(b);
-	if(!raw_data) {
-		free(raw_data);
+
+	if(!raw_data)
 		return GP_ERROR_NO_MEMORY;
-	}
+
 	if (!((gp_port_read(camera->port, (char *)raw_data, b)==b))) {
+		free(raw_data);
 		GP_DEBUG("Error in reading data\n");
 		return GP_ERROR;
 	}
 	frame_data = malloc(w * h);
 	if (!frame_data) {
-		free(frame_data);
+		free(raw_data);
 		return GP_ERROR_NO_MEMORY;
 	}
 	digi_decompress (frame_data, raw_data, w, h);
 	free(raw_data);
 	/* Now put the data into a PPM image file. */
 	ppm = malloc (w * h * 3 + 256);
-	if (!ppm)
+	if (!ppm) {
+		free(frame_data);
 		return GP_ERROR_NO_MEMORY;
+	}
 	snprintf ((char *)ppm, 64,
 		"P6\n"
 		"# CREATOR: gphoto2, SQ905C library\n"
@@ -449,7 +459,7 @@ camera_init(Camera *camera, GPContext *context)
 	/* Tell the CameraFilesystem where to get lists from */
 	gp_filesystem_set_funcs(camera->fs, &fsfuncs, camera);
 	camera->pl = malloc(sizeof (CameraPrivateLibrary));
-	if (!camera->pl) 
+	if (!camera->pl)
 		return GP_ERROR_NO_MEMORY;
 	camera->pl->catalog = NULL;
 	camera->pl->nb_entries = 0;
