@@ -1,4 +1,4 @@
-/* 
+/*
  * Program:	gPhoto2 driver for Mustek VDC 3500/Relisys Dimera 3500
  *
  * Note:
@@ -19,6 +19,8 @@
  *
  */
 
+#define _DEFAULT_SOURCE
+
 #include "config.h"
 
 #include "mesalib.h"
@@ -47,8 +49,6 @@
 
 #define GP_MODULE "dimera"
 
-#define ERROR(e) gp_log(GP_LOG_ERROR, GP_MODULE "/" __FILE__, (e))
-
 #ifndef MAX
 # define MAX(a, b) ((a) > (b) ? (a) : (b))
 #endif
@@ -69,7 +69,7 @@
 static const char Dimera_thumbhdr[] =
 "P5\n# Dimera 3500 Thumbnail written by gphoto2\n64 48\n255\n";
 
-static const char Dimera_viewhdr[] = 
+static const char Dimera_viewhdr[] =
 "P5\n# Dimera 3500 Viewfinder written by gphoto2\n128 96\n15\n";
 
 static const char Dimera_finehdr[] =
@@ -93,9 +93,9 @@ Dimera_Preview( long *size, Camera *camera, GPContext *context );
 /* Gphoto2 */
 
 struct _CameraPrivateLibrary {
-        unsigned exposure; 
-        int auto_exposure; 
-        int auto_flash; 
+        unsigned exposure;
+        int auto_exposure;
+        int auto_flash;
 };
 
 
@@ -186,7 +186,7 @@ static int
 conversion_chuck (const unsigned int width, const unsigned int height,
 	const unsigned char *src,unsigned char *dst
 ) {
-	int x, y;
+	unsigned int x, y;
 	int p1, p2, p3, p4;
 	int red, green, blue;
 
@@ -196,7 +196,7 @@ conversion_chuck (const unsigned int width, const unsigned int height,
 	 *
 	 * Convert the 4 cells into a single RGB pixel.
 	 * Colors are encoded as follows:
-	 * 
+	 *
 	 * 		000000000000000000........33
 	 * 		000000000011111111........11
 	 * 		012345678901234567........89
@@ -216,7 +216,7 @@ conversion_chuck (const unsigned int width, const unsigned int height,
 	 * The exception is when x or y are zero then the neighors used are
 	 * the +1 cells.
 	 */
-	
+
 	for (y = 0;y < height; y++)
 		for (x = 0;x < width; x++) {
 			p1 = ((y==0?y+1:y-1)*width) + (x==0?x+1:x-1);
@@ -279,7 +279,6 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	if (num < 0)
 		return num;
 
-	gp_file_set_name (file, filename);
 	switch (type) {
 	case GP_FILE_TYPE_NORMAL:
 		data = Dimera_Get_Full_Image (num, &size,
@@ -295,7 +294,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		newdata = malloc(size*3);
 		if (!newdata) return (GP_ERROR_NO_MEMORY);
 		conversion_chuck (width, height, data, newdata);
-		gp_file_append (file, newdata, size*3);
+		gp_file_append (file, (char *)newdata, size*3);
 		free (newdata);
 		free (data);
 		break;
@@ -306,15 +305,15 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 					      context);
 		if (!data)
 			return GP_ERROR;
-		gp_file_set_data_and_size (file, data, size); /* will take over data ptr ownership */
-		gp_file_set_mime_type (file, GP_MIME_RAW); 
+		gp_file_set_data_and_size (file, (char *)data, size); /* will take over data ptr ownership */
+		gp_file_set_mime_type (file, GP_MIME_RAW);
 		gp_file_adjust_name_for_mime_type (file);
 		break;
 	case GP_FILE_TYPE_PREVIEW:
 		data = Dimera_Get_Thumbnail (num,  &size, camera);
 		if (!data)
 			return GP_ERROR;
-		gp_file_set_data_and_size (file, data, size); /* will take over data ptr ownership */
+		gp_file_set_data_and_size (file, (char *)data, size); /* will take over data ptr ownership */
 		gp_file_set_mime_type (file, GP_MIME_PGM);
 		gp_file_adjust_name_for_mime_type (file);
 		break;
@@ -336,7 +335,7 @@ static int get_info_func (CameraFilesystem *fs, const char *folder, const char *
 
 	if ( (std_res = mesa_read_image_info( camera->port, num, NULL )) < 0 )
 	{
-		ERROR("Can't get Image Info");
+		gp_log(GP_LOG_ERROR, "dimera/dimera3500", "Can't get Image Info");
 		gp_context_error (context, _("Problem getting image information"));
 		return std_res;
 	}
@@ -347,9 +346,8 @@ static int get_info_func (CameraFilesystem *fs, const char *folder, const char *
 	info->preview.width = 64;
 	info->preview.height = 48;
 
-	info->file.fields = GP_FILE_INFO_ALL;
+	info->file.fields = GP_FILE_INFO_TYPE|GP_FILE_INFO_PERMISSIONS|GP_FILE_INFO_WIDTH|GP_FILE_INFO_HEIGHT|GP_FILE_INFO_SIZE;
 	strcpy(info->file.type, GP_MIME_PPM);
-	strcpy(info->file.name, filename);
 	info->file.permissions = GP_FILE_PERM_READ;
 
 	if (std_res) {
@@ -392,13 +390,12 @@ static int camera_capture_preview(Camera *camera, CameraFile *file, GPContext *c
         long size;
 	uint8_t *data;
 
-	gp_file_set_name (file, RAM_IMAGE_TEMPLATE);
 	gp_file_set_mime_type (file, GP_MIME_PGM);
 
         data = Dimera_Preview( &size, camera, context);
         if (!data)
                 return GP_ERROR;
-	return gp_file_set_data_and_size (file, data, size);
+	return gp_file_set_data_and_size (file, (char *)data, size);
 }
 
 static int camera_summary (Camera *camera, CameraText *summary, GPContext *context) {
@@ -444,7 +441,7 @@ static int camera_summary (Camera *camera, CameraText *summary, GPContext *conte
 			eeprom_capacity = eeprom_size_table[eeprom_info[11]];
 	}
 	/* Estimate the number of pictures that this size of flash can hold */
-	hi_pics_max = eeprom_capacity / 2; 
+	hi_pics_max = eeprom_capacity / 2;
 	lo_pics_max = (eeprom_capacity * 13) / 8;
 
 	if (features.feature_bits_lo & AC_PRESENT)
@@ -536,7 +533,7 @@ Dimera_Get_Thumbnail( int picnum, long *size, Camera *camera )
 	if ( !(image = (unsigned char *) malloc( MESA_THUMB_SZ +
 			sizeof( Dimera_thumbhdr ) - 1 )) )
 	{
-		ERROR( "Get Thumbnail, allocation failed" );
+		gp_log(GP_LOG_ERROR, "dimera/dimera3500",  "Get Thumbnail, allocation failed" );
 		*size = 0;
 		return NULL;
 	}
@@ -550,7 +547,7 @@ Dimera_Get_Thumbnail( int picnum, long *size, Camera *camera )
 	if ( (r = mesa_read_thumbnail( camera->port, picnum, image +
 			sizeof( Dimera_thumbhdr ) - 1 )) < 0 )
 	{
-		ERROR( "Get Thumbnail, read of thumbnail failed" );
+		gp_log(GP_LOG_ERROR, "dimera/dimera3500",  "Get Thumbnail, read of thumbnail failed" );
 		free( image );
 		*size = 0;
 		return NULL;
@@ -580,7 +577,7 @@ Dimera_Get_Full_Image (int picnum, long *size, int *width, int *height,
 		GP_DEBUG("Getting Image Info");
 		if ( (r = mesa_read_image_info( camera->port, picnum, NULL )) < 0 )
 		{
-			ERROR("Can't get Image Info");
+			gp_log(GP_LOG_ERROR, "dimera/dimera3500", "Can't get Image Info");
 			gp_context_error (context, _("Problem getting image information"));
 			return NULL;
 		}
@@ -598,7 +595,7 @@ Dimera_Get_Full_Image (int picnum, long *size, int *width, int *height,
 		GP_DEBUG("Loading Image");
 		if ( mesa_load_image( camera->port, picnum ) != GP_OK )
 		{
-			ERROR("Image Load failed");
+			gp_log(GP_LOG_ERROR, "dimera/dimera3500", "Image Load failed");
 			gp_context_error (context, _("Problem reading image from flash"));
 			return NULL;
 		}
@@ -648,7 +645,7 @@ Dimera_Get_Full_Image (int picnum, long *size, int *width, int *height,
 
 			if ( (s == GP_ERROR_TIMEOUT || s == GP_ERROR_CORRUPTED_DATA) && --retry > 0)
 			{
-				GP_DEBUG( "Dimera_Get_Full_Image: retrans"); 
+				GP_DEBUG( "Dimera_Get_Full_Image: retrans");
 				continue;
 			}
 			GP_DEBUG(
@@ -684,7 +681,7 @@ Dimera_Get_Full_Image (int picnum, long *size, int *width, int *height,
 
 			if ( (s == GP_ERROR_TIMEOUT || s == GP_ERROR_CORRUPTED_DATA) && --retry > 0)
 			{
-				GP_DEBUG("Dimera_Get_Full_Image: retrans"); 
+				GP_DEBUG("Dimera_Get_Full_Image: retrans");
 				continue;
 			}
 			GP_DEBUG(
@@ -747,7 +744,7 @@ static unsigned calc_new_exposure(unsigned exposure, unsigned brightness) {
 
 	/* Choose the correct curve for this camera scene */
 	for (i=0; exposure_tables[i].M; ++i) {
-		GP_DEBUG( "brightness %d, %f ", 
+		GP_DEBUG( "brightness %d, %f ",
 			brightness,
 			exp((double)((brightness/16.0) - exposure_tables[i].b) / exposure_tables[i].M));
 		if (exp((double)((brightness/16.0) - exposure_tables[i].b) / exposure_tables[i].M) > exposure)
@@ -782,7 +779,7 @@ Dimera_Preview( long *size, Camera *camera, GPContext *context )
 	if ( !(image = (unsigned char *) malloc( VIEWFIND_SZ +
 			sizeof( Dimera_viewhdr ) - 1 )) )
 	{
-		ERROR( "Get Preview, allocation failed" );
+		gp_log(GP_LOG_ERROR, "dimera/dimera3500",  "Get Preview, allocation failed" );
 		gp_context_error (context, _("Out of memory"));
 		return NULL;
 	}
@@ -796,7 +793,7 @@ Dimera_Preview( long *size, Camera *camera, GPContext *context )
 	if ( mesa_snap_view( camera->port, buffer, TRUE, 0, 0, 0, camera->pl->exposure,
 			VIEW_TYPE) < 0 )
 	{
-		ERROR( "Get Preview, mesa_snap_view failed" );
+		gp_log(GP_LOG_ERROR, "dimera/dimera3500",  "Get Preview, mesa_snap_view failed" );
 		free (image);
 		gp_context_error (context, _("Problem taking live image"));
 		return NULL;
@@ -814,7 +811,7 @@ Dimera_Preview( long *size, Camera *camera, GPContext *context )
 
 	/* Automatic exposure control */
 
-	/* Current picture brightness, where 0 is is dark and 255 is bright */
+	/* Current picture brightness, where 0 is dark and 255 is bright */
 	brightness = exposure_total / (VIEWFIND_SZ / 16);
 
 	GP_DEBUG(
@@ -874,6 +871,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("Exposure level on preview"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &wvalue);
 		camera->pl->exposure = MAX(MIN_EXPOSURE,MIN(MAX_EXPOSURE,atoi(wvalue)));
 		gp_setting_set ("dimera3500", "exposure", wvalue);
@@ -882,6 +880,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("Automatic exposure adjustment on preview"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &val);
 		camera->pl->auto_exposure = val;
 		sprintf(str, "%d", val);
@@ -891,6 +890,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	gp_widget_get_child_by_label (window, _("Automatic flash on capture"), &w);
 	if (gp_widget_changed (w)) {
+	        gp_widget_set_changed (w, 0);
 		gp_widget_get_value (w, &val);
 		camera->pl->auto_flash = val;
 		sprintf(str, "%d", val);
@@ -958,7 +958,7 @@ int camera_init (Camera *camera, GPContext *context) {
         GP_DEBUG("Opening port");
         if ( (ret = mesa_port_open(camera->port)) != GP_OK)
         {
-                ERROR("Camera Open Failed");
+                gp_log(GP_LOG_ERROR, "dimera/dimera3500", "Camera Open Failed");
 		free (camera->pl);
 		camera->pl = NULL;
 		gp_context_error (context, _("Problem opening port"));
@@ -968,7 +968,7 @@ int camera_init (Camera *camera, GPContext *context) {
         GP_DEBUG("Resetting camera");
         if ( (ret = mesa_reset(camera->port)) != GP_OK )
         {
-                ERROR("Camera Reset Failed");
+                gp_log(GP_LOG_ERROR, "dimera/dimera3500", "Camera Reset Failed");
 		free (camera->pl);
 		camera->pl = NULL;
 		gp_context_error (context, _("Problem resetting camera"));
@@ -978,7 +978,7 @@ int camera_init (Camera *camera, GPContext *context) {
         GP_DEBUG("Setting speed");
         if ( (ret = mesa_set_speed(camera->port, selected_speed)) != GP_OK )
         {
-                ERROR("Camera Speed Setting Failed");
+                gp_log(GP_LOG_ERROR, "dimera/dimera3500", "Camera Speed Setting Failed");
 		free (camera->pl);
 		camera->pl = NULL;
 		gp_context_error (context, _("Problem setting camera communication speed"));
@@ -991,13 +991,13 @@ int camera_init (Camera *camera, GPContext *context) {
         {
         case GP_ERROR_IO:
         case GP_ERROR_TIMEOUT:
-                ERROR("No or Unknown Response");
+                gp_log(GP_LOG_ERROR, "dimera/dimera3500", "No or Unknown Response");
 		free (camera->pl);
 		camera->pl = NULL;
 		gp_context_error (context, _("No response from camera"));
                 return GP_ERROR_TIMEOUT;
         case GP_ERROR_MODEL_NOT_FOUND:
-                ERROR("Probably a modem");
+                gp_log(GP_LOG_ERROR, "dimera/dimera3500", "Probably a modem");
 		free (camera->pl);
 		camera->pl = NULL;
 		gp_context_error (context, _("Looks like a modem, not a camera"));
