@@ -1,5 +1,5 @@
 /*
-* Copyright  2000-2001, Brian Beattie <beattie@aracnet.com>, et. al.
+* Copyright 2000-2001, Brian Beattie <beattie@aracnet.com>, et. al.
 *
 *	This software was created with the help of proprietary
 *      information belonging to StarDot Technologies
@@ -17,7 +17,7 @@
 *		* Add checksum to mesa_read_row()			  GDB
 *		* Add return with number of bytes to mesa_read_image()	  GDB
 *									  GDB
-* $Id: mesalib.c 9330 2006-10-14 11:54:47Z hun $
+* $Id$
 */
 #include "config.h"
 
@@ -25,7 +25,6 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <unistd.h>
 #include <sys/time.h>
 #include <string.h>
 
@@ -106,7 +105,7 @@ mesa_flush( GPPort *port, int timeout )
 
 	/* Wait for silence */
 	do {
-		if ( gp_port_read( port, b, sizeof( b ) ) > 0 )
+		if ( gp_port_read( port, (char *)b, sizeof( b ) ) > 0 )
 			/* reset timer */
 			gettimeofday( &start, NULL );
 		gettimeofday( &now, NULL );
@@ -115,7 +114,7 @@ mesa_flush( GPPort *port, int timeout )
 }
 
 /* Read exactly this number of bytes from the port within the given timeouts */
-int
+unsigned int
 mesa_read( GPPort *port, uint8_t *b, int s, int timeout2, int timeout1 )
 {
 	int		n = 0;
@@ -128,7 +127,7 @@ mesa_read( GPPort *port, uint8_t *b, int s, int timeout2, int timeout1 )
 	do
 	{
 		/* limit reads to 1k segment */
-		r = gp_port_read( port, &b[n], s>1024?1024:s );
+		r = gp_port_read( port, (char *)&b[n], s>1024?1024:s );
 		if ( r > 0 )
 		{
 			n += r;
@@ -147,7 +146,7 @@ mesa_send_command( GPPort *port, uint8_t *cmd, int n, int ackTimeout )
 {
 	uint8_t	c;
 
-	CHECK (gp_port_write( port, cmd, n ));
+	CHECK (gp_port_write( port, (char *)cmd, n ));
 
 	if ( mesa_read( port, &c, 1, ackTimeout, 0 ) != 1 )
 	{
@@ -205,7 +204,7 @@ mesa_set_speed( GPPort *port, int speed )
 	if (speed == 0)
 		speed = 115200;		/* use default speed */
 
-	GP_DEBUG( 
+	GP_DEBUG(
 	 "mesa_set_speed: speed %d", speed);
 
 	b[0] = SET_SPEED;
@@ -267,7 +266,7 @@ mesa_version( GPPort *port, char *version_string)
 		return GP_ERROR_TIMEOUT;
 	}
 
-	GP_DEBUG( 
+	GP_DEBUG(
 	 "mesa_version: %02x:%02x:%02x\n", r[0], r[1], r[2] );
 	sprintf(v, "%2x.%02x%c", r[1], r[0], r[2]);
 	/* highest byte must be < MESA_VERSION_SZ */
@@ -324,7 +323,7 @@ mesa_ram_test( GPPort *port )
  * *   The following contradicted by traces from my Win98 twain driver*	  GDB
  * ********************************************************************   GDB
  * The image is stored in the camera as 504 row (0 - 503), the first 8 and the
- * last 2 are blank. 
+ * last 2 are blank.
  * The rows are 18 dummy pixels, 2 blanks, 646 pixels, 13 bytes garbage,
  * 2 bytes blanks
  *
@@ -437,11 +436,11 @@ mesa_black_levels( GPPort *port, uint8_t r[2] )
  * the image can be full scale, 0, zoom 2x 1, zoom 4x, 2, and for
  * low res images, 8x, 3.
  *
- * row and column may be offset 0-197 and 0-223 for zoom. 
+ * row and column may be offset 0-197 and 0-223 for zoom.
  *
  * exposure is * 1/50000 seconds.
  *
- * download specifies how, of if thge image is downloaded.
+ * download specifies how, of if the image is downloaded.
  * 0-47		returns one 32 byte (64 pixel) row.
  * 128-223	returns on 64 byte row.
  * 249		returns all odd rows of a hi-res image 1536 bytes.
@@ -472,7 +471,7 @@ mesa_snap_view( GPPort *port, uint8_t *r, unsigned int hi_res,
 		return GP_ERROR_BAD_PARAMETERS;
 	else if ( download == 249 || download == 250 )
 		bytes = 1536;
-	else if ( download == 251 ) 
+	else if ( download == 251 )
 		bytes = 6144;
 	else if ( download == 252 )
 		bytes = 0;
@@ -521,7 +520,7 @@ mesa_snap_view( GPPort *port, uint8_t *r, unsigned int hi_res,
 			return GP_ERROR_CORRUPTED_DATA;
 		}
 	}
-	
+
 	return bytes;
 }
 
@@ -557,7 +556,7 @@ mesa_download_view( GPPort *port, uint8_t *r, uint8_t download )
 		return GP_ERROR_BAD_PARAMETERS;
 	else if ( download == 249 )
 		bytes = 1536;
-	else if ( download == 250 || download == 251 ) 
+	else if ( download == 250 || download == 251 )
 		bytes = 768;
 	else if ( download == 252 )
 		bytes = 0;
@@ -595,7 +594,7 @@ mesa_download_view( GPPort *port, uint8_t *r, uint8_t download )
 		if ( cksum != b[0] )
 			return GP_ERROR_CORRUPTED_DATA;
 	}
-	
+
 	return bytes;
 }
 
@@ -659,7 +658,7 @@ mesa_modem_check( GPPort *port )
 	b[1] = 'T';
 	b[2] = '\r';
 
-	CHECK (gp_port_write( port, b, sizeof( b ) ));
+	CHECK (gp_port_write( port, (char *)b, sizeof( b ) ));
 
 	/* Expect at least one byte */
 	if ( mesa_read( port, b, 1, 5, 0 ) < 1 )
@@ -754,7 +753,7 @@ mesa_recv_test( GPPort *port, uint8_t r[6] )
 {
 	uint8_t		b[7];
 	int		i;
-	
+
 	b[0] = RCV_TEST;
 	memcpy( &b[1], r, 6 );
 
