@@ -1,9 +1,9 @@
 /****************************************************************/
 /* gsmart300.c - Gphoto2 library for the Mustek gSmart 300      */
 /*                                                              */
-/* Copyright (C) 2002 Jrme Lodewyck                           */
+/* Copyright (C) 2002 Jérôme Lodewyck                           */
 /*                                                              */
-/* Author: Jrme Lodewyck <jerome.lodewyck@ens.fr>             */
+/* Author: Jérôme Lodewyck <jerome.lodewyck@ens.fr>             */
 /*                                                              */
 /* based on code by: Till Adam <till@adam-lilienthal.de>        */
 /*                                                              */
@@ -21,11 +21,11 @@
 /*                                                              */
 /* You should have received a copy of the GNU Library General   */
 /* Public License along with this library; if not, write to the */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
-/* Boston, MA 02111-1307, USA.                                  */
+/* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
+/* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
 
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 
 #include <config.h>
 
@@ -57,6 +57,10 @@
 #  define N_(String) (String)
 #endif
 
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+#define sleep(x)
+#endif
+
 #define GP_MODULE "gsmart300"
 static int gsmart300_download_data (CameraPrivateLibrary * lib,
 		int data_type, uint16_t index, unsigned int size,
@@ -140,8 +144,18 @@ gsmart300_request_file (CameraPrivateLibrary * lib, CameraFile *file,
 		  + (p[11] & 0xff);
 	qIndex = p[7] & 0x07;
 
+	if (qIndex >= 5) {
+		GP_DEBUG("qIndex %d is larger or equal than 5\n", qIndex);
+		return GP_ERROR_CORRUPTED_DATA;
+	}
+
 	file_size = data_size + GSMART_JPG_DEFAULT_HEADER_LENGTH + 1024 * 10;
 
+	if (flash_size < data_size) {
+		GP_DEBUG("flash_size %d is smaller than data_size %d\n", flash_size, data_size);
+		return GP_ERROR_CORRUPTED_DATA;
+	}
+
 	/* slurp in the image */
 	mybuf = malloc (flash_size);
 	if (!mybuf)
@@ -322,7 +336,7 @@ gsmart300_reset (CameraPrivateLibrary * lib)
 	GP_DEBUG ("* gsmart300_reset");
 	CHECK (gp_port_usb_msg_write (lib->gpdev, 0x02, 0x0000, 0x0003,
 				      NULL, 0));
-	sleep (1.5);
+	sleep (1);
 	return GP_OK;
 }
 
@@ -334,7 +348,7 @@ static int gsmart300_download_data (CameraPrivateLibrary * lib, int data_type,
 				 uint16_t index, unsigned int size, uint8_t * buf)
 {
 	uint16_t fat_index = 0x1fff - index;
-	int i;
+	unsigned int i;
 
 	if (data_type == __GS300_FAT)
 		CHECK (gp_port_usb_msg_write (lib->gpdev, 0x03,
@@ -362,7 +376,7 @@ static int
 gsmart300_get_FATs (CameraPrivateLibrary * lib)
 {
 	uint8_t type;
-	unsigned int index = 0;
+	int index = 0;
 	unsigned int file_index = 0;
 	uint8_t *p = NULL;
 	char buf[14];
@@ -371,11 +385,11 @@ gsmart300_get_FATs (CameraPrivateLibrary * lib)
 
 	if (lib->fats)
 		free (lib->fats);
-	lib->fats = malloc ((lib->num_files) * FLASH_PAGE_SIZE_300);
+	lib->fats = calloc (lib->num_files,FLASH_PAGE_SIZE_300);
 
 	if (lib->files)
 		free (lib->files);
-	lib->files = malloc (lib->num_files * sizeof (struct GsmartFile));
+	lib->files = calloc (lib->num_files,sizeof (struct GsmartFile));
 
 	p = lib->fats;
 
