@@ -15,11 +15,11 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 
 #include <config.h>
 
@@ -60,7 +60,7 @@ static struct {
    	unsigned short idProduct;
 } models[] = {
         {"iClick 5X",    GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9153},
-	{NULL,0,0}
+	{NULL,0,0,0}
 };
 
 int
@@ -111,7 +111,7 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *context)
 
 static int camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 {
-	strcpy(manual->text, 	
+	strcpy(manual->text,
 	_(
 	"Information regarding cameras with ID 0x2770:0x9153.\n\n"
 	"We do not recommend the use of a GUI program to access\n"
@@ -172,7 +172,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	unsigned char *ppm, *ptr;
 	unsigned char gtable[256];
 	int start;
-	int datasize, framesize, hdrsize, ppmsize;
+	int datasize, hdrsize, ppmsize;
 	int nb_frames=1;
 	unsigned char buf[0x8000];
 
@@ -239,9 +239,6 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		camera->pl->data_offset = start;
 	}
 
-	/* Retrieve frames */
-	framesize = datasize;
-
 	frame_data = malloc(datasize);
 	if (!frame_data) return GP_ERROR_NO_MEMORY;
 	icl_read_picture_data(camera->port, frame_data, datasize);
@@ -258,6 +255,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		if (icl_get_width_height (camera->pl, entry, &w, &h) >= GP_OK)
 			break; /* Known format, process image */
 		/* Unsupported format, fallthrough to raw */
+		/* fallthrough */
 	case GP_FILE_TYPE_RAW:
 		gp_file_set_mime_type (file, GP_MIME_RAW);
 		gp_file_adjust_name_for_mime_type (file);
