@@ -1,7 +1,7 @@
 /****************************************************************/
 /* library.c  - Gphoto2 library for the KBGear JamCam v2 and v3 */
 /*                                                              */
-/* Copyright  2001 Chris Pinkham                             */
+/* Copyright 2001 Chris Pinkham                                 */
 /*                                                              */
 /* Author: Chris Pinkham <cpinkham@infi.net>                    */
 /*                                                              */
@@ -19,8 +19,8 @@
 /*                                                              */
 /* You should have received a copy of the GNU Library General   */
 /* Public License along with this library; if not, write to the */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
-/* Boston, MA 02111-1307, USA.                                  */
+/* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
+/* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
 #include "config.h"
 
@@ -29,7 +29,6 @@
 #include <stdlib.h>
 #include <gphoto2/gphoto2.h>
 #include <time.h>
-#include <unistd.h>
 #include <math.h>
 
 #include "library.h"
@@ -56,13 +55,13 @@
 #define GP_MODULE
 
 static struct jamcam_file jamcam_files[1024];
-static int jamcam_count = 0;
+static unsigned int jamcam_count = 0;
 static int jamcam_mmc_card_size = 0;
 
 static int jamcam_read_packet (Camera *camera, unsigned char *packet, int length);
 static int jamcam_write_packet (Camera *camera, unsigned char *packet, int length);
 static int jamcam_fetch_memory( Camera *camera, CameraFile *file,
-		unsigned char *data, int start, int length, GPContext *context);
+		unsigned char *data, unsigned int start, unsigned int length, GPContext *context);
 static int jamcam_query_mmc_card (Camera *camera);
 
 static int jamcam_set_int_at_pos( unsigned char *buf, int pos, int value ) {
@@ -71,10 +70,10 @@ static int jamcam_set_int_at_pos( unsigned char *buf, int pos, int value ) {
 	buf[pos + 2] = ( value >> 16 ) & 0xff;
 	buf[pos + 3] = ( value >> 24 ) & 0xff;
 
-	return( value );
+	return value;
 }
 
-static int jamcam_get_int_at_pos( unsigned char *buf, int pos ) {
+static unsigned int jamcam_get_int_at_pos( unsigned char *buf, int pos ) {
 	int ret = 0;
 
 	ret += buf[pos + 0];
@@ -82,10 +81,10 @@ static int jamcam_get_int_at_pos( unsigned char *buf, int pos ) {
 	ret += buf[pos + 2] * 256 * 256;
 	ret += buf[pos + 3] * 256 * 256 * 256;
 
-	return( ret );
+	return ret;
 }
 
-static int jamcam_set_usb_mem_pointer( Camera *camera, int position ) {
+static int jamcam_set_usb_mem_pointer( Camera *camera, unsigned int position ) {
 	char reply[8];
 
 	GP_DEBUG ("* jamcam_set_usb_mem_pointer");
@@ -104,7 +103,7 @@ static int jamcam_set_usb_mem_pointer( Camera *camera, int position ) {
 		0,
 		reply, 8 );
 
-	return( GP_OK );
+	return GP_OK;
 }
 
 
@@ -113,9 +112,9 @@ static int jamcam_mmc_card_file_count (Camera *camera) {
 	unsigned char buf[16];
 	unsigned char reply[512];
 	unsigned int position = 0x40000000;
-	int data_incr;
-	int width;
-	int height;
+	unsigned int data_incr;
+	unsigned int width;
+	unsigned int height;
 
 	GP_DEBUG ("* jamcam_mmc_card_file_count");
 
@@ -141,13 +140,15 @@ static int jamcam_mmc_card_file_count (Camera *camera) {
 				jamcam_files[jamcam_count].height = height;
 				jamcam_files[jamcam_count].data_incr = data_incr;
 
+				if (jamcam_count+1 >= sizeof(jamcam_files)/sizeof(jamcam_files[0]))
+					break;
 				jamcam_count++;
 
 				position += data_incr;
 
 				jamcam_set_int_at_pos( buf, 4, position );
 				jamcam_write_packet( camera, buf, 8 );
-			
+
 				jamcam_read_packet( camera, reply, 16 );
 			}
 			break;
@@ -186,6 +187,8 @@ static int jamcam_mmc_card_file_count (Camera *camera) {
 				jamcam_files[jamcam_count].width = width;
 				jamcam_files[jamcam_count].height = height;
 				jamcam_files[jamcam_count].data_incr = data_incr;
+				if (jamcam_count+1 >= sizeof(jamcam_files)/sizeof(jamcam_files[0]))
+					break;
 				jamcam_count++;
 
 				position += data_incr;
@@ -216,7 +219,7 @@ static int jamcam_mmc_card_file_count (Camera *camera) {
 			break;
 	}
 
-	GP_DEBUG (		"*** returning with jamcam_count = %d", jamcam_count);
+	GP_DEBUG (		"*** returning with jamcam_count = %u", jamcam_count);
 	return( 0 );
 }
 
@@ -256,6 +259,8 @@ int jamcam_file_count (Camera *camera) {
 				jamcam_files[jamcam_count].width = width;
 				jamcam_files[jamcam_count].height = height;
 				jamcam_files[jamcam_count].data_incr = data_incr;
+				if (jamcam_count+1 >= sizeof(jamcam_files)/sizeof(jamcam_files[0]))
+					break;
 
 				jamcam_count++;
 
@@ -263,7 +268,7 @@ int jamcam_file_count (Camera *camera) {
 
 				jamcam_set_int_at_pos( buf, 4, position );
 				jamcam_write_packet( camera, buf, 8 );
-			
+
 				jamcam_read_packet( camera, reply, 16 );
 			}
 
@@ -292,6 +297,8 @@ int jamcam_file_count (Camera *camera) {
 				jamcam_files[jamcam_count].width = width;
 				jamcam_files[jamcam_count].height = height;
 				jamcam_files[jamcam_count].data_incr = data_incr;
+				if (jamcam_count+1 >= sizeof(jamcam_files)/sizeof(jamcam_files[0]))
+					break;
 				jamcam_count++;
 
 				position += data_incr;
@@ -314,19 +321,19 @@ int jamcam_file_count (Camera *camera) {
 		jamcam_count += jamcam_mmc_card_file_count( camera );
 	}
 
-	GP_DEBUG (		"*** returning jamcam_count = %d", jamcam_count);
+	GP_DEBUG (		"*** returning jamcam_count = %u", jamcam_count);
 	return( jamcam_count );
 }
 
 static int jamcam_fetch_memory( Camera *camera, CameraFile *file,
-		unsigned char *data, int start, int length, GPContext *context) {
+		unsigned char *data, unsigned int start, unsigned int length, GPContext *context) {
 	unsigned char tmp_buf[16];
 	unsigned char packet[16];
-	int new_start;
-	int new_end;
-	int bytes_read = 0;
-	int bytes_to_read;
-	int bytes_left = length;
+	unsigned int new_start;
+	unsigned int new_end;
+	unsigned int bytes_read = 0;
+	unsigned int bytes_to_read;
+	unsigned int bytes_left = length;
 	int res = GP_OK;
 	unsigned int id = 0;
 
@@ -378,7 +385,7 @@ static int jamcam_fetch_memory( Camera *camera, CameraFile *file,
 
 				break;
 		}
-				
+
 		bytes_left -= bytes_to_read;
 		bytes_read += bytes_to_read;
 
@@ -399,12 +406,12 @@ static int jamcam_fetch_memory( Camera *camera, CameraFile *file,
 	if ( res == GP_OK ) {
 		GP_DEBUG ("  * returning OK");
 	}
-	return( res );
+	return res;
 }
 
 int jamcam_request_image( Camera *camera, CameraFile *file,
-		char *buf, int *len, int number, GPContext *context ) {
-	int position;
+		char *buf, unsigned int *len, int number, GPContext *context ) {
+	unsigned int position;
 	int result;
 	unsigned char *tmp_buf;
 
@@ -425,9 +432,11 @@ int jamcam_request_image( Camera *camera, CameraFile *file,
 			0x0000,
 			NULL, 0 );
 	}
-	
-	result = jamcam_fetch_memory( camera, file, tmp_buf, position,
-		jamcam_files[number].data_incr, context );
+	if (jamcam_files[number].data_incr <= 640*480*3)
+		result = jamcam_fetch_memory( camera, file, tmp_buf, position,
+			jamcam_files[number].data_incr, context );
+	else
+		result = GP_ERROR;
 
 	/* this seems to reset the camera to a sane status */
 	if ( camera->port->type == GP_PORT_USB ) {
@@ -440,11 +449,12 @@ int jamcam_request_image( Camera *camera, CameraFile *file,
 
 	if ( result == GP_OK ) {
 		*len = jamcam_files[number].width * jamcam_files[number].height;
-		memcpy( buf, tmp_buf + 0x10, *len );
+		if (*len < 640*480*3)
+			memcpy( buf, tmp_buf + 0x10, *len );
 	}
 	free (tmp_buf);
 
-	return( result );
+	return result;
 }
 
 struct jamcam_file *jamcam_file_info(Camera *camera, int number)
@@ -454,14 +464,14 @@ struct jamcam_file *jamcam_file_info(Camera *camera, int number)
 }
 
 int jamcam_request_thumbnail( Camera *camera, CameraFile *file,
-		char *buf, int *len, int number, GPContext *context ) {
+		char *buf, unsigned int *len, int number, GPContext *context ) {
 	unsigned char line[2048];
 	char packet[16];
-	int position;
+	unsigned int position;
 	int x, y;
 	int res = GP_OK;
 	char *ptr;
-	int bytes_to_read;
+	unsigned int bytes_to_read;
 	unsigned int id;
 
 	GP_DEBUG ("* jamcam_request_thumbnail");
@@ -495,6 +505,11 @@ int jamcam_request_thumbnail( Camera *camera, CameraFile *file,
 		}
 	}
 
+	if (bytes_to_read > sizeof(line)) {
+		res = GP_ERROR;
+		goto resetcam;
+	}
+
 	/* fetch thumbnail lines and build the thumbnail */
 	position += 10 * jamcam_files[number].width;
 	id = gp_context_progress_start (context, 60.,
@@ -530,6 +545,7 @@ int jamcam_request_thumbnail( Camera *camera, CameraFile *file,
 	}
 	gp_context_progress_stop (context, id);
 
+resetcam:
 	/* this seems to reset the camera to a sane status */
 	if ( camera->port->type == GP_PORT_USB ) {
 		gp_port_usb_msg_write( camera->port,
@@ -539,7 +555,7 @@ int jamcam_request_thumbnail( Camera *camera, CameraFile *file,
 			NULL, 0 );
 	}
 
-	return( res );
+	return res;
 }
 
 static int jamcam_write_packet (Camera *camera, unsigned char *packet, int length) {
