@@ -1,3 +1,24 @@
+/* library.c
+ *
+ * Copyright (C) 2001,2002 Hubert Figuiere <hfiguiere@teaser.fr>
+ * Copyright (C) 2000,2001,2002 Scott Fritzinger
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 /*
   Kodak DC 240/280/3400/5000 driver.
   Maintainer:
@@ -39,6 +60,10 @@
 
 #define GP_MODULE "dc240"
 
+/* do not sleep during fuzzing */
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+# define usleep(x)
+#endif
 
 /* legacy from dc240.h */
 /*
@@ -89,7 +114,7 @@ static unsigned char *
 dc240_packet_new_path (const char *folder, const char *filename) {
     unsigned char *p;
     char buf[1024];
-    int x;
+    size_t x;
     unsigned char cs = 0;
 
     p = malloc(sizeof(char)*60);
@@ -129,7 +154,7 @@ static int dc240_packet_write (Camera *camera, unsigned char *packet, int size,
 write_again:
     /* If retry, give camera some recup time */
     if (x > 0) {
-        GP_SYSTEM_SLEEP(SLEEP_TIMEOUT);
+        usleep(SLEEP_TIMEOUT * 1000);
     }
 
     /* Return error if too many retries */
@@ -144,10 +169,13 @@ write_again:
 
     /* Read in the response from the camera if requested */
     while (read_response) {
-        if (gp_port_read(camera->port, in, 1) >= GP_OK) {
+	int ret;
+        if ((ret=gp_port_read(camera->port, in, 1)) >= GP_OK) {
             /* On error, read again */
 	    read_response = 0;
+            break;
 	}
+        if (ret == GP_ERROR_IO_READ) return ret; /* e.g. device detached? */
     }
 
     return GP_OK;
@@ -213,7 +241,7 @@ static int dc240_wait_for_completion (Camera *camera) {
 	    break;
 	case GP_ERROR_TIMEOUT:
 	    GP_DEBUG ("GP_ERROR_TIMEOUT\n");
-	    /* in busy state, GP_ERROR_IO_READ can happend */
+	    /* in busy state, GP_ERROR_IO_READ can happened */
 	    break;
 	default:
 	    done = 1;
@@ -237,7 +265,7 @@ static int dc240_wait_for_busy_completion (Camera *camera)
 	BUSY_RETRIES = 100
     };
     unsigned char p[8];
-    int retval;
+    int retval = 0;
     int x=0, done=0;
 
     /* Wait for command completion */
@@ -249,7 +277,7 @@ static int dc240_wait_for_busy_completion (Camera *camera)
 	    break;
 	case GP_ERROR_IO_READ:
 	case GP_ERROR_TIMEOUT:
-	    /* in busy state, GP_ERROR_IO_READ can happend */
+	    /* in busy state, GP_ERROR_IO_READ can happened */
 	    break;
 	default:
 	    if (*p != DC240_SC_BUSY) {
@@ -379,8 +407,7 @@ read_data_read_again:
     }
     gp_context_progress_stop (context, id);
     /* Read in command completed */
-    dc240_wait_for_completion(camera);
-    return (GP_OK);
+    return dc240_wait_for_completion(camera);
 }
 
 
@@ -438,7 +465,10 @@ static int dc240_get_file_size (Camera *camera, const char *folder, const char *
     if (dc240_packet_exchange(camera, f, p1, p2, &size, 256, context) < 0)
         size = 0;
     else {
-	gp_file_get_data_and_size (f, (const char**)&fdata, &fsize);
+	int ret;
+	ret = gp_file_get_data_and_size (f, (const char**)&fdata, &fsize);
+	if (ret < GP_OK) return ret;
+	if (!fdata || (fsize < 4)) return GP_ERROR;
         size = (fdata[offset]   << 24) |
                (fdata[offset+1] << 16) |
                (fdata[offset+2] << 8 ) |
@@ -509,7 +539,7 @@ int dc240_set_speed (Camera *camera, int speed)
     if (retval != GP_OK)
         goto fail;
 
-    GP_SYSTEM_SLEEP(300);
+    usleep(300 * 1000);
     retval = dc240_wait_for_completion(camera);
     if (retval != GP_OK)
 	goto fail;
@@ -595,7 +625,7 @@ const char * dc240_get_memcard_status_str(uint8_t status)
 
 
 /*
-  Feed manually the stucture from data.
+  Feed manually the structure from data.
  */
 static int dc240_load_status_data_to_table (const unsigned char *fdata, DC240StatusTable *table)
 {
@@ -678,16 +708,21 @@ int dc240_get_status (Camera *camera, DC240StatusTable *table, GPContext *contex
     retval = dc240_packet_exchange(camera, file, p, NULL, &size, 256, context);
 
     if (retval == GP_OK) {
-	gp_file_get_data_and_size (file, &fdata, &fsize);
+	retval = gp_file_get_data_and_size (file, &fdata, &fsize);
+	if (retval != GP_OK) goto exit;
 	if (fsize != 256) {
 	    GP_DEBUG ("wrong status packet size ! Size is %ld", fsize);
+	    retval = GP_ERROR;
+	    goto exit;
 	}
 	if (fdata [0] != 0x01) { /* see 2.6 for why 0x01 */
 	    GP_DEBUG ("not a status table. Is %d", fdata [0]);
+	    retval = GP_ERROR;
+	    goto exit;
 	}
 	dc240_load_status_data_to_table ((uint8_t *)fdata, table);
     }
-
+exit:
     gp_file_free(file);
     free (p);
 
@@ -698,7 +733,8 @@ int dc240_get_directory_list (Camera *camera, CameraList *list, const char *fold
                              unsigned char attrib, GPContext *context) {
 
     CameraFile *file;
-    int x, y=0, z, size=256;
+    unsigned int x, y=0, z;
+    int size=256;
     char buf[64];
     unsigned char *p1 = dc240_packet_new(0x99);
     unsigned char *p2 = dc240_packet_new_path(folder, NULL);
@@ -706,7 +742,7 @@ int dc240_get_directory_list (Camera *camera, CameraList *list, const char *fold
     unsigned long int fsize;
     int ret;
     int num_of_entries = 0; /* number of entries in the listing */
-    int total_size = 0; /* total useful size of the listing */
+    unsigned int total_size = 0; /* total useful size of the listing */
 
     gp_file_new(&file);
     ret = dc240_packet_exchange(camera, file, p1, p2, &size, 256, context);
@@ -718,13 +754,26 @@ int dc240_get_directory_list (Camera *camera, CameraList *list, const char *fold
     free(p2);
 
     /* Don't expect to have a fully useful buffer. */
-    gp_file_get_data_and_size (file, &fdata, &fsize);
+    ret = gp_file_get_data_and_size (file, &fdata, &fsize);
+    if (ret < 0) {
+	gp_file_free (file);
+        return ret;
+    }
+    if ((size < 1) || !fdata) {
+	gp_file_free (file);
+        return GP_ERROR;
+    }
 
     /* numbers in DC 240 are Big-Endian. */
     /* Conversion below should be endian neutral. */
     num_of_entries = be16atoh(&fdata [0]) + 1;
     total_size = 2 + (num_of_entries * 20);
     GP_DEBUG ("number of file entries : %d, size = %ld", num_of_entries, fsize);
+    if (total_size > fsize) {
+        GP_DEBUG ("total_size %d > fsize %ld", total_size, fsize);
+	gp_file_free (file);
+        return GP_ERROR;
+    }
     for (x = 2; x < total_size; x += 20) {
         if ((fdata[x] != '.') && (attrib == (unsigned char)fdata[x+11]))  {
             /* Files have attrib 0x00, Folders have attrib 0x10 */
@@ -763,9 +812,9 @@ int dc240_file_action (Camera *camera, int action, CameraFile *file,
     case DC240_ACTION_PREVIEW:
         cmd_packet[4] = 0x02;
         thumb = 1;
-        /* no break on purpose */
+	/* fallthrough */
     case DC240_ACTION_IMAGE:
-        if ((size = dc240_get_file_size(camera, folder, filename, thumb, context)) < 0) {
+        if ((size = dc240_get_file_size(camera, folder, filename, thumb, context)) < GP_OK) {
             retval = GP_ERROR;
             break;
         }
