@@ -7,15 +7,15 @@
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include <config.h>
@@ -30,7 +30,7 @@
 
 #include "lg_gsm.h"
 
-#define GP_MODULE "lg_gsm" 
+#define GP_MODULE "lg_gsm"
 
 #define MSGWRITE gp_port_usb_msg_write
 #define WRITE gp_port_write
@@ -42,7 +42,7 @@ static char get_firmware[] = "\x1\x0\x0\x0\x0\x0";
 static char list_all_photo[]= "\x04\x0\x08\x0\x0\x0\x40\x0\x0\x0\xff\xff\xff\xff";
 static char get_photo_cmd[]="\x0b\x0\x8a\x0\x0\x0\x40\x0\x0\x0";
 
-int lg_gsm_init (GPPort *port, Model *model, Info *info) 
+int lg_gsm_init (GPPort *port, Model *model, Info *info)
 {
 	char firmware[54];
 	char oknok[6];
@@ -66,6 +66,7 @@ int lg_gsm_init (GPPort *port, Model *model, Info *info)
 
 	/* This information, too. */
 	memcpy (info, &firmware[6], 40);
+	info[39] = 0;
 
 	GP_DEBUG("info = %s\n", info);
 	/*GP_DEBUG("info[20] = 0x%x\n", firmware[26]);*/
@@ -74,7 +75,7 @@ int lg_gsm_init (GPPort *port, Model *model, Info *info)
         return GP_OK;
 }
 
-unsigned int lg_gsm_get_picture_size  (GPPort *port, int pic) 
+unsigned int lg_gsm_get_picture_size  (GPPort *port, int pic)
 {
 
         unsigned int size;
@@ -107,7 +108,7 @@ unsigned int lg_gsm_get_picture_size  (GPPort *port, int pic)
 	size = (unsigned int)photodesc[138] + (unsigned int)photodesc[139]*0x100 + (unsigned int)photodesc[140]*0x10000+(unsigned int)photodesc[141]*0x1000000;
 	GP_DEBUG(" size of picture %i is 0x%x\n", pic, size);
 	/* max. 1280x960x24bits ? */
-	if ( (size >= 0x384000 ) ) {return GP_ERROR;} 
+	if ( (size >= 0x384000 ) ) {return GP_ERROR;}
 	MSGWRITE(port, 0x13, 0x6, 0x0, "", 0);
 	WRITE(port, sync_stop, 6);
 	READ(port, oknok, 6);
@@ -115,7 +116,7 @@ unsigned int lg_gsm_get_picture_size  (GPPort *port, int pic)
 	return size;
 }
 
-int lg_gsm_read_picture_data (GPPort *port, char *data, int size, int n) 
+int lg_gsm_read_picture_data (GPPort *port, char *data, unsigned int datasize, unsigned int n)
 {
 	char listphotos[] = "\x04\x0\x08\x0\x0\x0\x40\x0\x0\x0\x0\x0\x0\x0";
 
@@ -126,6 +127,7 @@ int lg_gsm_read_picture_data (GPPort *port, char *data, int size, int n)
 	char block[50000];
 	char oknok[6];
 
+	unsigned int size;
 	int pos=0;
 	int block_size=50000;
 	int header_size=8;
@@ -157,11 +159,15 @@ int lg_gsm_read_picture_data (GPPort *port, char *data, int size, int n)
 	/* then read 142 */
 	READ(port, photodesc, 142);
 	size = (int)photodesc[138] + (int)photodesc[139]*0x100 + (int)photodesc[140]*0x10000+(int)photodesc[141]*0x1000000;
-	GP_DEBUG(" size of picture %i is 0x%x\n", n, size);
+	GP_DEBUG(" size of picture %i is 0x%x", n, size);
 	/* max. 1280x960x24bits ? */
 	if ( (size >= 0x384000 ) ) {
 		return GP_ERROR;
 	}
+	if (datasize < size) {
+		GP_DEBUG("size of picture %u, previous read %u", datasize, size);
+		return GP_ERROR_CORRUPTED_DATA;
+	}
 
 	memcpy(getphoto, &get_photo_cmd[0], 10);
 	memcpy(getphoto +10, &n, 1); /*TODO: fix this*/
@@ -179,6 +185,7 @@ int lg_gsm_read_picture_data (GPPort *port, char *data, int size, int n)
 	for (i = 1 ; i <= nb_blocks ; i++)
 	{
 		remain = size - pos;
+		GP_DEBUG ("size %d, pos %d, remain %d, block_size %d, header_size %d", size, pos, remain, block_size, header_size);
 		if (remain >= block_size - header_size)
 		{
 			READ(port, block, block_size);
@@ -190,7 +197,7 @@ int lg_gsm_read_picture_data (GPPort *port, char *data, int size, int n)
 			memcpy(data+pos,&block[header_size],remain);
 			pos=pos+remain;
 		}
-		
+
 	}
 	/*port->timeout=5000;*/
 	/* syncstop */
@@ -202,13 +209,13 @@ int lg_gsm_read_picture_data (GPPort *port, char *data, int size, int n)
         return GP_OK;
 }
 
-int lg_gsm_list_files (GPPort *port, CameraList *list) 
+int lg_gsm_list_files (GPPort *port, CameraList *list)
 {
-	int num_pics;
-	int i;	
+	unsigned int num_pics;
+	unsigned int i;
 
 	char oknok[6];
-	char photonumber[22];
+	unsigned char photonumber[22];
 	char photolist[142000]; /* max_photos * 142 */
 	char name[44];
 	char value[88];
@@ -231,10 +238,13 @@ int lg_gsm_list_files (GPPort *port, CameraList *list)
 	/* lsphoto : write 0x04000800000040000000ffffffff */
 	MSGWRITE(port, 0x13, 0xe, 0x0, "", 0);
 	WRITE(port, list_all_photo, 0xe);
-	READ(port, photonumber, 0x16);
+	READ(port, (char*)photonumber, 0x16);
 
 	num_pics=photonumber[20]+256*photonumber[21];
 
+	GP_DEBUG ("num_pics = %d\n", num_pics);
+	if (num_pics > 1000) return GP_ERROR;
+
 	/* increase timeout to 20s */
 	/*port->timeout=20000;*/
 	/* read 142 * nb_photos */
@@ -243,7 +253,9 @@ int lg_gsm_list_files (GPPort *port, CameraList *list)
 	for (i = 0; i < num_pics; i++){
 		/* sprintf( name, "lg_gsm_pic%03i.jpg", i ); */
 		memcpy(name,&photolist[6+142*i],44);
+		name[43] = 0;
 		memcpy(value,&photolist[50+142*i],80);
+		value[80] = 0;
 		gp_list_append(list, name, value);
 	}
 	/* restore timeout to 5s */
