@@ -1,7 +1,7 @@
 /*
-        Copyright  2000 Mariusz Zynel <mariusz@mizar.org> (gPhoto port)
-        Copyright  2000 Fredrik Roubert <roubert@df.lth.se> (idea)
-        Copyright  1999 Galen Brooks <galen@nine.com> (DC1580 code)
+        Copyright 2000 Mariusz Zynel <mariusz@mizar.org> (gPhoto port)
+        Copyright 2000 Fredrik Roubert <roubert@df.lth.se> (idea)
+        Copyright 1999 Galen Brooks <galen@nine.com> (DC1580 code)
 
         This file is part of the gPhoto project and may only be used,  modified,
         and distributed under the terms of the gPhoto project license,  COPYING.
@@ -24,9 +24,7 @@
 
 #include <stdlib.h>
 #include <stdarg.h>
-#include <sys/time.h>
 #include <sys/stat.h>
-#include <unistd.h>
 #ifdef HAVE_MEMORY_H
 #include <memory.h>
 #endif
@@ -78,7 +76,7 @@ static uint8_t dsc2_checksum(char *buffer, int size) {
 
 static int dsc2_sendcmd(Camera *camera, uint8_t cmd, long int data, uint8_t sequence) {
 
-        int     i;
+        unsigned int     i;
 
         DEBUG_PRINT_MEDIUM(("Sending command: 0x%02x, data: %i, sequence: %i.", cmd, data, sequence));
 
@@ -100,7 +98,7 @@ static int dsc2_sendcmd(Camera *camera, uint8_t cmd, long int data, uint8_t sequ
 /* dsc2_retrcmd - retrieve command and its data from DSC */
 
 static int dsc2_retrcmd(Camera *camera) {
-        
+
         int     result = GP_ERROR;
         int     s;
 
@@ -128,7 +126,7 @@ static int dsc2_retrcmd(Camera *camera) {
 /* dsc2_connect - try hand shake with camera and establish connection */
 
 static int dsc2_connect(Camera *camera, int speed) {
-        
+
         DEBUG_PRINT_MEDIUM(("Connecting camera with speed: %i.", speed));
 
         if (dsc1_setbaudrate(camera, speed) != GP_OK)
@@ -153,7 +151,7 @@ static int dsc2_connect(Camera *camera, int speed) {
 /* dsc2_disconnect - reset camera, free buffers and close files */
 
 static int dsc2_disconnect(Camera *camera) {
-        
+
         DEBUG_PRINT_MEDIUM(("Disconnecting the camera."));
 
         if (dsc2_sendcmd(camera, DSC2_CMD_RESET, 0, 0) != GP_OK)
@@ -173,7 +171,7 @@ static int dsc2_disconnect(Camera *camera) {
 /* dsc2_getindex - retrieve the number of images stored in camera memory */
 
 static int dsc2_getindex(Camera *camera) {
-        
+
         int     result = GP_ERROR;
 
         DEBUG_PRINT_MEDIUM(("Retrieving the number of images."));
@@ -199,7 +197,7 @@ static int dsc2_getindex(Camera *camera) {
 /* dsc2_delete - delete image #index from camera memory */
 
 static int dsc2_delete(Camera *camera, int index) {
-        
+
         DEBUG_PRINT_MEDIUM(("Deleting image: %i.", index));
 
         if (index < 1)
@@ -220,7 +218,7 @@ static int dsc2_delete(Camera *camera, int index) {
 /* dsc2_selectimage - select image to download, return its size */
 
 static int dsc2_selectimage(Camera *camera, int index, int thumbnail) {
-        
+
         int     size = 0;
 
         DEBUG_PRINT_MEDIUM(("Selecting image: %i, thumbnail: %i.", index, thumbnail));
@@ -254,7 +252,7 @@ static int dsc2_selectimage(Camera *camera, int index, int thumbnail) {
 /* gp_port_readimageblock - read #block block (1024 bytes) of an image into buf */
 
 static int dsc2_readimageblock(Camera *camera, int block, char *buffer) {
-        
+
         DEBUG_PRINT_MEDIUM(("Reading image block: %i.", block));
 
         if (dsc2_sendcmd(camera, DSC2_CMD_GET_DATA, block, block) != GP_OK)
@@ -374,7 +372,7 @@ static int dsc2_writeimageblock(Camera *camera, int block, char *buffer, int siz
 
 #if 0
 static int dsc2_writeimage(Camera *camera, char *buffer, int size) {
-        
+
         int     blocks, blocksize, i;
 
         DEBUG_PRINT_MEDIUM(("Writing an image of size: %i.", size));
@@ -404,7 +402,7 @@ static int dsc2_writeimage(Camera *camera, char *buffer, int size) {
 
 #if 0
 static int dsc2_preview(Camera *camera, int index) {
-        
+
         if (index < 1)
                 RETURN_ERROR(EDSCBADNUM);
                 /* bad image number */
@@ -425,21 +423,21 @@ static int dsc2_preview(Camera *camera, int index) {
 /* Library interface functions */
 
 int camera_id (CameraText *id) {
-        
+
         strcpy(id->text, "panasonic-dc1580");
 
         return (GP_OK);
 }
 
 int camera_abilities (CameraAbilitiesList *list) {
-        
+
         CameraAbilities a;
         char    *models[] = {
                         "Panasonic:DC1580",
                         "Nikon:CoolPix 600",
                         NULL };
         int     i = 0, result;
-        
+
         while (models[i]) {
 		memset(&a, 0, sizeof(a));
 		a.status = GP_DRIVER_STATUS_PRODUCTION;
@@ -452,7 +450,7 @@ int camera_abilities (CameraAbilitiesList *list) {
                 a.speed[4]     = 115200;
                 a.speed[5]     = 0;
                 a.operations        = 	GP_OPERATION_NONE;
-                a.file_operations   = 	GP_FILE_OPERATION_DELETE | 
+                a.file_operations   = 	GP_FILE_OPERATION_DELETE |
 					GP_FILE_OPERATION_PREVIEW;
                 a.folder_operations = 	GP_FOLDER_OPERATION_PUT_FILE;
 
@@ -480,9 +478,9 @@ static int file_list_func (CameraFilesystem *fs, const char *folder,
 
         Camera  *camera = data;
         int     count, result;
-        
+
 	CHECK (count = dsc2_getindex(camera));
-        
+
         CHECK (gp_list_populate(list, DSC_FILENAMEFMT, count));
 
         return GP_OK;
@@ -499,12 +497,11 @@ static int get_info_func (CameraFilesystem *fs, const char *folder,
         CHECK (index = gp_filesystem_number(camera->fs, folder, filename, context));
         index++;
 
-	info->file.fields = GP_FILE_INFO_TYPE | GP_FILE_INFO_NAME | GP_FILE_INFO_SIZE;
+	info->file.fields = GP_FILE_INFO_TYPE | GP_FILE_INFO_SIZE;
 	strcpy(info->file.type, GP_MIME_JPEG);
-        sprintf(info->file.name, DSC_FILENAMEFMT, index);
         info->file.size = dsc2_selectimage(camera, index, DSC_FULLIMAGE);
 
-	info->preview.fields = GP_FILE_INFO_TYPE | GP_FILE_INFO_NAME | GP_FILE_INFO_SIZE;
+	info->preview.fields = GP_FILE_INFO_TYPE | GP_FILE_INFO_SIZE;
 	strcpy(info->preview.type, GP_MIME_JPEG);
         info->preview.size = dsc2_selectimage(camera, index, DSC_THUMBNAIL);
 
@@ -538,7 +535,6 @@ static int get_file_func (CameraFilesystem *fs, const char *folder,
 	if (size < 0)
 		return (size);
 
-	CHECK (gp_file_set_name(file, filename));
 	CHECK (gp_file_set_mime_type(file, GP_MIME_JPEG));
 
         blocks = (size - 1)/DSC_BLOCKSIZE + 1;
@@ -556,21 +552,19 @@ static int get_file_func (CameraFilesystem *fs, const char *folder,
         return GP_OK;
 }
 
-static int put_file_func (CameraFilesystem *fs, const char *folder,
-			  CameraFile *file, void *user_data,
+static int put_file_func (CameraFilesystem *fs, const char *folder, const char *name,
+			  CameraFileType type, CameraFile *file, void *user_data,
 			  GPContext *context) {
-        
+
 	Camera *camera = user_data;
         int             blocks, blocksize, i, result;
-	const char      *name;
 	const char      *data;
-	long int        size;
+	long unsigned int size;
 	unsigned int id;
 
-	gp_file_get_name(file, &name);
         gp_context_status(context, _("Uploading image: %s."), name);
 
-/*      We can not figure out file type, at least by now.
+/*      We can not figure out file type, at least by now. (? curious, mime type -Marcus)
 
         if (strcmp(file->type, "image/jpg") != 0) {
                 dsc_print_message(camera, "JPEG image format allowed only.");
@@ -581,7 +575,7 @@ static int put_file_func (CameraFilesystem *fs, const char *folder,
         if (size > DSC_MAXIMAGESIZE) {
                 gp_context_message (context, _("File size is %ld bytes. "
 				   "The size of the largest file possible to "
-				   "upload is: %i bytes."), size, 
+				   "upload is: %i bytes."), size,
 				   DSC_MAXIMAGESIZE);
                 return GP_ERROR;
         }
@@ -595,8 +589,8 @@ static int put_file_func (CameraFilesystem *fs, const char *folder,
                 blocksize = size - i*DSC_BLOCKSIZE;
                 if (DSC_BLOCKSIZE < blocksize)
                         blocksize = DSC_BLOCKSIZE;
-		result = dsc2_writeimageblock(camera, i, 
-					       (char*)&data[i*DSC_BLOCKSIZE], 
+		result = dsc2_writeimageblock(camera, i,
+					       (char*)&data[i*DSC_BLOCKSIZE],
 					       blocksize);
 		if (result != GP_OK)
 			return result;
@@ -612,7 +606,7 @@ static int put_file_func (CameraFilesystem *fs, const char *folder,
 static int delete_file_func (CameraFilesystem *fs, const char *folder,
 			     const char *filename, void *data,
 			     GPContext *context) {
-        
+
 	Camera *camera = data;
         int     index, result;
 
@@ -625,7 +619,7 @@ static int delete_file_func (CameraFilesystem *fs, const char *folder,
         return dsc2_delete(camera, index);
 }
 
-static int camera_about (Camera *camera, CameraText *about, GPContext *context) 
+static int camera_about (Camera *camera, CameraText *about, GPContext *context)
 {
         strcpy(about->text,
                         _("Panasonic DC1580 gPhoto2 library\n"
@@ -644,7 +638,7 @@ static CameraFilesystemFuncs fsfuncs = {
 	.del_file_func = delete_file_func,
 };
 
-int camera_init (Camera *camera, GPContext *context) 
+int camera_init (Camera *camera, GPContext *context)
 {
         GPPortSettings settings;
         int result, selected_speed;
