@@ -1,7 +1,7 @@
 /* pdc640.c
  *
- * Copyright  2001 Lutz Mller <lutz@users.sourceforge.net>
- * Copyright  2002 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright 2001 Lutz Mueller <lutz@users.sourceforge.net>
+ * Copyright 2002 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include "config.h"
 
@@ -204,7 +204,13 @@ static struct {
 			"dlink%04i.ppm"
 		}
 	},
-	{NULL,}
+	{NULL, 0, 0, {
+			0,
+			0,
+			NULL,
+			NULL
+		}
+	}
 };
 
 static int
@@ -511,7 +517,7 @@ pdc640_processtn (int width, int height, unsigned char **data, int size) {
 	int y;
 
 	/* Sanity checks */
-	if ((data == NULL) || (size < width * height))
+	if ((data == NULL) || (size  / width < height))
 		return (GP_ERROR_CORRUPTED_DATA);
 
 	/* Allocate a new buffer */
@@ -568,6 +574,7 @@ pdc640_deltadecode (int width, int height, unsigned char **rawdata, int *rawsize
 
 	GP_DEBUG ("pdc640_deltacode ()");
 
+	if (width < 2) return GP_ERROR_CORRUPTED_DATA; /* invalid, we implicitly assume it to be 2 or higher in the decoder */
 	/* Create a buffer to store RGB data in */
 	size = width * height;
 	data = malloc (size * sizeof (char));
@@ -697,7 +704,7 @@ pdc640_getpic (Camera *camera, int n, int thumbnail, int justraw,
 	CHECK_RESULT (pdc640_transmit_pic (camera->port, cmd, width, thumbnail,
 					*data, *size));
 
-	if (thumbnail || (compression_type == 0 )) {
+	if (thumbnail || ((compression_type & 3) == 0 )) {
 		/* Process uncompressed data */
 		CHECK_RESULT (pdc640_processtn (width, height,
 						data, *size));
@@ -725,7 +732,7 @@ pdc640_getpic (Camera *camera, int n, int thumbnail, int justraw,
 		return (GP_ERROR_NO_MEMORY);
 
 	/* Set header */
-	strcpy(outdata, ppmheader);
+	strcpy((char *)outdata, ppmheader);
 
 	/* Decode and interpolate the Bayer Mask */
 	result = gp_bayer_decode(*data, width, height,
