@@ -7,10 +7,10 @@
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
@@ -20,7 +20,7 @@
 /*
  * This is working, but unfinished!
  * - Event handling not finished.
- * - Some configure checking magic missing for the special header files 
+ * - Some configure checking magic missing for the special header files
  *   and functions.
  * - Not everything implementation correctly cross checked.
  * - Coolpix P3 does not give transfer status (image 000x/000y), and reports an
@@ -28,7 +28,8 @@
  *
  * Nikon WU-1* adapters might use 0011223344556677 as GUID always...
  */
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
+#define _DARWIN_C_SOURCE
 #include "config.h"
 
 #include <stdlib.h>
@@ -41,8 +42,11 @@
 #include <sys/select.h>
 #endif
 
+#include "ptpip-private.h"
+
 #ifdef WIN32
-# include <winsock.h>
+# include <winsock2.h>
+# include <ws2tcpip.h>
 #else
 # include <sys/socket.h>
 # include <netinet/in.h>
@@ -107,7 +111,17 @@ ptp_ptpip_sendreq (PTPParams* params, PTPContainer* req, int dataphase)
 	int		len = 18+req->Nparam*4;
 	unsigned char 	*request = malloc(len);
 
-	GP_LOG_D ("Sending PTP_OC 0x%0x (%s) request...", req->Code, ptp_get_opcode_name(params, req->Code));
+	switch (req->Nparam) {
+	default:
+	case 0: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) request...", req->Code, ptp_get_opcode_name(params, req->Code));
+		break;
+	case 1: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1);
+		break;
+	case 2: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x,0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1, req->Param2);
+		break;
+	case 3: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x,0x%x,0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1, req->Param2, req->Param3);
+		break;
+	}
 
 	ptp_ptpip_check_event (params);
 
@@ -122,20 +136,22 @@ ptp_ptpip_sendreq (PTPParams* params, PTPContainer* req, int dataphase)
 	htod32a(&request[ptpip_cmd_transid],req->Transaction_ID);
 
 	switch (req->Nparam) {
-	case 5: htod32a(&request[ptpip_cmd_param5],req->Param5);
-	case 4: htod32a(&request[ptpip_cmd_param4],req->Param4);
-	case 3: htod32a(&request[ptpip_cmd_param3],req->Param3);
-	case 2: htod32a(&request[ptpip_cmd_param2],req->Param2);
-	case 1: htod32a(&request[ptpip_cmd_param1],req->Param1);
+	case 5: htod32a(&request[ptpip_cmd_param5],req->Param5);/* fallthrough */
+	case 4: htod32a(&request[ptpip_cmd_param4],req->Param4);/* fallthrough */
+	case 3: htod32a(&request[ptpip_cmd_param3],req->Param3);/* fallthrough */
+	case 2: htod32a(&request[ptpip_cmd_param2],req->Param2);/* fallthrough */
+	case 1: htod32a(&request[ptpip_cmd_param1],req->Param1);/* fallthrough */
 	case 0:
 	default:
 		break;
 	}
 	GP_LOG_DATA ( (char*)request, len, "ptpip/oprequest data:");
-	ret = write (params->cmdfd, request, len);
+	ret = PTPSOCK_WRITE(params->cmdfd, request, len);
 	free (request);
-	if (ret == -1)
+	if (ret == PTPSOCK_ERR) {
 		perror ("sendreq/write to cmdfd");
+		return GP_ERROR_IO;
+	}
 	if (ret != len) {
 		GP_LOG_E ("ptp_ptpip_sendreq() len =%d but ret=%d", len, ret);
 		return PTP_RC_OK;
@@ -150,8 +166,8 @@ ptp_ptpip_generic_read (PTPParams *params, int fd, PTPIPHeader *hdr, unsigned ch
 
 	xhdr = (unsigned char*)hdr; curread = 0; len = sizeof (PTPIPHeader);
 	while (curread < len) {
-		ret = read (fd, xhdr + curread, len - curread);
-		if (ret == -1) {
+		ret = PTPSOCK_READ (fd, xhdr + curread, len - curread);
+		if (ret == PTPSOCK_ERR) {
 			perror ("read PTPIPHeader");
 			return PTP_RC_GeneralError;
 		}
@@ -174,8 +190,8 @@ ptp_ptpip_generic_read (PTPParams *params, int fd, PTPIPHeader *hdr, unsigned ch
 	}
 	curread = 0;
 	while (curread < len) {
-		ret = read (fd, (*data)+curread, len-curread);
-		if (ret == -1) {
+		ret = PTPSOCK_READ (fd, (*data)+curread, len-curread);
+		if (ret == PTPSOCK_ERR) {
 			GP_LOG_E ("error %d in reading PTPIP data", errno);
 			free (*data);*data = NULL;
 			return PTP_RC_GeneralError;
@@ -242,9 +258,11 @@ ptp_ptpip_senddata (PTPParams* params, PTPContainer* ptp,
 	htod32a(&request[ptpip_startdata_totallen + 8],size);
 	htod32a(&request[ptpip_startdata_unknown  + 8],0);
 	GP_LOG_DATA ((char*)request, sizeof(request), "ptpip/senddata header:");
-	ret = write (params->cmdfd, request, sizeof(request));
-	if (ret == -1)
+	ret = PTPSOCK_WRITE (params->cmdfd, request, sizeof(request));
+	if (ret == PTPSOCK_ERR) {
 		perror ("sendreq/write to cmdfd");
+		return GP_ERROR_IO;
+	}
 	if (ret != sizeof(request)) {
 		GP_LOG_E ("ptp_ptpip_senddata() len=%d but ret=%d", (int)sizeof(request), ret);
 		return PTP_RC_GeneralError;
@@ -277,8 +295,8 @@ ptp_ptpip_senddata (PTPParams* params, PTPContainer* ptp,
 		GP_LOG_DATA ((char*)xdata, towrite2, "ptpip/senddata data:");
 		written = 0;
 		while (written < towrite2) {
-			ret = write (params->cmdfd, xdata+written, towrite2-written);
-			if (ret == -1) {
+			ret = PTPSOCK_WRITE (params->cmdfd, xdata+written, towrite2-written);
+			if (ret == PTPSOCK_ERR) {
 				perror ("write in senddata failed");
 				free (xdata);
 				return PTP_RC_GeneralError;
@@ -388,22 +406,22 @@ retry:
 
 	switch (dtoh32(hdr.type)) {
 	case PTPIP_END_DATA_PACKET:
-		GP_LOG_D("PTPIP_END_DATA_PACKET");
 		resp->Transaction_ID	= dtoh32a(&data[0]);
+		GP_LOG_D("PTPIP_END_DATA_PACKET (tid = 0x%08x)", resp->Transaction_ID);
 		free (data);
 		data = NULL;
 		goto retry;
 	case PTPIP_CMD_RESPONSE:
-		GP_LOG_D("PTPIP_CMD_RESPONSE");
 		resp->Code		= dtoh16a(&data[ptpip_resp_code]);
 		resp->Transaction_ID	= dtoh32a(&data[ptpip_resp_transid]);
+		GP_LOG_D("PTPIP_CMD_RESPONSE (result=0x%04x, tid=0x%08x)", resp->Code, resp->Transaction_ID);
 		n = (dtoh32(hdr.length) - sizeof(hdr) - ptpip_resp_param1)/sizeof(uint32_t);
 		switch (n) {
-		case 5: resp->Param5 = dtoh32a(&data[ptpip_resp_param5]);
-		case 4: resp->Param4 = dtoh32a(&data[ptpip_resp_param4]);
-		case 3: resp->Param3 = dtoh32a(&data[ptpip_resp_param3]);
-		case 2: resp->Param2 = dtoh32a(&data[ptpip_resp_param2]);
-		case 1: resp->Param1 = dtoh32a(&data[ptpip_resp_param1]);
+		case 5: resp->Param5 = dtoh32a(&data[ptpip_resp_param5]);/* fallthrough */
+		case 4: resp->Param4 = dtoh32a(&data[ptpip_resp_param4]);/* fallthrough */
+		case 3: resp->Param3 = dtoh32a(&data[ptpip_resp_param3]);/* fallthrough */
+		case 2: resp->Param2 = dtoh32a(&data[ptpip_resp_param2]);/* fallthrough */
+		case 1: resp->Param1 = dtoh32a(&data[ptpip_resp_param1]);/* fallthrough */
 		case 0: break;
 		default:
 			GP_LOG_E ("response got %d parameters?", n);
@@ -429,10 +447,16 @@ ptp_ptpip_init_command_request (PTPParams* params)
 	unsigned int	i;
 	int 		len, ret;
 	unsigned char	guid[16];
-	
+
 	ptp_nikon_getptpipguid(guid);
+#if !defined (WIN32)
 	if (gethostname (hostname, sizeof(hostname)))
 		return PTP_RC_GeneralError;
+#else
+	DWORD hostname_size = (DWORD)sizeof(hostname);
+	if (!GetComputerNameA(hostname, &hostname_size))
+		return PTP_RC_GeneralError;
+#endif
 	len = ptpip_initcmd_name + (strlen(hostname)+1)*2 + 4;
 
 	cmdrequest = malloc(len);
@@ -449,9 +473,9 @@ ptp_ptpip_init_command_request (PTPParams* params)
 	htod16a(&cmdrequest[ptpip_initcmd_name+(strlen(hostname)+1)*2+2],PTPIP_VERSION_MAJOR);
 
 	GP_LOG_DATA ((char*)cmdrequest, len, "ptpip/init_cmd data:");
-	ret = write (params->cmdfd, cmdrequest, len);
+	ret = PTPSOCK_WRITE (params->cmdfd, cmdrequest, len);
 	free (cmdrequest);
-	if (ret == -1) {
+	if (ret == PTPSOCK_ERR) {
 		perror("write init cmd request");
 		return PTP_RC_GeneralError;
 	}
@@ -490,7 +514,7 @@ ptp_ptpip_init_command_ack (PTPParams* params)
 	memcpy (params->cameraguid, &data[ptpip_cmdack_guid], 16);
 	name = (unsigned short*)&data[ptpip_cmdack_name];
 	for (i=0;name[i];i++) /* EMPTY */;
-	params->cameraname = malloc((i+1)*sizeof(uint16_t));
+	params->cameraname = calloc((i+1),sizeof(uint16_t));
 	for (i=0;name[i];i++)
 		params->cameraname[i] = name[i];
 	free (data);
@@ -510,8 +534,8 @@ ptp_ptpip_init_event_request (PTPParams* params)
 	htod32a(&evtrequest[ptpip_eventinit_idx],params->eventpipeid);
 
 	GP_LOG_DATA ((char*)evtrequest, ptpip_eventinit_size, "ptpip/init_event data:");
-	ret = write (params->evtfd, evtrequest, ptpip_eventinit_size);
-	if (ret == -1) {
+	ret = PTPSOCK_WRITE (params->evtfd, evtrequest, ptpip_eventinit_size);
+	if (ret == PTPSOCK_ERR) {
 		perror("write init evt request");
 		return PTP_RC_GeneralError;
 	}
@@ -554,7 +578,6 @@ ptp_ptpip_init_event_ack (PTPParams* params)
 static uint16_t
 ptp_ptpip_event (PTPParams* params, PTPContainer* event, int wait)
 {
-#ifndef WIN32
 	fd_set		infds;
 	struct timeval	timeout;
 	int ret;
@@ -599,9 +622,9 @@ ptp_ptpip_event (PTPParams* params, PTPContainer* event, int wait)
 	event->Transaction_ID	= dtoh32a(&data[ptpip_event_transid]);
 	n = (dtoh32(hdr.length) - sizeof(hdr) - ptpip_event_param1)/sizeof(uint32_t);
 	switch (n) {
-	case 3: event->Param3 = dtoh32a(&data[ptpip_event_param3]);
-	case 2: event->Param2 = dtoh32a(&data[ptpip_event_param2]);
-	case 1: event->Param1 = dtoh32a(&data[ptpip_event_param1]);
+	case 3: event->Param3 = dtoh32a(&data[ptpip_event_param3]);/* fallthrough */
+	case 2: event->Param2 = dtoh32a(&data[ptpip_event_param2]);/* fallthrough */
+	case 1: event->Param1 = dtoh32a(&data[ptpip_event_param1]);/* fallthrough */
 	case 0: break;
 	default:
 		GP_LOG_E ("response got %d parameters?", n);
@@ -609,10 +632,12 @@ ptp_ptpip_event (PTPParams* params, PTPContainer* event, int wait)
 	}
 	free (data);
 	return PTP_RC_OK;
-#else
-	GP_LOG_E ("not supported currently on Windows");
-	return PTP_RC_OK;
-#endif
+}
+
+uint16_t
+ptp_ptpip_event_check_queue (PTPParams* params, PTPContainer* event) {
+	/* the fast check just takes 1ms, so lets keep it */
+	return ptp_ptpip_event (params, event, PTP_EVENT_CHECK_FAST);
 }
 
 uint16_t
@@ -630,7 +655,7 @@ ptp_ptpip_event_wait (PTPParams* params, PTPContainer* event) {
  *
  * This command gets the GUID of this machine. If it does not exists, it creates
  * one.
- *  
+ *
  * params:	PTPParams*
  *
  * Return values: Some PTP_RC_* code.
@@ -644,9 +669,9 @@ ptp_nikon_getptpipguid (unsigned char* guid) {
 	int valid;
 	char* endptr;
 	char* pos;
-	
+
 	gp_setting_get("ptp2_ip","guid",buffer);
-	
+
 	if (strlen(buffer) == 47) { /* 47 = 16*2 (numbers) + 15 (semi-colons) */
 		pos = buffer;
 		valid = 1;
@@ -667,9 +692,9 @@ ptp_nikon_getptpipguid (unsigned char* guid) {
 		if (valid)
 			return;
 	}
-	
+
 	/*fprintf(stderr, "Invalid GUID\n");*/
-	
+
 	/* Generate an ID */
 	srand(time(NULL));
 	buffer[0] = 0;
@@ -679,9 +704,9 @@ ptp_nikon_getptpipguid (unsigned char* guid) {
 		pos += sprintf(pos, "%02x:", guid[i]);
 	}
 	buffer[47] = 0;
-	
+
 	/*printf("New GUID: %s\n", buffer);*/
-	
+
 	gp_setting_set("ptp2_ip","guid",buffer);
 }
 
@@ -689,7 +714,7 @@ ptp_nikon_getptpipguid (unsigned char* guid) {
 int
 ptp_ptpip_connect (PTPParams* params, const char *address) {
 	char 		*addr, *s, *p;
-	int		port, tries;
+	int		port, eventport, tries;
 	struct sockaddr_in	saddr;
 	uint16_t	ret;
 
@@ -697,7 +722,6 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 	if (NULL == strchr (address,':'))
 		return GP_ERROR_BAD_PARAMETERS;
 
-#ifdef HAVE_INET_ATON
 	addr = strdup (address);
 	if (!addr)
 		return GP_ERROR_NO_MEMORY;
@@ -709,7 +733,7 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 	}
 	*s = '\0';
 	p = strchr (s+1,':');
-	port = 15740;
+	eventport = port = 15740;
 	if (p) {
 		*p = '\0';
 		if (!sscanf (p+1,"%d",&port)) {
@@ -717,8 +741,21 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 			free (addr);
 			return GP_ERROR_BAD_PARAMETERS;
 		}
+		/* different event port ? */
+		p = strchr (p+1,':');
+		if (p) {
+			if (!sscanf (p+1,"%d",&eventport)) {
+				fprintf(stderr,"failed to scan for eventport in %s\n", p+1);
+				free (addr);
+				return GP_ERROR_BAD_PARAMETERS;
+			}
+		}
 	}
+#ifdef HAVE_INET_ATON
 	if (!inet_aton (s+1,  &saddr.sin_addr)) {
+#else
+	if (inet_pton(AF_INET, s+1, &saddr.sin_addr) != 1) {
+#endif
 		fprintf(stderr,"failed to scan for addr in %s\n", s+1);
 		free (addr);
 		return GP_ERROR_BAD_PARAMETERS;
@@ -726,48 +763,54 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 	saddr.sin_port		= htons(port);
 	saddr.sin_family	= AF_INET;
 	free (addr);
-	params->cmdfd = socket (PF_INET, SOCK_STREAM, 0);
-	if (params->cmdfd == -1) {
+	PTPSOCK_SOCKTYPE cmdfd = params->cmdfd = socket (PF_INET, SOCK_STREAM, PTPSOCK_PROTO);
+	if (cmdfd == PTPSOCK_INVALID) {
 		perror ("socket cmd");
 		return GP_ERROR_BAD_PARAMETERS;
 	}
-	params->evtfd = socket (PF_INET, SOCK_STREAM, 0);
-	if (params->evtfd == -1) {
+	PTPSOCK_SOCKTYPE evtfd = params->evtfd = socket (PF_INET, SOCK_STREAM, PTPSOCK_PROTO);
+	if (evtfd == PTPSOCK_INVALID) {
 		perror ("socket evt");
-		close (params->cmdfd);
+		PTPSOCK_CLOSE (params->cmdfd);
 		return GP_ERROR_BAD_PARAMETERS;
 	}
-	if (-1 == connect (params->cmdfd, (struct sockaddr*)&saddr, sizeof(struct sockaddr_in))) {
+	if (PTPSOCK_ERR == connect (params->cmdfd, (struct sockaddr*)&saddr, sizeof(struct sockaddr_in))) {
 		perror ("connect cmd");
-		close (params->cmdfd);
-		close (params->evtfd);
+		PTPSOCK_CLOSE (params->cmdfd);
+		PTPSOCK_CLOSE (params->evtfd);
 		return GP_ERROR_IO;
 	}
 	ret = ptp_ptpip_init_command_request (params);
 	if (ret != PTP_RC_OK) {
-		close (params->cmdfd);
-		close (params->evtfd);
+		PTPSOCK_CLOSE (params->cmdfd);
+		PTPSOCK_CLOSE (params->evtfd);
 		return translate_ptp_result (ret);
 	}
 	ret = ptp_ptpip_init_command_ack (params);
 	if (ret != PTP_RC_OK) {
-		close (params->cmdfd);
-		close (params->evtfd);
+		PTPSOCK_CLOSE (params->cmdfd);
+		PTPSOCK_CLOSE (params->evtfd);
 		return translate_ptp_result (ret);
 	}
 	/* seen on Ricoh Theta, camera is not immediately ready. try again two times. */
 	tries = 2;
+	saddr.sin_port		= htons(eventport);
 	do {
-		if (-1 != connect (params->evtfd, (struct sockaddr*)&saddr, sizeof(struct sockaddr_in)))
+		if (PTPSOCK_ERR != connect (params->evtfd, (struct sockaddr*)&saddr, sizeof(struct sockaddr_in)))
 			break;
 		if ((errno == ECONNREFUSED) && (tries--)) {
 			GP_LOG_D ("event connect failed, retrying after short wait");
-			usleep(100*1000);
+			int sleep_ms = 100;
+#ifdef WIN32
+			Sleep(sleep_ms);
+#else
+			usleep(sleep_ms*1000);
+#endif
 			continue;
 		}
 		GP_LOG_E ("could not connect event");
-		close (params->cmdfd);
-		close (params->evtfd);
+		PTPSOCK_CLOSE (params->cmdfd);
+		PTPSOCK_CLOSE (params->evtfd);
 		return GP_ERROR_IO;
 	} while (1);
 	ret = ptp_ptpip_init_event_request (params);
@@ -778,8 +821,4 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 		return translate_ptp_result (ret);
 	GP_LOG_D ("ptpip connected!");
 	return GP_OK;
-#else
-	GP_LOG_E ("Windows currently not supported, neeeds a winsock port.");
-	return GP_ERROR_NOT_SUPPORTED;
-#endif
 }
