@@ -2,6 +2,7 @@
  *
  * Copyright (C) 2001-2004 Mariusz Woloszyn <emsi@ipartners.pl>
  * Copyright (C) 2003-2016 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright (C) 2003-2017 Marcus Meissner <marcus@jet.franken.de>
  * Copyright (C) 2006-2007 Linus Walleij <triad@df.lth.se>
  *
  * This library is free software; you can redistribute it and/or
@@ -20,7 +21,7 @@
  * Boston, MA  02110-1301  USA
  */
 
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 #include <config.h>
 #include "ptp.h"
 #include "ptp-private.h"
@@ -76,7 +77,17 @@ ptp_usb_sendreq (PTPParams* params, PTPContainer* req, int dataphase)
 	PTPUSBBulkContainer usbreq;
 	Camera *camera = ((PTPData *)params->data)->camera;
 
-	GP_LOG_D ("Sending PTP_OC 0x%0x (%s) request...", req->Code, ptp_get_opcode_name(params, req->Code));
+	switch (req->Nparam) {
+	default:
+	case 0: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) request...", req->Code, ptp_get_opcode_name(params, req->Code));
+		break;
+	case 1: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1);
+		break;
+	case 2: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x,0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1, req->Param2);
+		break;
+	case 3: GP_LOG_D ("Sending PTP_OC 0x%0x (%s) (0x%x,0x%x,0x%x) request...", req->Code, ptp_get_opcode_name(params, req->Code), req->Param1, req->Param2, req->Param3);
+		break;
+	}
 
 	/* build appropriate USB container */
 	usbreq.length=htod32(PTP_USB_BULK_REQ_LEN-
@@ -104,7 +115,7 @@ retry:
 			}
 		} else
 			GP_LOG_E ("PTP_OC 0x%04x sending req failed: wrote only %d of %d bytes", req->Code, res, towrite);
-		return PTP_ERROR_IO;
+		return translate_gp_result_to_ptp(res);
 	}
 	return PTP_RC_OK;
 }
@@ -114,7 +125,8 @@ ptp_usb_senddata (PTPParams* params, PTPContainer* ptp,
 		  uint64_t size, PTPDataHandler *handler
 ) {
 	uint16_t ret = PTP_RC_OK;
-	int res, wlen, datawlen;
+	int res, wlen;
+	unsigned long datawlen;
 	PTPUSBBulkContainer usbdata;
 	unsigned long bytes_left_to_transfer, written;
 	Camera *camera = ((PTPData *)params->data)->camera;
@@ -150,7 +162,7 @@ ptp_usb_senddata (PTPParams* params, PTPContainer* ptp,
 			GP_LOG_E ("PTP_OC 0x%04x sending data failed: %s (%d)", ptp->Code, gp_port_result_as_string(res), res);
 		else
 			GP_LOG_E ("PTP_OC 0x%04x sending data failed: wrote only %d of %d bytes", ptp->Code, res, wlen);
-		return PTP_ERROR_IO;
+		return translate_gp_result_to_ptp(res);
 	}
 	if (size <= datawlen) { /* nothing more to do */
 		written = wlen;
@@ -176,7 +188,7 @@ ptp_usb_senddata (PTPParams* params, PTPContainer* ptp,
 			break;
 		res = gp_port_write (camera->port, (char*)bytes, readlen);
 		if (res < 0) {
-			ret = PTP_ERROR_IO;
+			ret = translate_gp_result_to_ptp(res);
 			break;
 		}
 		bytes_left_to_transfer -= res;
@@ -204,7 +216,7 @@ finalize:
 }
 
 static uint16_t
-ptp_usb_getpacket(PTPParams *params, PTPUSBBulkContainer *packet, uint32_t *rlen)
+ptp_usb_getpacket(PTPParams *params, PTPUSBBulkContainer *packet, uint32_t maxsize, uint32_t *rlen)
 {
 	int		tries = 0, result;
 	Camera		*camera = ((PTPData *)params->data)->camera;
@@ -213,6 +225,10 @@ ptp_usb_getpacket(PTPParams *params, PTPUSBBulkContainer *packet, uint32_t *rlen
 	if (params->response_packet_size > 0) {
 		GP_LOG_D ("Returning previously buffered response packet.");
 		/* If there is a buffered packet, just use it. */
+
+		if (params->response_packet_size > maxsize)
+			params->response_packet_size = maxsize;
+
 		memcpy(packet, params->response_packet, params->response_packet_size);
 		*rlen = params->response_packet_size;
 		free(params->response_packet);
@@ -223,10 +239,10 @@ ptp_usb_getpacket(PTPParams *params, PTPUSBBulkContainer *packet, uint32_t *rlen
 	}
 retry:
 	/* A packet should come in a single read always. */
-	result = gp_port_read (camera->port, (char*)packet, sizeof(*packet));
+	result = gp_port_read (camera->port, (char*)packet, maxsize);
 	/* This might be a left over zero-write of the device at the end of the previous transmission */
 	if (result == 0)
-		result = gp_port_read (camera->port, (char*)packet, sizeof(*packet));
+		result = gp_port_read (camera->port, (char*)packet, maxsize);
 	if (result > 0) {
 		*rlen = result;
 		return PTP_RC_OK;
@@ -238,10 +254,10 @@ retry:
 		if (tries++ < 1)
 			goto retry;
 	}
-	return PTP_ERROR_IO;
+	return translate_gp_result_to_ptp(result);
 }
 
-#define READLEN 64*1024 /* read blob size */
+#define READLEN 512*1024 /* read blob size, mostly to avoid reading all of it at once. */
 
 uint16_t
 ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
@@ -257,9 +273,16 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 	GP_LOG_D ("Reading PTP_OC 0x%0x (%s) data...", ptp->Code, ptp_get_opcode_name(params, ptp->Code));
 	PTP_CNT_INIT(usbdata);
 
-	ret = ptp_usb_getpacket (params, &usbdata, &bytes_read);
+	ret = ptp_usb_getpacket (params, &usbdata, sizeof(usbdata), &bytes_read);
 	if (ret != PTP_RC_OK)
 		goto exit;
+
+	if (bytes_read < PTP_USB_BULK_HDR_LEN) {
+		GP_LOG_E ("Read short bulk packet in data phase %d vs %d vs min len %d", bytes_read, dtoh32(usbdata.length), (uint32_t)PTP_USB_BULK_HDR_LEN);
+		ret = PTP_ERROR_IO;
+		goto exit;
+	}
+
 	if (dtoh16(usbdata.type) != PTP_USB_CONTAINER_DATA) {
 		/* We might have got a response instead. On error for instance. */
 		if (dtoh16(usbdata.type) == PTP_USB_CONTAINER_RESPONSE) {
@@ -296,7 +319,39 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 		} else {
 			GP_LOG_E ("Read broken PTP header (Code is %04x vs %04x).",
 				  dtoh16(usbdata.code), ptp->Code );
+
 			ret = PTP_ERROR_IO;
+
+			/* if this is a valid packet that came late, drain it to allow reestablishing */
+
+			if (bytes_read >= dtoh32(usbdata.length))
+				goto exit;
+
+			bytes_to_read = dtoh32(usbdata.length) - bytes_read;
+			bytes_read -= PTP_USB_BULK_HDR_LEN;
+
+			data = malloc(READLEN);
+			if (!data) goto exit;
+			while (bytes_to_read > 0) {
+				unsigned long chunk_to_read = bytes_to_read;
+
+				/* if in read-until-short-packet mode, read one packet at a time */
+				/* else read in large blobs */
+				/* else read all but the last short packet depending on EP packetsize. */
+				if (chunk_to_read > READLEN)
+					chunk_to_read = READLEN;
+				else if (chunk_to_read > params->maxpacketsize)
+					chunk_to_read = chunk_to_read - (chunk_to_read % params->maxpacketsize);
+				res = gp_port_read (camera->port, (char*)data, chunk_to_read);
+				if (res == GP_ERROR_IO_READ)
+					break;
+				if (res <= 0)
+					break;
+				bytes_to_read -= res;
+				bytes_read += res;
+			}
+			free (data);
+			data = NULL;
 			goto exit;
 		}
 	}
@@ -354,6 +409,22 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 	if (!data) return PTP_RC_GeneralError;
 
 	report_progress = (bytes_to_read > 2*CONTEXT_BLOCK_SIZE) && (dtoh32(usbdata.length) != 0xffffffffU);
+
+	/* On partial reads, do not report progress, this might lead to flickering progress bars.
+	 * FIXME: would be better to pass in a flag
+	 */
+	if (	(ptp->Code == PTP_OC_GetPartialObject) 					||
+		(	(params->deviceinfo.VendorExtensionID == PTP_VENDOR_CANON)	&&
+			(	(ptp->Code == PTP_OC_CANON_EOS_GetPartialObject)	||
+				(ptp->Code == PTP_OC_CANON_EOS_GetPartialObject64)	||
+				(ptp->Code == PTP_OC_CANON_EOS_GetPartialObjectEx)	||
+				(ptp->Code == PTP_OC_CANON_EOS_GetPartialObjectEX64)
+			)
+		)									||
+		(ptp->Code == PTP_OC_ANDROID_GetPartialObject64)
+	)
+		report_progress = 0;
+
 	if (report_progress)
 		progress_id = gp_context_progress_start (context, (bytes_to_read/CONTEXT_BLOCK_SIZE), _("Downloading..."));
 	while (bytes_to_read > 0) {
@@ -376,10 +447,10 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 			do_retry = FALSE;
 			continue;
 		} else if (res <= 0) {
-			ret = PTP_ERROR_IO;
+			ret = translate_gp_result_to_ptp(res);
 			break;
 		} else
-			do_retry = FALSE; /* once we have succesfully read any data, don't try again */
+			do_retry = FALSE; /* once we have successfully read any data, don't try again */
 		ret = handler->putfunc (params, handler->priv, res, data);
 		if (ret != PTP_RC_OK)
 			break;
@@ -423,11 +494,17 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 	GP_LOG_D ("Reading PTP_OC 0x%0x (%s) response...", resp->Code, ptp_get_opcode_name(params, resp->Code));
 	PTP_CNT_INIT(usbresp);
 	/* read response, it should never be longer than sizeof(usbresp) */
-	ret = ptp_usb_getpacket(params, &usbresp, &rlen);
+	ret = ptp_usb_getpacket(params, &usbresp, sizeof(usbresp), &rlen);
 
 	if (ret!=PTP_RC_OK) {
 		ret = PTP_ERROR_IO;
 	} else
+	if (rlen < 12) {
+		ret = PTP_ERROR_IO;
+	} else
+	if (rlen != dtoh32(usbresp.length)) {
+		ret = PTP_ERROR_IO;
+	} else
 	if (dtoh16(usbresp.type)!=PTP_USB_CONTAINER_RESPONSE) {
 		ret = PTP_ERROR_RESP_EXPECTED;
 	} else
@@ -465,6 +542,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 /* PTP Events wait for or check mode */
 #define PTP_EVENT_CHECK			0x0000	/* waits for */
 #define PTP_EVENT_CHECK_FAST		0x0001	/* checks */
+#define PTP_EVENT_CHECK_QUEUE		0x0002	/* just looks in the queue */
 
 static inline uint16_t
 ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
@@ -496,14 +574,19 @@ ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 		if (result <= 0) result = gp_port_check_int (camera->port, (char*)&usbevent, sizeof(usbevent));
 		gp_port_set_timeout (camera->port, timeout);
 		break;
+	case PTP_EVENT_CHECK_QUEUE:
+		gp_port_get_timeout (camera->port, &timeout);
+		gp_port_set_timeout (camera->port, 0); /* indicates no waiting at all */
+		result = gp_port_check_int (camera->port, (char*)&usbevent, sizeof(usbevent));
+		gp_port_set_timeout (camera->port, timeout);
+		break;
 	default:
 		return PTP_ERROR_BADPARAM;
 	}
 	if (result < 0) {
-		GP_LOG_E ("Reading PTP event failed: %s (%d)", gp_port_result_as_string(result), result);
-		if (result == GP_ERROR_TIMEOUT)
-			return PTP_ERROR_TIMEOUT;
-		return PTP_ERROR_IO;
+		if ((result != GP_ERROR_TIMEOUT) || (wait != PTP_EVENT_CHECK_FAST))
+			GP_LOG_E ("Reading PTP event failed: %s (%d)", gp_port_result_as_string(result), result);
+		return translate_gp_result_to_ptp(result);
 	}
 	if (result == 0) {
 		GP_LOG_E ("Reading PTP event failed: a 0 read occurred, assuming timeout.");
@@ -544,6 +627,12 @@ ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 	return PTP_RC_OK;
 }
 
+uint16_t
+ptp_usb_event_check_queue (PTPParams* params, PTPContainer* event) {
+
+	return ptp_usb_event (params, event, PTP_EVENT_CHECK_QUEUE);
+}
+
 uint16_t
 ptp_usb_event_check (PTPParams* params, PTPContainer* event) {
 
@@ -558,11 +647,13 @@ ptp_usb_event_wait (PTPParams* params, PTPContainer* event) {
 
 uint16_t
 ptp_usb_control_get_extended_event_data (PTPParams *params, char *buffer, int *size) {
-	Camera	*camera = ((PTPData *)params->data)->camera;
-	int	ret;
+	Camera		*camera = ((PTPData *)params->data)->camera;
+	int		ret;
+	GPPortSettings	settings;
 
 	GP_LOG_D ("Getting extended event data.");
-	ret = gp_port_usb_msg_class_read (camera->port, 0x65, 0x0000, 0x0000, buffer, *size);
+	gp_port_get_settings (camera->port, &settings);
+	ret = gp_port_usb_msg_class_read (camera->port, 0x65, 0x0000, settings.usb.interface, buffer, *size);
 	if (ret < GP_OK)
 		return PTP_ERROR_IO;
 	*size = ret;
@@ -571,11 +662,13 @@ ptp_usb_control_get_extended_event_data (PTPParams *params, char *buffer, int *s
 
 uint16_t
 ptp_usb_control_device_reset_request (PTPParams *params) {
-	Camera	*camera = ((PTPData *)params->data)->camera;
-	int	ret;
+	Camera		*camera = ((PTPData *)params->data)->camera;
+	int		ret;
+	GPPortSettings	settings;
 
 	GP_LOG_D ("Sending usb device reset request.");
-	ret = gp_port_usb_msg_class_write (camera->port, 0x66, 0x0000, 0x0000, NULL, 0);
+	gp_port_get_settings (camera->port, &settings);
+	ret = gp_port_usb_msg_class_write (camera->port, 0x66, 0x0000, settings.usb.interface, NULL, 0);
 	if (ret < GP_OK)
 		return PTP_ERROR_IO;
 	return PTP_RC_OK;
@@ -583,10 +676,13 @@ ptp_usb_control_device_reset_request (PTPParams *params) {
 
 uint16_t
 ptp_usb_control_get_device_status (PTPParams *params, char *buffer, int *size) {
-	Camera	*camera = ((PTPData *)params->data)->camera;
-	int	ret;
+	Camera		*camera = ((PTPData *)params->data)->camera;
+	int		ret;
+	GPPortSettings	settings;
 
-	ret = gp_port_usb_msg_class_read (camera->port, 0x67, 0x0000, 0x0000, buffer, *size);
+	GP_LOG_D ("Getting usb device status.");
+	gp_port_get_settings (camera->port, &settings);
+	ret = gp_port_usb_msg_class_read (camera->port, 0x67, 0x0000, settings.usb.interface, buffer, *size);
 	if (ret < GP_OK)
 		return PTP_ERROR_IO;
 	*size = ret;
@@ -595,13 +691,17 @@ ptp_usb_control_get_device_status (PTPParams *params, char *buffer, int *size) {
 
 uint16_t
 ptp_usb_control_cancel_request (PTPParams *params, uint32_t transactionid) {
-	Camera	*camera = ((PTPData *)params->data)->camera;
-	int	ret;
+	Camera		*camera = ((PTPData *)params->data)->camera;
+	int		ret;
 	unsigned char	buffer[6];
+	GPPortSettings	settings;
+
+	GP_LOG_D ("Sending cancel request.");
+	gp_port_get_settings (camera->port, &settings);
 
 	htod16a(&buffer[0],PTP_EC_CancelTransaction);
 	htod32a(&buffer[2],transactionid);
-	ret = gp_port_usb_msg_class_write (camera->port, 0x64, 0x0000, 0x0000, (char*)buffer, sizeof (buffer));
+	ret = gp_port_usb_msg_class_write (camera->port, 0x64, 0x0000, settings.usb.interface, (char*)buffer, sizeof (buffer));
 	if (ret < GP_OK)
 		return PTP_ERROR_IO;
 	return PTP_RC_OK;
