@@ -7,16 +7,17 @@
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301  USA
  */
+#define _DARWIN_C_SOURCE
 #include "config.h"
 #include "library.h"
 
@@ -82,7 +83,7 @@ typedef enum _SierraPacket SierraPacket;
 #define		QUICKSLEEP	5
 
 int sierra_change_folder (Camera *camera, const char *folder, GPContext *context)
-{	
+{
 	int st = 0, i;
 	char target[128];
 
@@ -90,7 +91,7 @@ int sierra_change_folder (Camera *camera, const char *folder, GPContext *context
 	GP_DEBUG ("*** folder: %s", folder);
 
 	/*
-	 * Do not issue the command if the camera doesn't support folders 
+	 * Do not issue the command if the camera doesn't support folders
 	 * or if the folder is the current working folder
 	 */
 	if (!camera->pl->folders || !strcmp (camera->pl->folder, folder))
@@ -117,7 +118,7 @@ int sierra_change_folder (Camera *camera, const char *folder, GPContext *context
 			target[i] = '\0';
 			if (st == i - 1)
 				break;
-			CHECK (sierra_set_string_register (camera, 84, 
+			CHECK (sierra_set_string_register (camera, 84,
 							   target + st, strlen (target + st), context));
 
 			st = i + 1;
@@ -164,7 +165,7 @@ int sierra_list_files (Camera *camera, const char *folder, CameraList *list, GPC
 
 	/*
 	 * Get the filename of the first picture. Note that some cameras
-	 * that don't support filenames return 8 blanks instead of 
+	 * that don't support filenames return 8 blanks instead of
 	 * reporting an error. If this is indeed the case, just fill
 	 * the list with dummy entries and return.
 	 */
@@ -200,8 +201,8 @@ int sierra_list_files (Camera *camera, const char *folder, CameraList *list, GPC
 int sierra_list_folders (Camera *camera, const char *folder, CameraList *list,
 			 GPContext *context)
 {
-	int j, count;
-	unsigned int i, bsize;
+	int i, j, count;
+	unsigned int bsize;
 	char buf[1024];
 
 	/* List the folders only if the camera supports them */
@@ -217,7 +218,7 @@ int sierra_list_folders (Camera *camera, const char *folder, CameraList *list,
 		CHECK (sierra_set_int_register (camera, 83, i + 1, context));
 		bsize = 1024;
 		GP_DEBUG ("*** getting name of folder %i", i + 1);
-		CHECK (sierra_get_string_register (camera, 84, 0, 
+		CHECK (sierra_get_string_register (camera, 84, 0,
 						   NULL, (unsigned char *)buf,
 						   &bsize, context));
 
@@ -232,9 +233,9 @@ int sierra_list_folders (Camera *camera, const char *folder, CameraList *list,
 
 /**
  * sierra_get_picture_folder:
- * @camera : camera data stucture
+ * @camera : camera data structure
  * @folder : folder name (to be freed by the caller)
- *     
+ *
  * Return the name of the folder that stores pictures. It is assumed that
  * the camera conforms with the JEIDA standard. Thus, look for the picture
  * folder into the /DCIM directory.
@@ -286,8 +287,8 @@ int sierra_get_picture_folder (Camera *camera, char **folder)
 
 /**
  * sierra_check_battery_capacity:
- * @camera : camera data stucture
- *     
+ * @camera : camera data structure
+ *
  * Check if the battery capacity is high enough.
  *
  * Returns: a gphoto2 error code
@@ -318,10 +319,10 @@ int sierra_check_battery_capacity (Camera *camera, GPContext *context)
 
 /**
  * sierra_get_memory_left:
- * @camera : camera data stucture
+ * @camera : camera data structure
  * @memory : memory left
- *     
- * Provide the available memory left 
+ *
+ * Provide the available memory left
  *
  * Returns: a gphoto2 error code
  */
@@ -374,7 +375,7 @@ sierra_check_connection (Camera *camera, GPContext *context)
 		default:
 
 			/*
-			 * If any error (except timeout) has occurred, 
+			 * If any error (except timeout) has occurred,
 			 * report it.
 			 */
 			CHECK (ret);
@@ -421,7 +422,7 @@ sierra_write_packet (Camera *camera, char *packet, GPContext *context)
 	case SIERRA_PACKET_COMMAND:
 		switch (camera->port->type) {
 		case GP_PORT_SERIAL:
-			packet[1] = (camera->pl->first_packet ? 
+			packet[1] = (camera->pl->first_packet ?
 					SUBSIERRA_PACKET_COMMAND_FIRST :
 					SUBSIERRA_PACKET_COMMAND);
 			camera->pl->first_packet = 0;
@@ -435,7 +436,7 @@ sierra_write_packet (Camera *camera, char *packet, GPContext *context)
 		break;
 	}
 
-	/* Determing packet length */
+	/* Determining packet length */
 	if ((packet[0] == SIERRA_PACKET_COMMAND) ||
 	    (packet[0] == SIERRA_PACKET_DATA) ||
 	    (packet[0] == SIERRA_PACKET_DATA_END)) {
@@ -451,7 +452,7 @@ sierra_write_packet (Camera *camera, char *packet, GPContext *context)
 		for (x = 4; x < length - 2; x++)
 			checksum += (unsigned char)packet[x];
 		packet[length-2] = checksum & 0xff;
-		packet[length-1] = (checksum >> 8) & 0xff; 
+		packet[length-1] = (checksum >> 8) & 0xff;
 	}
 
 	if (camera->pl->flags & SIERRA_WRAP_USB_MASK) {
@@ -468,7 +469,7 @@ sierra_write_packet (Camera *camera, char *packet, GPContext *context)
 static int
 sierra_clear_usb_halt(Camera *camera)
 {
-	
+
 	if ( (camera->port->type == GP_PORT_USB) &&
 	     !(camera->pl->flags & SIERRA_WRAP_USB_MASK) &&
 	     !(camera->pl->flags & SIERRA_NO_USB_CLEAR) )
@@ -492,9 +493,9 @@ sierra_write_nak (Camera *camera, GPContext *context)
 
 /**
  * sierra_read_packet:
- * @camera : camera data stucture
+ * @camera : camera data structure
  * @packet : to return the read packet
- *     
+ *
  * Read a data packet from the camera.
  *
  * Method:
@@ -569,7 +570,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 			continue;
 		}
 		if (result == 0) {
-			GP_DEBUG ("Read got 0 bytes.."); 
+			GP_DEBUG ("Read got 0 bytes..");
 			if (++r > 2) {
 				sierra_clear_usb_halt(camera);
 				GP_DEBUG ("Giving up...");
@@ -581,7 +582,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 		br = result;
 
 		/*
-		 * If the first read byte is not known, 
+		 * If the first read byte is not known,
 		 * report an error and exit the processing.
 		 */
 		switch (packet[0]) {
@@ -644,8 +645,12 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 		 * data, and 2 for the checksum.
 		 */
 		length = packet[2] + (packet[3] * 256) + 6;
+		if (length > SIERRA_PACKET_SIZE) {
+			GP_DEBUG ("Packet too long (%d)!", length);
+			return GP_ERROR_IO;
+		}
 
-		/* 
+		/*
 		 * Read until the end of the packet is reached
 		 * or an error occurred.
 		 */
@@ -676,7 +681,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 			for (c = 0, i = 4; i < br - 2; i++)
 				c += packet[i];
 			c &= 0xffff;
-			if (c == (packet[br - 2] + (packet[br - 1] * 256)))
+			if (c == ((unsigned int)packet[br - 2] + ((unsigned int)packet[br - 1] * 256)))
 				break;
 
 			/*
@@ -694,8 +699,8 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 			if ((packet[br - 2] == 0x00) &&
 			    (packet[br - 1] == 0x00))
 				break;
-			
-					
+
+
 			GP_DEBUG ("Checksum wrong (calculated 0x%x, "
 				"found 0x%x)!", c,
 				packet[br - 2] + (packet[br - 1] * 256));
@@ -799,7 +804,7 @@ sierra_transmit_ack (Camera *camera, char *packet, GPContext *context)
 					"retries."));
 				return GP_ERROR;
 			}
-			
+
 			/*
                          * The camera has ended this session and
                          * reverted the speed back to 19200. Reinitialize
@@ -826,7 +831,7 @@ sierra_transmit_ack (Camera *camera, char *packet, GPContext *context)
 
 static int
 sierra_build_packet (Camera *camera, char type, char subtype,
-		     int data_length, char *packet) 
+		     int data_length, char *packet)
 {
 	packet[0] = type;
 	switch (type) {
@@ -851,13 +856,13 @@ sierra_build_packet (Camera *camera, char type, char subtype,
 }
 
 static int
-sierra_write_ack (Camera *camera, GPContext *context) 
+sierra_write_ack (Camera *camera, GPContext *context)
 {
 	char buf[4096];
 	int ret;
 
 	GP_DEBUG ("Writing acknowledgement...");
-	
+
 	buf[0] = ACK;
 	ret = sierra_write_packet (camera, buf, context);
 	sierra_clear_usb_halt(camera);
@@ -868,7 +873,7 @@ sierra_write_ack (Camera *camera, GPContext *context)
 }
 
 int
-sierra_init (Camera *camera, GPContext *context) 
+sierra_init (Camera *camera, GPContext *context)
 {
 	unsigned char buf[SIERRA_PACKET_SIZE], packet[4096];
 	int ret, r = 0;
@@ -913,7 +918,7 @@ sierra_init (Camera *camera, GPContext *context)
 		CHECK (ret);
 
 		switch (buf[0]) {
-		case SIERRA_PACKET_NAK: 
+		case SIERRA_PACKET_NAK:
 
 			/* Everything is fine. */
 			return GP_OK;
@@ -946,7 +951,7 @@ static struct {
 };
 
 int
-sierra_set_speed (Camera *camera, SierraSpeed speed, GPContext *context) 
+sierra_set_speed (Camera *camera, SierraSpeed speed, GPContext *context)
 {
 	GPPortSettings settings;
 	unsigned int i, bit_rate;
@@ -957,7 +962,7 @@ sierra_set_speed (Camera *camera, SierraSpeed speed, GPContext *context)
 
 	/*
 	 * Check that the requested speed is valid. We don't want to bug
-	 * the user with our coding errors, therefore if the requested 
+	 * the user with our coding errors, therefore if the requested
 	 * speed is invalid, we use 19200.
 	 */
 	for (i = 0; SierraSpeeds[i].bit_rate; i++)
@@ -979,7 +984,7 @@ sierra_set_speed (Camera *camera, SierraSpeed speed, GPContext *context)
 	 * they are currently operating at that speed.
 	 */
 	CHECK (gp_port_get_settings (camera->port, &settings));
-	if (settings.serial.speed == bit_rate)
+	if ((unsigned int)settings.serial.speed == bit_rate)
 		return GP_OK;
 
 	/*
@@ -1037,7 +1042,7 @@ sierra_action (Camera *camera, SierraAction action, GPContext *context)
 
 int
 sierra_set_int_register (Camera *camera, int reg, int value,
-			 GPContext *context) 
+			 GPContext *context)
 {
 	char p[4096];
 
@@ -1062,7 +1067,7 @@ sierra_set_int_register (Camera *camera, int reg, int value,
 	return GP_OK;
 }
 
-int sierra_get_int_register (Camera *camera, int reg, int *value, GPContext *context) 
+int sierra_get_int_register (Camera *camera, int reg, int *value, GPContext *context)
 {
 	int r = 0;
 	unsigned char p[4096], buf[SIERRA_PACKET_SIZE];
@@ -1072,7 +1077,7 @@ int sierra_get_int_register (Camera *camera, int reg, int *value, GPContext *con
 	/* Build and send the packet. */
 	CHECK (sierra_build_packet (camera, SIERRA_PACKET_COMMAND, 0, 2, (char *)p));
 	p[4] = 0x01;
-	p[5] = reg; 
+	p[5] = reg;
 	CHECK (sierra_write_packet (camera, (char *)p, context));
 
 	while (1) {
@@ -1140,13 +1145,13 @@ int sierra_get_int_register (Camera *camera, int reg, int *value, GPContext *con
 
 int
 sierra_set_string_register (Camera *camera, int reg, const char *s,
-			    long int length, GPContext *context) 
+			    long int length, GPContext *context)
 {
 
 	char packet[4096];
 	char type;
 	long int x=0;
-	int seq=0, size=0; 
+	int seq=0, size=0;
 	int do_percent;
 	unsigned int id = 0;
 
@@ -1206,7 +1211,7 @@ sierra_set_string_register (Camera *camera, int reg, const char *s,
 	return GP_OK;
 }
 
-int sierra_get_string_register (Camera *camera, int reg, int fnumber, 
+int sierra_get_string_register (Camera *camera, int reg, int fnumber,
                                 CameraFile *file, unsigned char *b,
 				unsigned int *b_len, GPContext *context)
 {
@@ -1235,7 +1240,7 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 
 	/* Build and send the request */
 	CHECK (sierra_build_packet (camera, SIERRA_PACKET_COMMAND, 0, 2, (char *)p));
-	/* 
+	/*
 	 * If the extended protocol is enabled, use code 0x06 so we can send
 	 * and receive 32k size packages, otherwise code 0x04 is used and
 	 * we have 2k size packages.
@@ -1260,7 +1265,7 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 	do {
 
 		/* Read one packet and retry on timeout. */
-		r = sierra_read_packet (camera, p, context); 
+		r = sierra_read_packet (camera, p, context);
 		if (r == GP_ERROR_TIMEOUT) {
 			if (++retries > RETRIES) {
 				in_function = 0;
@@ -1318,7 +1323,7 @@ sierra_delete_all (Camera *camera, GPContext *context)
 }
 
 int
-sierra_delete (Camera *camera, int picture_number, GPContext *context) 
+sierra_delete (Camera *camera, int picture_number, GPContext *context)
 {
 	/* Tell the camera which picture to delete and execute command. */
 	CHECK (sierra_set_int_register (camera, 4, picture_number, context));
@@ -1328,7 +1333,7 @@ sierra_delete (Camera *camera, int picture_number, GPContext *context)
 }
 
 int
-sierra_end_session (Camera *camera, GPContext *context) 
+sierra_end_session (Camera *camera, GPContext *context)
 {
 	CHECK (sierra_action (camera, SIERRA_ACTION_END, context));
 
@@ -1394,10 +1399,10 @@ sierra_capture (Camera *camera, CameraCaptureType type,
 	CHECK (gp_port_set_timeout (camera->port, timeout));
 
 	if (filepath != NULL) {
-		/* 
+		/*
 		 * After the picture is taken, register 4 is set to the
 		 * current picture.
-		 */ 
+		 */
 		GP_DEBUG ("Getting picture number.");
 		r = sierra_get_int_register (camera, 4, &n, context);
 		if (r == GP_OK) {
@@ -1405,7 +1410,7 @@ sierra_capture (Camera *camera, CameraCaptureType type,
 		}
 		/*
 		 * We need to tell the frontend where the new image can be
-		 * found.  Unfortunatelly, we can only figure out the
+		 * found.  Unfortunately, we can only figure out the
 		 * filename. Therefore, reset the CameraFilesystem and let
 		 * it search for the filename.
 		 *
@@ -1414,7 +1419,7 @@ sierra_capture (Camera *camera, CameraCaptureType type,
 		 *
 		 * Not that some cameras that don't support filenames will
 		 * return 8 blanks instead of reporting an error.
-		 * 
+		 *
 		 * Some newer cameras (Nikon*) do not return register 4, so
 		 * ignore errors from that.
 		 */
