@@ -1,23 +1,23 @@
 /* sierra.c:
  *
- * Copyright  2001 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright 2001 Lutz Mueller <lutz@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 
 #include "config.h"
 #include "sierra.h"
@@ -60,8 +60,8 @@ static int get_jpeg_data(const char *data, int data_size, char **jpeg_data, int
 /* Useful markers */
 static const char JPEG_SOI_MARKER[]  = { (char)0xFF, (char)0xD8, '\0' };
 static const char JPEG_SOF_MARKER[]  = { (char)0xFF, (char)0xD9, '\0' };
-static const char JPEG_APP1_MARKER[] = { (char)0xFF, (char)0xE1, '\0' };
-static const char TIFF_SOI_MARKER[]  = { (char)0x49, (char)0x49, (char)0x2A, (char)0x00, (char)0x08, '\0' };
+/*static const char JPEG_APP1_MARKER[] = { (char)0xFF, (char)0xE1, '\0' };*/
+/*static const char TIFF_SOI_MARKER[]  = { (char)0x49, (char)0x49, (char)0x2A, (char)0x00, (char)0x08, '\0' };*/
 
 static struct {
 	const char *manuf;
@@ -74,7 +74,7 @@ static struct {
 } sierra_cameras[] = {
 	/* Manufacturer, Camera Model, Sierra Model,
 	   USBvendor id, USB product id,
-	   USB wrapper protocol, CameraDesc) */ 
+	   USB wrapper protocol, CameraDesc) */
 	{"Agfa", "ePhoto 307", 	SIERRA_MODEL_DEFAULT, 	0, 0,
 		SIERRA_NO_51, NULL},
 	{"Agfa", "ePhoto 780", 	SIERRA_MODEL_DEFAULT, 	0, 0, 0, NULL },
@@ -82,7 +82,8 @@ static struct {
 		SIERRA_NO_51 | SIERRA_LOW_SPEED, NULL },
 	{"Agfa", "ePhoto 1280",	SIERRA_MODEL_DEFAULT,	0, 0, 0, NULL },
 	{"Agfa", "ePhoto 1680", SIERRA_MODEL_DEFAULT,	0, 0, 0, NULL },
-	{"Apple", "QuickTake 200", SIERRA_MODEL_DEFAULT,	0, 0, 0, NULL },
+	/* unclear, if driven by fuji driver or by sierra driver */
+	{"Apple", "QuickTake 200 (Sierra Mode)", SIERRA_MODEL_DEFAULT,	0, 0, 0, NULL },
 	{"Chinon", "ES-1000", 	SIERRA_MODEL_DEFAULT,	0, 0, 0, NULL },
 	{"Epson", "PhotoPC 500", SIERRA_MODEL_EPSON,	0, 0, 0, NULL },
 	{"Epson", "PhotoPC 550", SIERRA_MODEL_EPSON,	0, 0, 0, NULL },
@@ -120,6 +121,9 @@ static struct {
 	{"Nikon", "CoolPix 4300 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x010e,
 						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
 							&cp4300_cam_desc},
+	{"Nikon", "CoolPix 4100 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x012c,
+						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
+							&cp4300_cam_desc},
 	{"Nikon", "CoolPix 3500 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0110,
 						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
 							&cp880_cam_desc},
@@ -165,7 +169,7 @@ static struct {
 	{"Olympus", "C-900 Zoom", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0, NULL },
 	{"Olympus", "C-900L Zoom", SIERRA_MODEL_OLYMPUS,	0, 0, 0, NULL },
 
-	/* http://sourceforge.net/tracker/index.php?func=detail&aid=1082569&group_id=8874&atid=358874 
+	/* http://sourceforge.net/tracker/index.php?func=detail&aid=1082569&group_id=8874&atid=358874
 	 * strangely only works with low speed
 	 */
 	{"Olympus", "C-990 Zoom", 	SIERRA_MODEL_OLYMPUS,	0, 0, SIERRA_LOW_SPEED, NULL },
@@ -257,14 +261,14 @@ static struct {
 	{0, 0}
 };
 
-int camera_id (CameraText *id) 
+int camera_id (CameraText *id)
 {
 	strcpy(id->text, "sierra");
 
 	return (GP_OK);
 }
 
-int camera_abilities (CameraAbilitiesList *list) 
+int camera_abilities (CameraAbilitiesList *list)
 {
 	int x;
 	CameraAbilities a;
@@ -278,9 +282,12 @@ int camera_abilities (CameraAbilitiesList *list)
 		a.port     = GP_PORT_SERIAL;
 		if ((sierra_cameras[x].usb_vendor  > 0) &&
 		    (sierra_cameras[x].usb_product > 0)) {
+#ifdef linux
+			/* The USB SCSI generic passthrough driver only works on Linux currently */
 			if (sierra_cameras[x].flags & (SIERRA_WRAP_USB_OLYMPUS|SIERRA_WRAP_USB_PENTAX|SIERRA_WRAP_USB_NIKON))
 			    a.port |= GP_PORT_USB_SCSI;
                         else
+#endif
 			    a.port |= GP_PORT_USB;
 		}
 		a.speed[0] = 9600;
@@ -288,7 +295,7 @@ int camera_abilities (CameraAbilitiesList *list)
 		a.speed[2] = 38400;
 		if (sierra_cameras[x].flags & SIERRA_LOW_SPEED) {
 			/*
-			 * Camera supports 9600 -> 38400 
+			 * Camera supports 9600 -> 38400
 			 */
 			a.speed[3] = 0;
 		} else {
@@ -306,7 +313,7 @@ int camera_abilities (CameraAbilitiesList *list)
 		a.operations        = 	GP_OPERATION_CAPTURE_IMAGE |
 					GP_OPERATION_CAPTURE_PREVIEW |
 					GP_OPERATION_CONFIG;
-		a.file_operations   = 	GP_FILE_OPERATION_DELETE | 
+		a.file_operations   = 	GP_FILE_OPERATION_DELETE |
 					GP_FILE_OPERATION_PREVIEW |
 					GP_FILE_OPERATION_AUDIO;
 		a.folder_operations = 	GP_FOLDER_OPERATION_DELETE_ALL |
@@ -495,7 +502,7 @@ camera_start (Camera *camera, GPContext *context)
 }
 
 int
-camera_stop (Camera *camera, GPContext *context) 
+camera_stop (Camera *camera, GPContext *context)
 {
 	GP_DEBUG ("Closing connection");
 
@@ -511,7 +518,7 @@ camera_stop (Camera *camera, GPContext *context)
 }
 
 static int
-camera_exit (Camera *camera, GPContext *context) 
+camera_exit (Camera *camera, GPContext *context)
 {
 	GP_DEBUG ("sierra camera_exit");
 
@@ -533,8 +540,9 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	char *jpeg_data = NULL;
 	int jpeg_size;
 	const char *data, *mime_type;
-	long int size;
-	int download_size, audio_info[8], transferred;
+	long unsigned int size;
+	int download_size, audio_info[8];
+	unsigned int transferred;
 
 	/*
 	 * Get the file number from the CameraFileSystem.
@@ -576,15 +584,15 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	switch (type) {
 	case GP_FILE_TYPE_PREVIEW:
 	case GP_FILE_TYPE_EXIF:
-		sierra_get_size(camera, 13, n, &download_size, context);
+		CHECK_STOP (camera, sierra_get_size(camera, 13, n, &download_size, context));
 		break;
 	case GP_FILE_TYPE_NORMAL:
-		sierra_get_size(camera, 12, n, &download_size, context);
+		CHECK_STOP (camera, sierra_get_size(camera, 12, n, &download_size, context));
 		break;
 	case GP_FILE_TYPE_AUDIO:
-		sierra_get_string_register (camera, 43, n, NULL,
-			(unsigned char *) &audio_info, &transferred, context);
-		if (transferred == 0) 
+		CHECK_STOP (camera, sierra_get_string_register (camera, 43, n, NULL,
+			(unsigned char *) &audio_info, &transferred, context));
+		if (transferred == 0)
 			download_size = 0;
 		else
 			download_size = audio_info[0];
@@ -595,7 +603,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 
 	/* Get the file */
 	CHECK_STOP (camera, sierra_get_string_register (camera, regd, n,
-					file, NULL, &download_size,  context));
+					file, NULL, (unsigned int *)&download_size,  context));
 	if (download_size == 0) {
 		/*
 		 * Assume a failure. At least coolpix 880 returns no error
@@ -613,7 +621,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		/* * Thumbnails are always JPEG files (as far as we know...) */
 		CHECK (gp_file_set_mime_type (file, GP_MIME_JPEG));
 
-		/* 
+		/*
 		 * Some camera (e.g. Epson 3000z) send the EXIF application
 		 * data as thumbnail of a picture. The JPEG file need to be
 		 * extracted. Equally for movies, the JPEG thumbnail is
@@ -641,7 +649,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		if (!strcmp (mime_type, GP_MIME_RAW))
 			CHECK (gp_file_set_mime_type (file, GP_MIME_QUICKTIME));
 		break;
-	
+
 	case GP_FILE_TYPE_AUDIO:
 		CHECK (gp_file_set_mime_type (file, GP_MIME_WAV));
 		break;
@@ -673,8 +681,8 @@ delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
 	 * Mick Grant <mickgr@drahthaar.clara.net> found out that his
 	 * Nicon CoolPix 880 won't have deleted any picture at this point.
 	 * It seems that those cameras just acknowledge the command but do
-	 * nothing in the end. Therefore, we have to count the number of 
-	 * pictures that are now on the camera. If there indeed are any 
+	 * nothing in the end. Therefore, we have to count the number of
+	 * pictures that are now on the camera. If there indeed are any
 	 * pictures, return an error. libgphoto2 will then try to manually
 	 * delete them one-by-one.
 	 */
@@ -687,7 +695,7 @@ delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
 
 static int
 delete_file_func (CameraFilesystem *fs, const char *folder,
-		  const char *filename, void *data, GPContext *context) 
+		  const char *filename, void *data, GPContext *context)
 {
 	Camera *camera = data;
 	int n;
@@ -710,7 +718,7 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 	/*
 	 * The following command takes a while, and there is no way to add
 	 * a nice progress bar, since it is a single sierra_action call.
-	 * so multiple gp_context_progress_update () calls can not add 
+	 * so multiple gp_context_progress_update () calls can not add
 	 * anything.
 	 */
 	CHECK_STOP (camera, sierra_delete (camera, n + 1, context));
@@ -721,7 +729,7 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 
 static int
 camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
-		GPContext *context) 
+		GPContext *context)
 {
 	CHECK (camera_start (camera, context));
 	CHECK_STOP (camera, sierra_capture (camera, type, path, context));
@@ -748,7 +756,7 @@ put_file_func (CameraFilesystem * fs, const char *folder, const char *filename,
 	char *picture_folder;
 	int ret;
 	const char *data_file;
-	long data_size;
+	long unsigned int data_size;
 	int available_memory;
 
 	GP_DEBUG ("*** put_file_func");
@@ -757,7 +765,7 @@ put_file_func (CameraFilesystem * fs, const char *folder, const char *filename,
 
 	if (type != GP_FILE_TYPE_NORMAL)
 		return GP_ERROR_BAD_PARAMETERS;
-	
+
 	/* Check the size */
 	CHECK (gp_file_get_data_and_size (file, &data_file, &data_size));
 	if ( data_size == 0 ) {
@@ -774,7 +782,7 @@ put_file_func (CameraFilesystem * fs, const char *folder, const char *filename,
 
 	/* Check the available memory */
 	CHECK (sierra_get_memory_left(camera, &available_memory, context));
-	if (available_memory < data_size) {
+	if (available_memory < 0 || (unsigned int)available_memory < data_size) {
 		gp_context_error (context,
 				    _("Not enough memory available on the memory card"));
 		return GP_ERROR_NO_MEMORY;
@@ -861,7 +869,7 @@ static void dump_register (Camera *camera, GPContext *context)
 		"spot metering mode",		/* 70 */
 		"?",
 		"zoom",
-		"?", "?", "?", "?", "?", "?", 
+		"?", "?", "?", "?", "?", "?",
 		"current filename",
 		"?",				/* 80 */
 		"?", "?",
@@ -878,14 +886,14 @@ static void dump_register (Camera *camera, GPContext *context)
 		"?", "?", "?", "?", "?", "?", "?", "?", "?",
 		"?"				/* 130 */
 	};
-		
+
 
 	GP_DEBUG ("*** Register:");
 	for (i = 0; i < 128; i++) {
 		ret = sierra_get_int_register (camera, i, &value, context);
 		if (ret >= 0)
 			GP_DEBUG (
-					 "***  %3i: %12i (%s)", i, value, 
+					 "***  %3i: %12i (%s)", i, value,
 					 description [i]);
 	}
 }
@@ -919,7 +927,7 @@ camera_get_config_olympus (Camera *camera, CameraWidget **window, GPContext *con
 		gp_widget_add_choice (child, _("Standard"));
 		gp_widget_add_choice (child, _("High"));
 		gp_widget_add_choice (child, _("Best"));
-		
+
                 switch (value) {
 		case 0: strcpy (t, _("Auto"));
 			break;
@@ -940,9 +948,9 @@ camera_get_config_olympus (Camera *camera, CameraWidget **window, GPContext *con
 	/* Shutter Speed */
         ret = sierra_get_int_register (camera, 3, &value, context);
         if (ret >= 0) {
-		
-		gp_widget_new (GP_WIDGET_RANGE, 
-			       _("Shutter Speed (microseconds, 0 auto)"), 
+
+		gp_widget_new (GP_WIDGET_RANGE,
+			       _("Shutter Speed (microseconds, 0 auto)"),
 			       &child);
 		gp_widget_set_range (child, 0, 2000000, 1);
 		float_value = value;
@@ -1044,7 +1052,7 @@ camera_get_config_olympus (Camera *camera, CameraWidget **window, GPContext *con
         ret = sierra_get_int_register (camera, 19, &value, context);
         if (ret >= 0) {
 
-		gp_widget_new (GP_WIDGET_RADIO, _("Brightness/Contrast"), 
+		gp_widget_new (GP_WIDGET_RADIO, _("Brightness/Contrast"),
 			       &child);
 		gp_widget_add_choice (child, _("Normal"));
 		gp_widget_add_choice (child, _("Bright+"));
@@ -1074,7 +1082,7 @@ camera_get_config_olympus (Camera *camera, CameraWidget **window, GPContext *con
 	/* White Balance */
         ret = sierra_get_int_register (camera, 20, &value, context);
         if (ret >= 0) {
-		
+
 		gp_widget_new (GP_WIDGET_RADIO, _("White Balance"), &child);
 		gp_widget_add_choice (child, _("Auto"));
 		gp_widget_add_choice (child, _("Skylight"));
@@ -1129,8 +1137,8 @@ camera_get_config_olympus (Camera *camera, CameraWidget **window, GPContext *con
 	/* Spot Metering Mode */
 	ret = sierra_get_int_register (camera, 70, &value, context);
 	if (ret >= 0) {
-		
-		gp_widget_new (GP_WIDGET_RADIO, _("Spot Metering Mode"), 
+
+		gp_widget_new (GP_WIDGET_RADIO, _("Spot Metering Mode"),
 			       &child);
 		gp_widget_add_choice (child, _("On"));
 		gp_widget_add_choice (child, _("Off"));
@@ -1183,7 +1191,7 @@ camera_get_config_olympus (Camera *camera, CameraWidget **window, GPContext *con
 	/* Auto Off (host) */
 	ret = sierra_get_int_register (camera, 23, &value, context);
 	if (ret >= 0) {
-		
+
 		gp_widget_new (GP_WIDGET_RANGE, _("Auto Off (host) "
 				       "(in seconds)"), &child);
 		gp_widget_set_info (child, _("How long will it take until the "
@@ -1211,7 +1219,7 @@ camera_get_config_olympus (Camera *camera, CameraWidget **window, GPContext *con
 	/* LCD Brightness */
 	ret = sierra_get_int_register (camera, 35, &value, context);
 	if (ret >= 0) {
-		
+
 		gp_widget_new (GP_WIDGET_RANGE, _("LCD Brightness"), &child);
 		gp_widget_set_range (child, 1, 7, 1);
 		gp_widget_set_value (child, &value);
@@ -1249,6 +1257,7 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 	if ((gp_widget_get_child_by_label (window, _("Resolution"), &child)
 		>= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("Auto")) == 0) {
 			i = 0;
@@ -1262,12 +1271,13 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 			return (GP_ERROR_NOT_SUPPORTED);
 		CHECK_STOP (camera, sierra_set_int_register (camera, 1, i, context));
 	}
-	
+
 	/* Shutter Speed */
 	GP_DEBUG ("*** setting shutter speed");
-	if ((gp_widget_get_child_by_label (window, 
+	if ((gp_widget_get_child_by_label (window,
 		_("Shutter Speed (microseconds, 0 auto)"), &child) >= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &float_value);
 		i = float_value;
 		CHECK_STOP (camera, sierra_set_int_register (camera, 3, i, context));
@@ -1275,9 +1285,10 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 
 	/* Aperture */
 	GP_DEBUG ("*** setting aperture");
-	if ((gp_widget_get_child_by_label (window, _("Aperture"), &child) 
+	if ((gp_widget_get_child_by_label (window, _("Aperture"), &child)
 		>= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("Auto")) == 0) {
 			i = 0;
@@ -1297,6 +1308,7 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 	if ((gp_widget_get_child_by_label (window, _("Color Mode"), &child)
 		>= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("Normal")) == 0) {
 			i = 0;
@@ -1318,6 +1330,7 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 	if ((gp_widget_get_child_by_label (window, _("Flash Mode"), &child)
 		>= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("Auto")) == 0) {
 			i = 0;
@@ -1337,9 +1350,10 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 	/* Brightness/Contrast */
 	GP_DEBUG (
 			 "*** setting brightness/contrast");
-	if ((gp_widget_get_child_by_label (window, _("Brightness/Contrast"), 
+	if ((gp_widget_get_child_by_label (window, _("Brightness/Contrast"),
 		&child) >= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("Normal")) == 0) {
 			i = 0;
@@ -1361,6 +1375,7 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 	if ((gp_widget_get_child_by_label (window, _("White Balance"), &child)
 		>= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("Auto")) == 0) {
 			i = 0;
@@ -1382,6 +1397,7 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 	if ((gp_widget_get_child_by_label (window, _("Lens Mode"), &child)
 		>= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
                 if (strcmp (value, _("Macro")) == 0) {
 			i = 1;
@@ -1397,9 +1413,10 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 	/* Spot Metering Mode */
 	GP_DEBUG (
 			 "*** setting spot metering mode");
-	if ((gp_widget_get_child_by_label (window, _("Spot Metering Mode"), 
+	if ((gp_widget_get_child_by_label (window, _("Spot Metering Mode"),
 		&child) >= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("On")) == 0) {
 			i = 3;
@@ -1412,9 +1429,10 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 
 	/* Zoom */
 	GP_DEBUG ("*** setting zoom");
-	if ((gp_widget_get_child_by_label (window, _("Zoom"), &child) 
+	if ((gp_widget_get_child_by_label (window, _("Zoom"), &child)
 		>= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, "1x") == 0) {
 			i = 0;
@@ -1424,7 +1442,7 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 			i = 520;
 		} else if (strcmp (value, "2.5x") == 0) {
 			i = 1032;
-		} else 
+		} else
 			return (GP_ERROR_NOT_SUPPORTED);
 		CHECK_STOP (camera, sierra_set_int_register (camera, 72, i, context));
 	}
@@ -1434,6 +1452,7 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 	if ((gp_widget_get_child_by_label (window, _("Auto Off (host) "
 					  "(in seconds)"), &child) >= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &i);
                 CHECK_STOP (camera, sierra_set_int_register (camera, 23, i, context));
         }
@@ -1444,24 +1463,27 @@ camera_set_config_olympus (Camera *camera, CameraWidget *window, GPContext *cont
 	if ((gp_widget_get_child_by_label (window, _("Auto Off (field) "
 		"(in seconds)"), &child) >= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &i);
 		CHECK_STOP (camera, sierra_set_int_register (camera, 24, i, context));
 	}
 
         /* LCD Brightness */
 	GP_DEBUG ("*** setting lcd brightness");
-	if ((gp_widget_get_child_by_label (window, 
+	if ((gp_widget_get_child_by_label (window,
 		_("LCD Brightness"), &child) >= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &i);
 		CHECK_STOP (camera, sierra_set_int_register (camera, 35, i, context));
 	}
 
         /* LCD Auto Off */
 	GP_DEBUG ("*** setting lcd auto off");
-	if ((gp_widget_get_child_by_label (window, 
+	if ((gp_widget_get_child_by_label (window,
 		_("LCD Auto Off (in seconds)"), &child) >= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &i);
 		CHECK_STOP (camera, sierra_set_int_register (camera, 38, i, context));
         }
@@ -1505,13 +1527,13 @@ camera_get_config_epson (Camera *camera, CameraWidget **window, GPContext *conte
 			break;
                 case 2: strcpy (t, _("F2.8"));
 			break;
-                case 3: strcpy (t, _("F4"));    
+                case 3: strcpy (t, _("F4"));
 			break;
-		case 4: strcpy (t, _("F5.6"));  
+		case 4: strcpy (t, _("F5.6"));
 			break;
-		case 5: strcpy (t, _("F8"));    
+		case 5: strcpy (t, _("F8"));
 			break;
-		case 6: strcpy (t, _("auto"));  
+		case 6: strcpy (t, _("auto"));
 			break;
                 default:
 			sprintf(t, _("%i (unknown)"), value);
@@ -1602,7 +1624,7 @@ camera_get_config_epson (Camera *camera, CameraWidget **window, GPContext *conte
 		gp_widget_add_choice (child, _("fine"));
 		gp_widget_add_choice (child, _("superfine"));
 		gp_widget_add_choice (child, _("HyPict"));
-		
+
                 switch (value) {
 		case 1: strcpy (t, _("standard"));
 			break;
@@ -1645,7 +1667,7 @@ camera_get_config_epson (Camera *camera, CameraWidget **window, GPContext *conte
 	/* Auto Off (host) */
 	ret = sierra_get_int_register (camera, 23, &value, context);
 	if (ret >= 0) {
-		
+
 		gp_widget_new (GP_WIDGET_RANGE, _("Auto Off (host) "
 				       "(in seconds)"), &child);
 		gp_widget_set_info (child, _("How long will it take until the "
@@ -1681,7 +1703,7 @@ camera_get_config_epson (Camera *camera, CameraWidget **window, GPContext *conte
 		gp_widget_add_choice (child, _("Italian"));
 		gp_widget_add_choice (child, _("Japanese"));
 		gp_widget_add_choice (child, _("Spanish"));
-		gp_widget_add_choice (child, _("Portugese"));
+		gp_widget_add_choice (child, _("Portuguese"));
                 switch (value) {
                 case 1: strcpy (t, _("Korean"));
                         break;
@@ -1697,7 +1719,7 @@ camera_get_config_epson (Camera *camera, CameraWidget **window, GPContext *conte
 			break;
 		case 8: strcpy (t, _("Spanish"));
 			break;
-		case 9: strcpy (t, _("Portugese"));
+		case 9: strcpy (t, _("Portuguese"));
 			break;
                 default:
                         sprintf (t, _("%i (unknown)"), value);
@@ -1731,8 +1753,9 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 
 	/* Aperture */
 	GP_DEBUG ("*** setting aperture");
-	if ((gp_widget_get_child_by_label (window, _("Aperture"), &child) 
+	if ((gp_widget_get_child_by_label (window, _("Aperture"), &child)
 	     >= 0) && gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("F2")) == 0) {
 			i = 0;
@@ -1747,7 +1770,7 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 		} else if (strcmp (value, _("F8")) == 0) {
 			i = 5;
 		} else if (strcmp (value, _("auto")) == 0) {
-			i = 6;			
+			i = 6;
 		} else
 			return (GP_ERROR_NOT_SUPPORTED);
 		CHECK_STOP (camera, sierra_set_int_register (camera, 5, i, context));
@@ -1757,6 +1780,7 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 	GP_DEBUG ("*** setting flash mode");
 	if ((gp_widget_get_child_by_label (window, _("Flash Mode"), &child)
 	     >= 0) && gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("Auto")) == 0) {
 			i = 0;
@@ -1777,6 +1801,7 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 	GP_DEBUG ("*** setting white balance");
 	if ((gp_widget_get_child_by_label (window, _("White Balance"), &child)
 	     >= 0) && gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("Auto")) == 0) {
 			i = 0;
@@ -1794,6 +1819,7 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 	if ((gp_widget_get_child_by_label (window, _("Lens Mode"), &child)
 		>= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
                 if (strcmp (value, _("Macro")) == 0) {
 			i = 1;
@@ -1808,6 +1834,7 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 	GP_DEBUG ("*** setting resolution");
 	if ((gp_widget_get_child_by_label (window, _("Resolution"), &child)
 	     >= 0) && gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("standard")) == 0) {
 			i = 1;
@@ -1821,11 +1848,12 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 			return (GP_ERROR_NOT_SUPPORTED);
 		CHECK_STOP (camera, sierra_set_int_register (camera, 1, i, context));
 	}
-	
+
 	/* Color Mode */
 	GP_DEBUG ("*** setting color mode");
 	if ((gp_widget_get_child_by_label (window, _("Color Mode"), &child)
 	     >= 0) && gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("color")) == 0) {
 			i = 1;
@@ -1841,6 +1869,7 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 	if ((gp_widget_get_child_by_label (window, _("Auto Off (host) "
 					  "(in seconds)"), &child) >= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &i);
                 CHECK_STOP (camera, sierra_set_int_register (camera, 23, i, context));
         }
@@ -1851,6 +1880,7 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 	if ((gp_widget_get_child_by_label (window, _("Auto Off (field) "
 		"(in seconds)"), &child) >= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &i);
 		CHECK_STOP (camera, sierra_set_int_register (camera, 24, i, context));
 	}
@@ -1859,6 +1889,7 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 	GP_DEBUG ("*** setting language");
 	if ((gp_widget_get_child_by_label (window, _("Language"), &child)
 	     >= 0) && gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &value);
 		if (strcmp (value, _("Korean")) == 0) {
 			i = 1;
@@ -1874,7 +1905,7 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 			i = 7;
 		} else if (strcmp (value, _("Spanish")) == 0) {
 			i = 8;
-		} else if (strcmp (value, _("Portugese")) == 0) {
+		} else if (strcmp (value, _("Portuguese")) == 0) {
 			i = 9;
 		} else
 			return (GP_ERROR_NOT_SUPPORTED);
@@ -1886,6 +1917,7 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 	if ((gp_widget_get_child_by_label (window, _("Date & Time"), &child)
 		>= 0) &&
 	    gp_widget_changed (child)) {
+	        gp_widget_set_changed (child, FALSE);
 		gp_widget_get_value (child, &i);
 		CHECK_STOP (camera, sierra_set_int_register (camera, 2, i, context));
 	}
@@ -1909,7 +1941,7 @@ camera_set_config_default (Camera *camera, CameraWidget *window, GPContext *cont
 
 
 static int
-camera_summary (Camera *camera, CameraText *summary, GPContext *c) 
+camera_summary (Camera *camera, CameraText *summary, GPContext *c)
 {
 	char buf[1024*32];
 	int v, r;
@@ -1938,15 +1970,20 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *c)
 	}
 	/* Get all the string-related info */
 
-	if (sierra_get_string_register (camera, 27, 0, NULL, t, &v, c) >= 0)
+	if (sierra_get_string_register (camera, 27, 0, NULL,
+					(unsigned char *)t, (unsigned int *)&v, c) >= 0)
 		sprintf (buf+strlen(buf), _("Camera Model: %s\n"), t);
-	if (sierra_get_string_register (camera, 48, 0, NULL, t, &v, c) >= 0)
+	if (sierra_get_string_register (camera, 48, 0, NULL,
+					(unsigned char *)t, (unsigned int *)&v, c) >= 0)
 		sprintf (buf+strlen(buf), _("Manufacturer: %s\n"), t);
-	if (sierra_get_string_register (camera, 22, 0, NULL, t, &v, c) >= 0)
+	if (sierra_get_string_register (camera, 22, 0, NULL,
+					(unsigned char *)t, (unsigned int *)&v, c) >= 0)
 		sprintf (buf+strlen(buf), _("Camera ID: %s\n"), t);
-	if (sierra_get_string_register (camera, 25, 0, NULL, t, &v, c) >= 0)
+	if (sierra_get_string_register (camera, 25, 0, NULL,
+					(unsigned char *)t, (unsigned int *)&v, c) >= 0)
 		sprintf (buf+strlen(buf), _("Serial Number: %s\n"), t);
-	if (sierra_get_string_register (camera, 26, 0, NULL, t, &v, c) >= 0)
+	if (sierra_get_string_register (camera, 26, 0, NULL,
+					(unsigned char *)t, (unsigned int *)&v, c) >= 0)
 		sprintf (buf+strlen(buf), _("Software Rev.: %s\n"), t);
 
 	/* Get all the integer information */
@@ -2003,7 +2040,8 @@ storage_info_func (CameraFilesystem *fs,
 	sif->fields |= GP_STORAGEINFO_FILESYSTEMTYPE;
 	sif->fstype  = GP_STORAGEINFO_FST_DCF;
 
-	if (sierra_get_string_register (camera, 25, 0, NULL, t, &v, c) >= 0) {
+	if (sierra_get_string_register (camera, 25, 0, NULL,
+					(unsigned char *)t, (unsigned int *)&v, c) >= 0) {
 		sif->fields |= GP_STORAGEINFO_LABEL;
 		strcpy (sif->label, t);
 	}
@@ -2020,7 +2058,7 @@ storage_info_func (CameraFilesystem *fs,
 }
 
 static int
-camera_manual (Camera *camera, CameraText *manual, GPContext *context) 
+camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 {
 	GP_DEBUG ("*** sierra camera_manual");
 
@@ -2045,7 +2083,7 @@ camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 		break;
 	case SIERRA_MODEL_OLYMPUS:
 	default:
-		strcpy (manual->text, 
+		strcpy (manual->text,
 		  _("Some notes about Olympus cameras (and others?):\n"
 		    "(1) Camera Configuration:\n"
 		    "    A value of 0 will take the default one (auto).\n"
@@ -2068,11 +2106,11 @@ camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 }
 
 static int
-camera_about (Camera *camera, CameraText *about, GPContext *context) 
+camera_about (Camera *camera, CameraText *about, GPContext *context)
 {
 	GP_DEBUG ("*** sierra camera_about");
-	
-	strcpy (about->text, 
+
+	strcpy (about->text,
 		_("sierra SPARClite library\n"
 		"Scott Fritzinger <scottf@unr.edu>\n"
 		"Support for sierra-based digital cameras\n"
@@ -2090,7 +2128,7 @@ camera_about (Camera *camera, CameraText *about, GPContext *context)
  * @data: input data table
  * @data_size: table size
  * @jpeg_data: return JPEG data table (NULL if no valid data is found).
- *             To be free by the the caller.
+ *             To be free by the caller.
  * @jpeg_size: return size of the jpeg_data table.
  *
  * Extract JPEG data form the provided input data (looking for the SOI
@@ -2109,7 +2147,7 @@ int get_jpeg_data(const char *data, int data_size, char **jpeg_data, int *jpeg_s
 		if (memcmp(data+i, JPEG_SOF_MARKER, 2) == 0)
 			sof_marker = data + i;
 	}
-   
+
 	if (soi_marker && sof_marker) {
 		/* Valid JPEG data has been found: build the output table */
 		*jpeg_size = sof_marker - soi_marker + 2;
@@ -2140,7 +2178,7 @@ static CameraFilesystemFuncs fsfuncs = {
 };
 
 int
-camera_init (Camera *camera, GPContext *context) 
+camera_init (Camera *camera, GPContext *context)
 {
         int x = 0, ret, value;
         int vendor=0;
@@ -2221,7 +2259,7 @@ camera_init (Camera *camera, GPContext *context)
 			/* Find the speed into abilities.speeds array.
 			 * Highest speeds are last ... so look backwards ...
 			 */
-			for (i=0;i<sizeof(a.speed)/sizeof(a.speed[0]) && a.speed[i];i++)
+			for (i=0;(unsigned int)i<sizeof(a.speed)/sizeof(a.speed[0]) && a.speed[i];i++)
 				/*empty*/;
 			i--;
 			for (; i >= 0 ; i--) {
