@@ -7,10 +7,10 @@
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
@@ -56,14 +56,16 @@
 
 static int
 blink2_getnumpics(
-    GPPort *port, GPContext *context,int *numpics
+    GPPort *port, GPContext *context, unsigned int *numpics
 ) {
-    char buf[6];
+    unsigned char buf[2];
     int ret;
-    
-    ret = gp_port_usb_msg_read(port, BLINK2_GET_NUMPICS, 0x03, 0, buf, 2);
-    if (ret<GP_OK)
+
+    ret = gp_port_usb_msg_read(port, BLINK2_GET_NUMPICS, 0x03, 0, (char*)buf, 2);
+    if (ret < GP_OK)
 	return ret;
+    gp_log(GP_LOG_DEBUG, "blink2","ret is %d", ret);
+    if (ret < 2) return GP_ERROR_IO_READ;
     *numpics = (buf[0]<<8) | buf[1];
     return GP_OK;
 }
@@ -92,8 +94,11 @@ static int
 file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		void *data, GPContext *context)
 {
-	Camera *camera = data;
-	int	i, ret, numpics, bytes;
+	Camera		*camera = data;
+	unsigned int	i;
+	int		ret;
+	int		bytes;
+	unsigned int	numpics;
 	unsigned char	*xbuf, buf[8];
 
 	ret = blink2_getnumpics( camera->port, context, &numpics );
@@ -106,11 +111,13 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		free(xbuf);
 		return ret;
 	}
+	if (ret < 1) return GP_ERROR_IO_READ;
 	ret = gp_port_read( camera->port, (char*)xbuf, bytes);
 	if (ret < GP_OK) {
 		free(xbuf);
 		return ret;
 	}
+	if (ret < bytes) return GP_ERROR_IO_READ;
 	for ( i=0; i < numpics; i++) {
 		char name[20];
 		if (xbuf[8*(i+1)])
@@ -120,7 +127,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		gp_list_append( list, name, NULL);
 	}
 	free(xbuf);
-	return (GP_OK);
+	return GP_OK;
 }
 
 static int
@@ -129,8 +136,10 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	       GPContext *context)
 {
 	Camera *camera = data;
-        int image_no, result;
-	int	i, ret, numpics, bytes;
+	int image_no, result;
+	unsigned int i;
+	int ret, bytes;
+	unsigned int numpics;
 	unsigned char	*xbuf, buf[8];
 
 	struct xaddr {
@@ -155,12 +164,14 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		free(xbuf);
 		return ret;
 	}
+	if (ret < 1) return GP_ERROR_IO_READ;
 	ret = gp_port_read (camera->port, (char*)xbuf, bytes);
 	if (ret < GP_OK) {
 		free(addrs);
 		free(xbuf);
 		return ret;
 	}
+	if (ret < bytes) return GP_ERROR_IO_READ;
 	for ( i=0; i < numpics; i++) {
 		int end, start;
 
@@ -170,6 +181,10 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		end = 	(xbuf[8*i+13] << 16) |
 			(xbuf[8*i+14] <<  8) |
 			(xbuf[8*i+15]);
+
+		if (end < start)
+			return GP_ERROR_IO_READ;
+
 		addrs[i].start = start;
 		addrs[i].len = (end-start)/4;
 		addrs[i].type = xbuf[8*(i+1)];
@@ -182,14 +197,18 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		free(addrs);
                 return image_no;
 	}
+	if (image_no < 0 || (unsigned int)image_no >= numpics) {
+		free(addrs);
+		gp_log(GP_LOG_DEBUG, "blink2","image %d requested, but only %d pics on camera?", image_no, numpics);
+                return GP_ERROR;
+	}
         switch (type) {
         case GP_FILE_TYPE_NORMAL:
 #ifdef HAVE_LIBJPEG
 {
 		char *convline,*convline2,*rawline;
 		unsigned char	*jpegdata;
-		unsigned int start, len, pitch;
-		int curread;
+		unsigned int curread, start, len, pitch;
 		struct jpeg_decompress_struct	dinfo;
 		struct jpeg_source_mgr		xjsm;
 		struct jpeg_error_mgr		jerr;
@@ -201,6 +220,11 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
         		gp_file_set_mime_type (file, GP_MIME_JPEG);
 		start = addrs[image_no].start;
 		len = addrs[image_no].len;
+
+		if (len >= UINT_MAX / 8) {
+			result = GP_ERROR_NO_MEMORY;
+			break;
+		}
 		jpegdata = (unsigned char*)malloc (len*8);
 		if (!jpegdata) {
 			result = GP_ERROR_NO_MEMORY;
@@ -223,11 +247,12 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		curread  = 0;
 		do {
 			int res;
-			res = gp_port_read (camera->port, (char*)(jpegdata+curread), len );
+			res = gp_port_read (camera->port, (char*)(jpegdata+curread), len-curread );
 			if (res < GP_OK) {
 				result = GP_OK;
 				break;
 			}
+			if (!res) break;
 			curread += res;
 		} while (curread<=len);
 
@@ -262,7 +287,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			gp_file_append (file, foo, strlen(foo));
 		}
 		for (i = 0; i < dinfo.output_height ; i++ ) {
-			int j;
+			unsigned int j;
 			JSAMPROW row[1];
 			JSAMPARRAY arr = row;
 
@@ -297,9 +322,9 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
         case GP_FILE_TYPE_RAW: {
 		char buf2[4096];
 		unsigned int start, len;
-		int curread;
+		unsigned int curread;
 		memset( buf, 0, sizeof(buf));
-		
+
 		if (addrs[image_no].type)
         		gp_file_set_mime_type (file, GP_MIME_AVI);
 		else
@@ -320,15 +345,17 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			break;
 		len = len*8;
 		do {
+			int ret;
+
 			curread = 4096;
 			if (curread > len) curread = len;
-			curread = gp_port_read( camera->port, buf2, curread);
-			if (curread < GP_OK) {
+			ret = gp_port_read( camera->port, buf2, curread);
+			if (ret < GP_OK) {
 				result = GP_OK;
 				break;
 			}
-			len -= curread;
-			result = gp_file_append( file, buf2, curread);
+			len -= ret;
+			result = gp_file_append( file, buf2, ret);
 			if (result < GP_OK)
 				break;
 		} while (len>0);
@@ -355,16 +382,18 @@ delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
 	ret = gp_port_usb_msg_read( camera->port, BLINK2_DELETE_ALL, 0x03, 0, buf, 1);
 	if (ret < GP_OK)
 		return ret;
-        return (GP_OK);
+	if (ret < 1) return GP_ERROR_IO_READ;
+        return GP_OK;
 }
 
 static int
 camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
                 GPContext *context)
 {
-	int oldnumpics, numpics, ret;
+	int ret;
+	unsigned int oldnumpics, numpics;
 	char buf[1];
-	
+
 	ret = blink2_getnumpics (camera->port, context, &oldnumpics);
 	if (ret < GP_OK)
 		return ret;
@@ -372,10 +401,12 @@ camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
 	ret = gp_port_usb_msg_read( camera->port, BLINK2_INIT_CAPTURE, 0x03, 0, buf, 1);
 	if (ret < GP_OK)
 		return ret;
+	if (ret < 1) return GP_ERROR_IO_READ;
 	do {
 		ret = gp_port_usb_msg_read( camera->port, BLINK2_CHECK_CAPTURE_FINISH, 0x03, 0, buf, 1);
 		if (ret < GP_OK)
 			return ret;
+		if (ret < 1) return GP_ERROR_IO_READ;
 		sleep(1);
 	} while (buf[0] == 0x00);
 
@@ -391,7 +422,7 @@ camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
 
 
 int
-camera_abilities (CameraAbilitiesList *list) 
+camera_abilities (CameraAbilitiesList *list)
 {
 	CameraAbilities a;
 
@@ -419,7 +450,7 @@ camera_abilities (CameraAbilitiesList *list)
 }
 
 int
-camera_id (CameraText *id) 
+camera_id (CameraText *id)
 {
 	strcpy(id->text, "SiPix Blink2");
 	return (GP_OK);
@@ -432,7 +463,7 @@ static CameraFilesystemFuncs fsfuncs = {
 };
 
 int
-camera_init (Camera *camera, GPContext *context) 
+camera_init (Camera *camera, GPContext *context)
 {
 	char buf[6];
 	int ret;
@@ -451,8 +482,10 @@ camera_init (Camera *camera, GPContext *context)
 	ret = gp_port_usb_msg_read( camera->port, BLINK2_GET_FIRMWARE_ID, 0x03, 0, buf, 6);
 	if (ret < GP_OK)
 		return ret;
+	if (ret < 6) return GP_ERROR_IO_READ;
 	ret = gp_port_usb_msg_read( camera->port, 0x04, 0x03, 0, buf, 1);
 	if (ret < GP_OK)
 		return ret;
+	if (ret < 1) return GP_ERROR_IO_READ;
 	return GP_OK;
 }
