@@ -14,11 +14,11 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 
 #include <config.h>
 #include <stdio.h>
@@ -35,8 +35,13 @@
 #include "ultrapocket.h"
 #include "smal.h"
 
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+# define sleep(x)
+#endif
+
 #define GP_MODULE "Smal Ultrapocket"
 
+#include <locale.h>
 #ifdef ENABLE_NLS
 #  include <libintl.h>
 #  undef _
@@ -66,9 +71,9 @@ static int
 ultrapocket_command(GPPort *port, int iswrite, unsigned char *data, int datasize) {
     int ret;
     if (iswrite)
-	ret = gp_port_write(port, data, datasize);
+	ret = gp_port_write(port, (char *)data, datasize);
     else
-        ret = gp_port_read(port, data, datasize);
+        ret = gp_port_read(port, (char *)data, datasize);
 
     return ret;
 }
@@ -168,7 +173,7 @@ getpicture_logitech_pd(Camera *camera, GPContext *context, unsigned char **rd, c
     unsigned char *rawdata;
     int            ptc,pc,id;
 
-    memcpy(command+3, filename, 11); /* the id of the image to transfer */
+    strncpy((char*)(command+3), filename, 11); /* the id of the image to transfer */
 
     CHECK_RESULT(ultrapocket_command(port, 1, command, 0x10));
 
@@ -205,7 +210,7 @@ getpicture_logitech_pd(Camera *camera, GPContext *context, unsigned char **rd, c
  * use a different protocol - have to differetiate.
  */
 int
-ultrapocket_getrawpicture(Camera *camera, GPContext *context, 
+ultrapocket_getrawpicture(Camera *camera, GPContext *context,
    unsigned char **pdata, int *size, const char *filename)
 {
     char           ppmheader[200];
@@ -231,7 +236,7 @@ ultrapocket_getrawpicture(Camera *camera, GPContext *context,
 	imgstart = 0x29;
 	break;
      default:
-	break;
+        return GP_ERROR;
     }
 
    tile = BAYER_TILE_BGGR;
@@ -246,11 +251,13 @@ ultrapocket_getrawpicture(Camera *camera, GPContext *context,
    pmmhdr_len = strlen(ppmheader);
    outsize = ((long)width + 4) * height * 3 + pmmhdr_len;
    outdata = malloc(outsize);
-   if (!outdata)
+   if (!outdata) {
+     free(rawdata);
      return (GP_ERROR_NO_MEMORY);
+   }
 
    /* Set header */
-   strcpy(outdata, ppmheader);
+   strcpy((char *)outdata, ppmheader);
 
    /* Expand the Bayer tiles */
    result = gp_bayer_expand((rawdata+imgstart), width + 4, height,
@@ -258,13 +265,13 @@ ultrapocket_getrawpicture(Camera *camera, GPContext *context,
 
    /* and chop the spare 4 pixels off the RHS */
    for (pc = 1; pc < height; pc++) {
-      memmove(outdata + pmmhdr_len + ((long)width * pc * 3), 
-              outdata + pmmhdr_len + (((long)width + 4) * pc * 3), 
+      memmove(outdata + pmmhdr_len + ((long)width * pc * 3),
+              outdata + pmmhdr_len + (((long)width + 4) * pc * 3),
               ((long)width) * 3);
    }
    /* modify outsize to reflect trim */
    outsize = ((long)width) * height * 3 + pmmhdr_len;
-   
+
    free(rawdata);
    if (result < 0) {
       free (outdata);
@@ -322,7 +329,7 @@ ultrapocket_getpicture(Camera *camera, GPContext *context, unsigned char **pdata
 	    ", gamma %.2f"
 #endif
 	    "\n%d %d\n"
-	    "255\n", BayerTileNames[tile], 
+	    "255\n", BayerTileNames[tile],
 #if DO_GAMMA
 	    GAMMA_NUMBER,
 #endif
@@ -339,7 +346,7 @@ ultrapocket_getpicture(Camera *camera, GPContext *context, unsigned char **pdata
    }
 
    /* Set header */
-   strcpy(outdata, ppmheader);
+   strcpy((char *)outdata, ppmheader);
 
    /* Decode and interpolate the Bayer tiles */
    result = gp_bayer_decode((rawdata+imgstart), width+4, height,
@@ -347,8 +354,8 @@ ultrapocket_getpicture(Camera *camera, GPContext *context, unsigned char **pdata
 
    /* and chop the spare 4 pixels off the RHS */
    for (pc = 1; pc < height; pc++) {
-      memmove(outdata + pmmhdr_len + ((long)width * pc * 3), 
-              outdata + pmmhdr_len + (((long)width + 4) * pc * 3), 
+      memmove(outdata + pmmhdr_len + ((long)width * pc * 3),
+              outdata + pmmhdr_len + (((long)width + 4) * pc * 3),
               ((long)width) * 3);
    }
    /* modify outsize to reflect trim */
@@ -377,13 +384,20 @@ ultrapocket_getpicture(Camera *camera, GPContext *context, unsigned char **pdata
 static int
 ultrapocket_reset(Camera *camera)
 {
-   GPPortInfo oldpi;
-   GPPort *port = camera->port;
    CameraAbilities cab;
    unsigned char cmdbuf[0x10];
+   GPPort *port = camera->port;
+
    gp_camera_get_abilities(camera, &cab);
    GP_DEBUG ("First connect since camera was used - need to reset cam");
 
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+   GPPortType porttype;
+   GPPortInfo oldpi;
+   GPPortInfo newpi;
+   char *val;
+#endif
+
    /*
     * this resets the ultrapocket.  Messy, but it's what the windows
     * software does.   We only reset if it's been plugged in since
@@ -394,15 +408,26 @@ ultrapocket_reset(Camera *camera)
    cmdbuf[1] = 0x01;
    CHECK_RESULT(ultrapocket_command(port, 1, cmdbuf, 0x10));
    /* -------------- */
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
    sleep(4); /* This should do - _might_ need increasing */
    CHECK_RESULT(gp_port_get_info(port, &oldpi));
+   gp_port_info_new(&newpi);
+   gp_port_info_get_name (oldpi, &val);
+   gp_port_info_set_name (newpi, val);
+   gp_port_info_get_path (oldpi, &val);
+   gp_port_info_set_path (newpi, val);
+   gp_port_info_get_type (oldpi, &porttype);
+   gp_port_info_set_type (newpi, porttype);
+   /*gp_port_info_get_library_filename (oldpi, &val);
+   gp_port_info_set_library_filename (newpi, val);*/
+
    CHECK_RESULT(gp_port_free(port));
    CHECK_RESULT(gp_port_new(&port));
-   CHECK_RESULT(gp_port_set_info(port, oldpi));
-   CHECK_RESULT(gp_port_usb_find_device(port, 
-      cab.usb_vendor, cab.usb_product));
+   CHECK_RESULT(gp_port_set_info(port, newpi));
+   CHECK_RESULT(gp_port_usb_find_device(port, cab.usb_vendor, cab.usb_product));
    CHECK_RESULT(gp_port_open(port));
    camera->port = port;
+#endif
    return GP_OK;
 }
 
@@ -414,7 +439,7 @@ ultrapocket_skip(GPPort *port, int npackets)
 
    gp_port_get_timeout(port, &old_timeout);
    gp_port_set_timeout(port, 100);
-   for (; (npackets > 0) && gp_port_read(port, retbuf, 0x1000); npackets--);
+   for (; (npackets > 0) && gp_port_read(port, (char *)retbuf, 0x1000); npackets--);
    gp_port_set_timeout(port, old_timeout);
    return GP_OK;
 }
@@ -486,7 +511,7 @@ static int getpicsoverview_generic(
    char fn[20];
    int picid;
    int reset_needed;
-   
+
    CHECK_RESULT(ultrapocket_sync(camera));
 
    memset(command, 0, 16);
@@ -560,7 +585,7 @@ static int getpicsoverview_logitech_pd(
  *
  * Fuji Slimshot + Axia etc
  * The camera responds with 8 * 0x1000 bytes
- * 
+ *
  * Creative CardCam
  * The camera responds with 8 * 0x1000 bytes
  *
