@@ -19,10 +19,10 @@
 /*                                                              */
 /* You should have received a copy of the GNU Library General   */
 /* Public License along with this library; if not, write to the */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
-/* Boston, MA 02111-1307, USA.                                  */
+/* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
+/* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 
 #include "config.h"
 
@@ -124,7 +124,7 @@ models[] =
 	{"D-Link:DSC 350+", 0x084d, 0x0003,
 		BRIDGE_SPCA500, SPCA50X_FLASH},
         {"Minton:S-Cam F5", 0x084d, 0x0003,
-	        BRIDGE_SPCA500, SPCA50X_FLASH},	
+	        BRIDGE_SPCA500, SPCA50X_FLASH},
 	{"PureDigital:Ritz Disposable", 0x04fc, 0xffff,
 		BRIDGE_SPCA504B_PD, SPCA50X_FLASH},
 	{NULL, 0, 0, 0, 0}
@@ -192,7 +192,7 @@ camera_capture (Camera *camera, CameraCaptureType type,
 
 	/* Not all our cameras support capture */
 	gp_camera_get_abilities (camera, &a);
-	if (!a.operations & GP_OPERATION_CAPTURE_IMAGE)
+	if (!(a.operations & GP_OPERATION_CAPTURE_IMAGE))
 		return GP_ERROR_NOT_SUPPORTED;
 
 	if (cam_has_flash(camera->pl))
@@ -243,9 +243,13 @@ camera_exit (Camera *camera, GPContext *context)
 			free (camera->pl->fats);
 			camera->pl->fats = NULL;
 		}
-		if (camera->pl->files) {
-			free (camera->pl->files);
-			camera->pl->files = NULL;
+		if (camera->pl->flash_files) {
+			free (camera->pl->flash_files);
+			camera->pl->flash_files = NULL;
+		}
+		if (camera->pl->sdram_files) {
+			free (camera->pl->sdram_files);
+			camera->pl->sdram_files = NULL;
 		}
 		if (camera->pl->flash_toc) {
 			free (camera->pl->flash_toc);
@@ -330,7 +334,10 @@ file_list_func (CameraFilesystem *fs, const char *folder,
 			CHECK (spca50x_sdram_get_info (camera->pl));
 
 		for (i = 0; i < camera->pl->num_files_on_sdram; i++) {
-			strncpy (temp_file, camera->pl->files[i].name, 12);
+			if (camera->pl->sdram_files[i].name)
+				strncpy (temp_file, camera->pl->sdram_files[i].name, 12);
+			else
+				strcpy (temp_file, "BAD.BAD");
 			temp_file[12] = 0;
 			gp_list_append (list, temp_file, NULL);
 		}
@@ -348,7 +355,8 @@ get_file_func (CameraFilesystem *fs, const char *folder,
 
 	Camera *camera = user_data;
 	unsigned char *data = NULL;
-	int size, number, filetype, flash_file_count = 0;
+	int number, filetype, flash_file_count = 0;
+	unsigned int size;
 
 	CHECK (number =
 	       gp_filesystem_number (camera->fs, folder, filename, context));
@@ -409,7 +417,7 @@ get_file_func (CameraFilesystem *fs, const char *folder,
 	}
 	if (!data)
 		return GP_ERROR;
-	return gp_file_set_data_and_size (file, data, size);
+	return gp_file_set_data_and_size (file, (char *)data, size);
 }
 
 static int
@@ -569,10 +577,9 @@ camera_init (Camera *camera, GPContext *context)
 			break;
 	}
 
-	camera->pl = malloc (sizeof (CameraPrivateLibrary));
+	camera->pl = calloc (sizeof (CameraPrivateLibrary), 1);
 	if (!camera->pl)
 		return (GP_ERROR_NO_MEMORY);
-	memset (camera->pl, 0, sizeof (CameraPrivateLibrary));
 	camera->pl->gpdev = camera->port;
 	camera->pl->dirty_sdram = 1;
 	camera->pl->dirty_flash = 1;
@@ -605,7 +612,7 @@ camera_init (Camera *camera, GPContext *context)
 	if (camera->pl->fw_rev > 1) {
 		CHECK (spca50x_detect_storage_type (camera->pl));
 	}
-   
+
 	if (cam_has_flash(camera->pl) || cam_has_card(camera->pl) ) {
 		if ((camera->pl->bridge == BRIDGE_SPCA504) ||
 		    (camera->pl->bridge == BRIDGE_SPCA504B_PD))
