@@ -23,10 +23,10 @@
 /*                                                              */
 /* You should have received a copy of the GNU Library General   */
 /* Public License along with this library; if not, write to the */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
-/* Boston, MA 02111-1307, USA.                                  */
+/* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
+/* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 
 #include "config.h"
 #include <string.h>
@@ -57,11 +57,12 @@ static void
 free_files (CameraPrivateLibrary *pl)
 {
 	int i;
-	if (pl->files) {
-		for (i = 0; i < pl->num_files_on_flash; i++){
-			if(pl->files[i].thumb) free (pl->files[i].thumb);
+	if (pl->flash_files) {
+		for (i = 0; i < pl->num_files_on_flash; i++) {
+			if(pl->flash_files[i].thumb) free (pl->flash_files[i].thumb);
 		}
-		free(pl->files);
+		free(pl->flash_files);
+		pl->flash_files = NULL;
 	}
 }
 
@@ -72,7 +73,9 @@ spca50x_flash_wait_for_ready(CameraPrivateLibrary *pl)
 	int timeout = 30;
 	uint8_t ready = 0;
 	while (timeout--) {
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 		sleep(1);
+#endif
 		if (pl->bridge == BRIDGE_SPCA500) {
 			CHECK (gp_port_usb_msg_read (pl->gpdev,
 							0x00, 0x0000, 0x0101,
@@ -99,8 +102,11 @@ spca500_flash_84D_wait_while_busy(CameraPrivateLibrary *pl)
 {
 	int timeout = 30;
 	uint8_t ready = 0;
+
 	while (timeout--) {
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 		sleep(1);
+#endif
 	CHECK (gp_port_usb_msg_read (pl->gpdev,
 					0x00, 0x0000, 0x0100,
 					(char*)&ready, 0x01));
@@ -183,19 +189,19 @@ spca50x_flash_get_TOC(CameraPrivateLibrary *pl, int *filecount)
 
 	if (pl->bridge == BRIDGE_SPCA500) { /* For dsc350 type cams */
 		/* read the TOC from the cam */
-		CHECK (gp_port_read (pl->gpdev, pl->flash_toc, toc_size));
+		CHECK (gp_port_read (pl->gpdev, (char *)pl->flash_toc, toc_size));
 		/* reset to idle */
 		CHECK (gp_port_usb_msg_write(pl->gpdev, 0x00, 0x0000, 0x0100, NULL, 0x0));
 		*filecount = (int)pl->flash_toc[10];
 		/* Now, create the files info buffer */
 		free_files(pl);
 		/* NOTE: using calloc to ensure new block is "empty" */
-		pl->files = calloc (1, *filecount * sizeof (struct SPCA50xFile));
-		if (!pl->files)
+		pl->flash_files = calloc (*filecount , sizeof (struct SPCA50xFile));
+		if (!pl->flash_files)
 			return GP_ERROR_NO_MEMORY;
 	} else { /* all other cams with flash... */
 		/* read the TOC from the cam */
-		CHECK (gp_port_read (pl->gpdev, pl->flash_toc, toc_size));
+		CHECK (gp_port_read (pl->gpdev, (char *)pl->flash_toc, toc_size));
 	}
 	/* record that TOC has been updated - clear the "dirty" flag */
 	pl->num_files_on_flash = *filecount;
@@ -315,8 +321,10 @@ spca500_flash_capture (CameraPrivateLibrary *pl)
 
 		/* wait until the camera is not busy any more */
 		/* spca50x_flash_wait_for_ready doesn't work here */
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 		sleep(3);
-		
+#endif
+
 		/* invalidate TOC/info cache */
 		pl->dirty_flash = 1;
 		return GP_OK;
@@ -332,7 +340,7 @@ spca50x_flash_get_file_name (CameraPrivateLibrary *pl, int index, char *name)
 {
 	uint8_t *p;
 	if (pl->bridge == BRIDGE_SPCA500) {
-		char p[14];
+		char p[30];
 		int w, h, type, size;
 
 		memset (p, 0, sizeof(p));
@@ -351,7 +359,7 @@ spca50x_flash_get_file_name (CameraPrivateLibrary *pl, int index, char *name)
 		else{
 			snprintf (p, sizeof(p), "Unknown");
 		}
-		memcpy (name, p, sizeof(p) );
+		strcpy (name, p);
 	} else {
 		if (pl->fw_rev == 1) {
 			p = pl->flash_toc + index*2*32;
@@ -370,7 +378,7 @@ spca50x_flash_get_file_name (CameraPrivateLibrary *pl, int index, char *name)
 static int
 spca50x_flash_get_number_from_file_name (CameraPrivateLibrary *pl, int index, int *file_number)
 {
-	char name[13];
+	char name[14];
 
 	CHECK (spca50x_flash_get_file_name (pl, index, name));
 	if(sscanf(&(name[4]), "%d", file_number) != 1)    /* skip "DSC_" */
@@ -389,12 +397,12 @@ spca500_flash_84D_get_file_info (CameraPrivateLibrary * pl, int index,
 
 	/* First, check if the info. is already buffered in the cam private lib
 	 * We DO NOT want to be fetching the entire thumbnail just to get the file size...*/
-	 if ((pl->dirty_flash == 0) && (pl->files[index].type != 0)){
+	 if ((pl->dirty_flash == 0) && (pl->flash_files[index].type != 0)){
 		 /* This file must have been checked before */
-		*w = pl->files[index].width ;
-		*h = pl->files[index].height ;
-		*t = pl->files[index].type ;
-		*sz = pl->files[index].size ;
+		*w = pl->flash_files[index].width ;
+		*h = pl->flash_files[index].height ;
+		*t = pl->flash_files[index].type ;
+		*sz = pl->flash_files[index].size ;
 		return GP_OK;
 	 } else if (pl->dirty_flash != 0){ /* should never happen, but just in case... */
 		CHECK(spca50x_flash_get_TOC (pl, &i));
@@ -429,9 +437,9 @@ spca500_flash_84D_get_file_info (CameraPrivateLibrary * pl, int index,
 	{
 		int j;
 		uint8_t * buf;
-		if (pl->files[index].thumb){
-			free (pl->files[index].thumb); /* discard any previous thumbnail data */
-			pl->files[index].thumb = NULL;
+		if (pl->flash_files[index].thumb){
+			free (pl->flash_files[index].thumb); /* discard any previous thumbnail data */
+			pl->flash_files[index].thumb = NULL;
 		}
 		buf = malloc (38 * 256); /* create a new buffer to hold the thumbnail data */
 		if (buf){
@@ -441,12 +449,12 @@ spca500_flash_84D_get_file_info (CameraPrivateLibrary * pl, int index,
 				CHECK (gp_port_read (pl->gpdev, (char *)&buf[j], 256));
 				j += 256;
 			}
-			pl->files[index].thumb = buf;
+			pl->flash_files[index].thumb = buf;
 		} else { /* couldn't get a buffer - read in the data anyway and discard it */
 			for (i = 0; i < 38; i++) {/* Thumbnails ALWAYS download 38 blocks...*/
 				CHECK (gp_port_read (pl->gpdev, waste, 256));
 			}
-			pl->files[index].thumb = NULL;
+			pl->flash_files[index].thumb = NULL;
 		}
 	} /* end of thumbnail storing loop */
 
@@ -518,10 +526,10 @@ spca500_flash_84D_get_file_info (CameraPrivateLibrary * pl, int index,
 	if (pl->dirty_flash == 0){
 		/* Only update the files cache if it exists, i.e. if dirty is zero */
 		/* We should never get here if dirty is set, however ! */
-		pl->files[index].type = *t;
-		pl->files[index].width = *w;
-		pl->files[index].height = *h;
-		pl->files[index].size = *sz;
+		pl->flash_files[index].type = *t;
+		pl->flash_files[index].width = *w;
+		pl->flash_files[index].height = *h;
+		pl->flash_files[index].size = *sz;
 	}
 
 	return GP_OK;
@@ -563,7 +571,7 @@ spca50x_process_thumbnail (CameraPrivateLibrary *lib,	/*  context */
 	 * This should never fail; it would be nice to have an error
 	 * code like GP_ERROR_CAMLIB_INTERNAL for cases like this.
  */
-	hdrlen = snprintf(tmp, alloc_size, "P6 %d %d 255\n", w, h);
+	hdrlen = snprintf((char*)tmp, alloc_size, "P6 %d %d 255\n", w, h);
 	true_size = w * h * 3 + hdrlen;
 	if ( true_size > alloc_size ) {
 		free (tmp);
@@ -572,7 +580,7 @@ spca50x_process_thumbnail (CameraPrivateLibrary *lib,	/*  context */
 
 	yuv_p = buf;
 	rgb_p = tmp + hdrlen;
-	while (yuv_p < buf + file_size) {
+	while ((yuv_p < buf + file_size) && (rgb_p < tmp + hdrlen + w*h*3 - 6)) {
 		uint32_t u, v, y, y2;
 		uint32_t r, g, b;
 
@@ -609,8 +617,8 @@ spca50x_flash_process_image (CameraPrivateLibrary *pl,	/*  context */
 	uint8_t qIndex = 0, format;
 	int file_size;
 
-	int w = pl->files[index].width;
-	int h = pl->files[index].height;
+	int w = pl->flash_files[index].width;
+	int h = pl->flash_files[index].height;
 
 	/* qindex == 2 seems to be right for all the dsc350 images - so far!!! */
 	qIndex = 2 ; /*  FIXME - what to do about Qtable stuff? */
@@ -647,7 +655,7 @@ spca500_flash_84D_get_file (CameraPrivateLibrary * pl,
 	char tbuf[260];		/*  for the file data blocks */
 	int i, j;		/*  general loop vars */
 	int blks;		/*  number of 256 byte blocks to fetch */
-	unsigned int sz;	/*  number of bytes in image */
+	int sz;			/*  number of bytes in image */
 	uint8_t *buf;		/*  buffer for the read data */
 	int type;		/*  type of image reported by header */
 	int w, h;		/*  width, height of file being read */
@@ -662,11 +670,11 @@ spca500_flash_84D_get_file (CameraPrivateLibrary * pl,
 	}
 
 	/* OK, we've decided what type of file it is, now lets upload it... */
-	if ((thumbnail != 0) && (pl->files[index].thumb != NULL)){
+	if ((thumbnail != 0) && (pl->flash_files[index].thumb != NULL)){
 		/* We can cheat here - we already have the thumbnail data in a cache, so don't load it again */
 		blks = 38;
-		buf = pl->files[index].thumb;
-		pl->files[index].thumb = NULL; /* we will release buf, so this pointer must be "emptied" */
+		buf = pl->flash_files[index].thumb;
+		pl->flash_files[index].thumb = NULL; /* we will release buf, so this pointer must be "emptied" */
 	}
 	else {
 		/* We need to download this data from the cam... */
@@ -711,9 +719,8 @@ spca500_flash_84D_get_file (CameraPrivateLibrary * pl,
 
 		/* create a buffer to hold all the read in data */
 		buf = malloc (blks * 256);
-		if (!buf){
+		if (!buf)
 			return GP_ERROR_NO_MEMORY;
-		}
 
 		j = 0; /* k = 0; */
 		/* Now read in the image data */
@@ -731,7 +738,7 @@ spca500_flash_84D_get_file (CameraPrivateLibrary * pl,
 	sz = (blks * 256) - 1;
 	/* look for the last non-zero byte in the file...
 	 * I hope the file never *really* ends with a zero!*/
-	while (buf[sz] == 0) {
+	while ((buf[sz] == 0)  && (sz>0)) {
 		sz -= 1;
 	}
 	true_len = sz + 1;
@@ -806,6 +813,9 @@ spca50x_flash_get_file (CameraPrivateLibrary *lib, GPContext *context,
 	if (lib->fw_rev != 1 && thumbnail)
 		return GP_ERROR_NOT_SUPPORTED;
 
+	if (!lib->flash_toc)
+		return GP_ERROR;
+
 	if (thumbnail) {
 		p = lib->flash_toc + (index*2+1) * 32;
 	} else {
@@ -860,7 +870,7 @@ spca50x_flash_get_file (CameraPrivateLibrary *lib, GPContext *context,
 		free (buf);
 		return ret;
 	}
-	ret = gp_port_read (lib->gpdev, buf, aligned_size);
+	ret = gp_port_read (lib->gpdev, (char *)buf, aligned_size);
 	if (ret < GP_OK) {
 		free (buf);
 		return ret;
@@ -999,7 +1009,7 @@ spca50x_flash_init (CameraPrivateLibrary *pl, GPContext *context)
 						jpReg[i].val, jpReg[i].reg,
 						NULL, 0x00));
 			CHECK (gp_port_usb_msg_read (pl->gpdev, 0x00, 0x0000,
-						jpReg[i].reg, bytes, 0x01));
+						jpReg[i].reg, (char *)bytes, 0x01));
 		}
 		CHECK (gp_port_usb_msg_write (pl->gpdev, 0x00, 0x0001, 0x2501,
 					NULL, 0x00));
@@ -1014,11 +1024,11 @@ spca50x_flash_init (CameraPrivateLibrary *pl, GPContext *context)
 		 * command. Unfortunately it is of yet unclear, what the exact procedure
 		 * is. Until we know, let's keep this here. */
 		CHECK (gp_port_usb_msg_read (pl->gpdev, 0x01, 0x0000, 0x0001,
-					bytes, 0x01));
+					(char *)bytes, 0x01));
 		CHECK (gp_port_usb_msg_read (pl->gpdev, 0x01, 0x0000, 0x0001,
-					bytes, 0x01));
+					(char *)bytes, 0x01));
 		CHECK (gp_port_usb_msg_read (pl->gpdev, 0x01, 0x0000, 0x0001,
-					bytes, 0x01));
+					(char *)bytes, 0x01));
 
 		/* Set to idle? */
 		CHECK (gp_port_usb_msg_read (pl->gpdev, 0x01, 0x0000, 0x000f,
@@ -1030,11 +1040,11 @@ spca50x_flash_init (CameraPrivateLibrary *pl, GPContext *context)
 
 		/* firmware detection */
 		CHECK (gp_port_usb_msg_read (pl->gpdev, 0x20, 0x0000, 0x0000,
-					bytes, 0x01));
+					(char *)bytes, 0x01));
 		CHECK (gp_port_usb_msg_read (pl->gpdev, 0x20, 0x0000, 0x0000,
-					bytes, 0x05));
+					(char *)bytes, 0x05));
 		CHECK (gp_port_usb_msg_read (pl->gpdev, 0x21, 0x0000, 0x0000,
-					bytes, 0x01));
+					(char *)bytes, 0x01));
 
 		/*
 		 * The cam is supposed to sync up with the computer time here
@@ -1059,7 +1069,7 @@ spca50x_flash_init (CameraPrivateLibrary *pl, GPContext *context)
 
 		for (i=0;i<7;i++)
 			CHECK (gp_port_usb_msg_write (pl->gpdev, 0x29, 0x0000,
-						i, bytes+i, 0x01));
+						i, (char *)bytes+i, 0x01));
 
 	}
 
