@@ -2,7 +2,7 @@
 /* spca50x_sdram.c - Gphoto2 library for cameras with sunplus   */
 /*             spca50x chips                                    */
 /*                                                              */
-/* Copyright  2002, 2003 Till Adam                             */
+/* Copyright 2002, 2003 Till Adam                               */
 /*                                                              */
 /* Author: Till Adam <till@adam-lilienthal.de>                  */
 /*                                                              */
@@ -20,9 +20,11 @@
 /*                                                              */
 /* You should have received a copy of the GNU Library General   */
 /* Public License along with this library; if not, write to the */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
-/* Boston, MA 02111-1307, USA.                                  */
+/* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
+/* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
+#define _DEFAULT_SOURCE
+
 #include "config.h"
 
 #include <stdio.h>
@@ -95,7 +97,8 @@ spca50x_sdram_get_fat_page (CameraPrivateLibrary * lib, int index,
 					 SPCA50X_FAT_PAGE_SIZE, p));
 			break;
 		default:
-			break;
+			gp_log(GP_LOG_ERROR, "spca50x", "spca50x_sdram_get_fat_page: dramtype %d unhandled", dramtype);
+			return GP_ERROR;
 	}
 
 	return GP_OK;
@@ -114,24 +117,28 @@ spca50x_sdram_get_file_count_and_fat_count (CameraPrivateLibrary * lib,
 		uint8_t lower, upper;
 
 		CHECK (gp_port_usb_msg_write (lib->gpdev, 0x5, 0, 0, NULL, 0));
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 		sleep (1);
+#endif
 		CHECK (gp_port_usb_msg_read
 				(lib->gpdev, 0, 0, 0xe15,
-				 (uint8_t *) & lib->num_files_on_sdram, 1));
+				 (char *) & lib->num_files_on_sdram, 1));
 		LE32TOH (lib->num_files_on_sdram);
 
 		/*  get fatscount */
 		CHECK (gp_port_usb_msg_write
 				(lib->gpdev, 0x05, 0x0000, 0x0008, NULL, 0));
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 		sleep (1);
+#endif
 		CHECK (gp_port_usb_msg_read
 				(lib->gpdev, 0, 0, 0x0e19,
-				 (uint8_t *) & lower, 1));
+				 (char *)&lower, 1));
 		CHECK (gp_port_usb_msg_read
 				(lib->gpdev, 0, 0, 0x0e20,
-				 (uint8_t *) & upper, 1));
+				 (char *)&upper, 1));
 
-		lib->num_fats = ((upper & 0xFF << 8) | (lower & 0xFF));
+		lib->num_fats = (((upper & 0xFF) << 8) | (lower & 0xFF));
 	} else {
 		while (1) {
 			CHECK (spca50x_sdram_get_fat_page (lib, lib->num_fats,
@@ -163,7 +170,9 @@ spca50x_sdram_delete_file (CameraPrivateLibrary * lib, unsigned int index)
 
 	CHECK (gp_port_usb_msg_write
 	       (lib->gpdev, 0x06, fat_index, 0x0007, NULL, 0));
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 	sleep (1);
+#endif
 
 	/* Reread fats the next time it is accessed */
 	lib->dirty_sdram = 1;
@@ -182,7 +191,9 @@ spca50x_sdram_delete_all (CameraPrivateLibrary * lib)
 		CHECK (gp_port_usb_msg_write
 		       (lib->gpdev, 0x02, 0x0000, 0x0005, NULL, 0));
 	}
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 	sleep (3);
+#endif
 
 	/* Reread fats the next time it is accessed */
 	lib->dirty_sdram = 1;
@@ -216,6 +227,7 @@ spca50x_get_image (CameraPrivateLibrary * lib, uint8_t ** buf,
 	int omit_escape = 0;
 
 	p = g_file->fat;
+	if (!p) return GP_ERROR;
 
 	/* get the position in memory where the image is */
 	start = (p[1] & 0xff) + (p[2] & 0xff) * 0x100;
@@ -270,8 +282,10 @@ spca50x_get_image (CameraPrivateLibrary * lib, uint8_t ** buf,
 			free (mybuf);
 			return ret;
 		}
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 		sleep (1);
-		ret = gp_port_read (lib->gpdev, mybuf, size);
+#endif
+		ret = gp_port_read (lib->gpdev, (char *)mybuf, size);
 		if (ret < GP_OK) {
 			free (mybuf);
 			return ret;
@@ -304,7 +318,7 @@ spca50x_get_avi (CameraPrivateLibrary * lib, uint8_t ** buf,
 {
 	int i, j, length, ret;
 	int frame_count = 0, frames_per_fat = 0, fn = 0;
-	int size = 0;
+	unsigned int size = 0;
 	int file_size;
 	int index_size;
 	uint32_t frame_size = 0, frame_width = 0, frame_height = 0;
@@ -378,6 +392,7 @@ spca50x_get_avi (CameraPrivateLibrary * lib, uint8_t ** buf,
 		+ 8 + index_size	/* for index chunk */
 		+ 1024 * 10 * frame_count;
 
+	GP_DEBUG("file_size = %d", file_size);
 	avi = malloc (file_size);
 	if (!avi) {
 		free (avi_index);
@@ -424,6 +439,14 @@ spca50x_get_avi (CameraPrivateLibrary * lib, uint8_t ** buf,
 				+ ((p[51 + j * 3] & 0xFF) * 0x100) +
 				(p[50 + j * 3] & 0xFF);
 
+			GP_DEBUG("frame_size = %d", frame_size);
+
+			if ((file_size - (avi - start_of_file)) < SPCA50X_AVI_FRAME_HEADER_LENGTH) {
+				free (mybuf);
+				GP_DEBUG("BAD: writing more than we allocated (%ld, %ld vs total %d)", (avi-start_of_file), (file_size - (avi - start_of_file)), SPCA50X_AVI_FRAME_HEADER_LENGTH);
+				return GP_ERROR_CORRUPTED_DATA;
+			}
+
 			memcpy (avi, SPCA50xAviFrameHeader,
 					SPCA50X_AVI_FRAME_HEADER_LENGTH);
 
@@ -431,10 +454,21 @@ spca50x_get_avi (CameraPrivateLibrary * lib, uint8_t ** buf,
 			start_of_frame = avi;
 
 			/* jpeg starts here */
-			create_jpeg_from_data (avi, data, qIndex, frame_width,
+			if ((data - mybuf) + frame_size > size) {
+				free (mybuf);
+				GP_DEBUG("BAD: accessing more than we read (%u vs total %d)", (unsigned int)((data-mybuf)+frame_size), size);
+				return GP_ERROR_CORRUPTED_DATA;
+			}
+			ret = create_jpeg_from_data (avi, data, qIndex, frame_width,
 					       frame_height, 0x22, frame_size,
 					       &length, 1, 0);
 
+			if (ret != GP_OK) {
+				free (mybuf);
+				return ret;
+			}
+
+			GP_DEBUG("avi added length = %d, width %d , height %d", length, frame_width, frame_height);
 			data += (frame_size + 7) & 0xfffffff8;
 			avi += length;
 			/* Make sure the next frame is aligned */
@@ -520,7 +554,7 @@ spca50x_get_avi_thumbnail (CameraPrivateLibrary * lib, uint8_t ** buf,
 	uint32_t start;
 	uint8_t *mybuf;
 	int size, o_size, file_size;
-	int w, h, ret;
+	int ret;
 
 	/* FIXME */
 	if (lib->bridge == BRIDGE_SPCA500)
@@ -537,8 +571,6 @@ spca50x_get_avi_thumbnail (CameraPrivateLibrary * lib, uint8_t ** buf,
 		(p[52] & 0xff) * 0x10000 + (p[51] & 0xff) * 0x100 +
 		(p[50] & 0xff);
 	qIndex = p[7] & 0x0f;
-	w = (int) ((p[8] & 0xFF) * 16);
-	h = (int) ((p[9] & 0xFF) * 16);
 
 	/* align */
 	if (size % 64 != 0)
@@ -627,8 +659,10 @@ spca50x_get_image_thumbnail (CameraPrivateLibrary * lib, uint8_t ** buf,
 			free (mybuf);
 			return ret;
 		}
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 		sleep (1);
-		ret = gp_port_read (lib->gpdev, mybuf, size);
+#endif
+		ret = gp_port_read (lib->gpdev, (char *)mybuf, size);
 		if (ret < GP_OK) {
 			free (mybuf);
 			return ret;
@@ -642,7 +676,7 @@ spca50x_get_image_thumbnail (CameraPrivateLibrary * lib, uint8_t ** buf,
 	}
 
 	tmp = *buf;
-	snprintf (tmp, *len, "P6 %d %d 255\n", t_width, t_height);
+	snprintf ((char*)tmp, *len, "P6 %d %d 255\n", t_width, t_height);
 	tmp += headerlength;
 
 	yuv_p = mybuf;
@@ -677,7 +711,7 @@ spca50x_get_image_thumbnail (CameraPrivateLibrary * lib, uint8_t ** buf,
 int
 spca50x_sdram_get_info (CameraPrivateLibrary * lib)
 {
-	unsigned int index;
+	int index;
 	uint8_t dramtype = 0;
 	uint8_t *p;
 	uint32_t start_page, end_page;
@@ -697,7 +731,7 @@ spca50x_sdram_get_info (CameraPrivateLibrary * lib)
 
 		CHECK (gp_port_usb_msg_read
 		       (lib->gpdev, 0, 0, SPCA50X_REG_DramType,
-			(uint8_t *) & dramtype, 1));
+			(char *) & dramtype, 1));
 		dramtype &= 0xFF;
 	}
 
@@ -706,7 +740,11 @@ spca50x_sdram_get_info (CameraPrivateLibrary * lib)
 	if (lib->num_files_on_sdram > 0) {
 		CHECK (spca50x_get_FATs (lib, dramtype));
 
-		index = lib->files[lib->num_files_on_sdram - 1].fat_end;
+		index = lib->sdram_files[lib->num_files_on_sdram - 1].fat_end;
+		if (index >= lib->num_fats) {
+			gp_log(GP_LOG_ERROR, "spca50x", "%d exceeds num_fats %d", index, lib->num_fats);
+			return GP_ERROR;
+		}
 		p = lib->fats + SPCA50X_FAT_PAGE_SIZE * index;
 		/* p now points to the fat of the last image of the last file */
 
@@ -733,7 +771,7 @@ spca50x_sdram_get_file_info (CameraPrivateLibrary * lib, unsigned int index,
 {
 	if (lib->dirty_sdram)
 		CHECK (spca50x_sdram_get_info (lib));
-	*g_file = &(lib->files[index]);
+	*g_file = &(lib->sdram_files[index]);
 	return GP_OK;
 }
 static int
@@ -751,7 +789,7 @@ spca50x_is_idle (CameraPrivateLibrary * lib)
 	int mode;
 
 	CHECK (gp_port_usb_msg_read
-	       (lib->gpdev, 0, 0, SPCA50X_REG_CamMode, (uint8_t *) & mode, 1));
+	       (lib->gpdev, 0, 0, SPCA50X_REG_CamMode, (char *) & mode, 1));
 
 	return mode == SPCA50X_CamMode_Idle ? 1 : 0;
 }
@@ -790,13 +828,13 @@ spca50x_download_data (CameraPrivateLibrary * lib, uint32_t start,
 
 	CHECK (gp_port_usb_msg_read
 	       (lib->gpdev, 0, 0, SPCA50X_REG_VlcAddressL,
-		(uint8_t *) & vlcAddressL, 1));
+		(char *) & vlcAddressL, 1));
 	CHECK (gp_port_usb_msg_read
 	       (lib->gpdev, 0, 0, SPCA50X_REG_VlcAddressM,
-		(uint8_t *) & vlcAddressM, 1));
+		(char *) & vlcAddressM, 1));
 	CHECK (gp_port_usb_msg_read
 	       (lib->gpdev, 0, 0, SPCA50X_REG_VlcAddressH,
-		(uint8_t *) & vlcAddressH, 1));
+		(char *) & vlcAddressH, 1));
 
 	foo = start & 0xFF;
 	CHECK (gp_port_usb_msg_write
@@ -816,7 +854,7 @@ spca50x_download_data (CameraPrivateLibrary * lib, uint32_t start,
 	       (lib->gpdev, 0, SPCA50X_TrigDramFifo, SPCA50X_REG_Trigger, NULL,
 		0));
 
-	CHECK (gp_port_read (lib->gpdev, buf, size));
+	CHECK (gp_port_read (lib->gpdev, (char *)buf, size));
 
 	CHECK (gp_port_usb_msg_write
 	       (lib->gpdev, 0, vlcAddressL, SPCA50X_REG_VlcAddressL, NULL, 0));
@@ -836,7 +874,7 @@ spca50x_get_FATs (CameraPrivateLibrary * lib, int dramtype)
 	unsigned int index = 0;
 	unsigned int file_index = 0;
 	uint8_t *p = NULL;
-	uint8_t buf[14];
+	char buf[30];
 
 	/* Reset image and movie counter */
 	lib->num_images = lib->num_movies = 0;
@@ -846,17 +884,17 @@ spca50x_get_FATs (CameraPrivateLibrary * lib, int dramtype)
 		lib->fats = NULL;
 	}
 
-	if (lib->files) {
-		free (lib->files);
-		lib->files = NULL;
+	if (lib->sdram_files) {
+		free (lib->sdram_files);
+		lib->sdram_files = NULL;
 	}
 
-	lib->fats = malloc (lib->num_fats * SPCA50X_FAT_PAGE_SIZE);
-	lib->files = malloc (lib->num_files_on_sdram * sizeof (struct SPCA50xFile));
+	lib->fats = calloc (lib->num_fats , SPCA50X_FAT_PAGE_SIZE);
+	lib->sdram_files = calloc (lib->num_files_on_sdram , sizeof (struct SPCA50xFile));
 
 	p = lib->fats;
 	if (lib->bridge == BRIDGE_SPCA504) {
-		while (index < lib->num_fats) {
+		while (index < (unsigned int)lib->num_fats) {
 			CHECK (spca50x_sdram_get_fat_page (lib, index,
 						dramtype, p));
 			if (p[0] == 0xFF)
@@ -870,16 +908,23 @@ spca50x_get_FATs (CameraPrivateLibrary * lib, int dramtype)
 		spca50x_reset (lib);
 		CHECK (gp_port_usb_msg_write
 		       (lib->gpdev, 0x05, 0x00, 0x07, NULL, 0));
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 		sleep (1);
+#endif
 		CHECK (gp_port_read
-		       (lib->gpdev, lib->fats,
+		       (lib->gpdev, (char *)lib->fats,
 			lib->num_fats * SPCA50X_FAT_PAGE_SIZE));
 	}
 
 	p = lib->fats;
 	index = 0;
 
-	while (index < lib->num_fats) {
+	while (index < (unsigned int)lib->num_fats) {
+		if (file_index >= (unsigned int)lib->num_files_on_sdram) {
+			free (lib->fats); lib->fats = NULL;
+			free (lib->sdram_files); lib->sdram_files = NULL;
+			return GP_ERROR;
+		}
 
 		type = p[0];
 		/* While the spca504a indicates start of avi as 0x08 and cont.
@@ -887,29 +932,34 @@ spca50x_get_FATs (CameraPrivateLibrary * lib, int dramtype)
 		 * gets its own fat table with a sequence number at p[18]. */
 		if ((type == 0x80) || (type == 0x03 && (p[18] != 0x00))) {
 			/* continuation of an avi */
-			lib->files[file_index - 1].fat_end = index;
+			if (!file_index)
+				return GP_ERROR;
+			lib->sdram_files[file_index - 1].fat_end = index;
 		} else {
 			/* its an image */
 			if (type == 0x00 || type == 0x01) {
-				snprintf (buf, 13, "Image%03d.jpg",
+				snprintf (buf, sizeof(buf), "Image%03d.jpg",
 					  ++lib->num_images);
-				lib->files[file_index].mime_type =
+				lib->sdram_files[file_index].mime_type =
 					SPCA50X_FILE_TYPE_IMAGE;
 			} else if ((type == 0x08) || (type == 0x03)) {
 				/* its the start of an avi */
-				snprintf (buf, 13, "Movie%03d.avi",
+				snprintf (buf, sizeof(buf), "Movie%03d.avi",
 					  ++lib->num_movies);
-				lib->files[file_index].mime_type =
+				lib->sdram_files[file_index].mime_type =
 					SPCA50X_FILE_TYPE_AVI;
+			} else {
+				gp_log(GP_LOG_ERROR, "spca50x", "type %d unhandled - error", type);
+				return GP_ERROR;
 			}
-			lib->files[file_index].fat = p;
-			lib->files[file_index].fat_start = index;
-			lib->files[file_index].fat_end = index;
-			lib->files[file_index].name = strdup (buf);
+			lib->sdram_files[file_index].fat = p;
+			lib->sdram_files[file_index].fat_start = index;
+			lib->sdram_files[file_index].fat_end = index;
+			lib->sdram_files[file_index].name = strdup (buf);
 			if (lib->bridge == BRIDGE_SPCA504) {
-				lib->files[file_index].width =
+				lib->sdram_files[file_index].width =
 					(p[8] & 0xFF) * 16;
-				lib->files[file_index].height =
+				lib->sdram_files[file_index].height =
 					(p[9] & 0xFF) * 16;
 			} else if (lib->bridge == BRIDGE_SPCA500) {
 				int w, h;
@@ -921,10 +971,10 @@ spca50x_get_FATs (CameraPrivateLibrary * lib, int dramtype)
 					w = 640;
 					h = 480;
 				}
-				lib->files[file_index].width = w;
-				lib->files[file_index].height = h;
+				lib->sdram_files[file_index].width = w;
+				lib->sdram_files[file_index].height = h;
 			}
-			lib->files[file_index].thumb = NULL;
+			lib->sdram_files[file_index].thumb = NULL;
 			file_index++;
 		}
 		p += SPCA50X_FAT_PAGE_SIZE;
