@@ -25,7 +25,7 @@
 /* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
 /* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 
 #include "config.h"
 
@@ -78,7 +78,7 @@ spca50x_detect_storage_type (CameraPrivateLibrary *lib)
 	for (i=0;i<3;i++)
 	{
 		buf[i] = 0;  /* if no data returned, assume no capability */
-		CHECK (gp_port_usb_msg_read (lib->gpdev, 0x28, 0x0000, 
+		CHECK (gp_port_usb_msg_read (lib->gpdev, 0x28, 0x0000,
 					i, (char *)&buf[i], 0x01));
 	}
 
@@ -152,7 +152,9 @@ spca50x_reset (CameraPrivateLibrary * lib)
  			CHECK (spca50x_pd_enable (lib));
 		}
 	}
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 	usleep(200000);
+#endif
 	return GP_OK;
 }
 
@@ -189,15 +191,19 @@ yuv2rgb (uint32_t y, uint32_t u, uint32_t v, uint32_t *_r, uint32_t *_g, uint32_
 int
 spca50x_capture (CameraPrivateLibrary * lib)
 {
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 	sleep (2);
+#endif
         CHECK (gp_port_usb_msg_write
 		(lib->gpdev, 0x06, 0x0000, 0x0003, NULL, 0));
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 	sleep (3);
+#endif
 	return GP_OK;
 }
 
-void
-create_jpeg_from_data (uint8_t * dst, uint8_t * src, int qIndex, int w,
+int
+create_jpeg_from_data (uint8_t * dst, uint8_t * src, unsigned int qIndex, int w,
 		       int h, uint8_t format, int o_size, int *size,
 		       int omit_huffman_table, int omit_escape)
 {
@@ -211,6 +217,10 @@ create_jpeg_from_data (uint8_t * dst, uint8_t * src, int qIndex, int w,
 			SPCA50X_JPG_DEFAULT_HEADER_PART1_LENGTH);
 
 	/* modify quantization table */
+	if (qIndex >= sizeof(SPCA50xQTable)/sizeof(SPCA50xQTable[0])/2) {
+		gp_log(GP_LOG_ERROR,"create_jpeg_from_data","qIndex %d too large", qIndex);
+		return GP_ERROR;
+	}
 	memcpy (dst + 7, SPCA50xQTable[qIndex * 2], 64);
 	memcpy (dst + 72, SPCA50xQTable[qIndex * 2 + 1], 64);
 
@@ -238,18 +248,22 @@ create_jpeg_from_data (uint8_t * dst, uint8_t * src, int qIndex, int w,
 	dst += SPCA50X_JPG_DEFAULT_HEADER_PART3_LENGTH;
 
 	for (i = 0; i < o_size; i++) {
+		if (dst - start >= *size) return GP_ERROR;
 		value = *(src + i) & 0xFF;
 		*(dst) = value;
 		dst++;
 
 		if (value == 0xFF && !omit_escape) {
+			if (dst - start >= *size) return GP_ERROR;
 			*(dst) = 0x00;
 			dst++;
 		}
 	}
+	if (dst + 2 - start >= *size) return GP_ERROR;
 	/* Add end of image marker */
 	*(dst++) = 0xFF;
 	*(dst++) = 0xD9;
 
 	*size = dst - start;
+	return GP_OK;
 }
