@@ -13,18 +13,19 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 #include "config.h"
 
 #include <string.h>
 #include <stdlib.h>
-#ifdef HAVE_ICONV
+#if defined(HAVE_ICONV) && defined(HAVE_LANGINFO_H)
 # include <langinfo.h>
 #endif
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 # include <gd.h>
 #endif
 
@@ -69,7 +70,7 @@ camera_abilities (CameraAbilitiesList *list)
 	a.usb_vendor = 0x1403;
 	a.usb_product= 0x0001;
 	a.operations = GP_OPERATION_NONE;
-	a.folder_operations = GP_FOLDER_OPERATION_PUT_FILE | 
+	a.folder_operations = GP_FOLDER_OPERATION_PUT_FILE |
 			      GP_FOLDER_OPERATION_DELETE_ALL;
 	a.file_operations   = GP_FILE_OPERATION_DELETE | GP_FILE_OPERATION_RAW;
 	return gp_abilities_list_append (list, a);
@@ -129,7 +130,7 @@ static int get_file_idx(CameraPrivateLibrary *pl, const char *folder,
 	return i;
 }
 
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 static void
 rotate90 (gdImagePtr src, gdImagePtr dest)
 {
@@ -180,7 +181,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 {
 	Camera *camera = data;
 	int idx, size;
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 	int ret;
 	gdImagePtr im, rotated;
 	void *gdpng;
@@ -203,7 +204,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		return GP_OK;
 	}
 
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 	if (type != GP_FILE_TYPE_NORMAL)
 		return GP_ERROR_NOT_SUPPORTED;
 
@@ -236,7 +237,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	ret = gp_file_set_mime_type (file, GP_MIME_PNG);
 	if (ret < 0) { gdFree (gdpng); return ret; }
 
-	ret = gp_file_set_name (file, filename); 
+	ret = gp_file_set_name (file, filename);
 	if (ret < 0) { gdFree (gdpng); return ret; }
 
 	ret = gp_file_append (file, gdpng, size);
@@ -249,16 +250,16 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 }
 
 static int
-put_file_func (CameraFilesystem *fs, const char *folder, const char *name, 
+put_file_func (CameraFilesystem *fs, const char *folder, const char *name,
 	CameraFileType type, CameraFile *file, void *data, GPContext *context)
 {
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 	Camera *camera = data;
 	char *c, *in_name, *out_name, *filedata = NULL;
 	int ret, in_width, in_height, in_x, in_y;
 	size_t inc, outc;
 	double aspect_in, aspect_out;
-#ifdef HAVE_ICONV
+#if defined(HAVE_ICONV) && defined(HAVE_LANGINFO_H)
 	char *in, *out;
 #else
 	int i;
@@ -280,10 +281,10 @@ put_file_func (CameraFilesystem *fs, const char *folder, const char *name,
 	}
 
 	/* Convert name to ASCII */
-#ifdef HAVE_ICONV
+#if defined(HAVE_ICONV) && defined(HAVE_LANGINFO_H)
 	in = in_name;
 	out = out_name;
-	if (iconv (camera->pl->cd, &in, &inc, &out, &outc) == -1) {
+	if (iconv (camera->pl->cd, &in, &inc, &out, &outc) == (size_t)(-1)) {
 		free (in_name);
 		free (out_name);
 		gp_log (GP_LOG_ERROR, "iconv",
@@ -442,7 +443,7 @@ static int
 file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		void *data, GPContext *context)
 {
-	Camera *camera = data; 
+	Camera *camera = data;
 	int i, ret;
 
 	for (i = 0; i < ST2205_MAX_NO_FILES; i++) {
@@ -583,7 +584,7 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 }
 
 static int
-camera_exit (Camera *camera, GPContext *context) 
+camera_exit (Camera *camera, GPContext *context)
 {
 	char buf[2];
 
@@ -593,7 +594,7 @@ camera_exit (Camera *camera, GPContext *context)
 		gp_setting_set ("st2205", "syncdatetime", buf);
 		gp_setting_set ("st2205", "orientation", orientation_to_string
 						(camera->pl->orientation));
-#ifdef HAVE_ICONV
+#if defined(HAVE_ICONV) && defined(HAVE_LANGINFO_H)
 		if (camera->pl->cd != (iconv_t) -1)
 			iconv_close (camera->pl->cd);
 #endif
@@ -605,10 +606,10 @@ camera_exit (Camera *camera, GPContext *context)
 }
 
 int
-camera_init (Camera *camera, GPContext *context) 
+camera_init (Camera *camera, GPContext *context)
 {
 	int i, j, ret;
-#ifdef HAVE_ICONV
+#if defined(HAVE_ICONV) && defined(HAVE_LANGINFO_H)
 	char *curloc;
 #endif
 	char buf[256];
@@ -642,7 +643,7 @@ camera_init (Camera *camera, GPContext *context)
 			camera->pl->orientation = ret;
 	}
 
-#ifdef HAVE_ICONV
+#if defined(HAVE_ICONV) && defined(HAVE_LANGINFO_H)
 	curloc = nl_langinfo (CODESET);
 	if (!curloc)
 		curloc="UTF-8";
@@ -698,12 +699,20 @@ camera_init (Camera *camera, GPContext *context)
 
 	/* Sync time if requested */
 	if (camera->pl->syncdatetime) {
+#ifdef HAVE_LOCALTIME_R
 		struct tm tm;
+#endif
+		struct tm *xtm;
 		time_t t;
 
 		t = time (NULL);
-		if (localtime_r (&t , &tm)) {
-			ret = st2205_set_time_and_date (camera, &tm);
+
+#ifdef HAVE_LOCALTIME_R
+		if ((xtm = localtime_r (&t , &tm))) {
+#else
+		if ((xtm = localtime (&t))) {
+#endif
+			ret = st2205_set_time_and_date (camera, xtm);
 			if (ret != GP_OK) {
 				camera_exit (camera, context);
 				return ret;
