@@ -13,9 +13,12 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
+#define _DEFAULT_SOURCE
+
 #include "config.h"
 
 #include <stdio.h>
@@ -25,10 +28,9 @@
 #include <stdlib.h>
 #include <time.h>
 #include <fcntl.h>
-#include <sys/mman.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 #include <gd.h>
 #endif
 
@@ -76,7 +78,7 @@ tp6801_set_time_and_date(Camera *camera, struct tm *t)
 	char sense_buffer[32];
 
 	memset (cmd_buffer, 0, sizeof (cmd_buffer));
-	
+
 	cmd_buffer[0] = TP6801_SET_TIME;
 	cmd_buffer[1] = 0x11;
 	cmd_buffer[2] = 0x31;
@@ -321,7 +323,7 @@ int
 tp6801_max_filecount(Camera *camera)
 {
 	int free_mem, size;
-	
+
 	size = tp6801_filesize (camera);
 	free_mem = camera->pl->mem_size - TP6801_PICTURE_OFFSET(0, size);
 	free_mem -= TP6801_CONST_DATA_SIZE;
@@ -371,7 +373,7 @@ tp6801_check_file_present(Camera *camera, int idx)
 static int
 tp6801_decode_image(Camera *camera, char *_src, int **dest)
 {
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 	int x, y;
 	unsigned char *src = (unsigned char *)_src;
 
@@ -399,7 +401,7 @@ tp6801_decode_image(Camera *camera, char *_src, int **dest)
 static int
 tp6801_encode_image(Camera *camera, int **src, char *dest)
 {
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 	int x, y;
 
 	for (y = 0; y < camera->pl->height; y++) {
@@ -689,7 +691,7 @@ tp6801_commit(Camera *camera)
 		camera->pl->page_state[TP6801_PAT_PAGE] |= TP6801_PAGE_DIRTY;
 	}
 
-	/* And commit the block with the PAT */	
+	/* And commit the block with the PAT */
 	CHECK (tp6801_commit_block (camera, 0))
 
 	return GP_OK;
