@@ -6,12 +6,6 @@
 
 #include "samples.h"
 
-static void errordumper(GPLogLevel level, const char *domain, const char *format,
-                 va_list args, void *data) {
-  vfprintf(stdout, format, args);
-  fprintf(stdout, "\n");
-}
-
 /* Set the owner of the first camera.
  *
  * This program can autodetect a single camera and then sets its
@@ -29,9 +23,6 @@ int main(int argc, char **argv) {
 	GPContext	*context;
 
 	context = sample_create_context (); /* see context.c */
-
-	gp_log_add_func(GP_LOG_ERROR, errordumper, NULL);
-
 	gp_camera_new (&camera);
 
 	/* This call will autodetect cameras, take the
@@ -46,17 +37,17 @@ int main(int argc, char **argv) {
 		return 0;
 	}
 
-	ret = get_config_value_string (camera, "ownername", &owner, context);
+	ret = get_config_value_string (camera, "artist", &owner, context);
 	if (ret < GP_OK) {
 		printf ("Could not query owner.\n");
 		goto out;
 	}
 	printf("Current owner: %s\n", owner);
 	if (argc > 1) {
-		ret = set_config_value_string (camera, "ownername", argv[1], context);
+		ret = set_config_value_string (camera, "artist", argv[1], context);
 		if (ret < GP_OK) {
 			fprintf (stderr, "Failed to set camera owner to %s; %d\n", argv[1], ret);
-		} else 
+		} else
 			printf("New owner: %s\n", argv[1]);
 	}
 out:
