@@ -1,4 +1,4 @@
-/* 
+/*
  * This program does the equivalent of:
  * gphoto2 --wait-event-and-download
  *
@@ -41,7 +41,7 @@ camera_tether(Camera *camera, GPContext *context) {
 			path = (CameraFilePath*)evtdata;
 			printf("File added on the camera: %s/%s\n", path->folder, path->name);
 
-			fd = open(path->name, O_CREAT | O_WRONLY, 0644);
+			fd = open (path->name, O_CREAT | O_WRONLY | O_BINARY, 0644);
 			retval = gp_file_new_from_fd(&file, fd);
 			printf("  Downloading %s...\n", path->name);
 			retval = gp_camera_file_get(camera, path->folder, path->name,
@@ -50,10 +50,17 @@ camera_tether(Camera *camera, GPContext *context) {
 			printf("  Deleting %s on camera...\n", path->name);
 			retval = gp_camera_file_delete(camera, path->folder, path->name, context);
 			gp_file_free(file);
+			free(evtdata);
 			break;
 		case GP_EVENT_FOLDER_ADDED:
 			path = (CameraFilePath*)evtdata;
 			printf("Folder added on camera: %s / %s\n", path->folder, path->name);
+			free(evtdata);
+			break;
+		case GP_EVENT_FILE_CHANGED:
+			path = (CameraFilePath*)evtdata;
+			printf("File changed on camera: %s / %s\n", path->folder, path->name);
+			free(evtdata);
 			break;
 		case GP_EVENT_CAPTURE_COMPLETE:
 			printf("Capture Complete.\n");
