@@ -11,16 +11,16 @@
  * version 2 of the License, or (at your option) any later version.
  *
  * \note
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * \note
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef __GPHOTO2_CAMERA_H__
@@ -35,7 +35,7 @@ extern "C" {
  *
  * A Camera object represents a specific instance of a (physical of
  * virtual) camera attached to the system.
- * 
+ *
  * The abilities of this type of camera are stored in a CameraAbility
  * object.
  *
@@ -58,7 +58,7 @@ typedef struct _Camera Camera;
 extern "C" {
 #endif /* __cplusplus */
 
-/** 
+/**
  * \brief CameraText structure used in various functions.
  *
  * A text structure containing translated text returned
@@ -69,7 +69,7 @@ typedef struct {
 	char text [32 * 1024]; /**< \brief Character string containing the translated text. */
 } CameraText;
 
-/** 
+/**
  * \brief A structure created by the capture operation.
  *
  * A structure containing the folder and filename of an object
@@ -98,21 +98,21 @@ typedef enum {
  *
  * Used by gp_camera_wait_for_event() to specify what
  * event happened on the camera.
- * This functionality is still in development and might change.
  *
  */
 typedef enum {
-	GP_EVENT_UNKNOWN,	/**< unknown and unhandled event */
+	GP_EVENT_UNKNOWN,	/**< unknown and unhandled event. argument is a char* or NULL */
 	GP_EVENT_TIMEOUT,	/**< timeout, no arguments */
 	GP_EVENT_FILE_ADDED,	/**< CameraFilePath* = file path on camfs */
 	GP_EVENT_FOLDER_ADDED,	/**< CameraFilePath* = folder on camfs */
-	GP_EVENT_CAPTURE_COMPLETE	/**< last capture is complete */
+	GP_EVENT_CAPTURE_COMPLETE,	/**< last capture is complete */
+	GP_EVENT_FILE_CHANGED	/**< CameraFilePath* = file path on camfs */
 } CameraEventType;
 
 /**
  * \name Camera object member functions
  *
- * These functions must be implemented by a camlib and the camlib's 
+ * These functions must be implemented by a camlib and the camlib's
  * camera_init() function will add them to a Camera object.
  *
  * @{
@@ -133,9 +133,10 @@ typedef enum {
  * Implement this function if you need to any of this stuff, otherwise leave
  * it out.
  *
- * \returns a gphoto error code 
+ * \returns a gphoto error code
  */
 typedef int (*CameraExitFunc)      (Camera *camera, GPContext *context);
+
 /**
  * \brief Get a configuration tree for the camera and its driver
  *
@@ -157,21 +158,85 @@ typedef int (*CameraExitFunc)      (Camera *camera, GPContext *context);
  */
 typedef int (*CameraGetConfigFunc) (Camera *camera, CameraWidget **widget,
 				    GPContext *context);
+/**
+ * \brief Get a configuration widget for a specific configuration
+ *
+ * \param camera the current camera
+ * \param name the name of the widget
+ * \param widget pointer to store the toplevel widget of the tree
+ * \param context the active #GPContext
+ *
+ * A camera driver can support configuration of either its own behaviour
+ * or the camera device itself. To allow a flexible driver framework,
+ * the camera driver provides a generic configuration widget tree to the
+ * frontend, which then renders it, allows user input and sends it back
+ * via the #CameraSetConfigFunc function to have the driver configure itself
+ * or the camera.
+ *
+ * This specific function retrieves one specific named entry, and not the full
+ * tree to allow for querying specific settings faster.
+ *
+ * If you do not have configuration ability, there is no need to specify this
+ * function.
+ *
+ * \returns a gphoto error code
+ */
+typedef int (*CameraGetSingleConfigFunc) (Camera *camera, const char *name, CameraWidget **widget,
+				    GPContext *context);
+/**
+ * \brief List all configuration widgets for a specific configuration
+ *
+ * \param camera the current camera
+ * \param list the list of widgets available
+ * \param context the active #GPContext
+ *
+ * A camera driver can support configuration of either its own behaviour
+ * or the camera device itself. To allow a flexible driver framework,
+ * the camera driver provides a generic configuration widget tree to the
+ * frontend, which then renders it, allows user input and sends it back
+ * via the #CameraSetConfigFunc function to have the driver configure itself
+ * or the camera.
+ *
+ * This specific function retrieves all the available configuration values in a flat list.
+ *
+ * This is different than the GetConfigFunc, which returns a configuration tree.
+ *
+ * If you do not have configuration ability, there is no need to specify this
+ * function.
+ *
+ * \returns a gphoto error code
+ */
+typedef int (*CameraListConfigFunc) (Camera *camera, CameraList *list, GPContext *context);
 /**
  * \brief Set the configuration in the camera
  *
- * \param camera the current camera 
+ * \param camera the current camera
  * \param widget the configuration widget tree that was changed
  * \param context the active #GPContext
  *
  * This function is called in the driver after the configuration is set.
  * It is called directly after setting the value and might called multiple
  * times (or never) after just one #CameraGetConfigFunc.
- * 
+ *
  * \returns a gphoto error code
  */
 typedef int (*CameraSetConfigFunc) (Camera *camera, CameraWidget  *widget,
 				    GPContext *context);
+/**
+ * \brief Set a single configuration variable in the camera
+ *
+ * \param camera the current camera
+ * \param name the widget to set
+ * \param widget the configuration widget tree that was changed
+ * \param context the active #GPContext
+ *
+ * This function is called in the driver after the configuration value is set.
+ *
+ * \returns a gphoto error code
+ */
+typedef int (*CameraSetSingleConfigFunc) (Camera *camera, const char *name, CameraWidget  *widget,
+				    GPContext *context);
+
 typedef int (*CameraCaptureFunc)   (Camera *camera, CameraCaptureType type,
 				    CameraFilePath *path, GPContext *context);
 typedef int (*CameraTriggerCaptureFunc)   (Camera *camera, GPContext *context);
@@ -198,8 +263,8 @@ typedef int (*CameraWaitForEvent)  (Camera *camera, int timeout,
  * be initialized before or reset the after each access from
  * libgphoto2.
  *
- * For example, you would probably set the speed to the highest one 
- * right before downloading an image, and reset it to the default speed 
+ * For example, you would probably set the speed to the highest one
+ * right before downloading an image, and reset it to the default speed
  * afterwards so that other programs will not be affected by this speed
  * change.
  */
@@ -227,6 +292,10 @@ typedef struct _CameraFunctions {
 	CameraGetConfigFunc       get_config;	/**< \brief Called for requesting the configuration widgets. */
 	CameraSetConfigFunc       set_config;	/**< \brief Called after a configuration was changed */
 
+	CameraListConfigFunc      list_config;	/**< \brief Called for listing the available configuration widgets. */
+	CameraGetSingleConfigFunc get_single_config;	/**< \brief Called for requesteing a single widget. */
+	CameraSetSingleConfigFunc set_single_config;	/**< \brief Called for setting a single configuration widget. */
+
 	/* Capturing */
 	CameraCaptureFunc        capture;	/**< \brief Remote control the camera to capture */
 	CameraTriggerCaptureFunc trigger_capture;/**< \brief Remote control the camera to trigger capture */
@@ -271,7 +340,7 @@ struct _Camera {
 int gp_camera_new               (Camera **camera);
 
 
-/** \name Preparing initialization 
+/** \name Preparing initialization
  * @{
  */
 int gp_camera_set_abilities     (Camera *camera, CameraAbilities  abilities);
@@ -296,19 +365,19 @@ int gp_camera_get_port_speed    (Camera *camera);
 /**@}*/
 
 
-/** \name Initialization 
- * @{ 
+/** \name Initialization
+ * @{
  */
 int gp_camera_autodetect 	 (CameraList *list, GPContext *context);
 int gp_camera_init               (Camera *camera, GPContext *context);
 int gp_camera_exit               (Camera *camera, GPContext *context);
 
 /**@}*/
- 
 
 
-/** \name Operations on cameras 
- * @{ 
+
+/** \name Operations on cameras
+ * @{
  */
 int gp_camera_ref   		 (Camera *camera);
 int gp_camera_unref 		 (Camera *camera);
@@ -316,8 +385,14 @@ int gp_camera_free 		 (Camera *camera);
 
 int gp_camera_get_config	 (Camera *camera, CameraWidget **window,
 				  GPContext *context);
+int gp_camera_list_config	 (Camera *camera, CameraList *list,
+				  GPContext *context);
+int gp_camera_get_single_config	 (Camera *camera, const char *name, CameraWidget **widget,
+				  GPContext *context);
 int gp_camera_set_config	 (Camera *camera, CameraWidget  *window,
 				  GPContext *context);
+int gp_camera_set_single_config	 (Camera *camera, const char *name, CameraWidget  *widget,
+				  GPContext *context);
 int gp_camera_get_summary	 (Camera *camera, CameraText *summary,
 				  GPContext *context);
 int gp_camera_get_manual	 (Camera *camera, CameraText *manual,
@@ -339,12 +414,12 @@ int gp_camera_get_storageinfo    (Camera *camera, CameraStorageInformation**,
 /**@}*/
 
 
-/** \name Operations on folders 
+/** \name Operations on folders
  * @{
  */
-int gp_camera_folder_list_files   (Camera *camera, const char *folder, 
+int gp_camera_folder_list_files   (Camera *camera, const char *folder,
 				   CameraList *list, GPContext *context);
-int gp_camera_folder_list_folders (Camera *camera, const char *folder, 
+int gp_camera_folder_list_folders (Camera *camera, const char *folder,
 				   CameraList *list, GPContext *context);
 int gp_camera_folder_delete_all   (Camera *camera, const char *folder,
 				   GPContext *context);
@@ -359,29 +434,29 @@ int gp_camera_folder_remove_dir   (Camera *camera, const char *folder,
 /**@}*/
 
 
-/** \name Operations on files 
+/** \name Operations on files
  * @{
  */
-int gp_camera_file_get_info 	(Camera *camera, const char *folder, 
+int gp_camera_file_get_info 	(Camera *camera, const char *folder,
 				 const char *file, CameraFileInfo *info,
 				 GPContext *context);
-int gp_camera_file_set_info 	(Camera *camera, const char *folder, 
+int gp_camera_file_set_info 	(Camera *camera, const char *folder,
 				 const char *file, CameraFileInfo info,
 				 GPContext *context);
-int gp_camera_file_get		(Camera *camera, const char *folder, 
+int gp_camera_file_get		(Camera *camera, const char *folder,
 				 const char *file, CameraFileType type,
 				 CameraFile *camera_file, GPContext *context);
 int gp_camera_file_read		(Camera *camera, const char *folder, const char *file,
-		    		 CameraFileType type, 
+		    		 CameraFileType type,
 		    		 uint64_t offset, char *buf, uint64_t *size,
 		    		 GPContext *context);
-int gp_camera_file_delete     	(Camera *camera, const char *folder, 
+int gp_camera_file_delete     	(Camera *camera, const char *folder,
 				 const char *file, GPContext *context);
 /**@}*/
 
 
 /**
- * \name Some cameras need 'keep-alive-messages'. 
+ * \name Some cameras need 'keep-alive-messages'.
  * @{
  */
 typedef int          (* CameraTimeoutFunc)      (Camera *camera,
