@@ -9,10 +9,10 @@
  * version 2 of the License, or (at your option) any later version.
  *
  * \note
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * \note
  * You should have received a copy of the GNU Lesser General Public
@@ -25,7 +25,8 @@
  * external software modules is considered <strong>deprecated</strong>.
  */
 #define _POSIX_SOURCE
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
+#define _DARWIN_C_SOURCE
 
 #include "config.h"
 #include <gphoto2/gphoto2-file.h>
@@ -94,6 +95,8 @@ gp_file_new (CameraFile **file)
 
 
 /*! Create new #CameraFile object from a UNIX filedescriptor.
+ *
+ * This function takes ownership of the filedescriptor and will close it when closing the CameraFile.
  *
  * \param file a pointer to a #CameraFile
  * \param fd a UNIX filedescriptor
@@ -147,7 +150,7 @@ int gp_file_free (CameraFile *file)
 	C_PARAMS (file);
 
 	CHECK_RESULT (gp_file_clean (file));
-	
+
 	if (file->accesstype == GP_FILE_ACCESSTYPE_FD)
 		close (file->fd);
 
@@ -167,7 +170,7 @@ gp_file_ref (CameraFile *file)
 	C_PARAMS (file);
 
 	file->ref_count += 1;
-	
+
 	return (GP_OK);
 }
 
@@ -182,7 +185,7 @@ int
 gp_file_unref (CameraFile *file)
 {
 	C_PARAMS (file);
-	
+
 	file->ref_count -= 1;
 
 	if (file->ref_count == 0)
@@ -200,7 +203,7 @@ gp_file_unref (CameraFile *file)
  *
  **/
 int
-gp_file_append (CameraFile *file, const char *data, 
+gp_file_append (CameraFile *file, const char *data,
 		unsigned long int size)
 {
 	C_PARAMS (file);
@@ -212,7 +215,7 @@ gp_file_append (CameraFile *file, const char *data,
 		file->size += size;
 		break;
 	case GP_FILE_ACCESSTYPE_FD: {
-		unsigned long int curwritten = 0; 
+		unsigned long int curwritten = 0;
 		while (curwritten < size) {
 			ssize_t	res = write (file->fd, data+curwritten, size-curwritten);
 			if (res == -1) {
@@ -249,7 +252,7 @@ gp_file_append (CameraFile *file, const char *data,
  * Internal.
  **/
 int
-gp_file_slurp (CameraFile *file, char *data, 
+gp_file_slurp (CameraFile *file, char *data,
 	size_t size, size_t *readlen
 ) {
 	C_PARAMS (file);
@@ -263,7 +266,7 @@ gp_file_slurp (CameraFile *file, char *data,
 		if (readlen) *readlen = size;
 		break;
 	case GP_FILE_ACCESSTYPE_FD: {
-		unsigned long int curread = 0; 
+		unsigned long int curread = 0;
 		while (curread < size) {
 			ssize_t	res = read (file->fd, data+curread, size-curread);
 			if (res == -1) {
@@ -384,8 +387,12 @@ gp_file_set_data_and_size (CameraFile *file, char *data,
  *
  * Both data and size can be NULL and will then be ignored.
  *
- * The pointer to data that is returned is still owned by libgphoto2
- * and its lifetime is the same as the #file.
+ * For regular CameraFiles, the pointer to data that is returned is
+ * still owned by libgphoto2 and its lifetime is the same as the #file.
+ *
+ * For filedescriptor or handler based CameraFile types, the returned
+ * data pointer is owned by the caller and needs to be free()d to avoid
+ * memory leaks.
  *
  **/
 int
@@ -403,7 +410,7 @@ gp_file_get_data_and_size (CameraFile *file, const char **data,
 		break;
 	case GP_FILE_ACCESSTYPE_FD: {
 		off_t	offset;
-		unsigned long int curread = 0;
+		off_t	curread = 0;
 
 		if (-1 == lseek (file->fd, 0, SEEK_END)) {
 			if (errno == EBADF) return GP_ERROR_IO;
@@ -498,7 +505,7 @@ gp_file_save (CameraFile *file, const char *filename)
 		break;
 	case GP_FILE_ACCESSTYPE_FD: {
 		char *data;
-		unsigned long int curread = 0;
+		off_t	curread = 0;
 		off_t	offset;
 
 		if (-1 == lseek (file->fd, 0, SEEK_END))
@@ -531,7 +538,7 @@ gp_file_save (CameraFile *file, const char *filename)
 				unlink (filename);
 				return GP_ERROR_IO_READ;
 			}
-			if (fwrite (data, 1, res, fp) != res) {
+			if ((int)fwrite (data, 1, res, fp) != res) {
 				GP_LOG_E ("Not enough space on device in order to save '%s'.", filename);
 				free (data);
 				fclose (fp);
@@ -692,7 +699,7 @@ gp_file_open (CameraFile *file, const char *filename)
 int
 gp_file_clean (CameraFile *file)
 {
-        /* 
+        /*
 	 * Frees a CameraFile's components, not the CameraFile itself.
 	 * This is used to prep a CameraFile struct to be filled.
          */
@@ -744,7 +751,7 @@ gp_file_copy (CameraFile *destination, CameraFile *source)
 		(source->accesstype == GP_FILE_ACCESSTYPE_FD)
 	) {
 		off_t	offset;
-		unsigned long int curread = 0;
+		off_t	curread = 0;
 
 		free (destination->data);
 		destination->data = NULL;
@@ -793,7 +800,7 @@ gp_file_copy (CameraFile *destination, CameraFile *source)
 		lseek (source->fd, 0, SEEK_SET);
 		C_MEM (data = malloc (65536));
 		while (1) {
-			unsigned long curwritten = 0;
+			ssize_t curwritten = 0;
 			ssize_t res;
 
 			res = read (source->fd, data, 65536);
@@ -1091,6 +1098,7 @@ gp_file_adjust_name_for_mime_type (CameraFile *file)
 		GP_MIME_AVI,  "avi",
 		GP_MIME_CRW,  "crw",
 		GP_MIME_CR2,  "cr2",
+		GP_MIME_CR3,  "cr3",
 		GP_MIME_NEF,  "nef",
 		GP_MIME_TXT,  "txt",
 		NULL};
@@ -1109,7 +1117,7 @@ gp_file_adjust_name_for_mime_type (CameraFile *file)
 			break;
 		}
 	GP_LOG_D ("Name adjusted to '%s'.", file->name);
-	
+
 	return (GP_OK);
 }
 
