@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* gphoto2-port-usb.c
  *
- * Copyright (c) 2001 Lutz Mller <lutz@users.sf.net>
+ * Copyright (c) 2001 Lutz Mueller <lutz@users.sf.net>
  * Copyright (c) 1999-2000 Johannes Erdfelt <johannes@erdfelt.com>
  * Copyright (c) 2005, 2007 Hubert Figuiere <hub@figuiere.net>
  *
@@ -10,30 +10,24 @@
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
 #include <gphoto2/gphoto2-port-library.h>
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <sys/time.h>
-#ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
-#include <dirent.h>
 #include <string.h>
 #ifdef HAVE_MNTENT_H
 # include <mntent.h>
@@ -53,10 +47,6 @@
 #define endmntent(f) fclose(f)
 #endif
 
-#ifdef HAVE_HAL
-#include <hal/libhal.h>
-#endif
-
 #include <gphoto2/gphoto2-port.h>
 #include <gphoto2/gphoto2-port-result.h>
 #include <gphoto2/gphoto2-port-log.h>
@@ -96,117 +86,90 @@ int
 gp_port_library_list (GPPortInfoList *list)
 {
 	GPPortInfo info;
-#ifdef HAVE_HAL
-        LibHalContext *ctx;
-        DBusError error;
-        DBusConnection *dbus_connection;
-        int i;
-        int num_volumes;
-        char **volumes;
-        char *udi;
-
-        ctx = libhal_ctx_new ();
-        if (!ctx) {
-		gp_log(GP_LOG_DEBUG, "gphoto2-port/disk", 
-		       "failed to initialize HAL!\n");
-                goto generic;
-        }
-        dbus_error_init (&error);
-
-        dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
-
-        if (dbus_error_is_set (&error)) {
-		gp_log (GP_LOG_DEBUG, "gphoto2-port/disk", 
-			"hal_initialize failed: %s", 
-			error.message);
-		dbus_error_free (&error);
-		libhal_ctx_free(ctx);
-                goto generic;
-        }
-
-        libhal_ctx_set_dbus_connection (ctx, dbus_connection);
-
-        dbus_error_init (&error);
-        volumes = libhal_find_device_by_capability (ctx, "volume",
-                                                    &num_volumes, &error);
-        if (volumes == NULL) {
-		if (dbus_error_is_set (&error)) {
-			gp_log (GP_LOG_DEBUG, "gphoto2-port/disk", 
-				"libhal: %s", 
-				error.message);
-			dbus_error_free (&error);
-		}
-                goto generic;
-        }
-
-	gp_log(GP_LOG_DEBUG, "gphoto2-port/disk", "found %d volumes", 
-	       num_volumes); 
-        for (i = 0; i < num_volumes; i++) {
-                udi = volumes [i];
-		
-                /* don't attempt to mount already mounted volumes */
-                if (!libhal_device_property_exists (ctx, udi,
-                                                    "volume.is_mounted",
-                                                    NULL) ||
-                    libhal_device_get_property_bool (ctx, udi,
-                                                     "volume.is_mounted",
-                                                     NULL)) 
-		{
-			char *mountpoint = NULL;
-			char *mediainfo = NULL;
-
-			if (!libhal_device_property_exists (ctx, udi,
-							   "volume.mount_point",
-							    NULL)) {
-				continue;
-			}
- 			mountpoint = libhal_device_get_property_string(ctx, udi,
-								       "volume.mount_point",
-								       &error);
-			if (mountpoint == NULL) {
-				if (dbus_error_is_set (&error)) {
-					gp_log (GP_LOG_DEBUG, 
-						"gphoto2-port/disk",
-						"libhal: %s", error.message);
-					dbus_error_free (&error);
-				}
-				continue;
-			}
-			mediainfo = libhal_device_get_property_string(ctx, udi,
-								       "info.product",
-								       &error);
-			info.type = GP_PORT_DISK;
-			snprintf (info.name, sizeof(info.name), _("Media '%s'"), 
-				  (mediainfo?mediainfo:_("(unknown)")));
-			snprintf (info.path, sizeof(info.path), "disk:%s",
-				  mountpoint);
-			CHECK (gp_port_info_list_append (list, info));
-			libhal_free_string(mountpoint);
-			if (mediainfo) {
-				libhal_free_string(mediainfo);
-			}
-		}
-		
-        }
-
-	libhal_free_string_array (volumes);
-	libhal_ctx_free (ctx);
-	dbus_connection_unref (dbus_connection);
-#else
 # ifdef HAVE_MNTENT_H
 	FILE *mnt;
 	struct mntent *mntent;
 	char	path[1024];
+	char	*s;
 	struct stat stbuf;
 
-	info.type = GP_PORT_DISK;
-
 	mnt = setmntent ("/etc/fstab", "r");
 	if (mnt) {
 		while ((mntent = getmntent (mnt))) {
 			/* detect floppies so we don't access them with the stat() below */
-			gp_log(GP_LOG_DEBUG, "gphoto2-port/disk",
-			       "found fstab fsname %s", mntent->mnt_fsname);
+			GP_LOG_D ("found fstab fsname %s", mntent->mnt_fsname);
+
+			if ((NULL != strstr(mntent->mnt_fsname,"fd"))	||
+			    (NULL != strstr(mntent->mnt_fsname,"floppy")) ||
+			    (NULL != strstr(mntent->mnt_fsname,"fuse"))	||
+			    (NULL != strstr(mntent->mnt_fsname,"nfs"))	||
+			    (NULL != strstr(mntent->mnt_fsname,"cifs"))	||
+			    (NULL != strstr(mntent->mnt_fsname,"smbfs"))||
+			    (NULL != strstr(mntent->mnt_fsname,"afs"))	||
+			    (NULL != strstr(mntent->mnt_fsname,"davfs"))||
+			    (NULL != strstr(mntent->mnt_fsname,"autofs"))||
+                            (NULL != strstr(mntent->mnt_fsname,"cgroup"))||
+                            (NULL != strstr(mntent->mnt_fsname,"systemd"))||
+                            (NULL != strstr(mntent->mnt_fsname,"mqueue"))||
+                            (NULL != strstr(mntent->mnt_fsname,"securityfs"))||
+                            (NULL != strstr(mntent->mnt_fsname,"proc"))||
+                            (NULL != strstr(mntent->mnt_fsname,"devtmpfs"))||
+                            (NULL != strstr(mntent->mnt_fsname,"devpts"))||
+                            (NULL != strstr(mntent->mnt_fsname,"sysfs"))||
+			    (NULL != strstr(mntent->mnt_fsname,"gphotofs"))||
+			/* fstype based */
+			    (NULL != strstr(mntent->mnt_type,"autofs"))	||
+			    (NULL != strstr(mntent->mnt_type,"davfs"))	||
+			    (NULL != strstr(mntent->mnt_type,"nfs"))	||
+			    (NULL != strstr(mntent->mnt_type,"smbfs"))||
+			    (NULL != strstr(mntent->mnt_type,"proc"))||
+			    (NULL != strstr(mntent->mnt_type,"sysfs"))||
+			    (NULL != strstr(mntent->mnt_type,"cifs"))||
+			    (NULL != strstr(mntent->mnt_type,"afs")) ||
+			/* mount options */
+				/* x-systemd.automount or similar */
+			    (NULL != strstr(mntent->mnt_opts,"automount"))
+			) {
+				continue;
+			}
+
+			/* Whitelist some fuse based filesystems, e.g. to help exfat mounts */
+			/* In general, if we are backed by a device, it is probably good(tm) */
+			if (NULL != strstr(mntent->mnt_type,"fuse")) {
+				if (!strstr(mntent->mnt_fsname,"/dev/"))
+					continue;
+			}
+
+			snprintf (path, sizeof(path), "%s/DCIM", mntent->mnt_dir);
+			if (-1 == stat(path, &stbuf)) {
+				snprintf (path, sizeof(path), "%s/dcim", mntent->mnt_dir);
+				if (-1 == stat(path, &stbuf))
+					continue;
+			}
+			s = malloc (strlen(_("Media '%s'"))+strlen(mntent->mnt_fsname)+1);
+			sprintf (s, _("Media '%s'"), mntent->mnt_fsname);
+			gp_port_info_new (&info);
+			gp_port_info_set_type (info, GP_PORT_DISK);
+			gp_port_info_set_name (info, s);
+			free (s);
+
+			s = malloc (strlen("disk:")+strlen(mntent->mnt_dir)+1);
+			sprintf (s, "disk:%s", mntent->mnt_dir);
+			gp_port_info_set_path (info, s);
+			if (gp_port_info_list_lookup_path (list, s) >= GP_OK) {
+				free (s);
+				continue;
+			}
+			free(s);
+			CHECK (gp_port_info_list_append (list, info));
+		}
+		endmntent(mnt);
+	}
+	mnt = setmntent ("/etc/mtab", "r");
+	if (mnt) {
+		while ((mntent = getmntent (mnt))) {
+			/* detect floppies so we don't access them with the stat() below */
+			GP_LOG_D ("found mtab fsname %s", mntent->mnt_fsname);
 
 			if ((NULL != strstr(mntent->mnt_fsname,"fd"))	||
 			    (NULL != strstr(mntent->mnt_fsname,"floppy")) ||
@@ -216,58 +179,36 @@ gp_port_library_list (GPPortInfoList *list)
 			    (NULL != strstr(mntent->mnt_fsname,"smbfs"))||
 			    (NULL != strstr(mntent->mnt_fsname,"afs"))	||
 			    (NULL != strstr(mntent->mnt_fsname,"autofs"))||
-			    (NULL != strstr(mntent->mnt_fsname,"cgroup"))||
-			    (NULL != strstr(mntent->mnt_fsname,"systemd"))||
-			    (NULL != strstr(mntent->mnt_fsname,"mqueue"))||
-			    (NULL != strstr(mntent->mnt_fsname,"securityfs"))||
-			    (NULL != strstr(mntent->mnt_fsname,"proc"))||
-			    (NULL != strstr(mntent->mnt_fsname,"devtmpfs"))||
-			    (NULL != strstr(mntent->mnt_fsname,"devpts"))||
-			    (NULL != strstr(mntent->mnt_fsname,"sysfs"))||
-			    (NULL != strstr(mntent->mnt_fsname,"gphotofs"))
+			    (NULL != strstr(mntent->mnt_fsname,"davfs"))||
+                            (NULL != strstr(mntent->mnt_fsname,"cgroup"))||
+                            (NULL != strstr(mntent->mnt_fsname,"systemd"))||
+                            (NULL != strstr(mntent->mnt_fsname,"mqueue"))||
+                            (NULL != strstr(mntent->mnt_fsname,"securityfs"))||
+                            (NULL != strstr(mntent->mnt_fsname,"proc"))||
+                            (NULL != strstr(mntent->mnt_fsname,"devtmpfs"))||
+                            (NULL != strstr(mntent->mnt_fsname,"devpts"))||
+                            (NULL != strstr(mntent->mnt_fsname,"sysfs"))||
+			    (NULL != strstr(mntent->mnt_fsname,"gphotofs"))||
+			/* fstype based */
+			    (NULL != strstr(mntent->mnt_type,"autofs"))	||
+			    (NULL != strstr(mntent->mnt_type,"davfs"))	||
+			    (NULL != strstr(mntent->mnt_type,"nfs"))	||
+			    (NULL != strstr(mntent->mnt_type,"smbfs"))||
+			    (NULL != strstr(mntent->mnt_type,"proc"))||
+			    (NULL != strstr(mntent->mnt_type,"sysfs"))||
+			    (NULL != strstr(mntent->mnt_type,"cifs"))||
+			    (NULL != strstr(mntent->mnt_type,"afs")) ||
+			/* options */
+			    (NULL != strstr(mntent->mnt_opts,"automount"))
 			) {
 				continue;
 			}
-			snprintf (path, sizeof(path), "%s/DCIM", mntent->mnt_dir);
-			if (-1 == stat(path, &stbuf)) {
-				snprintf (path, sizeof(path), "%s/dcim", mntent->mnt_dir);
-				if (-1 == stat(path, &stbuf))
+			/* Whitelist some fuse based filesystems, e.g. to help exfat mounts */
+			/* In general, if we are backed by a device, it is probably good(tm) */
+			if (NULL != strstr(mntent->mnt_type,"fuse")) {
+				if (!strstr(mntent->mnt_fsname,"/dev/"))
 					continue;
 			}
-			snprintf (info.name, sizeof(info.name), _("Media '%s'"), mntent->mnt_fsname),
-			snprintf (info.path, sizeof(info.path), "disk:%s", mntent->mnt_dir);
-			if (gp_port_info_list_lookup_path (list, info.path) >= GP_OK)
-				continue;
-			CHECK (gp_port_info_list_append (list, info));
-		}
-		endmntent(mnt);
-	}
-	mnt = setmntent ("/etc/mtab", "r");
-	if (mnt) {
-		while ((mntent = getmntent (mnt))) {
-			/* detect floppies so we don't access them with the stat() below */
-			gp_log(GP_LOG_DEBUG, "gphoto2-port/disk",
-			       "found mtab fsname %s", mntent->mnt_fsname);
-
-			if ((NULL != strstr(mntent->mnt_fsname,"fd"))	||
-			    (NULL != strstr(mntent->mnt_fsname,"floppy")) ||
-			    (NULL != strstr(mntent->mnt_fsname,"fuse"))	||
-			    (NULL != strstr(mntent->mnt_fsname,"nfs"))	||
-			    (NULL != strstr(mntent->mnt_fsname,"smbfs"))||
-			    (NULL != strstr(mntent->mnt_fsname,"afs"))	||
-			    (NULL != strstr(mntent->mnt_fsname,"autofs"))||
-			    (NULL != strstr(mntent->mnt_fsname,"cgroup"))||
-			    (NULL != strstr(mntent->mnt_fsname,"systemd"))||
-			    (NULL != strstr(mntent->mnt_fsname,"mqueue"))||
-			    (NULL != strstr(mntent->mnt_fsname,"securityfs"))||
-			    (NULL != strstr(mntent->mnt_fsname,"proc"))||
-			    (NULL != strstr(mntent->mnt_fsname,"devtmpfs"))||
-			    (NULL != strstr(mntent->mnt_fsname,"devpts"))||
-			    (NULL != strstr(mntent->mnt_fsname,"sysfs"))||
-			    (NULL != strstr(mntent->mnt_fsname,"gphotofs"))
-			) {
-				continue;
-			}
 
 			snprintf (path, sizeof(path), "%s/DCIM", mntent->mnt_dir);
 			if (-1 == stat(path, &stbuf)) {
@@ -282,11 +223,21 @@ gp_port_library_list (GPPortInfoList *list)
 			if (NULL != strstr(mntent->mnt_fsname, "automount")) {
 				continue;
 			}
-			info.type = GP_PORT_DISK;
-			snprintf (info.name, sizeof(info.name), _("Media '%s'"), mntent->mnt_fsname),
-			snprintf (info.path, sizeof(info.path), "disk:%s", mntent->mnt_dir);
-			if (gp_port_info_list_lookup_path (list, info.path) >= GP_OK)
+			gp_port_info_new (&info);
+			gp_port_info_set_type (info, GP_PORT_DISK);
+			s = malloc (strlen(_("Media '%s'"))+strlen(mntent->mnt_fsname)+1);
+			sprintf (s, _("Media '%s'"),  mntent->mnt_fsname);
+			gp_port_info_set_name (info, s);
+			free (s);
+
+			s = malloc (strlen("disk:")+strlen(mntent->mnt_dir)+1);
+			sprintf (s, "disk:%s", mntent->mnt_dir);
+			gp_port_info_set_path (info, s);
+			if (gp_port_info_list_lookup_path (list, s) >= GP_OK) {
+				free (s);
 				continue;
+			}
+			free (s);
 			CHECK (gp_port_info_list_append (list, info));
 		}
 		endmntent(mnt);
@@ -348,23 +299,18 @@ gp_port_library_list (GPPortInfoList *list)
 	}
 #  endif
 # endif
-#endif
-generic:
 	/* generic disk:/xxx/ matcher */
-	info.type = GP_PORT_DISK;
-	memset (info.name, 0, sizeof(info.name));
-	snprintf (info.path, sizeof(info.path), "^disk:");
-	CHECK (gp_port_info_list_append (list, info));
+	gp_port_info_new (&info);
+	gp_port_info_set_type (info, GP_PORT_DISK);
+	gp_port_info_set_name (info, "");
+	gp_port_info_set_path (info, "^disk:");
+	gp_port_info_list_append (list, info); /* do not check return */
 	return GP_OK;
 }
 
 static int gp_port_disk_init (GPPort *dev)
 {
-	dev->pl = malloc (sizeof (GPPortPrivateLibrary));
-	if (!dev->pl) {
-		return GP_ERROR_NO_MEMORY;
-	}
-	memset (dev->pl, 0, sizeof(GPPortPrivateLibrary));
+	C_MEM (dev->pl = calloc (1, sizeof (GPPortPrivateLibrary)));
 
 	return GP_OK;
 }
@@ -372,10 +318,8 @@ static int gp_port_disk_init (GPPort *dev)
 static int
 gp_port_disk_exit (GPPort *port)
 {
-	if (port->pl) {
-		free (port->pl);
-		port->pl = NULL;
-	}
+	free (port->pl);
+	port->pl = NULL;
 
 	return GP_OK;
 }
@@ -409,11 +353,9 @@ gp_port_library_operations (void)
 {
 	GPPortOperations *ops;
 
-	ops = malloc (sizeof (GPPortOperations));
-	if (!ops) {
+	ops = calloc (1, sizeof (GPPortOperations));
+	if (!ops)
 		return NULL;
-	}
-	memset (ops, 0, sizeof (GPPortOperations));
 
 	ops->init   = gp_port_disk_init;
 	ops->exit   = gp_port_disk_exit;
