@@ -1,6 +1,6 @@
 /** \file
  *
- * \author Copyright 2001 Lutz Mller <lutz@users.sf.net>
+ * \author Copyright 2001 Lutz Mueller <lutz@users.sf.net>
  *
  * \par License
  * This library is free software; you can redistribute it and/or
@@ -9,18 +9,19 @@
  * version 2 of the License, or (at your option) any later version.
  *
  * \par
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #define _GNU_SOURCE
+#define _DARWIN_C_SOURCE
 
 #include "config.h"
 
@@ -31,8 +32,10 @@
 #include <stdio.h>
 #ifdef HAVE_REGEX
 #include <regex.h>
+#elif defined(_MSC_VER)
+#pragma message("We need regex.h, but it has not been detected.")
 #else
-#error We need regex.h, but it has not been detected.
+#warning We need regex.h, but it has not been detected.
 #endif
 
 #include <ltdl.h>
@@ -51,7 +54,7 @@
 #    define N_(String) gettext_noop (String)
 #  else
 #    define N_(String) (String)
-#  endif 
+#  endif
 #else
 #  define textdomain(String) (String)
 #  define gettext(String) (String)
@@ -75,7 +78,6 @@ struct _GPPortInfoList {
 	unsigned int iolib_count;
 };
 
-#define CHECK_NULL(x) {if (!(x)) return (GP_ERROR_BAD_PARAMETERS);}
 #define CR(x)         {int r=(x);if (r<0) return (r);}
 
 
@@ -87,7 +89,7 @@ struct _GPPortInfoList {
  *
  * This function is called by the gp_message_codeset() function, there is
  * no need to call it yourself.
- * 
+ *
  * \param codeset new codeset to use
  * \return the previous codeset
  */
@@ -109,7 +111,7 @@ gp_port_message_codeset (const char *codeset) {
 int
 gp_port_info_list_new (GPPortInfoList **list)
 {
-	CHECK_NULL (list);
+	C_PARAMS (list);
 
 	/*
 	 * We put this in here because everybody needs to call this function
@@ -117,10 +119,7 @@ gp_port_info_list_new (GPPortInfoList **list)
 	 */
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 
-	*list = malloc (sizeof (GPPortInfoList));
-	if (!*list)
-		return (GP_ERROR_NO_MEMORY);
-	memset (*list, 0, sizeof (GPPortInfoList));
+	C_MEM (*list = calloc (1, sizeof (GPPortInfoList)));
 
 	return (GP_OK);
 }
@@ -136,10 +135,10 @@ gp_port_info_list_new (GPPortInfoList **list)
 int
 gp_port_info_list_free (GPPortInfoList *list)
 {
-	CHECK_NULL (list);
+	C_PARAMS (list);
 
 	if (list->info) {
-		int i;
+		unsigned int i;
 
 		for (i=0;i<list->count;i++) {
 			free (list->info[i]->name);
@@ -169,39 +168,28 @@ gp_port_info_list_free (GPPortInfoList *list)
  * Appends an entry to the list. This function is typically called by
  * an io-driver during #gp_port_library_list. If you leave info.name blank,
  * #gp_port_info_list_lookup_path will try to match non-existent paths
- * against info.path and - if successfull - will append this entry to the 
+ * against info.path and - if successful - will append this entry to the
  * list.
  *
- * \note This returns index - number of generic entries, not the correct index.
- *
- * \return A non-negative number or a gphoto2 error code
+ * \return A gphoto2 error code, or an index into the port list (excluding generic entries).
+ *         which can be used for gp_port_info_list_get_info.
  **/
 int
 gp_port_info_list_append (GPPortInfoList *list, GPPortInfo info)
 {
-	int generic, i;
-	GPPortInfo *new_info;
+	unsigned int generic, i;
 
-	CHECK_NULL (list);
+	C_PARAMS (list);
 
-	if (!list->info)
-		new_info = malloc (sizeof (GPPortInfo));
-	else
-		new_info = realloc (list->info, sizeof (GPPortInfo) *
-							(list->count + 1));
-	if (!new_info)
-		return (GP_ERROR_NO_MEMORY);
-
-	list->info = new_info;
+	C_MEM (list->info = realloc (list->info, sizeof (GPPortInfo) * (list->count + 1)));
 	list->count++;
-
 	list->info[list->count - 1] = info;
 
 	/* Ignore generic entries */
 	for (generic = i = 0; i < list->count; i++)
 		if (!strlen (list->info[i]->name))
 			generic++;
-	return (list->count - 1 - generic);
+        return (list->count - 1 - generic);
 }
 
 
@@ -216,21 +204,18 @@ foreach_func (const char *filename, lt_ptr data)
 	unsigned int j, old_size = list->count;
 	int result;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-		_("Called for filename '%s'."), filename );
+	GP_LOG_D ("Called for filename '%s'.", filename );
 
 	lh = lt_dlopenext (filename);
 	if (!lh) {
-		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-			_("Could not load '%s': '%s'."), filename, lt_dlerror ());
+		GP_LOG_D ("Could not load '%s': '%s'.", filename, lt_dlerror ());
 		return (0);
 	}
 
 	lib_type = lt_dlsym (lh, "gp_port_library_type");
 	lib_list = lt_dlsym (lh, "gp_port_library_list");
 	if (!lib_type || !lib_list) {
-		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-			_("Could not find some functions in '%s': '%s'."),
+		GP_LOG_D ("Could not find some functions in '%s': '%s'.",
 			filename, lt_dlerror ());
 		lt_dlclose (lh);
 		return (0);
@@ -241,18 +226,18 @@ foreach_func (const char *filename, lt_ptr data)
 		if (list->info[j]->type == type)
 			break;
 	if (j != list->count) {
-		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-			_("'%s' already loaded"), filename);
+		GP_LOG_D ("'%s' already loaded", filename);
 		lt_dlclose (lh);
 		return (0);
 	}
 
 	result = lib_list (list);
+#if !defined(VALGRIND)
 	lt_dlclose (lh);
+#endif
 	if (result < 0) {
-		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-			_("Could not load port driver list: '%s'."),
-			gp_port_result_as_string (result));
+		GP_LOG_E ("Error during assembling of port list: '%s' (%d).",
+			gp_port_result_as_string (result), result);
 	}
 
 	if (old_size != list->count) {
@@ -263,8 +248,7 @@ foreach_func (const char *filename, lt_ptr data)
 		list->iolib_count++;
 
 		for (j = old_size; j < list->count; j++){
-			gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-				_("Loaded '%s' ('%s') from '%s'."),
+			GP_LOG_D ("Loaded '%s' ('%s') from '%s'.",
 				list->info[j]->name, list->info[j]->path,
 				filename);
 			list->info[j]->library_filename = strdup (filename);
@@ -277,7 +261,7 @@ foreach_func (const char *filename, lt_ptr data)
 
 /**
  * \brief Load system ports
- * 
+ *
  * \param list a #GPPortInfoList
  *
  * Searches the system for io-drivers and appends them to the list. You would
@@ -294,11 +278,9 @@ gp_port_info_list_load (GPPortInfoList *list)
 	const char *iolibs = (iolibs_env != NULL)?iolibs_env:IOLIBS;
 	int result;
 
-	CHECK_NULL (list);
+	C_PARAMS (list);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-		_("Using ltdl to load io-drivers from '%s'..."),
-		iolibs);
+	GP_LOG_D ("Using ltdl to load io-drivers from '%s'...", iolibs);
 	lt_dlinit ();
 	lt_dladdsearchdir (iolibs);
 	result = lt_dlforeachfile (iolibs, foreach_func, list);
@@ -306,8 +288,7 @@ gp_port_info_list_load (GPPortInfoList *list)
 	if (result < 0)
 		return (result);
 	if (list->iolib_count == 0) {
-		gp_log (GP_LOG_ERROR, "gphoto2-port-info-list",
-                        "No iolibs found in '%s'", iolibs);
+		GP_LOG_E ("No iolibs found in '%s'", iolibs);
 		return GP_ERROR_LIBRARY;
 	}
         return (GP_OK);
@@ -324,16 +305,11 @@ gp_port_info_list_load (GPPortInfoList *list)
 int
 gp_port_info_list_count (GPPortInfoList *list)
 {
-	int count, i;
+	unsigned int count, i;
 
-	CHECK_NULL (list);
+	C_PARAMS (list);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-	ngettext(
-		"Counting entries (%i available)...",
-		"Counting entries (%i available)...",
-		list->count
-		), list->count);
+	GP_LOG_D ("Counting entries (%i available)...", list->count);
 
 	/* Ignore generic entries */
 	count = list->count;
@@ -341,12 +317,7 @@ gp_port_info_list_count (GPPortInfoList *list)
 		if (!strlen (list->info[i]->name))
 			count--;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-		ngettext(
-			"%i regular entry available.",
-			"%i regular entries available.",
-			count
-		), count);
+	GP_LOG_D ("%i regular entries available.", count);
 	return count;
 }
 
@@ -365,22 +336,20 @@ gp_port_info_list_count (GPPortInfoList *list)
 int
 gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 {
-	int i, result, generic;
+	unsigned int i;
+	int result, generic;
+#ifdef HAVE_REGEX
 	regex_t pattern;
 #ifdef HAVE_GNU_REGEX
 	const char *rv;
 #else
 	regmatch_t match;
+#endif
 #endif
 
-	CHECK_NULL (list && path);
+	C_PARAMS (list && path);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-		ngettext(
-		"Looking for path '%s' (%i entry available)...",
-		"Looking for path '%s' (%i entries available)...",
-		list->count
-		), path, list->count);
+	GP_LOG_D ("Looking for path '%s' (%i entries available)...", path, list->count);
 
 	/* Exact match? */
 	for (generic = i = 0; i < list->count; i++)
@@ -389,17 +358,16 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 		else if (!strcmp (list->info[i]->path, path))
 			return (i - generic);
 
+#ifdef HAVE_REGEX
 	/* Regex match? */
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-		_("Starting regex search for '%s'..."), path);
+	GP_LOG_D ("Starting regex search for '%s'...", path);
 	for (i = 0; i < list->count; i++) {
 		GPPortInfo newinfo;
 
 		if (strlen (list->info[i]->name))
 			continue;
 
-		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-			_("Trying '%s'..."), list->info[i]->path);
+		GP_LOG_D ("Trying '%s'...", list->info[i]->path);
 
 		/* Compile the pattern */
 #ifdef HAVE_GNU_REGEX
@@ -407,8 +375,7 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 		rv = re_compile_pattern (list->info[i]->path,
 					 strlen (list->info[i]->path), &pattern);
 		if (rv) {
-			gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-				"%s", rv);
+			GP_LOG_D ("%s", rv);
 			continue;
 		}
 #else
@@ -416,11 +383,9 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 		if (result) {
 			char buf[1024];
 			if (regerror (result, &pattern, buf, sizeof (buf)))
-				gp_log (GP_LOG_ERROR, "gphoto2-port-info-list",
-					"%s", buf);
+				GP_LOG_E ("%s", buf);
 			else
-				gp_log (GP_LOG_ERROR, "gphoto2-port-info-list",
-					_("regcomp failed"));
+				GP_LOG_E ("regcomp failed");
 			return (GP_ERROR_UNKNOWN_PORT);
 		}
 #endif
@@ -430,16 +395,14 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 		result = re_match (&pattern, path, strlen (path), 0, NULL);
 		regfree (&pattern);
 		if (result < 0) {
-			gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-				_("re_match failed (%i)"), result);
+			GP_LOG_D ("re_match failed (%i)", result);
 			continue;
 		}
 #else
 		result = regexec (&pattern, path, 1, &match, 0);
 		regfree (&pattern);
 		if (result) {
-			gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-				_("regexec failed"));
+			GP_LOG_D ("regexec failed");
 			continue;
 		}
 #endif
@@ -451,6 +414,7 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 		CR (result = gp_port_info_list_append (list, newinfo));
 		return result;
 	}
+#endif /* HAVE_REGEX */
 
 	return (GP_ERROR_UNKNOWN_PORT);
 }
@@ -467,12 +431,11 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 int
 gp_port_info_list_lookup_name (GPPortInfoList *list, const char *name)
 {
-	int i, generic;
+	unsigned int i, generic;
 
-	CHECK_NULL (list && name);
+	C_PARAMS (list && name);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", _("Looking up entry "
-		"'%s'..."), name);
+	GP_LOG_D ("Looking up entry '%s'...", name);
 
 	/* Ignore generic entries */
 	for (generic = i = 0; i < list->count; i++)
@@ -499,24 +462,17 @@ gp_port_info_list_get_info (GPPortInfoList *list, int n, GPPortInfo *info)
 {
 	int i;
 
-	CHECK_NULL (list && info);
+	C_PARAMS (list && info);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-		ngettext(
-		"Getting info of entry %i (%i available)...",
-		"Getting info of entry %i (%i available)...",
-		list->count
-		), n, list->count);
+	GP_LOG_D ("Getting info of entry %i (%i available)...", n, list->count);
 
-	if (n < 0 || n >= list->count)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS ((n >= 0) && (unsigned int)n < list->count);
 
 	/* Ignore generic entries */
 	for (i = 0; i <= n; i++)
 		if (!strlen (list->info[i]->name)) {
 			n++;
-			if (n >= list->count)
-				return GP_ERROR_BAD_PARAMETERS;
+			C_PARAMS ((unsigned int)n < list->count);
 		}
 
 	*info = list->info[n];
@@ -551,7 +507,7 @@ gp_port_info_get_name (GPPortInfo info, char **name) {
  **/
 int
 gp_port_info_set_name (GPPortInfo info, const char *name) {
-	info->name = strdup (name);
+	C_MEM (info->name = strdup (name));
 	return GP_OK;
 }
 
@@ -582,7 +538,7 @@ gp_port_info_get_path (GPPortInfo info, char **path) {
  **/
 int
 gp_port_info_set_path (GPPortInfo info, const char *path) {
-	info->path = strdup (path);
+	C_MEM (info->path = strdup (path));
 	return GP_OK;
 }
 
@@ -628,8 +584,6 @@ gp_port_info_set_type (GPPortInfo info, GPPortType type) {
  **/
 int
 gp_port_info_new (GPPortInfo *info) {
-	*info = calloc (sizeof(struct _GPPortInfo),1);
-	if (!*info)
-		return GP_ERROR_NO_MEMORY;
+	C_MEM (*info = calloc (1, sizeof(struct _GPPortInfo)));
 	return GP_OK;
 }
