@@ -1,6 +1,6 @@
 /** \file
  *
- * \author Copyright 2001 Lutz Mller <lutz@users.sf.net>
+ * \author Copyright 2001 Lutz Mueller <lutz@users.sf.net>
  *
  * \par License
  * This library is free software; you can redistribute it and/or
@@ -9,19 +9,19 @@
  * version 2 of the License, or (at your option) any later version.
  *
  * \par
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
-#define _BSD_SOURCE
+#define _DEFAULT_SOURCE
 
 #include "config.h"
 #include <gphoto2/gphoto2-port-log.h>
@@ -48,14 +48,14 @@
 #endif
 
 #ifndef DISABLE_DEBUGGING
-/** 
+/**
  * \brief Internal logging function entry.
- * 
+ *
  * Internal structure to remember the logging functions.
  * Use gp_log_add_func() and gp_log_remove_func() to access it.
  */
 typedef struct {
-	unsigned int id;	/**< Internal id */
+	int id;			/**< Internal id */
 	GPLogLevel   level;	/**< Internal loglevel */
 	GPLogFunc    func;	/**< Internal function pointer to call */
 	void        *data;	/**< Private data supplied by caller */
@@ -81,28 +81,49 @@ static unsigned int log_funcs_count = 0;
 int
 gp_log_add_func (GPLogLevel level, GPLogFunc func, void *data)
 {
-	LogFunc *new_log_funcs;
+	static int logfuncid = 0;
 
-	if (!func)
-		return (GP_ERROR_BAD_PARAMETERS);
-
-	if (!log_funcs)
-		new_log_funcs = malloc (sizeof (LogFunc));
-	else
-		new_log_funcs = realloc (log_funcs, sizeof (LogFunc) * 
-					 (log_funcs_count + 1));
-	if (!new_log_funcs)
-		return (GP_ERROR_NO_MEMORY);
-
-	log_funcs = new_log_funcs;
+	C_PARAMS (func);
+	C_MEM (log_funcs = realloc (log_funcs, sizeof (LogFunc) *
+				(log_funcs_count + 1)));
 	log_funcs_count++;
 
-	log_funcs[log_funcs_count - 1].id = log_funcs_count;
+	log_funcs[log_funcs_count - 1].id = ++logfuncid;
 	log_funcs[log_funcs_count - 1].level = level;
 	log_funcs[log_funcs_count - 1].func = func;
 	log_funcs[log_funcs_count - 1].data = data;
 
-	return (log_funcs_count);
+	return logfuncid;
+}
+
+
+char*
+gpi_vsnprintf (const char* format, va_list args)
+{
+	va_list xargs;
+	int strsize;
+	char *str;
+
+#ifdef HAVE_VA_COPY
+	va_copy (xargs, args);
+#else
+	/* according to 'the web', the only interesting compiler without va_copy is MSVC
+	 * and there a simple assignment is the way to go */
+	xargs = args;
+#endif
+
+	/* query the size necessary for the string, add the terminating '\0' */
+	strsize = vsnprintf (NULL, 0, format, xargs) + 1;
+	va_end (xargs);
+
+	str = malloc(strsize);
+	if (!str)
+		return NULL;
+
+	/* actually print the string into the buffer */
+	vsnprintf (str, strsize, format, args);
+
+	return str;
 }
 
 /**
@@ -116,17 +137,20 @@ gp_log_add_func (GPLogLevel level, GPLogFunc func, void *data)
 int
 gp_log_remove_func (int id)
 {
-	if (id < 1 || id > log_funcs_count)
-		return (GP_ERROR_BAD_PARAMETERS);
+	unsigned int i;
 
-	memmove (log_funcs + id - 1, log_funcs + id, log_funcs_count - id);
-	log_funcs_count--;
-
-	return (GP_OK);
+	for (i=0;i<log_funcs_count;i++) {
+		if (log_funcs[i].id == id) {
+			memmove (log_funcs + i, log_funcs + i + 1, sizeof(LogFunc) * (log_funcs_count - i - 1));
+			log_funcs_count--;
+			return GP_OK;
+		}
+	}
+	return GP_ERROR_BAD_PARAMETERS;
 }
 
 /**
- * Width of offset field in characters. Note that HEXDUMP_COMPLETE_LINE 
+ * Width of offset field in characters. Note that HEXDUMP_COMPLETE_LINE
  * needs to be changed when this value is changed.
  */
 #define HEXDUMP_OFFSET_WIDTH 4
@@ -151,7 +175,7 @@ gp_log_remove_func (int id)
 
 /**
  * We are lazy and do our typing only once. Please note that you have
- * to add/remove some lines when increasing/decreasing the values of 
+ * to add/remove some lines when increasing/decreasing the values of
  * HEXDUMP_BLOCK_DISTANCE and/or HEXDUMP_OFFSET_WIDTH.
  */
 #define HEXDUMP_COMPLETE_LINE {\
@@ -176,37 +200,45 @@ gp_log_remove_func (int id)
  * Takes the data and creates a formatted hexdump string. If you would like
  * to log text messages, use #gp_log instead.
  **/
+/* coverity[-tainted_sink] */
 void
-gp_log_data (const char *domain, const char *data, unsigned int size)
+gp_log_data (const char *domain, const char *data, unsigned int size, const char *format, ...)
 {
+	va_list args;
 	static const char hexchars[16] = "0123456789abcdef";
-	char *curline, *result;
+	char *curline, *result = 0, *msg = 0;
 	int x = HEXDUMP_INIT_X;
 	int y = HEXDUMP_INIT_Y;
-	int index;
+	unsigned int index, original_size = size;
 	unsigned char value;
 
+	va_start (args, format);
+	msg = gpi_vsnprintf(format, args);
+	va_end (args);
+	if (!msg) {
+		GP_LOG_E ("Malloc for expanding format string '%s' failed.", format);
+		goto exit;
+	}
+
 	if (!data) {
-		gp_log (GP_LOG_DATA, domain, _("No hexdump (NULL buffer)"));
-		return;
+		gp_log (GP_LOG_DATA, domain, "%s (no hexdump, NULL buffer)", msg);
+		goto exit;
 	}
 
 	if (!size) {
-		gp_log (GP_LOG_DATA, domain, _("Empty hexdump of empty buffer"));
-		return;
+		gp_log (GP_LOG_DATA, domain, "%s (empty hexdump of empty buffer)", msg);
+		goto exit;
 	}
 
 	if (size > 1024*1024) {
 		/* Does not make sense for 200 MB movies */
-		gp_log (GP_LOG_DATA, domain, _("Truncating dump from %d bytes to 1MB"), size);
 		size = 1024*1024;
 	}
 
 	curline = result = malloc ((HEXDUMP_LINE_WIDTH+1)*(((size-1)/16)+1)+1);
 	if (!result) {
-		gp_log (GP_LOG_ERROR, "gphoto2-log", _("Malloc for %i bytes "
-			"failed"), (HEXDUMP_LINE_WIDTH+1)*(((size-1)/16)+1)+1);
-		return;
+		GP_LOG_E ("Malloc for %i bytes failed", (HEXDUMP_LINE_WIDTH+1)*(((size-1)/16)+1)+1);
+		goto exit;
 	}
 
 	for (index = 0; index < size; ++index) {
@@ -235,8 +267,13 @@ gp_log_data (const char *domain, const char *data, unsigned int size)
         }
         curline[0] = '\0';
 
-	gp_log (GP_LOG_DATA, domain, _("Hexdump of %i = 0x%x bytes follows:\n%s"),
-		size, size, result);
+        if (size == original_size)
+                gp_log (GP_LOG_DATA, domain, "%s (hexdump of %d bytes)\n%s", msg, size, result);
+        else
+                gp_log (GP_LOG_DATA, domain, "%s (hexdump of the first %d of %d bytes)\n%s", msg, size, original_size, result);
+
+exit:
+	free (msg);
 	free (result);
 }
 
@@ -256,51 +293,24 @@ gp_log_data (const char *domain, const char *data, unsigned int size)
  * \param args the va_list corresponding to format
  *
  * Logs a message at the given log level. You would normally use this
- * function to log as yet unformatted strings. 
+ * function to log as yet unformatted strings.
  **/
 void
 gp_logv (GPLogLevel level, const char *domain, const char *format,
 	 va_list args)
 {
-	int i;
-#ifdef HAVE_VA_COPY
-	va_list xargs;
-#else
-#define xargs args
-#endif
-	int strsize = 1000;
-	char *str;
-	int n;
+	unsigned int i;
+	char *str = 0;
 
 	if (!log_funcs_count)
 		return;
 
-	str = malloc(strsize);
-	if (!str) return;
-#ifdef HAVE_VA_COPY
-	va_copy (xargs, args);
-#endif
-	n = vsnprintf (str, strsize, format, xargs);
-#ifdef HAVE_VA_COPY
-	va_end(xargs);
-#endif
-	if (n+1>strsize) {
-		free (str);
-		str = malloc(n+1);
-		if (!str) {
-			va_end(args);
-			return;
-		}
-		strsize = n+1;
-#ifdef HAVE_VA_COPY
-		va_copy (xargs, args);
-#endif
-		n = vsnprintf (str, strsize, format, xargs);
-#ifdef HAVE_VA_COPY
-		va_end(xargs);
-#endif
+	str = gpi_vsnprintf(format, args);
+	if (!str) {
+		GP_LOG_E ("Malloc for expanding format string '%s' failed.", format);
+		return;
 	}
-	va_end(args);
+
 	for (i = 0; i < log_funcs_count; i++)
 		if (log_funcs[i].level >= level)
 			log_funcs[i].func (level, domain, str, log_funcs[i].data);
@@ -327,6 +337,21 @@ gp_log (GPLogLevel level, const char *domain, const char *format, ...)
 	va_end (args);
 }
 
+void
+gp_log_with_source_location(GPLogLevel level, const char *file, int line, const char *func, const char *format, ...)
+{
+	va_list args;
+        char domain[100];
+
+        /* Only display filename without any path/directory part */
+        file = strrchr(file, '/') ? strrchr(file, '/') + 1 : file;
+        snprintf(domain, sizeof(domain), "%s [%s:%d]", func, file, line);
+
+	va_start (args, format);
+	gp_logv (level, domain, format, args);
+	va_end (args);
+}
+
 #else /* DISABLE_DEBUGGING */
 
 /*
@@ -350,6 +375,9 @@ gp_log (GPLogLevel level, const char *domain, const char *format, ...)
 #ifdef gp_log
 #undef gp_log
 #endif
+#ifdef gp_log_with_source_location
+#undef gp_log_with_source_location
+#endif
 
 int
 gp_log_add_func (GPLogLevel level, GPLogFunc func, void *data)
@@ -364,7 +392,7 @@ gp_log_remove_func (int id)
 }
 
 void
-gp_log_data (const char *domain, const char *data, unsigned int size)
+gp_log_data (const char *domain, const char *data, unsigned int size, const char *format, ...)
 {
 }
 
@@ -378,16 +406,21 @@ void
 gp_log (GPLogLevel level, const char *domain, const char *format, ...)
 {
 }
+
+void
+gp_log_with_source_location(GPLogLevel level, const char *file, int line, const char *func, const char *format, ...)
+{
+}
 #endif /* DISABLE_DEBUGGING */
 
 
 #ifdef _GPHOTO2_INTERNAL_CODE
 
-/** 
+/**
  * \brief (Internal) translate a enumeration code to a string
  */
 const char *
-gpi_enum_to_string(unsigned int _enum, 
+gpi_enum_to_string(unsigned int _enum,
 		   const StringFlagItem *map)
 {
 	int i;
@@ -400,7 +433,7 @@ gpi_enum_to_string(unsigned int _enum,
 	return NULL;
 }
 
-/** 
+/**
  * \brief (Internal) translate a string to an enumeration code
  */
 int
@@ -419,7 +452,7 @@ gpi_string_to_enum(const char *str,
 }
 
 void
-gpi_flags_to_string_list(unsigned int flags, 
+gpi_flags_to_string_list(unsigned int flags,
 			 const StringFlagItem *map,
 			 string_item_func func, void *data)
 {
@@ -429,13 +462,13 @@ gpi_flags_to_string_list(unsigned int flags,
 			func(map[i].str, data);
 			break;
 		} else if (0 != (flags & map[i].flag)) {
-			func(map[i].str, data);      
+			func(map[i].str, data);
 		}
 	}
 }
 
-unsigned int 
-gpi_string_to_flag(const char *str, 
+unsigned int
+gpi_string_to_flag(const char *str,
 	       const StringFlagItem *map)
 {
 	int i;
@@ -447,8 +480,8 @@ gpi_string_to_flag(const char *str,
   return 0;
 }
 
-int 
-gpi_string_or_to_flags(const char *str, 
+int
+gpi_string_or_to_flags(const char *str,
 		       unsigned int *flags,
 		       const StringFlagItem *map)
 {
@@ -468,7 +501,7 @@ gpi_string_or_to_flags(const char *str,
 }
 
 unsigned int
-gpi_string_list_to_flags(const char *str[], 
+gpi_string_list_to_flags(const char *str[],
 		     const StringFlagItem *map)
 {
 	int i;
