@@ -7,10 +7,10 @@
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
@@ -30,6 +30,7 @@ const char **gp_port_library_version(GPVersionVerbosity verbose)
 	static const char *shrt[] =
 		{
 			PACKAGE_VERSION,
+			"iolibs: " IOLIB_LIST,
 #ifdef HAVE_CC
 			HAVE_CC,
 #else
@@ -40,6 +41,11 @@ const char **gp_port_library_version(GPVersionVerbosity verbose)
 #else
 			"no ltdl",
 #endif
+#ifdef HAVE_LIBEXIF
+			"EXIF",
+#else
+			"no EXIF",
+#endif
 #if defined(HAVE_LIBUSB) || defined(HAVE_LIBUSB1) || defined(HAVE_LIBUSB_WIN32)
 			"USB",
 #else
@@ -66,15 +72,21 @@ const char **gp_port_library_version(GPVersionVerbosity verbose)
 	static const char *verb[] =
 		{
 			PACKAGE_VERSION,
+			"iolibs: " IOLIB_LIST,
 #ifdef HAVE_CC
 			HAVE_CC " (C compiler used)",
 #else
 			"unknown (C compiler used)",
 #endif
 #ifdef HAVE_LTDL
-			"ltdl (for portable loading of camlibs)",
+			"ltdl (for portable loading of iolibs)",
 #else
-			"no ltdl (for portable loading of camlibs)",
+			"no ltdl (for portable loading of iolibs)",
+#endif
+#ifdef HAVE_LIBEXIF
+			"EXIF (for vusb)",
+#else
+			"no EXIF (for vusb)",
 #endif
 #ifdef HAVE_LIBUSB_WIN32
 			"USB (libusb-win32, for USB cameras)",
