@@ -10,10 +10,10 @@
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
@@ -120,6 +120,17 @@ static int translate_libusb_error( int error, int default_return )
 	}
 }
 
+struct _PrivateIrqCompleted {
+	struct _PrivateIrqCompleted	*next;
+	enum libusb_transfer_status	status;
+	int							data_len;
+	unsigned char				*data;
+};
+
+#define NB_INTERRUPT_TRANSFERS 10
+/* FIXME: safe size? */
+#define INTERRUPT_BUFFER_SIZE 256
+
 struct _GPPortPrivateLibrary {
 	libusb_context *ctx;
 	libusb_device *d;
@@ -136,11 +147,15 @@ struct _GPPortPrivateLibrary {
 	struct libusb_device_descriptor	*descs;
 	libusb_device			**devs;
 
-#define INTERRUPT_TRANSFERS 10
-	struct libusb_transfer		*transfers[INTERRUPT_TRANSFERS];
-	int 				nrofirqs;
-	unsigned char			**irqs;
-	int 				*irqlens;
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+	/* for dumping the read usb content */
+	int				logfd;
+#endif
+
+	struct libusb_transfer		*transfers[NB_INTERRUPT_TRANSFERS];
+	int				nrofactiveinttransfers;
+	struct _PrivateIrqCompleted	*irqs_head;
+	struct _PrivateIrqCompleted	*irqs_tail;
 };
 
 GPPortType
@@ -198,6 +213,11 @@ gp_port_library_list (GPPortInfoList *list)
 	gp_port_info_list_append (list, info); /* do not check return value, it might be -1 */
 
 	nrofdevs = libusb_get_device_list (ctx, &devs);
+	if (!nrofdevs) {
+		libusb_exit (ctx); /* should free all stuff above */
+		goto nodevices;
+	}
+
 	C_MEM (descs = calloc (nrofdevs, sizeof(descs[0])));
 	for (i=0;i<nrofdevs;i++)
 		LOG_ON_LIBUSB_E (libusb_get_device_descriptor(devs[i], &descs[i]));
@@ -223,7 +243,7 @@ gp_port_library_list (GPPortInfoList *list)
 			}
 			for (i1 = 0; i1 < config->bNumInterfaces; i1++)
 				for (i2 = 0; i2 < config->interface[i1].num_altsetting; i2++) {
-					const struct libusb_interface_descriptor *intf = &config->interface[i1].altsetting[i2]; 
+					const struct libusb_interface_descriptor *intf = &config->interface[i1].altsetting[i2];
 					if (	(intf->bInterfaceClass == LIBUSB_CLASS_HID)	||
 						(intf->bInterfaceClass == LIBUSB_CLASS_PRINTER)	||
 						(intf->bInterfaceClass == LIBUSB_CLASS_COMM)	||
@@ -239,7 +259,7 @@ gp_port_library_list (GPPortInfoList *list)
 			continue;
 		/* Note: We do not skip USB storage. Some devices can support both,
 		 * and the Ricoh erronously reports it.
-		 */ 
+		 */
 		nrofdevices++;
 	}
 
@@ -249,7 +269,7 @@ gp_port_library_list (GPPortInfoList *list)
 	 */
 	/* We never want to return just "usb:" ... also return "usb:XXX,YYY", and
 	 * let upper layers filter out the usb: */
-	if (nrofdevices <= 1) 
+	if (nrofdevices <= 1)
 		return (GP_OK);
 #endif
 
@@ -278,7 +298,7 @@ gp_port_library_list (GPPortInfoList *list)
 			}
 			for (i1 = 0; i1 < config->bNumInterfaces; i1++)
 				for (i2 = 0; i2 < config->interface[i1].num_altsetting; i2++) {
-					const struct libusb_interface_descriptor *intf = &config->interface[i1].altsetting[i2]; 
+					const struct libusb_interface_descriptor *intf = &config->interface[i1].altsetting[i2];
 					if (	(intf->bInterfaceClass == LIBUSB_CLASS_HID)	||
 						(intf->bInterfaceClass == LIBUSB_CLASS_PRINTER)	||
 						(intf->bInterfaceClass == LIBUSB_CLASS_COMM))
@@ -292,7 +312,7 @@ gp_port_library_list (GPPortInfoList *list)
 			continue;
 		/* Note: We do not skip USB storage. Some devices can support both,
 		 * and the Ricoh erronously reports it.
-		 */ 
+		 */
 		C_GP (gp_port_info_new (&info));
 		gp_port_info_set_type (info, GP_PORT_USB);
 		gp_port_info_set_name (info, "Universal Serial Bus");
@@ -308,6 +328,7 @@ gp_port_library_list (GPPortInfoList *list)
 	libusb_exit (ctx); /* should free all stuff above */
 	free (descs);
 
+nodevices:
 	/* This will only be added if no other device was ever added.
 	 * Users doing "usb:" usage will enter the regular expression matcher case. */
 	if (nrofdevices == 0) {
@@ -332,6 +353,10 @@ static int gp_libusb1_init (GPPort *port)
 		port->pl = NULL;
 		return GP_ERROR_IO;
 	}
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+	unlink("usblog.raw");
+	port->pl->logfd = open("usblog.raw",O_CREAT|O_WRONLY,0644);
+#endif
 #if 0
 	libusb_set_debug (port->pl->ctx, 255);
 #endif
@@ -343,6 +368,9 @@ gp_libusb1_exit (GPPort *port)
 {
 	if (port->pl) {
 		free (port->pl->descs);
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+		if (port->pl->logfd >=0 ) close (port->pl->logfd);
+#endif
 		if (port->pl->nrofdevs)
 			libusb_free_device_list (port->pl->devs, 1);
 		libusb_exit (port->pl->ctx);
@@ -421,7 +449,9 @@ gp_libusb1_open (GPPort *port)
 		return GP_ERROR_IO_USB_CLAIM;
 	}
 
-	gp_libusb1_queue_interrupt_urbs (port);
+	ret = gp_libusb1_queue_interrupt_urbs (port);
+	if (ret)
+		return ret;
 
 	return GP_OK;
 }
@@ -429,7 +459,7 @@ gp_libusb1_open (GPPort *port)
 static int
 _close_async_interrupts(GPPort *port)
 {
-	int i, haveone;
+	unsigned int i, haveone;
 	struct timeval tv;
 
 	C_PARAMS (port);
@@ -509,11 +539,19 @@ gp_libusb1_close (GPPort *port)
 
 	libusb_close (port->pl->dh);
 
-	free (port->pl->irqs);
-	port->pl->irqs = NULL;
-	free (port->pl->irqlens);
-	port->pl->irqlens = NULL;
-	port->pl->nrofirqs = 0;
+	struct _PrivateIrqCompleted *irq_iter;
+	struct _PrivateIrqCompleted *irq_next;
+
+	irq_iter=port->pl->irqs_head;
+	while (irq_iter != NULL) {
+		if (irq_iter->data)
+			free(irq_iter->data);
+		irq_next = irq_iter->next;
+		free(irq_iter);
+		irq_iter = irq_next;
+	}
+	port->pl->irqs_head = NULL;
+	port->pl->irqs_tail = NULL;
 	port->pl->dh = NULL;
 	return GP_OK;
 }
@@ -570,6 +608,9 @@ gp_libusb1_read(GPPort *port, char *bytes, int size)
 			(unsigned char*)bytes, size, &curread, port->timeout),
 		  GP_ERROR_IO_READ );
 
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+	write(port->pl->logfd, bytes, curread);
+#endif
         return curread;
 }
 
@@ -589,44 +630,66 @@ gp_libusb1_reset(GPPort *port)
 static void LIBUSB_CALL
 _cb_irq(struct libusb_transfer *transfer)
 {
+	struct _PrivateIrqCompleted *irq_new = NULL;
 	struct _GPPortPrivateLibrary *pl = transfer->user_data;
-	int i;
+	unsigned int i;
+	int ret;
 
 	GP_LOG_D("%p with status %d", transfer, transfer->status);
 
+	if ((transfer->status != LIBUSB_TRANSFER_CANCELLED) &&
+		(transfer->status != LIBUSB_TRANSFER_TIMED_OUT)
+	) {
+		irq_new = (struct _PrivateIrqCompleted *) calloc(1, sizeof (struct _PrivateIrqCompleted));
+		irq_new->status = transfer->status;
+
+		/* Add the irq to the list */
+		if (pl->irqs_tail)
+			pl->irqs_tail->next = irq_new;
+		pl->irqs_tail = irq_new;
+		if (!pl->irqs_head)
+			pl->irqs_head = irq_new;
+	}
+
 	if (	(transfer->status == LIBUSB_TRANSFER_CANCELLED) ||
 		(transfer->status == LIBUSB_TRANSFER_TIMED_OUT) || /* on close */
-		(transfer->status == LIBUSB_TRANSFER_NO_DEVICE) /* on removing camera */
+		(transfer->status == LIBUSB_TRANSFER_NO_DEVICE) || /* on removing camera */
+		(transfer->status != LIBUSB_TRANSFER_COMPLETED) /* any error */
 	) {
+		if (transfer->status != LIBUSB_TRANSFER_COMPLETED) {
+			/* So far we don't requeue for any error, previous behavior, maybe we should?
+			 * Note: some times in case of device removed, libusb encounter an
+			 * errno == -71, that results in a LIBUSB_TRANSFER_ERROR (1)
+			 */
+			GP_LOG_D("Transfer %p should be in LIBUSB_TRANSFER_COMPLETED, but is %d!", transfer, transfer->status);
+		}
 		/* Only requeue the global transfers, not temporary ones */
 		for (i = 0; i < sizeof(pl->transfers)/sizeof(pl->transfers[0]); i++) {
 			if (pl->transfers[i] == transfer) {
 				libusb_free_transfer (transfer);
 				pl->transfers[i] = NULL;
+				pl->nrofactiveinttransfers--;
 				return;
 			}
 		}
 		return;
 	}
 
-	if (transfer->status != LIBUSB_TRANSFER_COMPLETED) {
-		GP_LOG_D("transfer %p should be in LIBUSB_TRANSFER_COMPLETED, but is %d!", transfer, transfer->status);
-		return;
-	}
 	if (transfer->actual_length) {
 		GP_LOG_DATA ((char*)transfer->buffer, transfer->actual_length, "interrupt");
 
-		pl->irqs 	= realloc(pl->irqs, sizeof(pl->irqs[0])*(pl->nrofirqs+1));
-		pl->irqlens	= realloc(pl->irqlens, sizeof(pl->irqlens[0])*(pl->nrofirqs+1));
-
-		pl->irqlens[pl->nrofirqs] = transfer->actual_length;
-		pl->irqs[pl->nrofirqs] = malloc(transfer->actual_length);
-		memcpy(pl->irqs[pl->nrofirqs],transfer->buffer,transfer->actual_length);
-		pl->nrofirqs++;
+		irq_new->data_len = transfer->actual_length;
+		// Steal the transfer buffer and replace it with a new one for reusing the transfer
+		irq_new->data = transfer->buffer;
+		transfer->buffer = malloc(INTERRUPT_BUFFER_SIZE);
+		transfer->length = INTERRUPT_BUFFER_SIZE;
 	}
 
-	GP_LOG_D("requeuing complete transfer %p", transfer);
-	LOG_ON_LIBUSB_E(libusb_submit_transfer(transfer));
+	GP_LOG_D("Requeuing completed transfer %p", transfer);
+	ret = LOG_ON_LIBUSB_E(libusb_submit_transfer (transfer));
+	if (ret < LIBUSB_SUCCESS) {
+		pl->nrofactiveinttransfers--;
+	}
 	return;
 }
 
@@ -634,18 +697,30 @@ static int
 gp_libusb1_queue_interrupt_urbs (GPPort *port)
 {
 	unsigned int i;
+	int ret = 0;
+
+	/* no interrupt endpoint */
+	if (port->settings.usb.intep == -1)
+		return GP_OK;
+
 
 	for (i = 0; i < sizeof(port->pl->transfers)/sizeof(port->pl->transfers[0]); i++) {
 		unsigned char *buf;
+		if (port->pl->transfers[i] != NULL)
+			continue;
 		port->pl->transfers[i] = libusb_alloc_transfer(0);
-
-		buf = malloc(256); /* FIXME: safe size? */
+		buf = malloc(INTERRUPT_BUFFER_SIZE);
 		libusb_fill_interrupt_transfer(port->pl->transfers[i], port->pl->dh, port->settings.usb.intep,
-			buf, 256, _cb_irq, port->pl, 0
+			buf, INTERRUPT_BUFFER_SIZE, _cb_irq, port->pl, 0
 		);
 		port->pl->transfers[i]->flags |= LIBUSB_TRANSFER_FREE_BUFFER;
-		LOG_ON_LIBUSB_E(libusb_submit_transfer (port->pl->transfers[i]));
-		
+		ret = LOG_ON_LIBUSB_E(libusb_submit_transfer (port->pl->transfers[i]));
+		if (ret < LIBUSB_SUCCESS) {
+			libusb_free_transfer (port->pl->transfers[i]);
+			port->pl->transfers[i] = NULL;
+			return translate_libusb_error(ret, GP_ERROR_IO);
+		}
+		port->pl->nrofactiveinttransfers++;
 	}
 	return GP_OK;
 }
@@ -655,18 +730,32 @@ gp_libusb1_check_int (GPPort *port, char *bytes, int size, int timeout)
 {
 	int 		ret;
 	struct timeval	tv;
+	struct _PrivateIrqCompleted *irq_cur = NULL;
 
 	C_PARAMS (port && port->pl->dh && timeout >= 0);
 
-	if (port->pl->nrofirqs)
+	if (port->pl->irqs_head != NULL)
 		goto handleirq;
 
+	if (!timeout)
+		return GP_ERROR_TIMEOUT;
+
+	/* If we have lost all the queued transfers, we should probably restart them
+	 * if there are long running error, like "no more device". That would be
+	 * reported upstream, so upstream can take care of that.
+	 */
+	if (port->pl->nrofactiveinttransfers < NB_INTERRUPT_TRANSFERS) {
+		ret = gp_libusb1_queue_interrupt_urbs(port);
+		if (ret != GP_OK)
+			return ret;
+	}
+
 	tv.tv_sec = timeout/1000;
 	tv.tv_usec = (timeout%1000)*1000;
 
 	ret = LOG_ON_LIBUSB_E (libusb_handle_events_timeout(port->pl->ctx, &tv));
 
-	if (port->pl->nrofirqs)
+	if (port->pl->irqs_head != NULL)
 		goto handleirq;
 
 	if (ret < LIBUSB_SUCCESS)
@@ -675,14 +764,57 @@ gp_libusb1_check_int (GPPort *port, char *bytes, int size, int timeout)
 	return GP_ERROR_TIMEOUT;
 
 handleirq:
-	if (size > port->pl->irqlens[0])
-		size = port->pl->irqlens[0];
-	memcpy(bytes, port->pl->irqs[0], size);
+	irq_cur = port->pl->irqs_head;
 
-	memmove(port->pl->irqs,port->pl->irqs+1,sizeof(port->pl->irqs[0])*(port->pl->nrofirqs-1));
-	memmove(port->pl->irqlens,port->pl->irqlens+1,sizeof(port->pl->irqlens[0])*(port->pl->nrofirqs-1));
-	port->pl->nrofirqs--;
-        return size;
+	switch (irq_cur->status) {
+	case LIBUSB_TRANSFER_COMPLETED:
+		ret = GP_OK;
+		break;
+	case LIBUSB_TRANSFER_NO_DEVICE:
+		ret = GP_ERROR_IO_USB_FIND;
+		/* Agglomerate similar errors to only report once. */
+		while ((irq_cur->next) &&
+			   (irq_cur->next->status == LIBUSB_TRANSFER_NO_DEVICE)
+		) {
+			port->pl->irqs_head = irq_cur->next;
+			if (irq_cur->data)
+				free(irq_cur->data);
+			free(irq_cur);
+			irq_cur = port->pl->irqs_head;
+		}
+		break;
+	default:
+		ret = GP_ERROR_IO;
+		/* Agglomerate similar errors to only report once. */
+		while ((irq_cur->next) &&
+			   (irq_cur->next->status != LIBUSB_TRANSFER_COMPLETED) &&
+			   (irq_cur->next->status != LIBUSB_TRANSFER_NO_DEVICE)
+        ) {
+			port->pl->irqs_head = irq_cur->next;
+			if (irq_cur->data)
+				free(irq_cur->data);
+			free(irq_cur);
+			irq_cur = port->pl->irqs_head;
+		}
+		break;
+	}
+
+	if (size > irq_cur->data_len)
+		size = irq_cur->data_len;
+	if (irq_cur->data) {
+		if (size > 0)
+			memcpy(bytes, irq_cur->data, size);
+		free(irq_cur->data);
+	}
+	port->pl->irqs_head = irq_cur->next;
+	if (port->pl->irqs_head == NULL)
+		port->pl->irqs_tail = NULL;
+	free(irq_cur);
+
+	if (ret != GP_OK)
+		return ret;
+
+	return size;
 }
 
 static int
@@ -716,12 +848,12 @@ gp_libusb1_msg_read_lib(GPPort *port, int request, int value, int index,
 					GP_ERROR_IO_READ);
 }
 
-/* The next two functions support the nonstandard request types 0x41 (write) 
- * and 0xc1 (read), which are occasionally needed. 
+/* The next two functions support the nonstandard request types 0x41 (write)
+ * and 0xc1 (read), which are occasionally needed.
  */
 
 static int
-gp_libusb1_msg_interface_write_lib(GPPort *port, int request, 
+gp_libusb1_msg_interface_write_lib(GPPort *port, int request,
 	int value, int index, char *bytes, int size)
 {
 	return gp_libusb1_msg (port, request, value, index, bytes, size,
@@ -731,7 +863,7 @@ gp_libusb1_msg_interface_write_lib(GPPort *port, int request,
 
 
 static int
-gp_libusb1_msg_interface_read_lib(GPPort *port, int request, 
+gp_libusb1_msg_interface_read_lib(GPPort *port, int request,
 	int value, int index, char *bytes, int size)
 {
 	return gp_libusb1_msg (port, request, value, index, bytes, size,
@@ -740,12 +872,12 @@ gp_libusb1_msg_interface_read_lib(GPPort *port, int request,
 }
 
 
-/* The next two functions support the nonstandard request types 0x21 (write) 
- * and 0xa1 (read), which are occasionally needed. 
+/* The next two functions support the nonstandard request types 0x21 (write)
+ * and 0xa1 (read), which are occasionally needed.
  */
 
 static int
-gp_libusb1_msg_class_write_lib(GPPort *port, int request, 
+gp_libusb1_msg_class_write_lib(GPPort *port, int request,
 	int value, int index, char *bytes, int size)
 {
 	return gp_libusb1_msg (port, request, value, index, bytes, size,
@@ -755,7 +887,7 @@ gp_libusb1_msg_class_write_lib(GPPort *port, int request,
 
 
 static int
-gp_libusb1_msg_class_read_lib(GPPort *port, int request, 
+gp_libusb1_msg_class_read_lib(GPPort *port, int request,
 	int value, int index, char *bytes, int size)
 {
 	return gp_libusb1_msg (port, request, value, index, bytes, size,
@@ -812,7 +944,7 @@ gp_libusb1_update (GPPort *port)
 	/* The interface changed. release the old, claim the new ... */
 	if (port->settings.usb.interface != port->pl->interface) {
 		GP_LOG_D ("changing interface %d -> %d", port->pl->interface, port->settings.usb.interface);
-		if (LOG_ON_LIBUSB_E (libusb_release_interface (port->pl->dh, port->pl->interface))) {
+		if ((port->pl->interface != -1) && (LOG_ON_LIBUSB_E (libusb_release_interface (port->pl->dh, port->pl->interface)))) {
 			/* Not a hard error for now. -Marcus */
 		} else {
 			GP_LOG_D ("claiming interface %d", port->settings.usb.interface);
@@ -824,10 +956,10 @@ gp_libusb1_update (GPPort *port)
 	}
 	if (port->settings.usb.config != port->pl->config) {
 		GP_LOG_D ("changing config %d -> %d", port->pl->config, port->settings.usb.config);
-		/* This can only be changed with the interface released. 
+		/* This can only be changed with the interface released.
 		 * This is a hard requirement since 2.6.12.
 		 */
-		if (LOG_ON_LIBUSB_E (libusb_release_interface (port->pl->dh, port->settings.usb.interface))) {
+		if ((port->pl->config != -1) && (LOG_ON_LIBUSB_E (libusb_release_interface (port->pl->dh, port->settings.usb.interface)))) {
 			ifacereleased = FALSE;
 		} else {
 			ifacereleased = TRUE;
@@ -840,7 +972,7 @@ gp_libusb1_update (GPPort *port)
 					   port->settings.usb.interface,
 					   port->settings.usb.config,
 					   strerror(saved_errno));
-			return GP_ERROR_IO_UPDATE;	
+			return GP_ERROR_IO_UPDATE;
 #endif
 			GP_LOG_E ("setting configuration from %d to %d failed, but continuing...", port->pl->config, port->settings.usb.config);
 		}
@@ -1129,7 +1261,7 @@ gp_libusb1_match_mtp_device(struct libusb_device *dev,int *configno, int *interf
 				xifaces++;
 
 				for (k = 0; k < dev->config[i].interface[j].num_altsetting; k++) {
-					struct usb_interface_descriptor *intf = &dev->config[i].interface[j].altsetting[k]; 
+					struct usb_interface_descriptor *intf = &dev->config[i].interface[j].altsetting[k];
 					if (	(intf->bInterfaceClass == LIBUSB_CLASS_HID)	||
 						(intf->bInterfaceClass == LIBUSB_CLASS_PRINTER)	||
 						(intf->bInterfaceClass == LIBUSB_CLASS_AUDIO)	||
@@ -1150,8 +1282,8 @@ gp_libusb1_match_mtp_device(struct libusb_device *dev,int *configno, int *interf
 		return 0;
 
 	/*
-	 * Loop over the device configurations and interfaces. Nokia MTP-capable 
-	 * handsets (possibly others) typically have the string "MTP" in their 
+	 * Loop over the device configurations and interfaces. Nokia MTP-capable
+	 * handsets (possibly others) typically have the string "MTP" in their
 	 * MTP interface descriptions, that's how they can be detected, before
 	 * we try the more esoteric "OS descriptors" (below).
 	 */
@@ -1163,9 +1295,9 @@ gp_libusb1_match_mtp_device(struct libusb_device *dev,int *configno, int *interf
 				int k;
 				for (k = 0; k < dev->config[i].interface[j].num_altsetting; k++) {
 					buf[0] = '\0';
-					ret = usb_get_string_simple(devh, 
-						dev->config[i].interface[j].altsetting[k].iInterface, 
-						(char *) buf, 
+					ret = usb_get_string_simple(devh,
+						dev->config[i].interface[j].altsetting[k].iInterface,
+						(char *) buf,
 						1024);
 					if (ret < 3)
 						continue;
