@@ -8,22 +8,21 @@
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
 #include <gphoto2/gphoto2-port-library.h>
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
@@ -102,10 +101,10 @@ _ptpip_resolved (
 	char		path[200];
 
 	if (errorCode != kDNSServiceErr_NoError) {
-		gp_log (GP_LOG_ERROR, "ptpip", "Error on 2nd level query.");
+		GP_LOG_E ("Error on 2nd level query.");
 		return;
 	}
-	gp_log (GP_LOG_DEBUG, "ptpip", "fullname %s, hosttarget %s, port %d", fullname, hosttarget, htons(port));
+	GP_LOG_D ("fullname %s, hosttarget %s, port %d", fullname, hosttarget, htons(port));
 	cnt = TXTRecordGetCount (txtLen, txtRecord);
 	for (i=0;i<cnt;i++) {
 		char	key[256];
@@ -114,11 +113,11 @@ _ptpip_resolved (
 
 		valuelen = 0;
 		if (kDNSServiceErr_NoError == TXTRecordGetItemAtIndex (txtLen, txtRecord, i, sizeof(key), key, &valuelen, &value))
-			gp_log (GP_LOG_DEBUG, "ptpip", "%d: %s:%s", i, key, (char*)value);
+			GP_LOG_D ("%d: %s:%s", i, key, (char*)value);
 	}
 	hent = gethostbyname (hosttarget);
 	if (!hent) {
-		gp_log (GP_LOG_ERROR, "ptpip", "Could not resolve the returned host: %s", hosttarget);
+		GP_LOG_E ("Could not resolve the returned host: %s", hosttarget);
 		return;
 	}
 	memcpy(&inaddr.s_addr,hent->h_addr_list[0],hent->h_length);
@@ -141,10 +140,10 @@ _ptpip_enumerate (
 	DNSServiceRef		sd;
 
 	if (errorCode != kDNSServiceErr_NoError) {
-		gp_log (GP_LOG_ERROR, "ptpip", "Error on _ptp._tcp query.");
+		GP_LOG_E ("Error on _ptp._tcp query.");
 		return;
 	}
-	gp_log (GP_LOG_DEBUG, "ptpip", "found %s, %s, %s", serviceName, regtype, replyDomain);
+	GP_LOG_D ("found %s, %s, %s", serviceName, regtype, replyDomain);
 	mdnsi.list = context;
 	mdnsi.name = serviceName;
 	DNSServiceResolve (&sd,
@@ -184,8 +183,20 @@ gp_port_library_list (GPPortInfoList *list)
 	gp_port_info_set_type (info, GP_PORT_PTPIP);
 	gp_port_info_set_name (info, "");
 	gp_port_info_set_path (info, "^ptpip:");
+	gp_port_info_list_append (list, info); /* do not check return */
+
+	gp_port_info_new (&info);
+	gp_port_info_set_type (info, GP_PORT_IP);
+	gp_port_info_set_name (info, _("IP Connection"));
+	gp_port_info_set_path (info, "ip:");
 	CHECK (gp_port_info_list_append (list, info));
 
+	/* Generic matcher so you can pass any IP address */
+	gp_port_info_new (&info);
+	gp_port_info_set_type (info, GP_PORT_IP);
+	gp_port_info_set_name (info, "");
+	gp_port_info_set_path (info, "^ip:");
+	gp_port_info_list_append (list, info); /* do not check return */
 #ifdef HAVE_MDNS_BONJOUR
 	ret = DNSServiceBrowse (
 		&sd,
@@ -199,7 +210,7 @@ gp_port_library_list (GPPortInfoList *list)
 	/* We need to make it a non-blocking query */
 	fd = DNSServiceRefSockFD(sd);
 	if (fd != -1) {
-		FD_ZERO (&infds); FD_SET (fd, &infds); 
+		FD_ZERO (&infds); FD_SET (fd, &infds);
 		tv.tv_sec = 0; tv.tv_usec = 1;
 		/* If we have input, we can try to process a result */
 		if (1 == select (fd+1, &infds, NULL, NULL, &tv))
@@ -212,20 +223,16 @@ gp_port_library_list (GPPortInfoList *list)
 
 static int gp_port_ptpip_init (GPPort *dev)
 {
-	dev->pl = malloc (sizeof (GPPortPrivateLibrary));
-	if (!dev->pl)
-		return GP_ERROR_NO_MEMORY;
-	memset (dev->pl, 0, sizeof(GPPortPrivateLibrary));
+	C_MEM (dev->pl = calloc (1, sizeof (GPPortPrivateLibrary)));
+
 	return GP_OK;
 }
 
 static int
 gp_port_ptpip_exit (GPPort *port)
 {
-	if (port->pl) {
-		free (port->pl);
-		port->pl = NULL;
-	}
+	free (port->pl);
+	port->pl = NULL;
 
 	return GP_OK;
 }
@@ -266,10 +273,10 @@ gp_port_library_operations (void)
 {
 	GPPortOperations *ops;
 
-	ops = malloc (sizeof (GPPortOperations));
+	ops = calloc (1, sizeof (GPPortOperations));
 	if (!ops)
 		return NULL;
-	memset (ops, 0, sizeof (GPPortOperations));
+
 	ops->init   = gp_port_ptpip_init;
 	ops->exit   = gp_port_ptpip_exit;
 	ops->open   = gp_port_ptpip_open;
