@@ -1,43 +1,43 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
- * Copyright  2001 Lutz Mller <lutz@users.sf.net>
- * Copyright  2000 Philippe Marzouk <pmarzouk@bigfoot.com>
- * Copyright  2000 Edouard Lafargue <Edouard.Lafargue@bigfoot.com>
- * Copyright  1999 Johannes Erdfelt <johannes@erdfelt.com>
- * Copyright  1999 Scott Fritzinger <scottf@unr.edu>
+ * Copyright 2001 Lutz Mueller <lutz@users.sf.net>
+ * Copyright 2000 Philippe Marzouk <pmarzouk@bigfoot.com>
+ * Copyright 2000 Edouard Lafargue <Edouard.Lafargue@bigfoot.com>
+ * Copyright 1999 Johannes Erdfelt <johannes@erdfelt.com>
+ * Copyright 1999 Scott Fritzinger <scottf@unr.edu>
  *
  * Based on work by:
- * Copyright  1999 Beat Christen <spiff@longstreet.ch>
+ * Copyright 1999 Beat Christen <spiff@longstreet.ch>
  * 	for the toshiba gPhoto library.
- *                   
+ *
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _DEFAULT_SOURCE
+#define _DARWIN_C_SOURCE
+/* Solaris needs this */
+#define __EXTENSIONS__
+
 #include "config.h"
 #include <gphoto2/gphoto2-port-library.h>
 
-/* Solaris needs this */
-#define __EXTENSIONS__
 
 #include <stdlib.h>
 #include <stdio.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -67,10 +67,6 @@
 #  include <sgtty.h>
 #endif
 
-#ifdef HAVE_RESMGR
-#  include <resmgr.h>
-#endif
-
 #ifdef HAVE_TTYLOCK
 #  include <ttylock.h>
 #elif defined(HAVE_LOCKDEV)
@@ -89,7 +85,7 @@
 #    define N_(String) gettext_noop (String)
 #  else
 #    define N_(String) (String)
-#  endif 
+#  endif
 #else
 #  define textdomain(String) (String)
 #  define gettext(String) (String)
@@ -187,10 +183,17 @@
 /* IRIX */
 #if defined(__sgi)
 #define GP_PORT_SERIAL_PREFIX "/dev/ttyd%i"
-#define GP_PORT_SERIAL_RANGE_LOW       1 
+#define GP_PORT_SERIAL_RANGE_LOW       1
 #define GP_PORT_SERIAL_RANGE_HIGH     11
 #endif
 
+/* MidnightBSD */
+#ifdef __MidnightBSD__
+#define GP_PORT_SERIAL_PREFIX   "/dev/cuad%x"
+#define GP_PORT_SERIAL_RANGE_LOW        0
+#define GP_PORT_SERIAL_RANGE_HIGH       (0xf)
+#endif
+
 /* Others? */
 
 /* Default */
@@ -200,8 +203,6 @@
 #define GP_PORT_SERIAL_RANGE_HIGH       0
 #endif
 
-#define GP_MODULE "serial"
-
 struct _GPPortPrivateLibrary {
 	int fd;       /* Device handle */
 	int baudrate; /* Current speed */
@@ -210,7 +211,7 @@ struct _GPPortPrivateLibrary {
 static int gp_port_serial_check_speed (GPPort *dev);
 
 GPPortType
-gp_port_library_type () 
+gp_port_library_type ()
 {
         return (GP_PORT_SERIAL);
 }
@@ -222,16 +223,7 @@ gp_port_serial_lock (GPPort *dev, const char *path)
 	int pid;
 #endif
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-serial",
-		"Trying to lock '%s'...", path);
-
-#ifdef HAVE_RESMGR
-	if (!rsm_lock_device(path)) {
-		return GP_OK;
-	}
-	/* fallthrough */
-#define __HAVE_LOCKING	
-#endif
+	GP_LOG_D ("Trying to lock '%s'...", path);
 
 #if defined(HAVE_TTYLOCK)
 	if (ttylock ((char*) path)) {
@@ -240,7 +232,7 @@ gp_port_serial_lock (GPPort *dev, const char *path)
 						  "'%s'"), path);
 		return (GP_ERROR_IO_LOCK);
 	}
-#define __HAVE_LOCKING	
+#define __HAVE_LOCKING
 #elif defined(HAVE_LOCKDEV)
 	pid = dev_lock (path);
 	if (pid) {
@@ -255,14 +247,14 @@ gp_port_serial_lock (GPPort *dev, const char *path)
 		}
 		return (GP_ERROR_IO_LOCK);
 	}
-#define __HAVE_LOCKING	
+#define __HAVE_LOCKING
 #endif
 
 #ifndef __HAVE_LOCKING
 # ifdef __GCC__
-#  warning No locking library found. 
+#  warning No locking library found.
 #  warning You will run into problems if you use
-#  warning gphoto2 with a serial (RS232) camera in 
+#  warning gphoto2 with a serial (RS232) camera in
 #  warning combination with Konqueror (KDE) or Nautilus (GNOME).
 #  warning This will *not* concern USB cameras.
 # endif
@@ -274,11 +266,6 @@ gp_port_serial_lock (GPPort *dev, const char *path)
 static int
 gp_port_serial_unlock (GPPort *dev, const char *path)
 {
-#ifdef HAVE_RESMGR
-	if (!rsm_unlock_device(path))
-		return GP_OK;
-	/* fallback through other unlock styles */
-#endif
 
 #if defined(HAVE_TTYLOCK)
 	if (ttyunlock ((char*) path)) {
@@ -315,22 +302,24 @@ gp_port_library_list (GPPortInfoList *list)
 {
 	GPPortInfo info;
 	char path[1024], prefix[1024];
-        int x, fd;
-        struct stat s;
+	int x;
+	struct stat s;
 #ifdef OS2
-        int r, fh, option;
+	int r, fh, option;
 #endif
 
-        /* Copy in the serial port prefix */
-        strcpy (prefix, GP_PORT_SERIAL_PREFIX);
+	/* Copy in the serial port prefix */
+	strcpy (prefix, GP_PORT_SERIAL_PREFIX);
 
 	/* On Linux systems, check for devfs */
 #ifdef __linux
-        if (!stat ("/dev/tts", &s))
+	if (!stat ("/dev/tts", &s))
 		strcpy (prefix, "/dev/tts/%i");
 #endif
 
-        for (x=GP_PORT_SERIAL_RANGE_LOW; x<=GP_PORT_SERIAL_RANGE_HIGH; x++) {
+	for (x=GP_PORT_SERIAL_RANGE_LOW; x<=GP_PORT_SERIAL_RANGE_HIGH; x++) {
+		char *xname;
+
 		sprintf (path, prefix, x);
 
 		/* OS/2 seems to need an additional check */
@@ -346,85 +335,61 @@ gp_port_library_list (GPPortInfoList *list)
 		 * there is no need to try locking. */
 		if ((stat (path, &s) == -1) && ((errno == ENOENT) || (errno == ENODEV)))
 			continue;
-		/* Just let it fail later on open, lock&open for just enumerating is taking too long. */
-		/* This is a significant part of the "empty" startup time. */
-#if 0
-		/* First of all, try to lock the device */
-		if (gp_port_serial_lock (NULL, path) < 0)
-			continue;
-			
-		/* Device locked. Try to open the device. */
-		fd = -1;
-#ifdef HAVE_RESMGR
-		/* resmgr has its own API, which calls to a server and
-		 * communicates over UNIX domain sockets.
-		 */
-		fd = rsm_open_device(path, O_RDONLY | O_NDELAY);
-		/* fall through to standard open if this failed */
-#endif
-		if (fd == -1)
-			fd = open (path, O_RDONLY | O_NONBLOCK);
-		if (fd < 0) {
-			gp_port_serial_unlock (NULL, path);
-			continue;
-		}
 
-		/*
-		 * Device locked and opened. Close it, unlock it, and add
-		 * it to the list.
-		 */
-		close (fd);
-		gp_port_serial_unlock (NULL, path);
-#endif
-		info.type = GP_PORT_SERIAL;
-		strncpy (info.path, "serial:", sizeof (info.path));
-		strncat (info.path, path, sizeof (info.path) - strlen (info.path) - 1);
-		snprintf (info.name, sizeof (info.name),
-			  _("Serial Port %i"), x);
+		gp_port_info_new (&info);
+		gp_port_info_set_type (info, GP_PORT_SERIAL);
+		C_MEM (xname = malloc (strlen("serial:")+strlen(path)+1));
+		strcpy (xname, "serial:");
+		strcat (xname, path);
+		gp_port_info_set_path (info, xname);
+		free (xname);
+		C_MEM (xname = malloc (100));
+		snprintf (xname, 100, _("Serial Port %i"), x);
+		gp_port_info_set_name (info, xname);
+		free (xname);
 		CHECK (gp_port_info_list_append (list, info));
-        }
+	}
 
 	/*
 	 * Generic support. Append it to the list without checking for
 	 * return values, because this entry will not be counted.
 	 */
-	info.type = GP_PORT_SERIAL;
-	strncpy (info.path, "^serial", sizeof (info.path));
-	memset (info.name, 0, sizeof (info.name));
-	gp_port_info_list_append (list, info);
+	gp_port_info_new (&info);
+	gp_port_info_set_type (info, GP_PORT_SERIAL);
+	gp_port_info_set_path (info, "serial:");
+	gp_port_info_set_name (info, _("Serial Port Device"));
+	CHECK (gp_port_info_list_append (list, info));
 
-        return (GP_OK);
+	gp_port_info_new (&info);
+	gp_port_info_set_type (info, GP_PORT_SERIAL);
+	gp_port_info_set_path (info, "^serial:");
+	gp_port_info_set_name (info, "");
+	gp_port_info_list_append (list, info); /* do not check */
+	return GP_OK;
 }
 
 static int
 gp_port_serial_init (GPPort *dev)
 {
-	if (!dev)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (dev);
 
-	dev->pl = malloc (sizeof (GPPortPrivateLibrary));
-	if (!dev->pl)
-		return (GP_ERROR_NO_MEMORY);
-	memset (dev->pl, 0, sizeof (GPPortPrivateLibrary));
+	C_MEM (dev->pl = calloc (1, sizeof (GPPortPrivateLibrary)));
 
 	/* There is no default speed. */
 	dev->pl->baudrate = -1;
 
-        return GP_OK;
+	return GP_OK;
 }
 
 static int
 gp_port_serial_exit (GPPort *dev)
 {
-	if (!dev)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (dev);
 
-	if (dev->pl) {
-		free (dev->pl);
-		dev->pl = NULL;
-	}
+	free (dev->pl);
+	dev->pl = NULL;
 
-        return GP_OK;
+	return GP_OK;
 }
 
 static int
@@ -435,9 +400,16 @@ gp_port_serial_open (GPPort *dev)
 	int fd;
 #endif
 	char *port;
+	GPPortInfo	info;
+
+	result = gp_port_get_info (dev, &info);
+	if (result < GP_OK) return result;
+	result = gp_port_info_get_path (info, &port);
+	if (result < GP_OK) return result;
+	GP_LOG_D ("opening %s", port);
 
 	/* Ports are named "serial:/dev/whatever/port" */
-	port = strchr (dev->settings.serial.port, ':');
+	port = strchr (port, ':');
 	if (!port)
 		return GP_ERROR_UNKNOWN_PORT;
 	port++;
@@ -448,35 +420,32 @@ gp_port_serial_open (GPPort *dev)
 			result = gp_port_serial_lock (dev, port);
 			if (result == GP_OK)
 				break;
-			gp_log (GP_LOG_DEBUG, "gphoto2-port-serial",
-				"Failed to get a lock, trying again...");
+			GP_LOG_D ("Failed to get a lock, trying again...");
 			sleep (1);
 		}
 		CHECK (result);
 	}
 	dev->pl->fd = -1;
 
-#if defined(__FreeBSD__) || defined(__APPLE__) || defined(__DragonFly__)
-        dev->pl->fd = open (port, O_RDWR | O_NOCTTY | O_NONBLOCK);
+#if defined(__FreeBSD__) || defined(__APPLE__) || defined(__DragonFly__) || defined(__MidnightBSD__)
+	dev->pl->fd = open (port, O_RDWR | O_NOCTTY | O_NONBLOCK);
 #elif OS2
-        fd = open (port, O_RDWR | O_BINARY);
+	fd = open (port, O_RDWR | O_BINARY);
 	dev->pl->fd = open (port, O_RDWR | O_BINARY);
-        close(fd);
+	close(fd);
 #else
-# ifdef HAVE_RESMGR
-        dev->pl->fd = rsm_open_device (port, O_RDWR | O_NOCTTY | O_SYNC | O_NONBLOCK);
-	/* fall back trying old style open if not possible */
-# endif
 	if (dev->pl->fd == -1)
 		dev->pl->fd = open (port, O_RDWR | O_NOCTTY | O_SYNC | O_NONBLOCK);
 #endif
-        if (dev->pl->fd == -1) {
-		gp_port_set_error (dev, _("Failed to open '%s' (%m)."), port);
+	if (dev->pl->fd == -1) {
+		int saved_errno = errno;
+		gp_port_set_error (dev, _("Failed to open '%s' (%s)."),
+				   port, strerror(saved_errno));
 		dev->pl->fd = 0;
 		return GP_ERROR_IO;
-        }
+	}
 
-        return GP_OK;
+	return GP_OK;
 }
 
 static int
@@ -489,8 +458,11 @@ gp_port_serial_close (GPPort *dev)
 
 	if (dev->pl->fd) {
 		if (close (dev->pl->fd) == -1) {
+			int saved_errno = errno;
 			gp_port_set_error (dev, _("Could not close "
-				"'%s' (%m)."), dev->settings.serial.port);
+						  "'%s' (%s)."),
+					   dev->settings.serial.port,
+					   strerror(saved_errno));
 	                return GP_ERROR_IO;
 	        }
 		dev->pl->fd = 0;
@@ -498,7 +470,7 @@ gp_port_serial_close (GPPort *dev)
 
 	/* Unlock the port */
 	path = strchr (dev->settings.serial.port, ':');
-	if (!path) return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (path);
 	path++;
 	CHECK (gp_port_serial_unlock (dev, path));
 
@@ -510,16 +482,15 @@ gp_port_serial_close (GPPort *dev)
 	dev->pl->baudrate = 0;
 #endif
 
-        return GP_OK;
+	return GP_OK;
 }
 
 static int
 gp_port_serial_write (GPPort *dev, const char *bytes, int size)
 {
-        int len, ret;
+	int len, ret;
 
-	if (!dev)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (dev);
 
 	/* The device needs to be opened for that operation */
 	if (!dev->pl->fd)
@@ -528,23 +499,25 @@ gp_port_serial_write (GPPort *dev, const char *bytes, int size)
 	/* Make sure we are operating at the specified speed */
 	CHECK (gp_port_serial_check_speed (dev));
 
-        len = 0;
+	len = 0;
         while (len < size) {
-		
+
 		/*
 		 * Make sure we write all data while handling
 		 * the harmless errors
 		 */
 		ret = write (dev->pl->fd, bytes, size - len);
-                if (ret == -1) {
-                        switch (errno) {
-                        case EAGAIN:
-                        case EINTR:
+		if (ret == -1) {
+			int saved_errno = errno;
+			switch (saved_errno) {
+			case EAGAIN:
+			case EINTR:
                                 ret = 0;
                                 break;
                         default:
 				gp_port_set_error (dev, _("Could not write "
-					"to port (%m)"));
+							  "to port (%s)"),
+						   strerror(saved_errno));
                                 return GP_ERROR_IO_WRITE;
                         }
 		}
@@ -570,8 +543,7 @@ gp_port_serial_read (GPPort *dev, char *bytes, int size)
         fd_set readfs;          /* file descriptor set */
         int readen = 0, now;
 
-	if (!dev)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (dev);
 
 	/* The device needs to be opened for that operation */
 	if (!dev->pl->fd)
@@ -587,7 +559,7 @@ gp_port_serial_read (GPPort *dev, char *bytes, int size)
 
 		/* Set timeout value within input loop */
                 timeout.tv_usec = (dev->timeout % 1000) * 1000;
-                timeout.tv_sec = (dev->timeout / 1000); 
+                timeout.tv_sec = (dev->timeout / 1000);
 
 		/* Any data available? */
                 if (!select (dev->pl->fd + 1, &readfs, NULL, NULL, &timeout))
@@ -613,7 +585,7 @@ gp_port_serial_read (GPPort *dev, char *bytes, int size)
 			 * as 0xff 0x00 sequence.
 			 *
 			 * 0xff sent by the camera are escaped as
-			 * 0xff 0xff sequence. 
+			 * 0xff 0xff sequence.
 			 *
 			 * All other 0xff 0xXX sequences are errors.
 			 *
@@ -681,16 +653,17 @@ gp_port_serial_get_pin (GPPort *dev, GPPin pin, GPLevel *level)
 	int j, bit;
 #endif
 
-	if (!dev || !level)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (dev && level);
 
 	*level = 0;
 
 #ifdef HAVE_TERMIOS_H
 	CHECK (get_termios_bit (dev, pin, &bit));
         if (ioctl (dev->pl->fd, TIOCMGET, &j) < 0) {
+		int saved_errno = errno;
 		gp_port_set_error (dev, _("Could not get level of pin %i "
-				   "(%m)."), pin);
+					  "(%s)."),
+				   pin, strerror(saved_errno));
                 return GP_ERROR_IO;
         }
         *level = j & bit;
@@ -710,8 +683,7 @@ gp_port_serial_set_pin (GPPort *dev, GPPin pin, GPLevel level)
         int bit, request;
 #endif
 
-	if (!dev)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (dev);
 
 #ifdef HAVE_TERMIOS_H
 	CHECK (get_termios_bit (dev, pin, &bit));
@@ -724,8 +696,10 @@ gp_port_serial_set_pin (GPPort *dev, GPPin pin, GPLevel level)
 		break;
         }
         if (ioctl (dev->pl->fd, request, &bit) < 0) {
+		int saved_errno = errno;
 		gp_port_set_error (dev, _("Could not set level of pin %i to "
-				   "%i (%m)."), pin, level);
+					  "%i (%s)."),
+				   pin, level, strerror(saved_errno));
 		return GP_ERROR_IO;
 	}
 #else
@@ -749,8 +723,10 @@ gp_port_serial_flush (GPPort *dev, int direction)
 
 #ifdef HAVE_TERMIOS_H
 	if (tcflush (dev->pl->fd, direction ? TCOFLUSH : TCIFLUSH) < 0) {
-		gp_port_set_error (dev, _("Could not flush '%s' (%m)."),
-			dev->settings.serial.port);
+		int saved_errno = errno;
+		gp_port_set_error (dev, _("Could not flush '%s' (%s)."),
+				   dev->settings.serial.port,
+				   strerror(saved_errno));
 		return (GP_ERROR_IO);
 	}
 #else
@@ -809,8 +785,7 @@ gp_port_serial_baudconv (int baudrate)
 #endif
         default:
 		ret = (speed_t) baudrate;
-		gp_log (GP_LOG_DEBUG, "gphoto2-port-serial", "Baudrate %d "
-			"unknown - using as is", baudrate);
+		GP_LOG_D ("Baudrate %d unknown - using as is", baudrate);
         }
 
         return ret;
@@ -843,8 +818,7 @@ gp_port_serial_check_speed (GPPort *dev)
 	if (dev->pl->baudrate == dev->settings.serial.speed)
 		return (GP_OK);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-serial",
-		"Setting baudrate to %d...", dev->settings.serial.speed);
+	GP_LOG_D ("Setting baudrate to %d...", dev->settings.serial.speed);
 	speed = gp_port_serial_baudconv (dev->settings.serial.speed);
 
 #ifdef OS2
@@ -896,13 +870,13 @@ gp_port_serial_check_speed (GPPort *dev)
 	cfsetispeed (&tio, speed);
 	cfsetospeed (&tio, speed);
         if (tcsetattr (dev->pl->fd, TCSANOW, &tio) < 0) {
-		GP_DEBUG ("Error on 'tcsetattr'.");
+		GP_LOG_E ("Error on 'tcsetattr'.");
                 return GP_ERROR_IO_SERIAL_SPEED;
         }
 
 	/* Clear O_NONBLOCK. */
         if (fcntl (dev->pl->fd, F_SETFL, 0) < 0) {
-		GP_DEBUG ("Error on 'fcntl'.");
+		GP_LOG_E ("Error on 'fcntl'.");
                 return GP_ERROR_IO_SERIAL_SPEED;
         }
 
@@ -915,12 +889,12 @@ gp_port_serial_check_speed (GPPort *dev)
 	 */
 	if (speed != B0) {
 		if (tcgetattr (dev->pl->fd, &tio)) {
-			GP_DEBUG ("Error on 'tcgetattr'.");
+			GP_LOG_E ("Error on 'tcgetattr'.");
 			return (GP_ERROR_IO_SERIAL_SPEED);
 		}
 		if ((cfgetispeed (&tio) != speed) ||
 		    (cfgetospeed (&tio) != speed)) {
-			GP_DEBUG ("Cannot set baudrate to %d...",
+			GP_LOG_E ("Cannot set baudrate to %d...",
 				  dev->settings.serial.speed);
 			return (GP_ERROR_NOT_SUPPORTED);
 		}
@@ -987,7 +961,7 @@ gp_port_library_operations ()
         ops = malloc (sizeof (GPPortOperations));
 	if (!ops)
 		return (NULL);
-        memset (ops, 0, sizeof (GPPortOperations)); 
+        memset (ops, 0, sizeof (GPPortOperations));
 
         ops->init   = gp_port_serial_init;
         ops->exit   = gp_port_serial_exit;
