@@ -1,29 +1,31 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* gphoto2-port-usb.c
  *
- * Copyright  2001 Lutz Mller <lutz@users.sf.net>
- * Copyright  1999-2000 Johannes Erdfelt <johannes@erdfelt.com>
+ * Copyright 2001 Lutz Mueller <lutz@users.sf.net>
+ * Copyright 1999-2000 Johannes Erdfelt <johannes@erdfelt.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
+
+#define _DEFAULT_SOURCE
+
 #include "config.h"
 #include <gphoto2/gphoto2-port-library.h>
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -34,6 +36,10 @@
 
 #include <usb.h>
 
+#ifndef ENODATA
+#	define ENODATA     120     /* No data available */
+#endif
+
 #include <gphoto2/gphoto2-port.h>
 #include <gphoto2/gphoto2-port-result.h>
 #include <gphoto2/gphoto2-port-log.h>
@@ -84,19 +90,17 @@ gp_port_library_list (GPPortInfoList *list)
 	struct usb_device *dev;
 	int nrofdevices = 0, i, i1, i2, unknownint;
 
-	/* default port first */
-	info.type = GP_PORT_USB;
 	/* generic matcher. This will catch passed XXX,YYY entries for instance. */
-	memset (info.name, 0, sizeof(info.name));
-	strcpy (info.path, "^usb:");
-	CHECK (gp_port_info_list_append (list, info));
+	gp_port_info_new (&info);
+	gp_port_info_set_type (info, GP_PORT_USB);
+	gp_port_info_set_name (info, "");
+	gp_port_info_set_path (info, "^usb:");
+	gp_port_info_list_append (list, info); /* do not check here */
 
 	usb_init ();
 	usb_find_busses ();
 	usb_find_devices ();
 
-	strcpy (info.name, "Universal Serial Bus");
-
 	bus = usb_get_busses();
 
 	/* Look and enumerate all USB ports. */
@@ -106,7 +110,8 @@ gp_port_library_list (GPPortInfoList *list)
 			if (	(dev->descriptor.bDeviceClass == USB_CLASS_HUB)		||
 				(dev->descriptor.bDeviceClass == USB_CLASS_HID)		||
 				(dev->descriptor.bDeviceClass == USB_CLASS_PRINTER)	||
-				(dev->descriptor.bDeviceClass == USB_CLASS_COMM)
+				(dev->descriptor.bDeviceClass == USB_CLASS_COMM)	||
+				(dev->descriptor.bDeviceClass == 0xe0)	/* wireless / bluetooth */
 			)
 				continue;
 			/* excepts HUBs, usually the interfaces have the classes, not
@@ -119,10 +124,12 @@ gp_port_library_list (GPPortInfoList *list)
 				}
 				for (i1 = 0; i1 < dev->config[i].bNumInterfaces; i1++)
 					for (i2 = 0; i2 < dev->config[i].interface[i1].num_altsetting; i2++) {
-						struct usb_interface_descriptor *intf = &dev->config[i].interface[i1].altsetting[i2]; 
+						struct usb_interface_descriptor *intf = &dev->config[i].interface[i1].altsetting[i2];
 						if (	(intf->bInterfaceClass == USB_CLASS_HID)	||
 							(intf->bInterfaceClass == USB_CLASS_PRINTER)	||
-							(intf->bInterfaceClass == USB_CLASS_COMM))
+							(intf->bInterfaceClass == USB_CLASS_COMM)	||
+							(intf->bInterfaceClass == 0xe0)	/* wireless/bluetooth*/
+						)
 							continue;
 						unknownint++;
 					}
@@ -132,18 +139,19 @@ gp_port_library_list (GPPortInfoList *list)
 				continue;
 			/* Note: We do not skip USB storage. Some devices can support both,
 			 * and the Ricoh erronously reports it.
-			 */ 
+			 */
 			nrofdevices++;
 		}
 		bus = bus->next;
 	}
 
 #if 0
-	/* Do not filter out usb:XXX,YYY, we want it for consistency... */
 	/* If we already added usb:, and have 0 or 1 devices we have nothing to do.
 	 * This should be the standard use case.
 	 */
-	if (nrofdevices <= 1) 
+	/* We never want to return just "usb:" ... also return "usb:XXX,YYY", and
+	 * let upper layers filter out the usb: */
+	if (nrofdevices <= 1)
 		return (GP_OK);
 #endif
 
@@ -154,6 +162,7 @@ gp_port_library_list (GPPortInfoList *list)
 	while (bus) {
 		for (dev = bus->devices; dev; dev = dev->next) {
 			char *s;
+			char path[200];
 			/* Devices which are definitely not cameras. */
 			if (	(dev->descriptor.bDeviceClass == USB_CLASS_HUB)		||
 				(dev->descriptor.bDeviceClass == USB_CLASS_HID)		||
@@ -171,7 +180,7 @@ gp_port_library_list (GPPortInfoList *list)
 				}
 				for (i1 = 0; i1 < dev->config[i].bNumInterfaces; i1++)
 					for (i2 = 0; i2 < dev->config[i].interface[i1].num_altsetting; i2++) {
-						struct usb_interface_descriptor *intf = &dev->config[i].interface[i1].altsetting[i2]; 
+						struct usb_interface_descriptor *intf = &dev->config[i].interface[i1].altsetting[i2];
 						if (	(intf->bInterfaceClass == USB_CLASS_HID)	||
 							(intf->bInterfaceClass == USB_CLASS_PRINTER)	||
 							(intf->bInterfaceClass == USB_CLASS_COMM))
@@ -184,10 +193,14 @@ gp_port_library_list (GPPortInfoList *list)
 				continue;
 			/* Note: We do not skip USB storage. Some devices can support both,
 			 * and the Ricoh erronously reports it.
-			 */ 
-			sprintf (info.path, "usb:%s,%s", bus->dirname, dev->filename);
+			 */
+			gp_port_info_new (&info);
+			gp_port_info_set_type (info, GP_PORT_USB);
+			gp_port_info_set_name (info, "Universal Serial Bus");
+			snprintf (path,sizeof(path), "usb:%s,%s", bus->dirname, dev->filename);
 			/* On MacOS X we might get usb:006,002-04a9-3139-00-00. */
-			s = strchr(info.path, '-');if (s) *s='\0';
+			s = strchr(path, '-');if (s) *s='\0';
+			gp_port_info_set_path (info, path);
 			CHECK (gp_port_info_list_append (list, info));
 		}
 		bus = bus->next;
@@ -195,8 +208,10 @@ gp_port_library_list (GPPortInfoList *list)
 	/* This will only be added if no other device was ever added.
 	 * Users doing "usb:" usage will enter the regular expression matcher case. */
 	if (nrofdevices == 0) {
-		strcpy (info.name, "Universal Serial Bus");
-		strcpy (info.path, "usb:");
+		gp_port_info_new (&info);
+		gp_port_info_set_type (info, GP_PORT_USB);
+		gp_port_info_set_name (info, "Universal Serial Bus");
+		gp_port_info_set_path (info, "usb:");
 		CHECK (gp_port_info_list_append (list, info));
 	}
 	return (GP_OK);
@@ -204,10 +219,7 @@ gp_port_library_list (GPPortInfoList *list)
 
 static int gp_port_usb_init (GPPort *port)
 {
-	port->pl = malloc (sizeof (GPPortPrivateLibrary));
-	if (!port->pl)
-		return (GP_ERROR_NO_MEMORY);
-	memset (port->pl, 0, sizeof (GPPortPrivateLibrary));
+	C_MEM (port->pl = calloc (1, sizeof (GPPortPrivateLibrary)));
 
 	port->pl->config = port->pl->interface = port->pl->altsetting = -1;
 
@@ -221,23 +233,26 @@ static int gp_port_usb_init (GPPort *port)
 static int
 gp_port_usb_exit (GPPort *port)
 {
-	if (port->pl) {
-		free (port->pl);
-		port->pl = NULL;
-	}
+	free (port->pl);
+	port->pl = NULL;
 
 	return (GP_OK);
 }
 
+static int gp_port_usb_find_path_lib(GPPort *);
+
 static int
 gp_port_usb_open (GPPort *port)
 {
 	int ret;
 	char name[64];
 
-	gp_log (GP_LOG_DEBUG,"libusb","gp_port_usb_open()");
-	if (!port || !port->pl->d)
-		return GP_ERROR_BAD_PARAMETERS;
+	GP_LOG_D ("gp_port_usb_open(%p)", port);
+	C_PARAMS (port);
+	if (!port->pl->d) {
+		ret = gp_port_usb_find_path_lib(port);
+		C_PARAMS (port->pl->d);
+	}
 
         /*
 	 * Open the device using the previous usb_handle returned by
@@ -245,7 +260,9 @@ gp_port_usb_open (GPPort *port)
 	 */
 	port->pl->dh = usb_open (port->pl->d);
 	if (!port->pl->dh) {
-		gp_port_set_error (port, _("Could not open USB device (%m)."));
+		int saved_errno = errno;
+		gp_port_set_error (port, _("Could not open USB device (%s)."),
+				   strerror(saved_errno));
 		return GP_ERROR_IO;
 	}
 #if defined(LIBUSB_HAS_GET_DRIVER_NP) && defined(LIBUSB_HAS_DETACH_KERNEL_DRIVER_NP)
@@ -253,16 +270,16 @@ gp_port_usb_open (GPPort *port)
 	ret = usb_get_driver_np (port->pl->dh, port->settings.usb.interface,
 		name, sizeof(name)
 	);
+	if (name[0] != 0)
+		GP_LOG_D ("Device has driver '%s' attached.", name);
 	if (strstr(name,"usbfs") || strstr(name,"storage")) {
-		/* other gphoto instance most likely
-		 * Also let mass storage be served by regular means.
-		 */
+		/* other gphoto instance most likely */
 		gp_port_set_error (port, _("Camera is already in use."));
 		return GP_ERROR_IO_LOCK;
 	}
 
 	if (ret >= 0) {
-		gp_log (GP_LOG_DEBUG,"libusb",_("Device has driver '%s' attached, detaching it now."), name);
+		GP_LOG_D ("Device has driver '%s' attached, detaching it now.", name);
 		ret = usb_detach_kernel_driver_np (port->pl->dh, port->settings.usb.interface);
 		if (ret < 0)
 			gp_port_set_error (port, _("Could not detach kernel driver '%s' of camera device."),name);
@@ -274,15 +291,29 @@ gp_port_usb_open (GPPort *port)
 	}
 #endif
 
-	gp_log (GP_LOG_DEBUG,"libusb","claiming interface %d", port->settings.usb.interface);
+	GP_LOG_D ("claiming interface %d", port->settings.usb.interface);
 	ret = usb_claim_interface (port->pl->dh,
 				   port->settings.usb.interface);
 	if (ret < 0) {
-		gp_port_set_error (port, _("Could not claim "
-			"interface %d (%m). Make sure no other program "
-			"or kernel module (such as %s) is using the device "
-			"and you have read/write access to the device."),
-			port->settings.usb.interface, "sdc2xx, stv680, spca50x");
+		int saved_errno = errno;
+		gp_port_set_error (port, _("Could not claim interface %d (%s). "
+					   "Make sure no other program (%s) "
+					   "or kernel module (such as %s) "
+					   "is using the device and you have "
+					   "read/write access to the device."
+					),
+				   port->settings.usb.interface,
+				   strerror(saved_errno),
+#ifdef __linux__
+				   "gvfs-gphoto2-volume-monitor",
+#else
+#if defined(__APPLE__)
+				   N_("MacOS PTPCamera service"),
+#else
+				   N_("unknown libgphoto2 using program"),
+#endif
+#endif
+				   "sdc2xx, stv680, spca50x");
 		return GP_ERROR_IO_USB_CLAIM;
 	}
 	return GP_OK;
@@ -291,25 +322,29 @@ gp_port_usb_open (GPPort *port)
 static int
 gp_port_usb_close (GPPort *port)
 {
-	if (!port || !port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (port && port->pl->dh);
 
 	if (usb_release_interface (port->pl->dh,
 				   port->settings.usb.interface) < 0) {
-		gp_port_set_error (port, _("Could not "
-			"release interface %d (%m)."),
-			port->settings.usb.interface);
+		int saved_errno = errno;
+		gp_port_set_error (port, _("Could not release "
+					   "interface %d (%s)."),
+				   port->settings.usb.interface,
+				   strerror(saved_errno));
 		return (GP_ERROR_IO);
 	}
 
 #if 0
-	/* This does break Canon EOS and some PowerShots ... -Marcus */
+	/* This confuses the EOS 5d camera and possible other EOSs. *sigh* */
 	/* This is only for our very special Canon cameras which need a good
 	 * whack after close, otherwise they get timeouts on reconnect.
 	 */
 	if (port->pl->d->descriptor.idVendor == 0x04a9) {
 		if (usb_reset (port->pl->dh) < 0) {
-			gp_port_set_error (port, _("Could not reset USB port (%m)."));
+			int saved_errno = errno;
+			gp_port_set_error (port, _("Could not reset "
+						   "USB port (%s)."),
+					   strerror(saved_errno));
 			return (GP_ERROR_IO);
 		}
 	}
@@ -329,14 +364,16 @@ gp_port_usb_close (GPPort *port)
 			command.ioctl_code = USBDEVFS_CONNECT;
 			command.data = NULL;
 			if (ioctl(fd, USBDEVFS_IOCTL, &command))
-				gp_log (GP_LOG_DEBUG,"libusb","reattach kernel driver failed");
+				GP_LOG_D ("reattach kernel driver failed");
 			close(fd);
 		}
 	}
 #endif
 
 	if (usb_close (port->pl->dh) < 0) {
-		gp_port_set_error (port, _("Could not close USB port (%m)."));
+		int saved_errno = errno;
+		gp_port_set_error (port, _("Could not close USB port (%s)."),
+				   strerror(saved_errno));
 		return (GP_ERROR_IO);
 	}
 
@@ -350,8 +387,7 @@ gp_port_usb_clear_halt_lib(GPPort *port, int ep)
 {
 	int ret=0;
 
-	if (!port || !port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (port && port->pl->dh);
 
 	switch (ep) {
 	case GP_PORT_USB_ENDPOINT_IN :
@@ -376,8 +412,7 @@ gp_port_usb_write (GPPort *port, const char *bytes, int size)
 {
         int ret;
 
-	if (!port || !port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+        C_PARAMS (port && port->pl->dh);
 
 	ret = usb_bulk_write (port->pl->dh, port->settings.usb.outep,
                            (char *) bytes, size, port->timeout);
@@ -392,8 +427,7 @@ gp_port_usb_read(GPPort *port, char *bytes, int size)
 {
 	int ret;
 
-	if (!port || !port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (port && port->pl->dh);
 
 	ret = usb_bulk_read(port->pl->dh, port->settings.usb.inep,
 			     bytes, size, port->timeout);
@@ -403,14 +437,30 @@ gp_port_usb_read(GPPort *port, char *bytes, int size)
         return ret;
 }
 
+static int
+gp_port_usb_reset(GPPort *port)
+{
+	int ret;
+
+	GP_LOG_D ("Reseting port");
+	C_PARAMS (port && port->pl->dh);
+
+	ret = usb_reset(port->pl->dh);
+        if (ret < 0) {
+		GP_LOG_E ("gp_port_reset: %d", ret);
+		return GP_ERROR_IO_READ;
+	}
+        return GP_OK;
+}
+
 static int
 gp_port_usb_check_int (GPPort *port, char *bytes, int size, int timeout)
 {
 	int ret;
 
-	if (!port || !port->pl->dh || timeout < 0)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (port && port->pl->dh && timeout >= 0);
 
+	/* 0 timeout is okish ... libusb 0.x it will do 1 poll though */
 	ret = usb_interrupt_read(port->pl->dh, port->settings.usb.intep,
 			     bytes, size, timeout);
         if (ret < 0) {
@@ -425,8 +475,7 @@ static int
 gp_port_usb_msg_write_lib(GPPort *port, int request, int value, int index,
 	char *bytes, int size)
 {
-	if (!port || !port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (port && port->pl->dh);
 
 	return usb_control_msg(port->pl->dh,
 		USB_TYPE_VENDOR | USB_RECIP_DEVICE,
@@ -437,69 +486,64 @@ static int
 gp_port_usb_msg_read_lib(GPPort *port, int request, int value, int index,
 	char *bytes, int size)
 {
-	if (!port || !port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (port && port->pl->dh);
 
 	return usb_control_msg(port->pl->dh,
 		USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN,
 		request, value, index, bytes, size, port->timeout);
 }
 
-/* The next two functions support the nonstandard request types 0x41 (write) 
- * and 0xc1 (read), which are occasionally needed. 
+/* The next two functions support the nonstandard request types 0x41 (write)
+ * and 0xc1 (read), which are occasionally needed.
  */
 
 static int
-gp_port_usb_msg_interface_write_lib(GPPort *port, int request, 
+gp_port_usb_msg_interface_write_lib(GPPort *port, int request,
 	int value, int index, char *bytes, int size)
 {
-	if (!port || !port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (port && port->pl->dh);
 
-	return usb_control_msg(port->pl->dh, 
+	return usb_control_msg(port->pl->dh,
 		USB_TYPE_VENDOR | USB_RECIP_INTERFACE,
 		request, value, index, bytes, size, port->timeout);
 }
 
 
 static int
-gp_port_usb_msg_interface_read_lib(GPPort *port, int request, 
+gp_port_usb_msg_interface_read_lib(GPPort *port, int request,
 	int value, int index, char *bytes, int size)
 {
-	if (!port || !port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (port && port->pl->dh);
 
-	return usb_control_msg(port->pl->dh, 
+	return usb_control_msg(port->pl->dh,
 		USB_TYPE_VENDOR | USB_RECIP_INTERFACE | USB_ENDPOINT_IN,
 		request, value, index, bytes, size, port->timeout);
 }
 
 
-/* The next two functions support the nonstandard request types 0x21 (write) 
- * and 0xa1 (read), which are occasionally needed. 
+/* The next two functions support the nonstandard request types 0x21 (write)
+ * and 0xa1 (read), which are occasionally needed.
  */
 
 static int
-gp_port_usb_msg_class_write_lib(GPPort *port, int request, 
+gp_port_usb_msg_class_write_lib(GPPort *port, int request,
 	int value, int index, char *bytes, int size)
 {
-	if (!port || !port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (port && port->pl->dh);
 
-	return usb_control_msg(port->pl->dh, 
+	return usb_control_msg(port->pl->dh,
 		USB_TYPE_CLASS | USB_RECIP_INTERFACE,
 		request, value, index, bytes, size, port->timeout);
 }
 
 
 static int
-gp_port_usb_msg_class_read_lib(GPPort *port, int request, 
+gp_port_usb_msg_class_read_lib(GPPort *port, int request,
 	int value, int index, char *bytes, int size)
 {
-	if (!port || !port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (port && port->pl->dh);
 
-	return usb_control_msg(port->pl->dh, 
+	return usb_control_msg(port->pl->dh,
 		USB_TYPE_CLASS | USB_RECIP_INTERFACE | USB_ENDPOINT_IN,
 		request, value, index, bytes, size, port->timeout);
 }
@@ -519,16 +563,18 @@ gp_port_usb_update (GPPort *port)
 {
 	int ret, ifacereleased = FALSE;
 
-	gp_log (GP_LOG_DEBUG, "libusb", "gp_port_usb_update(old int=%d, conf=%d, alt=%d), (new int=%d, conf=%d, alt=%d)",
+	C_PARAMS (port);
+
+	GP_LOG_D ("gp_port_usb_update(old int=%d, conf=%d, alt=%d) port %s, (new int=%d, conf=%d, alt=%d), port %s",
 		port->settings.usb.interface,
 		port->settings.usb.config,
 		port->settings.usb.altsetting,
+		port->settings.usb.port,
 		port->settings_pending.usb.interface,
 		port->settings_pending.usb.config,
-		port->settings_pending.usb.altsetting
+		port->settings_pending.usb.altsetting,
+		port->settings_pending.usb.port
 	);
-	if (!port)
-		return GP_ERROR_BAD_PARAMETERS;
 
 #if 0
 	if (port->pl->interface == -1) port->pl->interface = port->settings.usb.interface;
@@ -540,44 +586,44 @@ gp_port_usb_update (GPPort *port)
 	memcpy(&port->settings.usb.port, &port->settings_pending.usb.port,
 		sizeof(port->settings.usb.port));
 
- 	if (!port->pl->dh)
-		return GP_ERROR_BAD_PARAMETERS;
+	if (!port->pl->dh)
+		return GP_OK; /* the port might not be opened, yet. that is ok */
 
 	memcpy(&port->settings.usb, &port->settings_pending.usb,
 		sizeof(port->settings.usb));
 
 	/* The interface changed. release the old, claim the new ... */
 	if (port->settings.usb.interface != port->pl->interface) {
-		gp_log (GP_LOG_DEBUG, "libusb", "changing interface %d -> %d",
+		GP_LOG_D ("changing interface %d -> %d",
 			port->pl->interface,
 			port->settings.usb.interface
 		);
 		if (usb_release_interface (port->pl->dh,
 					   port->pl->interface) < 0) {
-			gp_log (GP_LOG_DEBUG, "gphoto2-port-usb","releasing the iface for config failed.");
+			GP_LOG_D ("releasing the iface for config failed.");
 			/* Not a hard error for now. -Marcus */
 		} else {
-			gp_log (GP_LOG_DEBUG,"libusb","claiming interface %d", port->settings.usb.interface);
+			GP_LOG_D ("claiming interface %d", port->settings.usb.interface);
 			ret = usb_claim_interface (port->pl->dh,
 						   port->settings.usb.interface);
 			if (ret < 0) {
-				gp_log (GP_LOG_DEBUG, "gphoto2-port-usb","reclaiming the iface for config failed.");
+				GP_LOG_D ("reclaiming the iface for config failed.");
 				return GP_ERROR_IO_UPDATE;
 			}
 			port->pl->interface = port->settings.usb.interface;
 		}
 	}
 	if (port->settings.usb.config != port->pl->config) {
-		gp_log (GP_LOG_DEBUG, "libusb", "changing config %d -> %d",
+		GP_LOG_D ("changing config %d -> %d",
 			port->pl->config,
 			port->settings.usb.config
 		);
-		/* This can only be changed with the interface released. 
+		/* This can only be changed with the interface released.
 		 * This is a hard requirement since 2.6.12.
 		 */
 		if (usb_release_interface (port->pl->dh,
 					   port->settings.usb.interface) < 0) {
-			gp_log (GP_LOG_DEBUG, "gphoto2-port-usb","releasing the iface for config failed.");
+			GP_LOG_D ("releasing the iface for config failed.");
 			ifacereleased = FALSE;
 		} else {
 			ifacereleased = TRUE;
@@ -586,27 +632,25 @@ gp_port_usb_update (GPPort *port)
 				     port->settings.usb.config);
 		if (ret < 0) {
 #if 0 /* setting the configuration failure is not fatal */
+			int saved_errno = errno;
 			gp_port_set_error (port,
-				_("Could not set config %d/%d (%m)"),
-				port->settings.usb.interface,
-				port->settings.usb.config);
-			return GP_ERROR_IO_UPDATE;	
+					   _("Could not set config %d/%d (%s)"),
+					   port->settings.usb.interface,
+					   port->settings.usb.config,
+					   strerror(saved_errno));
+			return GP_ERROR_IO_UPDATE;
 #endif
-			gp_log (GP_LOG_ERROR, "gphoto2-port-usb","setting configuration from %d to %d failed with ret = %d, but continue...", port->pl->config, port->settings.usb.config, ret);
+			GP_LOG_E ("setting configuration from %d to %d failed with ret = %d, but continue...", port->pl->config, port->settings.usb.config, ret);
 		}
 
-		gp_log (GP_LOG_DEBUG, "gphoto2-port-usb",
-			"Changed usb.config from %d to %d",
-			port->pl->config,
-			port->settings.usb.config);
+		GP_LOG_D ("Changed usb.config from %d to %d", port->pl->config, port->settings.usb.config);
 
 		if (ifacereleased) {
-			gp_log (GP_LOG_DEBUG,"libusb","claiming interface %d", port->settings.usb.interface);
+			GP_LOG_D ("claiming interface %d", port->settings.usb.interface);
 			ret = usb_claim_interface (port->pl->dh,
 						   port->settings.usb.interface);
-			if (ret < 0) {
-				gp_log (GP_LOG_DEBUG, "gphoto2-port-usb","reclaiming the iface for config failed.");
-			}
+			if (ret < 0)
+				GP_LOG_D ("reclaiming the iface for config failed.");
 		}
 		/*
 		 * Copy at once if something else fails so that this
@@ -619,18 +663,17 @@ gp_port_usb_update (GPPort *port)
 	if (port->settings.usb.altsetting != port->pl->altsetting) {
 		ret = usb_set_altinterface(port->pl->dh, port->settings.usb.altsetting);
 		if (ret < 0) {
-			gp_port_set_error (port, 
-				_("Could not set altsetting from %d "
-				"to %d (%m)"),
-				port->pl->altsetting,
-				port->settings.usb.altsetting);
+			int saved_errno = errno;
+			gp_port_set_error (port,
+					   _("Could not set altsetting from %d "
+					     "to %d (%s)"),
+					   port->pl->altsetting,
+					   port->settings.usb.altsetting,
+					   strerror(saved_errno));
 			return GP_ERROR_IO_UPDATE;
 		}
 
-		gp_log (GP_LOG_DEBUG, "gphoto2-port-usb",
-			"Changed usb.altsetting from %d to %d",
-			port->pl->altsetting,
-			port->settings.usb.altsetting);
+		GP_LOG_D ("Changed usb.altsetting from %d to %d", port->pl->altsetting, port->settings.usb.altsetting);
 		port->pl->altsetting = port->settings.usb.altsetting;
 	}
 
@@ -687,8 +730,7 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 	char *s;
 	char busname[64], devname[64];
 
-	if (!port)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (port);
 
 	s = strchr (port->settings.usb.port,':');
 	busname[0] = devname[0] = '\0';
@@ -733,10 +775,7 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 
 				port->pl->d = dev;
 
-				gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
-					"Looking for USB device "
-					"(vendor 0x%x, product 0x%x)... found.", 
-					idvendor, idproduct);
+				GP_LOG_D ("Looking for USB device (vendor 0x%x, product 0x%x)... found.", idvendor, idproduct);
 
 				/* Use the first config, interface and altsetting we find */
 				gp_port_usb_find_first_altsetting(dev, &config, &interface, &altsetting);
@@ -747,11 +786,9 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 
 					if (dev->config[config].interface[interface].altsetting[altsetting].bInterfaceClass
 					    == USB_CLASS_MASS_STORAGE) {
-						gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
-							_("USB device (vendor 0x%x, product 0x%x) is a mass"
+						GP_LOG_D ("USB device (vendor 0x%x, product 0x%x) is a mass"
 							  " storage device, and might not function with gphoto2."
-							  " Reference: %s"),
-							idvendor, idproduct, URL_USB_MASSSTORAGE);
+							  " Reference: %s", idvendor, idproduct, URL_USB_MASSSTORAGE);
 					}
 					port->settings.usb.config = dev->config[config].bConfigurationValue;
 					port->settings.usb.interface = dev->config[config].interface[interface].altsetting[altsetting].bInterfaceNumber;
@@ -762,18 +799,15 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 					port->settings.usb.intep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_IN, USB_ENDPOINT_TYPE_INTERRUPT);
 
 					port->settings.usb.maxpacketsize = 0;
-					gp_log (GP_LOG_DEBUG, "gphoto2-port-usb", "inep to look for is %02x", port->settings.usb.inep);
+					GP_LOG_D ("inep to look for is %02x", port->settings.usb.inep);
 					for (i=0;i<dev->config[config].interface[interface].altsetting[altsetting].bNumEndpoints;i++) {
 						if (port->settings.usb.inep == dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].bEndpointAddress) {
 							port->settings.usb.maxpacketsize = dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].wMaxPacketSize;
 							break;
 						}
 					}
-					gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
-						"Detected defaults: config %d, "
-						"interface %d, altsetting %d, "
-						"inep %02x, outep %02x, intep %02x, "
-						"class %02x, subclass %02x",
+					GP_LOG_D ("Detected defaults: config %d, interface %d, altsetting %d, "
+						  "inep %02x, outep %02x, intep %02x, class %02x, subclass %02x",
 						port->settings.usb.config,
 						port->settings.usb.interface,
 						port->settings.usb.altsetting,
@@ -798,11 +832,98 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 	return GP_ERROR_IO_USB_FIND;
 }
 
+static int
+gp_port_usb_find_path_lib(GPPort *port)
+{
+	struct usb_bus *bus;
+	struct usb_device *dev;
+	char *s;
+	char busname[64], devname[64];
+
+	C_PARAMS (port);
+
+	s = strchr (port->settings.usb.port,':');
+	busname[0] = devname[0] = '\0';
+	C_PARAMS (s && s[1] != '\0'); /* generic usb: match ... can't do */
+
+	if (s && (s[1] != '\0')) { /* usb:%d,%d */
+		strncpy(busname,s+1,sizeof(busname));
+		busname[sizeof(busname)-1] = '\0';
+
+		s = strchr(busname,',');
+		C_PARAMS (s); /* generic usb: match ... can't do */
+
+		strncpy(devname, s+1,sizeof(devname));
+		devname[sizeof(devname)-1] = '\0';
+		*s = '\0';
+	}
+	for (bus = usb_busses; bus; bus = bus->next) {
+		if (strcmp(busname, bus->dirname))
+			continue;
+
+		for (dev = bus->devices; dev; dev = dev->next) {
+			int config = -1, interface = -1, altsetting = -1;
+
+			if ((dev->filename != strstr(dev->filename, devname)))
+				continue;
+
+			port->pl->d = dev;
+
+			GP_LOG_D ("Found device for path %s", port->settings.usb.port);
+
+			/* Use the first config, interface and altsetting we find */
+			gp_port_usb_find_first_altsetting(dev, &config, &interface, &altsetting);
+
+			/* Set the defaults */
+			if (dev->config) {
+				int i;
+
+				port->settings.usb.config = dev->config[config].bConfigurationValue;
+				port->settings.usb.interface = dev->config[config].interface[interface].altsetting[altsetting].bInterfaceNumber;
+				port->settings.usb.altsetting = dev->config[config].interface[interface].altsetting[altsetting].bAlternateSetting;
+
+				port->settings.usb.inep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_IN, USB_ENDPOINT_TYPE_BULK);
+				port->settings.usb.outep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_OUT, USB_ENDPOINT_TYPE_BULK);
+				port->settings.usb.intep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_IN, USB_ENDPOINT_TYPE_INTERRUPT);
+
+				port->settings.usb.maxpacketsize = 0;
+				GP_LOG_D ("inep to look for is %02x", port->settings.usb.inep);
+				for (i=0;i<dev->config[config].interface[interface].altsetting[altsetting].bNumEndpoints;i++) {
+					if (port->settings.usb.inep == dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].bEndpointAddress) {
+						port->settings.usb.maxpacketsize = dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].wMaxPacketSize;
+						break;
+					}
+				}
+				GP_LOG_D ("Detected defaults: config %d, interface %d, altsetting %d, "
+					  "inep %02x, outep %02x, intep %02x, class %02x, subclass %02x",
+					port->settings.usb.config,
+					port->settings.usb.interface,
+					port->settings.usb.altsetting,
+					port->settings.usb.inep,
+					port->settings.usb.outep,
+					port->settings.usb.intep,
+					dev->config[config].interface[interface].altsetting[altsetting].bInterfaceClass,
+					dev->config[config].interface[interface].altsetting[altsetting].bInterfaceSubClass
+					);
+			}
+			return GP_OK;
+		}
+	}
+
+#if 0
+	gp_port_set_error (port, _("Could not find USB device "
+		"(vendor 0x%x, product 0x%x). Make sure this device "
+		"is connected to the computer."), idvendor, idproduct);
+#endif
+	return GP_ERROR_IO_USB_FIND;
+}
+
 /* This function reads the Microsoft OS Descriptor and looks inside to
  * find if it is a MTP device. This is the similar to the way that
  * Windows Media Player 10 uses.
  * It is documented to some degree on various internet pages.
  */
+#if 0
 static int
 gp_port_usb_match_mtp_device(struct usb_device *dev,int *configno, int *interfaceno, int *altsettingno)
 {
@@ -815,8 +936,8 @@ gp_port_usb_match_mtp_device(struct usb_device *dev,int *configno, int *interfac
 	if ((dev->descriptor.bDeviceClass!=0xff) && (dev->descriptor.bDeviceClass!=0))
 		return 0;
 #endif
+	xifaces = xnocamifaces = 0;
 	if (dev->config) {
-		xifaces = xnocamifaces = 0;
 		for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
 			unsigned int j;
 
@@ -825,7 +946,7 @@ gp_port_usb_match_mtp_device(struct usb_device *dev,int *configno, int *interfac
 				xifaces++;
 
 				for (k = 0; k < dev->config[i].interface[j].num_altsetting; k++) {
-					struct usb_interface_descriptor *intf = &dev->config[i].interface[j].altsetting[k]; 
+					struct usb_interface_descriptor *intf = &dev->config[i].interface[j].altsetting[k];
 					if (	(intf->bInterfaceClass == USB_CLASS_HID)	||
 						(intf->bInterfaceClass == USB_CLASS_PRINTER)	||
 						(intf->bInterfaceClass == USB_CLASS_AUDIO)	||
@@ -850,63 +971,65 @@ gp_port_usb_match_mtp_device(struct usb_device *dev,int *configno, int *interfac
 		return 0;
 
 	/*
-	 * Loop over the device configurations and interfaces. Nokia MTP-capable 
-	 * handsets (possibly others) typically have the string "MTP" in their 
+	 * Loop over the device configurations and interfaces. Nokia MTP-capable
+	 * handsets (possibly others) typically have the string "MTP" in their
 	 * MTP interface descriptions, that's how they can be detected, before
 	 * we try the more esoteric "OS descriptors" (below).
 	 */
-	for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
-		unsigned int j;
-		for (j = 0; j < dev->config[i].bNumInterfaces; j++) {
-			int k;
-			for (k = 0; k < dev->config[i].interface[j].num_altsetting; k++) {
-				buf[0] = '\0';
-				ret = usb_get_string_simple(devh, 
-					dev->config[i].interface[j].altsetting[k].iInterface, 
-					(char *) buf, 
-					1024);
-				if (ret < 3)
-					continue;
-				if (strcmp((char *) buf, "MTP") == 0) {
-					gp_log (GP_LOG_DEBUG, "mtp matcher", "Configuration %d, interface %d, altsetting %d:\n", i, j, k);
-					gp_log (GP_LOG_DEBUG, "mtp matcher", "   Interface description contains the string \"MTP\"\n");
-					gp_log (GP_LOG_DEBUG, "mtp matcher", "   Device recognized as MTP, no further probing.\n");
-					goto found;
+	if (dev->config) {
+		for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
+			unsigned int j;
+
+			for (j = 0; j < dev->config[i].bNumInterfaces; j++) {
+				int k;
+				for (k = 0; k < dev->config[i].interface[j].num_altsetting; k++) {
+					buf[0] = '\0';
+					ret = usb_get_string_simple(devh,
+						dev->config[i].interface[j].altsetting[k].iInterface,
+						(char *) buf,
+						1024);
+					if (ret < 3)
+						continue;
+					if (strcmp((char *) buf, "MTP") == 0) {
+						GP_LOG_D ("Configuration %d, interface %d, altsetting %d:\n", i, j, k);
+						GP_LOG_D ("   Interface description contains the string \"MTP\"\n");
+						GP_LOG_D ("   Device recognized as MTP, no further probing.\n");
+						goto found;
+					}
 				}
 			}
 		}
 	}
-
 	/* get string descriptor at 0xEE */
 	ret = usb_get_descriptor (devh, 0x03, 0xee, buf, sizeof(buf));
-	if (ret > 0) gp_log_data("get_MS_OSD",buf, ret);
+	if (ret > 0) GP_LOG_DATA (buf, ret, "get_MS_OSD");
 	if (ret < 10) goto errout;
 	if (!((buf[2] == 'M') && (buf[4]=='S') && (buf[6]=='F') && (buf[8]=='T')))
 		goto errout;
 	cmd = buf[16];
 	ret = usb_control_msg (devh, USB_ENDPOINT_IN|USB_RECIP_DEVICE|USB_TYPE_VENDOR, cmd, 0, 4, buf, sizeof(buf), 1000);
 	if (ret == -1) {
-		gp_log (GP_LOG_ERROR, "mtp matcher", "control message says %d\n", ret);
+		GP_LOG_E ("control message says %d\n", ret);
 		goto errout;
 	}
 	if (buf[0] != 0x28) {
-		gp_log (GP_LOG_ERROR, "mtp matcher", "ret is %d, buf[0] is %x\n", ret, buf[0]);
+		GP_LOG_E ("ret is %d, buf[0] is %x\n", ret, buf[0]);
 		goto errout;
 	}
-	if (ret > 0) gp_log_data("get_MS_ExtDesc",buf, ret);
+	if (ret > 0) GP_LOG_DATA (buf, ret, "get_MS_ExtDesc");
 	if ((buf[0x12] != 'M') || (buf[0x13] != 'T') || (buf[0x14] != 'P')) {
-		gp_log (GP_LOG_ERROR, "mtp matcher", "buf at 0x12 is %02x%02x%02x\n", buf[0x12], buf[0x13], buf[0x14]);
+		GP_LOG_E ("buf at 0x12 is %02x%02x%02x\n", buf[0x12], buf[0x13], buf[0x14]);
 		goto errout;
 	}
 	ret = usb_control_msg (devh, USB_ENDPOINT_IN|USB_RECIP_DEVICE|USB_TYPE_VENDOR, cmd, 0, 5, buf, sizeof(buf), 1000);
 	if (ret == -1) goto errout;
 	if (buf[0] != 0x28) {
-		gp_log (GP_LOG_ERROR, "mtp matcher", "ret is %d, buf[0] is %x\n", ret, buf[0]);
+		GP_LOG_E ("ret is %d, buf[0] is %x\n", ret, buf[0]);
 		goto errout;
 	}
-	if (ret > 0) gp_log_data("get_MS_ExtProp",buf, ret);
+	if (ret > 0) GP_LOG_DATA (buf, ret, "get_MS_ExtProp");
 	if ((buf[0x12] != 'M') || (buf[0x13] != 'T') || (buf[0x14] != 'P')) {
-		gp_log (GP_LOG_ERROR, "mtp matcher", "buf at 0x12 is %02x%02x%02x\n", buf[0x12], buf[0x13], buf[0x14]);
+		GP_LOG_E ("buf at 0x12 is %02x%02x%02x\n", buf[0x12], buf[0x13], buf[0x14]);
 		goto errout;
 	}
 
@@ -916,19 +1039,19 @@ found:
 	/* Now chose a nice interface for us to use ... Just take the first. */
 
 	if (dev->descriptor.bNumConfigurations > 1)
-		gp_log (GP_LOG_ERROR, "mtp matcher", "The device has %d configurations!\n", dev->descriptor.bNumConfigurations);
+		GP_LOG_E ("The device has %d configurations!\n", dev->descriptor.bNumConfigurations);
 	for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
 		struct usb_config_descriptor *config =
 			&dev->config[i];
 
 		if (config->bNumInterfaces > 1)
-			gp_log (GP_LOG_ERROR, "mtp matcher", "The configuration has %d interfaces!\n", config->bNumInterfaces);
+			GP_LOG_E ("The configuration has %d interfaces!\n", config->bNumInterfaces);
 		for (i1 = 0; i1 < config->bNumInterfaces; i1++) {
 			struct usb_interface *interface =
 				&config->interface[i1];
 
 			if (interface->num_altsetting > 1)
-				gp_log (GP_LOG_ERROR, "mtp matcher", "The interface has %d altsettings!\n", interface->num_altsetting);
+				GP_LOG_E ("The interface has %d altsettings!\n", interface->num_altsetting);
 			for (i2 = 0; i2 < interface->num_altsetting; i2++) {
 				*configno = i;
 				*interfaceno = i1;
@@ -943,14 +1066,17 @@ errout:
 	return 0;
 #endif
 }
+#endif
 
 static int
 gp_port_usb_match_device_by_class(struct usb_device *dev, int class, int subclass, int protocol, int *configno, int *interfaceno, int *altsettingno)
 {
 	int i, i1, i2;
 
+#if 0
 	if (class == 666) /* Special hack for MTP devices with MS OS descriptors. */
 		return gp_port_usb_match_mtp_device (dev, configno, interfaceno, altsettingno);
+#endif
 
 	if (dev->descriptor.bDeviceClass == class &&
 	    (subclass == -1 ||
@@ -1000,8 +1126,7 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 	char *s;
 	char busname[64], devname[64];
 
-	if (!port)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (port);
 
 	busname[0] = devname[0] = '\0';
 	s = strchr (port->settings.usb.port,':');
@@ -1024,8 +1149,7 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 	 * Should the USB layer report that ? I don't know.
 	 * Better to check here.
 	 */
-	if (!class)
-		return GP_ERROR_BAD_PARAMETERS;
+	C_PARAMS (class);
 
 	for (bus = usb_busses; bus; bus = bus->next) {
 		if ((busname[0] != '\0') && strcmp(busname, bus->dirname))
@@ -1036,20 +1160,14 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 
 			if ((devname[0] != '\0') && strcmp(devname, dev->filename))
 				continue;
-			gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
-				"Looking for USB device "
-				"(class 0x%x, subclass, 0x%x, protocol 0x%x)...", 
-				class, subclass, protocol);
+			GP_LOG_D ("Looking for USB device (class 0x%x, subclass, 0x%x, protocol 0x%x)...", class, subclass, protocol);
 
 			ret = gp_port_usb_match_device_by_class(dev, class, subclass, protocol, &config, &interface, &altsetting);
 			if (!ret)
 				continue;
 
 			port->pl->d = dev;
-			gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
-				"Found USB class device "
-				"(class 0x%x, subclass, 0x%x, protocol 0x%x).", 
-				class, subclass, protocol);
+			GP_LOG_D ("Found USB class device (class 0x%x, subclass, 0x%x, protocol 0x%x)", class, subclass, protocol);
 			/* Set the defaults */
 			if (dev->config) {
 				int i;
@@ -1062,18 +1180,15 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 				port->settings.usb.outep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_OUT, USB_ENDPOINT_TYPE_BULK);
 				port->settings.usb.intep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_IN, USB_ENDPOINT_TYPE_INTERRUPT);
 				port->settings.usb.maxpacketsize = 0;
-				gp_log (GP_LOG_DEBUG, "gphoto2-port-usb", "inep to look for is %02x", port->settings.usb.inep);
+				GP_LOG_D ("inep to look for is %02x", port->settings.usb.inep);
 				for (i=0;i<dev->config[config].interface[interface].altsetting[altsetting].bNumEndpoints;i++) {
 					if (port->settings.usb.inep == dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].bEndpointAddress) {
 						port->settings.usb.maxpacketsize = dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].wMaxPacketSize;
 						break;
 					}
 				}
-				gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
-					"Detected defaults: config %d, "
-					"interface %d, altsetting %d, "
-					"idVendor ID %04x, idProduct %04x, "
-					"inep %02x, outep %02x, intep %02x",
+				GP_LOG_D ("Detected defaults: config %d, interface %d, altsetting %d, "
+					  "idVendor ID %04x, idProduct %04x, inep %02x, outep %02x, intep %02x",
 					port->settings.usb.config,
 					port->settings.usb.interface,
 					port->settings.usb.altsetting,
@@ -1101,16 +1216,16 @@ gp_port_library_operations (void)
 {
 	GPPortOperations *ops;
 
-	ops = malloc (sizeof (GPPortOperations));
+	ops = calloc (1, sizeof (GPPortOperations));
 	if (!ops)
 		return (NULL);
-	memset (ops, 0, sizeof (GPPortOperations));
 
 	ops->init   = gp_port_usb_init;
 	ops->exit   = gp_port_usb_exit;
 	ops->open   = gp_port_usb_open;
 	ops->close  = gp_port_usb_close;
 	ops->read   = gp_port_usb_read;
+	ops->reset  = gp_port_usb_reset;
 	ops->write  = gp_port_usb_write;
 	ops->check_int = gp_port_usb_check_int;
 	ops->update = gp_port_usb_update;
