@@ -1,103 +1,5 @@
 var NAVTREEINDEX4 =
 {
-"gphoto2-setting_8c.html":[2,0,2,17],
-"gphoto2-setting_8c.html#a4b50249c04f6a48156bad839827e67b7":[2,0,2,17,3],
-"gphoto2-setting_8c.html#a6100f1cf5f3e90c2491807b6db1796af":[2,0,2,17,4],
-"gphoto2-setting_8c.html#a7f8c30ecb636e41b160ace9b51501caa":[2,0,2,17,2],
-"gphoto2-setting_8c.html#ad3d8a3bd0c0b677acef144f2c2ef6d73":[2,0,2,17,1],
-"gphoto2-setting_8h.html":[2,0,1,8],
-"gphoto2-setting_8h.html#a4b50249c04f6a48156bad839827e67b7":[2,0,1,8,0],
-"gphoto2-setting_8h.html#a6100f1cf5f3e90c2491807b6db1796af":[2,0,1,8,1],
-"gphoto2-setting_8h_source.html":[2,0,1,8],
-"gphoto2-version_8c.html":[2,0,2,18],
-"gphoto2-version_8c.html#a039629a84ef4c735b1124901438a7468":[2,0,2,18,0],
-"gphoto2-version_8h.html":[2,0,1,9],
-"gphoto2-version_8h.html#a039629a84ef4c735b1124901438a7468":[2,0,1,9,0],
-"gphoto2-version_8h_source.html":[2,0,1,9],
-"gphoto2-widget_8c.html":[2,0,2,19],
-"gphoto2-widget_8c.html#a01706284163682eb459b045b651a2fcf":[2,0,2,19,17],
-"gphoto2-widget_8c.html#a160a340c58e006a1a4de0ac77e02e136":[2,0,2,19,15],
-"gphoto2-widget_8c.html#a20783a5aec3e3a7cd4c8ce3af7a92361":[2,0,2,19,16],
-"gphoto2-widget_8c.html#a3017f318e2dadaf82d5d55140594bcdf":[2,0,2,19,9],
-"gphoto2-widget_8c.html#a467afee62da7ca075c8b24c71cfd812a":[2,0,2,19,7],
-"gphoto2-widget_8c.html#a4c4ea20c4d069c418399754822644174":[2,0,2,19,25],
-"gphoto2-widget_8c.html#a582ecd400d10bb040b35c317c48cf638":[2,0,2,19,27],
-"gphoto2-widget_8c.html#a58d57830ff9a7ce66b1b4b9faa93948a":[2,0,2,19,29],
-"gphoto2-widget_8c.html#a5b80fa207622d0efc676c3219c6150eb":[2,0,2,19,23],
-"gphoto2-widget_8c.html#a677590f6805dbf542d84cfe5a2718789":[2,0,2,19,31],
-"gphoto2-widget_8c.html#a6e6287e13a841e6f7631d06d72c06237":[2,0,2,19,30],
-"gphoto2-widget_8c.html#a7148abb7985c2d5d6310ef6b25d38256":[2,0,2,19,22],
-"gphoto2-widget_8c.html#a7788e7259ec60c96088f1093670e6d8d":[2,0,2,19,20],
-"gphoto2-widget_8c.html#a7a547c3a01c9ba10c8552187aca0a43f":[2,0,2,19,5],
-"gphoto2-widget_8c.html#a7a9e6b495cbdf50dfeeb9f49f58e186e":[2,0,2,19,28],
-"gphoto2-widget_8c.html#a8e84d6d4a4593ddef222d5246fa207ac":[2,0,2,19,21],
-"gphoto2-widget_8c.html#a91689d5c09fdc861080faafaf10767a7":[2,0,2,19,13],
-"gphoto2-widget_8c.html#aa46b3ea59abaafb0d83948fee80e57af":[2,0,2,19,8],
-"gphoto2-widget_8c.html#ab3365698f977ba94534148d1df5eb9d8":[2,0,2,19,2],
-"gphoto2-widget_8c.html#ac2407563a7f8c22de8df8d5009f0e4e1":[2,0,2,19,6],
-"gphoto2-widget_8c.html#acc79e22647c2cda8c053da21e296f303":[2,0,2,19,14],
-"gphoto2-widget_8c.html#ad15d33230a8a7b3d039d63bc5875c0ac":[2,0,2,19,10],
-"gphoto2-widget_8c.html#ad90a0c554f7fcc61914fd54d4c612f34":[2,0,2,19,24],
-"gphoto2-widget_8c.html#adaf6cc55afadae959d07adebecc85c65":[2,0,2,19,3],
-"gphoto2-widget_8c.html#ae0bafe1d2115b44a8cc0ba1cc82f7560":[2,0,2,19,19],
-"gphoto2-widget_8c.html#ae41c7eb76c0bb21770879ad8e1dc58b9":[2,0,2,19,1],
-"gphoto2-widget_8c.html#aed8693ca96a9d877810329fc744323f1":[2,0,2,19,12],
-"gphoto2-widget_8c.html#af3c1033ec2158a70fe987767b20d908a":[2,0,2,19,11],
-"gphoto2-widget_8c.html#af769b52499b4e6f9134d345326f59897":[2,0,2,19,18],
-"gphoto2-widget_8c.html#afa478d063b2470df8ba5ee29fc33b5bf":[2,0,2,19,4],
-"gphoto2-widget_8c.html#afee1b0fb138aa4fd73c69b5f925e5afd":[2,0,2,19,26],
-"gphoto2-widget_8h.html":[2,0,1,10],
-"gphoto2-widget_8h.html#a01706284163682eb459b045b651a2fcf":[2,0,1,10,19],
-"gphoto2-widget_8h.html#a160a340c58e006a1a4de0ac77e02e136":[2,0,1,10,17],
-"gphoto2-widget_8h.html#a20783a5aec3e3a7cd4c8ce3af7a92361":[2,0,1,10,18],
-"gphoto2-widget_8h.html#a3017f318e2dadaf82d5d55140594bcdf":[2,0,1,10,11],
-"gphoto2-widget_8h.html#a467afee62da7ca075c8b24c71cfd812a":[2,0,1,10,9],
-"gphoto2-widget_8h.html#a4c4ea20c4d069c418399754822644174":[2,0,1,10,27],
-"gphoto2-widget_8h.html#a4ff53441fd74eb630a997d461220f781":[2,0,1,10,1],
-"gphoto2-widget_8h.html#a582ecd400d10bb040b35c317c48cf638":[2,0,1,10,29],
-"gphoto2-widget_8h.html#a58d57830ff9a7ce66b1b4b9faa93948a":[2,0,1,10,31],
-"gphoto2-widget_8h.html#a5b80fa207622d0efc676c3219c6150eb":[2,0,1,10,25],
-"gphoto2-widget_8h.html#a677590f6805dbf542d84cfe5a2718789":[2,0,1,10,33],
-"gphoto2-widget_8h.html#a6e6287e13a841e6f7631d06d72c06237":[2,0,1,10,32],
-"gphoto2-widget_8h.html#a7148abb7985c2d5d6310ef6b25d38256":[2,0,1,10,24],
-"gphoto2-widget_8h.html#a76b1b482abd434a39ad607e9c72ee606":[2,0,1,10,30],
-"gphoto2-widget_8h.html#a7788e7259ec60c96088f1093670e6d8d":[2,0,1,10,22],
-"gphoto2-widget_8h.html#a7a547c3a01c9ba10c8552187aca0a43f":[2,0,1,10,7],
-"gphoto2-widget_8h.html#a8c7e7c5b04d992022d9a821a0d3f9c30":[2,0,1,10,2],
-"gphoto2-widget_8h.html#a8c7e7c5b04d992022d9a821a0d3f9c30a4da6bf6564c6535715ed7a51faf97d9e":[2,0,1,10,2,3],
-"gphoto2-widget_8h.html#a8c7e7c5b04d992022d9a821a0d3f9c30a82006459973ca4f17729b99e2493242f":[2,0,1,10,2,7],
-"gphoto2-widget_8h.html#a8c7e7c5b04d992022d9a821a0d3f9c30a82fd9cfdb47df56973dfebaa1ebb5356":[2,0,1,10,2,1],
-"gphoto2-widget_8h.html#a8c7e7c5b04d992022d9a821a0d3f9c30a9d2fb92d970fe5a9d4abb2d11379f64f":[2,0,1,10,2,4],
-"gphoto2-widget_8h.html#a8c7e7c5b04d992022d9a821a0d3f9c30aab0990bda50305e4314fa5d267ad8b2c":[2,0,1,10,2,5],
-"gphoto2-widget_8h.html#a8c7e7c5b04d992022d9a821a0d3f9c30ab9247accefebceecdbebf6d8cbb7edf8":[2,0,1,10,2,8],
-"gphoto2-widget_8h.html#a8c7e7c5b04d992022d9a821a0d3f9c30ac06d6140c294874ef251ba08834c506d":[2,0,1,10,2,0],
-"gphoto2-widget_8h.html#a8c7e7c5b04d992022d9a821a0d3f9c30acdd16f51f1a6b63ef044a9fa6c9b851c":[2,0,1,10,2,6],
-"gphoto2-widget_8h.html#a8c7e7c5b04d992022d9a821a0d3f9c30aed35470a4719ed91a2e163072112e835":[2,0,1,10,2,2],
-"gphoto2-widget_8h.html#a8e84d6d4a4593ddef222d5246fa207ac":[2,0,1,10,23],
-"gphoto2-widget_8h.html#a91689d5c09fdc861080faafaf10767a7":[2,0,1,10,15],
-"gphoto2-widget_8h.html#aa46b3ea59abaafb0d83948fee80e57af":[2,0,1,10,10],
-"gphoto2-widget_8h.html#ab3365698f977ba94534148d1df5eb9d8":[2,0,1,10,4],
-"gphoto2-widget_8h.html#ac2407563a7f8c22de8df8d5009f0e4e1":[2,0,1,10,8],
-"gphoto2-widget_8h.html#ac9da2516e7b6c1b10aa3c1656db72adc":[2,0,1,10,0],
-"gphoto2-widget_8h.html#acc79e22647c2cda8c053da21e296f303":[2,0,1,10,16],
-"gphoto2-widget_8h.html#ad15d33230a8a7b3d039d63bc5875c0ac":[2,0,1,10,12],
-"gphoto2-widget_8h.html#ad90a0c554f7fcc61914fd54d4c612f34":[2,0,1,10,26],
-"gphoto2-widget_8h.html#adaf6cc55afadae959d07adebecc85c65":[2,0,1,10,5],
-"gphoto2-widget_8h.html#ae0bafe1d2115b44a8cc0ba1cc82f7560":[2,0,1,10,21],
-"gphoto2-widget_8h.html#ae41c7eb76c0bb21770879ad8e1dc58b9":[2,0,1,10,3],
-"gphoto2-widget_8h.html#aed8693ca96a9d877810329fc744323f1":[2,0,1,10,14],
-"gphoto2-widget_8h.html#af3c1033ec2158a70fe987767b20d908a":[2,0,1,10,13],
-"gphoto2-widget_8h.html#af769b52499b4e6f9134d345326f59897":[2,0,1,10,20],
-"gphoto2-widget_8h.html#afa478d063b2470df8ba5ee29fc33b5bf":[2,0,1,10,6],
-"gphoto2-widget_8h.html#afee1b0fb138aa4fd73c69b5f925e5afd":[2,0,1,10,28],
-"gphoto2-widget_8h_source.html":[2,0,1,10],
-"gphoto2_8h.html":[2,0,1,11],
-"gphoto2_8h_source.html":[2,0,1,11],
-"index.html":[],
-"jpeg_8c.html":[2,0,2,20],
-"jpeg_8c.html#a038573b7f439be589bd905b2e310490d":[2,0,2,20,11],
-"jpeg_8c.html#a157a3d714ccab9eba11a83ab8c44ef2e":[2,0,2,20,7],
-"jpeg_8c.html#a15c75310906f82a5922655361510dcf5":[2,0,2,20,24],
 "jpeg_8c.html#a2a3b3ac76f7224d6e5f635f1a831e6c7":[2,0,2,20,8],
 "jpeg_8c.html#a2a7832a90410c88f8f4d1f99ff64c461":[2,0,2,20,15],
 "jpeg_8c.html#a2cc14f63e9eb5bac86a036771bfb1758":[2,0,2,20,12],
@@ -106,7 +8,6 @@ var NAVTREEINDEX4 =
 "jpeg_8c.html#a5bec69a612f0edb89186ecbc9cc8647b":[2,0,2,20,6],
 "jpeg_8c.html#a621dd0efba334f3b4f8c76e131d18f1e":[2,0,2,20,22],
 "jpeg_8c.html#a7f53c979d313bd96c6c179e51ffe9ad1":[2,0,2,20,3],
-"jpeg_8c.html#a7f8c30ecb636e41b160ace9b51501caa":[2,0,2,20,0],
 "jpeg_8c.html#a85517cf2a159ce5b1fd68b6ecde2a699":[2,0,2,20,2],
 "jpeg_8c.html#a900e68ccf955cb2c1093a5521e33adb5":[2,0,2,20,18],
 "jpeg_8c.html#a9fc326f857d0d4a8eb62180d9cfa6476":[2,0,2,20,4],
@@ -128,6 +29,7 @@ var NAVTREEINDEX4 =
 "jpeg_8h.html#a23992c28c13fcf5ed0906e7bb8cefa60":[2,0,2,21,3],
 "jpeg_8h.html#a2a3b3ac76f7224d6e5f635f1a831e6c7":[2,0,2,21,10],
 "jpeg_8h.html#a2a7832a90410c88f8f4d1f99ff64c461":[2,0,2,21,17],
+"jpeg_8h.html#a2c27579eadababab8c65bbbabeefc6d4":[2,0,2,21,2],
 "jpeg_8h.html#a2cc14f63e9eb5bac86a036771bfb1758":[2,0,2,21,14],
 "jpeg_8h.html#a3b490d263fb18683ec488cb84c524b61":[2,0,2,21,4],
 "jpeg_8h.html#a4aee2400e444196497aaf51b5bda674b":[2,0,2,21,19],
@@ -150,7 +52,6 @@ var NAVTREEINDEX4 =
 "jpeg_8h.html#ad06d178d14aae0033c1c51e5728afbdda86ce7335618ccd615eb9836d6d1c5a63":[2,0,2,21,5,1],
 "jpeg_8h.html#ad06d178d14aae0033c1c51e5728afbddac5c308abd323898ed35b2eac42237280":[2,0,2,21,5,5],
 "jpeg_8h.html#ad06d178d14aae0033c1c51e5728afbddae64f063f5f4a0d160ce5b8c2e4fb6b62":[2,0,2,21,5,3],
-"jpeg_8h.html#add022e5f235487028fc7a862de4459ac":[2,0,2,21,2],
 "jpeg_8h.html#ae08d9f698f1bdfc5b7a4ce0193fd177f":[2,0,2,21,11],
 "jpeg_8h.html#ae61037acbc754622dffb58feca3cea03":[2,0,2,21,25],
 "jpeg_8h.html#aee946402a45cc98e3b6489b52ae64988":[2,0,2,21,7],
@@ -187,23 +88,15 @@ var NAVTREEINDEX4 =
 "structCameraFilePath.html#a576f3bdaaf2905795120cb4a8b274364":[1,0,31,0],
 "structCameraText.html":[1,0,32],
 "structCameraText.html#af37011bda3c08a5b85d996a8a96a1175":[1,0,32,0],
-"structExifData.html":[1,0,35],
-"structExifData.html#a0855a0f999ff6318fc6164eac9558739":[1,0,35,3],
-"structExifData.html#a507ec464eb69af9dcf6643529474fe25":[1,0,35,4],
-"structExifData.html#a777d4a75a71a323d679e36040eefe949":[1,0,35,5],
-"structExifData.html#a817025910c9a21ad11e36cc6291f74d7":[1,0,35,1],
-"structExifData.html#aa1ffe358b9a4d06c3e6d77ad60bfed7f":[1,0,35,6],
-"structExifData.html#aca6f46963e0f2c21eb0a0947d9a5cbf9":[1,0,35,0],
-"structExifData.html#afea5ad4801e3718debc333caf6e4a410":[1,0,35,2],
-"structLogFunc.html":[1,0,38],
-"structLogFunc.html#a068941612eb97ecd67e9f215079da7b7":[1,0,38,2],
-"structLogFunc.html#a9924d784a044385a061cf38ba69acfa1":[1,0,38,0],
-"structLogFunc.html#a9dc0273791c4d020e92867460e6fb605":[1,0,38,3],
-"structLogFunc.html#adfa343a5175adcb7a670caf959615b86":[1,0,38,1],
-"structSetting.html":[1,0,39],
-"structSetting.html#a13b7fdbd033bf614f735995debcc0bfe":[1,0,39,2],
-"structSetting.html#a48642590d108a71ef0f4a4b243d0556b":[1,0,39,1],
-"structSetting.html#a4946f76797007123198a74dbe3ae1a01":[1,0,39,0],
+"structLogFunc.html":[1,0,36],
+"structLogFunc.html#a8cd5c8a503a285c7598a241cc3ec789e":[1,0,36,2],
+"structLogFunc.html#a9924d784a044385a061cf38ba69acfa1":[1,0,36,0],
+"structLogFunc.html#a9dc0273791c4d020e92867460e6fb605":[1,0,36,3],
+"structLogFunc.html#adfa343a5175adcb7a670caf959615b86":[1,0,36,1],
+"structSetting.html":[1,0,37],
+"structSetting.html#a13b7fdbd033bf614f735995debcc0bfe":[1,0,37,2],
+"structSetting.html#a48642590d108a71ef0f4a4b243d0556b":[1,0,37,1],
+"structSetting.html#a4946f76797007123198a74dbe3ae1a01":[1,0,37,0],
 "struct__Camera.html":[1,0,0],
 "struct__Camera.html#a4e8cee96ef5ef4f271c866d8d2c0ce9d":[1,0,0,0],
 "struct__Camera.html#a640193a6f18e834a87e71bc9dd3fedd2":[1,0,0,3],
@@ -212,6 +105,7 @@ var NAVTREEINDEX4 =
 "struct__Camera.html#af55d6e519c6b49eb11e518c419b1627f":[1,0,0,1],
 "struct__CameraAbilitiesList.html":[1,0,1],
 "struct__CameraAbilitiesList.html#a3a18bfd44cf3ae22ebea7587153debb9":[1,0,1,0],
+"struct__CameraAbilitiesList.html#a8293a0fc888d18b6f3c6ee5a424fb884":[1,0,1,2],
 "struct__CameraAbilitiesList.html#ad57e34bd38b93bcdaf2b47629105531d":[1,0,1,1],
 "struct__CameraFile.html":[1,0,2],
 "struct__CameraFile.html#a05fa8fdab784c185cabe2485c7820875":[1,0,2,10],
@@ -249,5 +143,111 @@ var NAVTREEINDEX4 =
 "struct__CameraFileInfoFile.html#af076fe6da9612a0167acf489c40f31e8":[1,0,6,1],
 "struct__CameraFileInfoPreview.html":[1,0,7],
 "struct__CameraFileInfoPreview.html#a08eb4d953d2c9f4f7e8ce54223bb8ea9":[1,0,7,3],
-"struct__CameraFileInfoPreview.html#a12d6115703a502a24cd5b193b75d666a":[1,0,7,5]
+"struct__CameraFileInfoPreview.html#a12d6115703a502a24cd5b193b75d666a":[1,0,7,5],
+"struct__CameraFileInfoPreview.html#a2cc5f80f66aa90d7c276df9ecd530490":[1,0,7,1],
+"struct__CameraFileInfoPreview.html#a42e378c8d0668052b39c0cca22382dd7":[1,0,7,4],
+"struct__CameraFileInfoPreview.html#a872f4d1625b0008c625138f33ef3e4cb":[1,0,7,0],
+"struct__CameraFileInfoPreview.html#a9a3b70779dc8cea4d520b9349bee6ba4":[1,0,7,2],
+"struct__CameraFilesystem.html":[1,0,8],
+"struct__CameraFilesystem.html#a1df04156f0ffe145f2e9d99eea9641ee":[1,0,8,13],
+"struct__CameraFilesystem.html#a2f676eede09ea213a0ef7e8711d6ddf3":[1,0,8,5],
+"struct__CameraFilesystem.html#a31e9fa7b813074658672a512b683a334":[1,0,8,1],
+"struct__CameraFilesystem.html#a32c4b277105e1a86ade9700dbc1cf891":[1,0,8,4],
+"struct__CameraFilesystem.html#a38fcb7940178fa609cd62b8aa9ea5c29":[1,0,8,10],
+"struct__CameraFilesystem.html#a48290da7bdd03d2a9f24a4bf6fa840dd":[1,0,8,12],
+"struct__CameraFilesystem.html#a4f95116fa657186a1cb011004451d503":[1,0,8,6],
+"struct__CameraFilesystem.html#a6a7b0038371ba041f35153ad55e302ba":[1,0,8,16],
+"struct__CameraFilesystem.html#a93e23f96c30c239d9db008156eb9ecf1":[1,0,8,9],
+"struct__CameraFilesystem.html#aa1c2393b2247bf5d80b94681d76aea10":[1,0,8,2],
+"struct__CameraFilesystem.html#ab051f9ac8d59e105dec10946f5ddb2ce":[1,0,8,11],
+"struct__CameraFilesystem.html#ab728495997f9343caee158097d63800c":[1,0,8,0],
+"struct__CameraFilesystem.html#ad07389f2a3311bf7084db695a17f90ab":[1,0,8,15],
+"struct__CameraFilesystem.html#ae3712591919ddad59a8ad49ef03fa804":[1,0,8,3],
+"struct__CameraFilesystem.html#ae8a6640dcf9d03a2cc884e57fdca046f":[1,0,8,8],
+"struct__CameraFilesystem.html#aeb3b5d62783eef1c5fa5fcbee840ef34":[1,0,8,7],
+"struct__CameraFilesystem.html#afbd79b65268a16e645bd783d084fbbe2":[1,0,8,14],
+"struct__CameraFilesystemFile.html":[1,0,9],
+"struct__CameraFilesystemFile.html#a02698fe765cf3f992a3fbdecbd8b9784":[1,0,9,11],
+"struct__CameraFilesystemFile.html#a433ed6bdeff42ed48e0ef04a55b60ac3":[1,0,9,6],
+"struct__CameraFilesystemFile.html#a44f3914de96b4d19d575f5ea2312f35c":[1,0,9,8],
+"struct__CameraFilesystemFile.html#a7219141059d867b7283590a9814ac341":[1,0,9,7],
+"struct__CameraFilesystemFile.html#a87ab89c82038e4a0022d76d7484b31c7":[1,0,9,2],
+"struct__CameraFilesystemFile.html#aa38832a89a3372580d17cbb1035cc9a7":[1,0,9,4],
+"struct__CameraFilesystemFile.html#aa97be6dd8694cb76a9031a9a7ec9bb28":[1,0,9,3],
+"struct__CameraFilesystemFile.html#aae68888fd56fcfdf2792e437164becb5":[1,0,9,0],
+"struct__CameraFilesystemFile.html#ad315850e708dd65c724db870023cf8f3":[1,0,9,10],
+"struct__CameraFilesystemFile.html#ad6d4c870b50398fe68737e677534396e":[1,0,9,5],
+"struct__CameraFilesystemFile.html#aeb320c929ddcc332d43bc0616241ad6d":[1,0,9,9],
+"struct__CameraFilesystemFile.html#af4b8f53099f2244ef750587c36f50d83":[1,0,9,1],
+"struct__CameraFilesystemFolder.html":[1,0,10],
+"struct__CameraFilesystemFolder.html#a3224f1b104e97740b2698c000f465391":[1,0,10,5],
+"struct__CameraFilesystemFolder.html#a5e4407759a7af5ab27a4d63c8e2341e1":[1,0,10,3],
+"struct__CameraFilesystemFolder.html#a7400f0d5ec56b57f1f301a6048bb6416":[1,0,10,0],
+"struct__CameraFilesystemFolder.html#a8f656af8a27f2192a55c95f8c62a6e56":[1,0,10,4],
+"struct__CameraFilesystemFolder.html#a8fb991ae3bb60c1bab6cf8a648fa9a64":[1,0,10,1],
+"struct__CameraFilesystemFolder.html#ae55b6cc0dac97f7f239ccd4b72f52110":[1,0,10,2],
+"struct__CameraFilesystemFuncs.html":[1,0,11],
+"struct__CameraFilesystemFuncs.html#a26913f33b0cc3180e3186d6a3f77421a":[1,0,11,2],
+"struct__CameraFilesystemFuncs.html#a72cf3ad48a579a9ce74f5fe17a9c93bf":[1,0,11,11],
+"struct__CameraFilesystemFuncs.html#a826af65d3d1e787ee72c1be8f76f7bd9":[1,0,11,3],
+"struct__CameraFilesystemFuncs.html#aa228cbb46f365a55323293a63bf3d8c4":[1,0,11,9],
+"struct__CameraFilesystemFuncs.html#aadc59812a3613cef34077b1b33d670a0":[1,0,11,5],
+"struct__CameraFilesystemFuncs.html#ab1337f319ee4ed71caece9cda170ddc1":[1,0,11,4],
+"struct__CameraFilesystemFuncs.html#ace327a54414a27e328bbade7c1d13d98":[1,0,11,10],
+"struct__CameraFilesystemFuncs.html#ad1a22b81b884ac28e7a5a2488853e503":[1,0,11,1],
+"struct__CameraFilesystemFuncs.html#ad28ae496c5198bce124b78595f6a6522":[1,0,11,8],
+"struct__CameraFilesystemFuncs.html#ad575f8fa1ad7dc256363d51bce8a62b3":[1,0,11,6],
+"struct__CameraFilesystemFuncs.html#adb2500fecfad53a073fff9ae5141c089":[1,0,11,7],
+"struct__CameraFilesystemFuncs.html#aecc8d4ddaebcb1ee047331a594ce6c20":[1,0,11,0],
+"struct__CameraFilesystemFuncs.html#afedc6d1a7e923d67db12c5aa2df101b8":[1,0,11,12],
+"struct__CameraFunctions.html":[1,0,12],
+"struct__CameraFunctions.html#a05e1ec8563ae62acbcda7476379004f0":[1,0,12,14],
+"struct__CameraFunctions.html#a0ed0d7256aa2ed3c82fa390b860486d2":[1,0,12,13],
+"struct__CameraFunctions.html#a0fc999ffd155ec790a698527bf8232c4":[1,0,12,19],
+"struct__CameraFunctions.html#a115074d29c4356a3f20bdb7f192932d0":[1,0,12,11],
+"struct__CameraFunctions.html#a1e83ac8e74419d73229b6dd9e149212a":[1,0,12,5],
+"struct__CameraFunctions.html#a26e1c20ccec7a5983c0c563f5a21e053":[1,0,12,2],
+"struct__CameraFunctions.html#a284268b41f1a849f320058a6cfc7aad4":[1,0,12,8],
+"struct__CameraFunctions.html#a3034c0d26ca790676b7a2e344780f869":[1,0,12,7],
+"struct__CameraFunctions.html#a3333e72219a225bec0d294f3a4c87d18":[1,0,12,22],
+"struct__CameraFunctions.html#a515f9abe8387cd85f0cfce4b6717f305":[1,0,12,17],
+"struct__CameraFunctions.html#a595f135aff3af5099f05d14dbd04802a":[1,0,12,15],
+"struct__CameraFunctions.html#a8a5dce8827fb65c09aba4bb48a95bc62":[1,0,12,9],
+"struct__CameraFunctions.html#a97669d9398b238a1cad0104da22d358b":[1,0,12,4],
+"struct__CameraFunctions.html#aa247ba281ffdefea96c0a7c82c4d74d1":[1,0,12,0],
+"struct__CameraFunctions.html#aa3e662eb999c79dd80bebb7333588fb1":[1,0,12,18],
+"struct__CameraFunctions.html#aa42394477d9a4f3c3cb3a8b093722075":[1,0,12,20],
+"struct__CameraFunctions.html#aa70e56a844c2f906211a2d1fd44bb162":[1,0,12,3],
+"struct__CameraFunctions.html#aaa16da1dc894cdca2d10b52f4eec9104":[1,0,12,21],
+"struct__CameraFunctions.html#aab8784a8dc7b2bd19b844852a76b0242":[1,0,12,1],
+"struct__CameraFunctions.html#ad2ae9500ae590351db5ee8395e2bd492":[1,0,12,6],
+"struct__CameraFunctions.html#aded55ef8e46bf8c840705df1dda1b2a7":[1,0,12,16],
+"struct__CameraFunctions.html#ae993834fa4d16eca83cd069cdea37881":[1,0,12,10],
+"struct__CameraFunctions.html#afc4f6db6b18855216c91aa7239a048e6":[1,0,12,12],
+"struct__CameraList.html":[1,0,13],
+"struct__CameraList.html#a21dfe1d464296a2ff0f234375134f1fe":[1,0,13,2],
+"struct__CameraList.html#a58aa63bdcd5ede08345c1071baf93468":[1,0,13,1],
+"struct__CameraList.html#ab4abed5ba8587ecfc18304365fb6aa2c":[1,0,13,3],
+"struct__CameraList.html#addac3264e9d1334085fceae522553b00":[1,0,13,0],
+"struct__CameraPrivateCore.html":[1,0,14],
+"struct__CameraPrivateCore.html#a039d8a8bd44993082c3b488c6f1ba454":[1,0,14,12],
+"struct__CameraPrivateCore.html#a03c3c731fca44db76358b1505e7f8d00":[1,0,14,3],
+"struct__CameraPrivateCore.html#a21a208a4f6988bce450c941793a0c2f1":[1,0,14,9],
+"struct__CameraPrivateCore.html#a229f165b8966f51cd5ad757af37973be":[1,0,14,4],
+"struct__CameraPrivateCore.html#a26f8e18411844ac32241b097b152e04a":[1,0,14,2],
+"struct__CameraPrivateCore.html#a386988024845dfa8758e9bfa9352a16a":[1,0,14,6],
+"struct__CameraPrivateCore.html#a43e64612d700e65db556df8dbb44c10f":[1,0,14,10],
+"struct__CameraPrivateCore.html#a5ae2b2fca5b63ce803bc48a942631592":[1,0,14,0],
+"struct__CameraPrivateCore.html#a759fbadf9dba5d5c09b935078b629db7":[1,0,14,7],
+"struct__CameraPrivateCore.html#a7b939f99bd3a061195e69592a5a9586e":[1,0,14,11],
+"struct__CameraPrivateCore.html#aadf69d4199b50b1af3d70426c903b86b":[1,0,14,1],
+"struct__CameraPrivateCore.html#ab69f041f4fe9690fc9ff3c5e965453c3":[1,0,14,8],
+"struct__CameraPrivateCore.html#ad39ab7bf131dab6f24b2a5b2a074f2d2":[1,0,14,5],
+"struct__CameraStorageInformation.html":[1,0,15],
+"struct__CameraStorageInformation.html#a0076093883131f7a90b6961c2455e573":[1,0,15,6],
+"struct__CameraStorageInformation.html#a10890d64e63a7e95624ad3258ea94199":[1,0,15,4],
+"struct__CameraStorageInformation.html#a1d192e8ebbd2367b90d1e0bcb69f02e4":[1,0,15,3],
+"struct__CameraStorageInformation.html#a2a263c93947db1b47bca9e7298840eca":[1,0,15,1],
+"struct__CameraStorageInformation.html#a468551e2079facf00b9acc97798fa30f":[1,0,15,2],
+"struct__CameraStorageInformation.html#a54d368755af52b0b4991e5cd8e706271":[1,0,15,5]
 };
