@@ -1,23 +1,23 @@
 /* print-camera-list - print libgphoto2 camera list in different formats
  *
- * Copyright  2002,2005 Hans Ulrich Niedermann <hun@users.sourceforge.net>
- * Portions Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
- * Portions Copyright  2005 Julien BLACHE <jblache@debian.org>
+ * Copyright 2002,2005 Hans Ulrich Niedermann <hun@users.sourceforge.net>
+ * Portions Copyright 2002 Lutz Mueller <lutz@users.sourceforge.net>
+ * Portions Copyright 2005 Julien BLACHE <jblache@debian.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 
@@ -92,7 +92,7 @@ typedef int (* begin_func_t)  (const func_params_t *params,
 			       void **data);
 typedef int (* middle_func_t)  (const func_params_t *params,
 			       void **data);
-typedef int (* camera_func_t) (const func_params_t *params, 
+typedef int (* camera_func_t) (const func_params_t *params,
 			       const int i,
 			       const int total,
 			       const CameraAbilities *ca,
@@ -165,9 +165,9 @@ print_version_comment(FILE *out,
 	unsigned int n;
 	if (out == NULL) { FATAL("Internal error: NULL out in print_version_comment()"); }
 	if (firstline != NULL) { fputs(firstline, out); }
-	fputs(startline, out);
+	if (startline != NULL) { fputs(startline, out); }
 	fputs("Created from this library:", out);
-	fputs(endline, out);
+	if (endline != NULL) { fputs(endline, out); }
 	for (n=0; (module_versions[n].name != NULL) && (module_versions[n].version_func != NULL); n++) {
 		const char *name = module_versions[n].name;
 		GPVersionFunc func = module_versions[n].version_func;
@@ -204,7 +204,7 @@ hotplug_begin_func (const func_params_t *params, void **data)
 
 /* print_usb_usermap
  *
- * Print out lines that can be included into usb.usermap 
+ * Print out lines that can be included into usb.usermap
  * - for all cams supported by our instance of libgphoto2.
  *
  * usb.usermap is a file used by
@@ -212,7 +212,7 @@ hotplug_begin_func (const func_params_t *params, void **data)
  */
 
 static int
-hotplug_camera_func (const func_params_t *params, 
+hotplug_camera_func (const func_params_t *params,
 		     const int i,
 		     const int total,
 		     const CameraAbilities *a,
@@ -220,8 +220,7 @@ hotplug_camera_func (const func_params_t *params,
 {
 	int flags = 0;
 	int class = 0, subclass = 0, proto = 0;
-	int usb_vendor = 0, usb_product = 0;
-	const char *usermap_script = 
+	const char *usermap_script =
 		((*params->argv)[0] != NULL)
 		?((*params->argv)[0])
 		:(GP_USB_HOTPLUG_SCRIPT);
@@ -231,10 +230,8 @@ hotplug_camera_func (const func_params_t *params,
 			class = 0;
 			subclass = 0;
 			proto = 0;
-			flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
+			flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID
 				 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
-			usb_vendor = a->usb_vendor;
-			usb_product = a->usb_product;
 		} else if ((a->usb_class) && (a->usb_class != 666)) {
 			class = a->usb_class;
 			subclass = a->usb_subclass;
@@ -248,8 +245,6 @@ hotplug_camera_func (const func_params_t *params,
 				flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
 			else
 				proto = 0;
-			usb_vendor = 0;
-			usb_product = 0;
 		}
 	} else {
 		/* not a USB camera */
@@ -257,7 +252,7 @@ hotplug_camera_func (const func_params_t *params,
 	}
 
 	if (params->add_comments) {
-		printf ("# %s\n", 
+		printf ("# %s\n",
 			a->model);
 	}
 	/* The first 3 lone bytes are the device class.
@@ -269,7 +264,7 @@ hotplug_camera_func (const func_params_t *params,
 		"0x0000      0x00         0x00            "
 		"0x00            0x%02x            0x%02x               "
 		"0x%02x               0x00000000\n",
-		usermap_script, flags, 
+		usermap_script, flags,
 		a->usb_vendor, a->usb_product,
 		class, subclass, proto);
 	return 0;
@@ -322,7 +317,7 @@ path_basename (const char *pathname)
 	char *result, *tmp;
 	/* remove path part from camlib name */
 	for (result=tmp=(char *)pathname; (*tmp!='\0'); tmp++) {
-		if ((*tmp == gp_system_dir_delim) 
+		if ((*tmp == gp_system_dir_delim)
 		    && (*(tmp+1) != '\0')) {
 			result = tmp+1;
 		}
@@ -332,8 +327,8 @@ path_basename (const char *pathname)
 
 
 static int
-human_camera_func (const func_params_t *params, 
-		   const int i, 
+human_camera_func (const func_params_t *params,
+		   const int i,
 		   const int total,
 		   const CameraAbilities *a,
 		   void *data)
@@ -341,7 +336,7 @@ human_camera_func (const func_params_t *params,
 	const char *camlib_basename;
 	camlib_basename = path_basename(a->library);
 	printf("%3d|%-20s|%-20s|%s\n",
-	       i+1, 
+	       i+1,
 	       camlib_basename,
 	       a->id,
 	       a->model);
@@ -350,8 +345,8 @@ human_camera_func (const func_params_t *params,
 
 
 static int
-idlist_camera_func (const func_params_t *params, 
-		    const int i, 
+idlist_camera_func (const func_params_t *params,
+		    const int i,
 		    const int total,
 		    const CameraAbilities *a,
 		    void *data)
@@ -370,7 +365,8 @@ typedef enum {
 		UDEV_PRE_0_98 = 0,
 		UDEV_0_98 = 1,
 		UDEV_136 = 2,
-		UDEV_175 = 3
+		UDEV_175 = 3,
+		UDEV_201 = 4
 } udev_version_t;
 
 static const StringFlagItem udev_version_t_map[] = {
@@ -378,6 +374,7 @@ static const StringFlagItem udev_version_t_map[] = {
 	{ "0.98", UDEV_0_98 },
 	{ "136", UDEV_136 },
 	{ "175", UDEV_175 },
+	{ "201", UDEV_201 },
 	{ NULL, 0 }
 };
 
@@ -402,13 +399,13 @@ udev_parse_params (const func_params_t *params, void **data)
 	 */
 	static const char * const begin_strings[] = {
 		/* UDEV_PRE_0_98 */
-		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"ACTION!=\"add|bind\", GOTO=\"libgphoto2_rules_end\"\n"
 		"BUS!=\"usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n",
 		/* UDEV_0_98 */
-		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"ACTION!=\"add|bind\", GOTO=\"libgphoto2_rules_end\"\n"
 		"SUBSYSTEM!=\"usb|usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n",
 		/* UDEV_136 */
-		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"ACTION!=\"add|bind\", GOTO=\"libgphoto2_rules_end\"\n"
 		"SUBSYSTEM!=\"usb\", GOTO=\"libgphoto2_usb_end\"\n"
 		"ENV{DEVTYPE}!=\"usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n"
 		"ENV{ID_USB_INTERFACES}==\"\", IMPORT{program}=\"usb_id --export %%p\"\n"
@@ -418,7 +415,7 @@ udev_parse_params (const func_params_t *params, void **data)
 		 * more rules . It will be completed in udev_begin_func() */
 		"ENV{ID_USB_INTERFACES}==\"*:060101:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\", ",
 		/* UDEV_175 */
-		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n"
+		"ACTION!=\"add|bind\", GOTO=\"libgphoto2_rules_end\"\n"
 		"SUBSYSTEM!=\"usb\", GOTO=\"libgphoto2_usb_end\"\n"
 		"ENV{DEVTYPE}!=\"usb_device\", GOTO=\"libgphoto2_usb_end\"\n\n"
 		"ENV{ID_USB_INTERFACES}==\"\", IMPORT{builtin}=\"usb_id\"\n"
@@ -426,7 +423,15 @@ udev_parse_params (const func_params_t *params, void **data)
 		"ENV{ID_USB_INTERFACES}==\"*:08*:*\", GOTO=\"libgphoto2_usb_end\"\n"
 		/* shortcut the most common camera driver, ptp class, so we avoid parsing 1000
 		 * more rules . It will be completed in udev_begin_func() */
-		"ENV{ID_USB_INTERFACES}==\"*:060101:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\", "
+		"ENV{ID_USB_INTERFACES}==\"*:060101:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\", ",
+
+		/* UDEV_201 ... regular stuff is done via hwdb, only scsi generic here. */
+		"ACTION!=\"add|bind\", GOTO=\"libgphoto2_rules_end\"\n"
+		"SUBSYSTEM!=\"usb\", GOTO=\"libgphoto2_usb_end\"\n"
+		"ENV{ID_USB_INTERFACES}==\"\", IMPORT{builtin}=\"usb_id\"\n"
+		/* shortcut the most common camera driver, ptp class, so we avoid parsing 1000
+		 * more rules . It will be completed in udev_begin_func() */
+		"ENV{ID_USB_INTERFACES}==\"*:060101:*\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"PTP\", ",
 	};
 	static const char * const usbcam_strings[] = {
 		/* UDEV_PRE_0_98 */
@@ -436,7 +441,9 @@ udev_parse_params (const func_params_t *params, void **data)
 		/* UDEV_136 */
 		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\"",
 		/* UDEV_175 */
-		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\""
+		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\"",
+		/* UDEV_201 */
+		""
 	};
 	static const char * const usbdisk_strings[] = {
 		/* UDEV_PRE_0_98 */
@@ -446,6 +453,8 @@ udev_parse_params (const func_params_t *params, void **data)
 		/* UDEV_136 */
 		"KERNEL==\"%s\", ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\"",
 		/* UDEV_175 */
+		"KERNEL==\"%s\", ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\"",
+		/* UDEV_201 */
 		"KERNEL==\"%s\", ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ENV{ID_GPHOTO2}=\"1\", ENV{GPHOTO2_DRIVER}=\"proprietary\""
 	};
 	udev_persistent_data_t *pdata;
@@ -456,7 +465,7 @@ udev_parse_params (const func_params_t *params, void **data)
 	if (1) {
 		int i;
 		char *key = NULL, *val = NULL;
-		for (i=0; ((key=(*params->argv)[i])   != NULL) && 
+		for (i=0; ((key=(*params->argv)[i])   != NULL) &&
 			  ((val=(*params->argv)[i+1]) != NULL); i+=2) {
 			if (0) {
 				/* nothing */
@@ -490,11 +499,13 @@ udev_parse_params (const func_params_t *params, void **data)
 	    && (pdata->version <= UDEV_0_98)) {
 		FATAL("Either <script> or <mode,group,owner> parameters must be given.");
 	}
-	if ((pdata->script != NULL) && (pdata->mode != NULL 
-					|| pdata->group != NULL 
+	if ((pdata->script != NULL) && (pdata->mode != NULL
+					|| pdata->group != NULL
 					|| pdata->owner != NULL)) {
 		FATAL("The <script> parameter conflicts with the <mode,group,owner> parameters.");
 	}
+	if (pdata->version >= UDEV_201)
+		fprintf(stderr,"NOTE: You need to generate a hwdb too, this file just contains the scsi generic device entries.\n");
 
 	pdata->begin_string = begin_strings[pdata->version];
 	pdata->usbcam_string = usbcam_strings[pdata->version];
@@ -524,7 +535,7 @@ udev_begin_func (const func_params_t *params, void **data)
 		printf ("# this file is autogenerated, local changes will be LOST on upgrades\n");
 		printf ("%s", pdata->begin_string);
 
-		if (pdata->version == UDEV_136 || pdata->version == UDEV_175) {
+		if (pdata->version == UDEV_136 || pdata->version == UDEV_175 || pdata->version == UDEV_201) {
 			if (pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL) {
 				if (pdata->mode != NULL) {
 					printf("MODE=\"%s\", ", pdata->mode);
@@ -566,7 +577,7 @@ udev_end_func (const func_params_t *params, void *data)
 
 
 static int
-udev_camera_func (const func_params_t *params, 
+udev_camera_func (const func_params_t *params,
 		  const int i,
 		  const int total,
 		  const CameraAbilities *a,
@@ -574,7 +585,6 @@ udev_camera_func (const func_params_t *params,
 {
 	int flags = 0;
 	int class = 0, subclass = 0, proto = 0;
-	int usb_vendor = 0, usb_product = 0;
 	int has_valid_rule = 0;
 	udev_persistent_data_t *pdata = (udev_persistent_data_t *) data;
 	ASSERT(pdata != NULL);
@@ -582,14 +592,14 @@ udev_camera_func (const func_params_t *params,
 	if (!(a->port & GP_PORT_USB))
 		return 0;
 
+	if (pdata->version == UDEV_201) return 0;
+
 	if (a->usb_vendor) { /* usb product id may be zero! */
 		class = 0;
 		subclass = 0;
 		proto = 0;
-		flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
+		flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID
 			 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
-		usb_vendor = a->usb_vendor;
-		usb_product = a->usb_product;
 	} else {
 		if (a->usb_class) {
 			class = a->usb_class;
@@ -604,8 +614,6 @@ udev_camera_func (const func_params_t *params,
 				flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
 			else
 				proto = 0;
-			usb_vendor = 0;
-			usb_product = 0;
 		}
 	}
 
@@ -676,7 +684,7 @@ udev_camera_func (const func_params_t *params,
 
 
 static int
-udev_camera_func2 (const func_params_t *params, 
+udev_camera_func2 (const func_params_t *params,
 		   const int i,
 		   const int total,
 		   const CameraAbilities *a,
@@ -730,6 +738,91 @@ udev_camera_func2 (const func_params_t *params,
 }
 
 
+static int
+hwdb_begin_func (const func_params_t *params, void **data)
+{
+	fprintf(stderr,"NOTE: You should generate a udev rules file with udev-version 201\n");
+	fprintf(stderr,"or later to support cameras that use SCSI tunneling support, like\n");
+	fprintf(stderr,"various picture frames, Olympus remote control support.\n");
+	printf ("# hardware database file for libgphoto2 devices\n");
+	return 0;
+}
+
+
+static int
+hwdb_camera_func (const func_params_t *params,
+		  const int i,
+		  const int total,
+		  const CameraAbilities *a,
+		  void *data)
+{
+	int flags = 0;
+	int class = 0, subclass = 0, proto = 0;
+	int usb_vendor = 0, usb_product = 0;
+	int has_valid_rule = 0;
+
+	if (!(a->port & GP_PORT_USB))
+		return 0;
+
+	if (a->usb_vendor) { /* usb product id may be zero! */
+		flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID
+			 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
+		usb_vendor = a->usb_vendor;
+		usb_product = a->usb_product;
+	} else {
+		if (a->usb_class) {
+			class = a->usb_class;
+			subclass = a->usb_subclass;
+			proto = a->usb_protocol;
+			flags = GP_USB_HOTPLUG_MATCH_INT_CLASS;
+			if (subclass != -1)
+				flags |= GP_USB_HOTPLUG_MATCH_INT_SUBCLASS;
+			else
+				subclass = 0;
+			if (proto != -1)
+				flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
+			else
+				proto = 0;
+		}
+	}
+
+	printf ("\n# %s\n", a->model);
+
+	if (flags & GP_USB_HOTPLUG_MATCH_INT_CLASS) {
+		if ((flags & (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) == (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) {
+			/* device class matcher ... */
+			/*printf("usb:v*p*d*dc%02ddsc%02dp%02d*\"\n GPHOTO2_DRIVER=PTP\n", class, subclass, proto);*/
+			/* but we need an interface class matcher, ptp is a interface */
+			printf("usb:v*ic%02disc%02dip%02d*\n GPHOTO2_DRIVER=PTP\n", class, subclass, proto);
+			has_valid_rule = 1;
+		} else {
+			if (class == 666) {
+				printf("# not working yet\n");
+			} else {
+				fprintf(stderr, "unhandled interface match flags %x\n", flags);
+			}
+		}
+	} else {
+		if (flags & GP_USB_HOTPLUG_MATCH_VENDOR_ID) {
+			if (strstr(a->library, "ptp"))
+				printf ("usb:v%04Xp%04X*\n GPHOTO2_DRIVER=PTP\n", usb_vendor, usb_product);
+			else
+				printf ("usb:v%04Xp%04X*\n GPHOTO2_DRIVER=proprietary\n", usb_vendor, usb_product);
+			has_valid_rule = 1;
+		} else {
+			fprintf (stderr, "Error: Trying to output device %d/%d with incorrect match flags.\n",
+				usb_vendor, usb_product
+			);
+		}
+	}
+	if (has_valid_rule != 0) {
+		printf(" ID_GPHOTO2=1\n");
+		if (a->device_type & GP_DEVICE_AUDIO_PLAYER)
+			printf(" ID_MEDIA_PLAYER=1\n");
+	}
+	return 0;
+}
+
 static int
 empty_begin_func (const func_params_t *params, void **data)
 {
@@ -773,7 +866,7 @@ ddb_list_out_func(const char *str, void *data)
 
 
 static void
-ddb_delayed_head(const func_params_t *params, 
+ddb_delayed_head(const func_params_t *params,
 		 const int i,
 		 const int total,
 		 const CameraAbilities *a,
@@ -790,26 +883,18 @@ ddb_delayed_head(const func_params_t *params,
 	printf("device \"%s\" {\n", a->model);
 
 	printf("    device_type");
-	first = 1;
-	gpi_enum_to_string(a->device_type, 
-			   gpi_gphoto_device_type_map,
-			   ddb_list_out_func,
-			   (void *) &first);
-	printf(";\n");
+	printf(" %s;\n", gpi_enum_to_string(a->device_type,
+					    gpi_gphoto_device_type_map);
 
 	printf("    driver \"%s\";\n", camlib_basename);
 
 	printf("    driver_status");
-	first = 1;
-	gpi_enum_to_string(a->status, 
-			   gpi_camera_driver_status_map,
-			   ddb_list_out_func,
-			   (void *) &first);
-	printf(";\n");
+	printf(" %s;\n", gpi_enum_to_string(a->status,
+					    gpi_camera_driver_status_map);
 
 	printf("    operations");
 	first = 1;
-	gpi_flags_to_string_list(a->operations, 
+	gpi_flags_to_string_list(a->operations,
 				 gpi_camera_operation_map,
 				 ddb_list_out_func,
 				 (void *) &first);
@@ -817,7 +902,7 @@ ddb_delayed_head(const func_params_t *params,
 
 	printf("    file_operations");
 	first = 1;
-	gpi_flags_to_string_list(a->file_operations, 
+	gpi_flags_to_string_list(a->file_operations,
 				 gpi_file_operation_map,
 				 ddb_list_out_func,
 				 (void *) &first);
@@ -825,7 +910,7 @@ ddb_delayed_head(const func_params_t *params,
 
 	printf("    folder_operations");
 	first = 1;
-	gpi_flags_to_string_list(a->folder_operations, 
+	gpi_flags_to_string_list(a->folder_operations,
 				 gpi_folder_operation_map,
 				 ddb_list_out_func,
 				 (void *) &first);
@@ -834,7 +919,7 @@ ddb_delayed_head(const func_params_t *params,
 
 
 static int
-ddb_camera_func (const func_params_t *params, 
+ddb_camera_func (const func_params_t *params,
 		 const int i,
 		 const int total,
 		 const CameraAbilities *a,
@@ -858,7 +943,7 @@ ddb_camera_func (const func_params_t *params,
 		if (a->speed[0] != 0) {
 			unsigned int i;
 			printf("        speeds");
-			for (i=0; 
+			for (i=0;
 			     ((i < (sizeof(a->speed)/sizeof(a->speed[0]))) &&
 			      (a->speed[i] != 0)); i++) {
 				printf("%s %d", (first)?"":",", a->speed[i]);
@@ -870,7 +955,7 @@ ddb_camera_func (const func_params_t *params,
 	}
 
 	if ((a->port & GP_PORT_USB)) {
-		if (a->usb_vendor) { 
+		if (a->usb_vendor) {
 			/* usb product id may have the legal value zero! */
 			DELAYED_HEAD();
 			printf("    interface usb {\n");
@@ -917,7 +1002,7 @@ ddb_camera_func (const func_params_t *params,
 
 /* print_fdi_map
  *
- * Print FDI Device Information file for HAL with information on 
+ * Print FDI Device Information file for HAL with information on
  * all cams supported by our instance of libgphoto2.
  *
  * For specs, look around on http://freedesktop.org/ and search
@@ -934,12 +1019,14 @@ fdi_begin_func (const func_params_t *params, void **data)
 	printf("<deviceinfo version=\"0.2\">\n");
 	printf(" <device>\n");
 	printf("  <match key=\"info.subsystem\" string=\"usb\">\n");
+	printf("   <!-- skip USB Mass Storage Devices -->\n");
+	printf("   <match key=\"usb.interface.class\" compare_ne=\"8\">\n");
 	return 0;
 }
 
 
 static int
-fdi_camera_func (const func_params_t *params, 
+fdi_camera_func (const func_params_t *params,
 		 const int i,
 		 const int total,
 		 const CameraAbilities *a,
@@ -973,62 +1060,62 @@ fdi_camera_func (const func_params_t *params,
 			return 0;
 		}
 		if (a->usb_vendor) { /* usb product id might be 0! */
-			printf("   <match key=\"usb.vendor_id\" int=\"%d\">\n", a->usb_vendor);
-			printf("    <match key=\"usb.product_id\" int=\"%d\">\n", a->usb_product);
+			printf("    <match key=\"usb.vendor_id\" int=\"%d\">\n", a->usb_vendor);
+			printf("     <match key=\"usb.product_id\" int=\"%d\">\n", a->usb_product);
 			if (a->usb_vendor == 0x05ac) { /* Apple iPhone, PTP user. */
-				printf("     <match key=\"usb.interface.class\" int=\"6\">\n");
-				printf("      <match key=\"usb.interface.subclass\" int=\"1\">\n");
-				printf("       <match key=\"usb.interface.protocol\" int=\"1\">\n");
+				printf("      <match key=\"usb.interface.class\" int=\"6\">\n");
+				printf("       <match key=\"usb.interface.subclass\" int=\"1\">\n");
+				printf("        <match key=\"usb.interface.protocol\" int=\"1\">\n");
 			}
 			if (a->device_type & GP_DEVICE_AUDIO_PLAYER) {
-				printf("     <merge key=\"info.category\" type=\"string\">portable_audio_player</merge>\n");
-				printf("     <addset key=\"info.capabilities\" type=\"strlist\">portable_audio_player</addset>\n");
-				printf("     <merge key=\"portable_audio_player.access_method\" type=\"string\">user</merge>\n");
-				printf("     <merge key=\"portable_audio_player.type\" type=\"string\">mtp</merge>\n");
-				
+				printf("      <merge key=\"info.category\" type=\"string\">portable_audio_player</merge>\n");
+				printf("      <addset key=\"info.capabilities\" type=\"strlist\">portable_audio_player</addset>\n");
+				printf("      <merge key=\"portable_audio_player.access_method\" type=\"string\">user</merge>\n");
+				printf("      <merge key=\"portable_audio_player.type\" type=\"string\">mtp</merge>\n");
+
 				/* FIXME: needs true formats ... But all of them can do MP3 */
-				printf("     <append key=\"portable_audio_player.output_formats\" type=\"strlist\">audio/mpeg</append>\n");
+				printf("      <append key=\"portable_audio_player.output_formats\" type=\"strlist\">audio/mpeg</append>\n");
 			} else {
-				printf("     <merge key=\"info.category\" type=\"string\">camera</merge>\n");
-				printf("     <addset key=\"info.capabilities\" type=\"strlist\">camera</addset>\n");
+				printf("      <merge key=\"info.category\" type=\"string\">camera</merge>\n");
+				printf("      <addset key=\"info.capabilities\" type=\"strlist\">camera</addset>\n");
 
 				/* HACK alert ... but the HAL / gnome-volume-manager guys want that */
 				if (NULL!=strstr(a->library,"ptp"))
-					printf("     <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
+					printf("      <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
 				else
-					printf("     <merge key=\"camera.access_method\" type=\"string\">proprietary</merge>\n");
+					printf("      <merge key=\"camera.access_method\" type=\"string\">proprietary</merge>\n");
 			}
 			/* leave them here even for audio players */
-			printf("     <merge key=\"camera.libgphoto2.name\" type=\"string\">%s</merge>\n", model);
-			printf("     <merge key=\"camera.libgphoto2.support\" type=\"bool\">true</merge>\n");
-			if (a->usb_vendor == 0x05ac) { /* Apple iPhone */
-				printf("       </match>\n");
-				printf("      </match>\n");
-				printf("     </match>\n");
-			}
-			printf("    </match>\n");
-			printf("   </match>\n");
-			
-		} else if ((a->usb_class) && (a->usb_class != 666)) {
-			printf("   <match key=\"usb.interface.class\" int=\"%d\">\n", a->usb_class);
-			printf("    <match key=\"usb.interface.subclass\" int=\"%d\">\n", a->usb_subclass);
-			printf("     <match key=\"usb.interface.protocol\" int=\"%d\">\n", a->usb_protocol);
-			printf("      <merge key=\"info.category\" type=\"string\">camera</merge>\n");
-			printf("      <addset key=\"info.capabilities\" type=\"strlist\">camera</addset>\n");
-			if (a->usb_class == 6) {
-				printf("      <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
-			} else {
-				if (a->usb_class == 8) {
-					printf("      <merge key=\"camera.access_method\" type=\"string\">storage</merge>\n");
-				} else {
-					printf("      <merge key=\"camera.access_method\" type=\"string\">proprietary</merge>\n");
-				}
-			}
 			printf("      <merge key=\"camera.libgphoto2.name\" type=\"string\">%s</merge>\n", model);
 			printf("      <merge key=\"camera.libgphoto2.support\" type=\"bool\">true</merge>\n");
+			if (a->usb_vendor == 0x05ac) { /* Apple iPhone */
+				printf("        </match>\n");
+				printf("       </match>\n");
+				printf("      </match>\n");
+			}
+			printf("     </match>\n");
+			printf("    </match>\n");
+
+		} else if ((a->usb_class) && (a->usb_class != 666)) {
+			printf("    <match key=\"usb.interface.class\" int=\"%d\">\n", a->usb_class);
+			printf("     <match key=\"usb.interface.subclass\" int=\"%d\">\n", a->usb_subclass);
+			printf("      <match key=\"usb.interface.protocol\" int=\"%d\">\n", a->usb_protocol);
+			printf("       <merge key=\"info.category\" type=\"string\">camera</merge>\n");
+			printf("       <addset key=\"info.capabilities\" type=\"strlist\">camera</addset>\n");
+			if (a->usb_class == 6) {
+				printf("       <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
+			} else {
+				if (a->usb_class == 8) {
+					printf("       <merge key=\"camera.access_method\" type=\"string\">storage</merge>\n");
+				} else {
+					printf("       <merge key=\"camera.access_method\" type=\"string\">proprietary</merge>\n");
+				}
+			}
+			printf("       <merge key=\"camera.libgphoto2.name\" type=\"string\">%s</merge>\n", model);
+			printf("       <merge key=\"camera.libgphoto2.support\" type=\"bool\">true</merge>\n");
+			printf("      </match>\n");
 			printf("     </match>\n");
 			printf("    </match>\n");
-			printf("   </match>\n");
 		}
 	} /* camera has USB connection */
 	return 0;
@@ -1037,6 +1124,7 @@ fdi_camera_func (const func_params_t *params,
 static int
 fdi_end_func (const func_params_t *params, void *data)
 {
+	printf("   </match>\n");
 	printf("  </match>\n");
 	printf(" </device>\n");
 	printf("</deviceinfo>\n");
@@ -1046,7 +1134,7 @@ fdi_end_func (const func_params_t *params, void *data)
 
 /* print_fdi_device_map
  *
- * Print FDI Device Information file for HAL with information on 
+ * Print FDI Device Information file for HAL with information on
  * all cams supported by our instance of libgphoto2.
  *
  * For specs, look around on http://freedesktop.org/ and search
@@ -1063,12 +1151,14 @@ fdi_device_begin_func (const func_params_t *params, void **data)
 	printf("<deviceinfo version=\"0.2\">\n");
 	printf(" <device>\n");
 	printf("  <match key=\"info.subsystem\" string=\"usb\">\n");
+	printf("   <!-- skip USB Mass Storage Devices -->\n");
+	printf("   <match key=\"usb.interface.class\" compare_ne=\"8\">\n");
 	return 0;
 }
 
 
 static int
-fdi_device_camera_func (const func_params_t *params, 
+fdi_device_camera_func (const func_params_t *params,
 		 	const int i,
 			const int total,
 		 	const CameraAbilities *a,
@@ -1106,28 +1196,28 @@ fdi_device_camera_func (const func_params_t *params,
 			/* do not set category. We don't really know what this device really is.
 			 * But we do now that is capable of being a camera, so add to capabilities
 			 */
-			printf("   <match key=\"usb_device.vendor_id\" int=\"%d\">\n", a->usb_vendor);
-			printf("    <match key=\"usb_device.product_id\" int=\"%d\">\n", a->usb_product);
+			printf("    <match key=\"usb_device.vendor_id\" int=\"%d\">\n", a->usb_vendor);
+			printf("     <match key=\"usb_device.product_id\" int=\"%d\">\n", a->usb_product);
 			if (params->add_comments) {
 				printf("     <!-- %s -->\n", a->model);
 			}
 			if (a->device_type & GP_DEVICE_AUDIO_PLAYER)
-				printf("     <append key=\"info.capabilities\" type=\"strlist\">portable_audio_player</append>\n");
+				printf("      <append key=\"info.capabilities\" type=\"strlist\">portable_audio_player</append>\n");
 			else
-				printf("     <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+				printf("      <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+			printf("     </match>\n");
 			printf("    </match>\n");
-			printf("   </match>\n");
 		}
 #if 0
 		/* would need to be able to merge upwards ... but cannot currently */
 		else if ((a->usb_class) && (a->usb_class != 666)) {
-			printf("   <match key=\"usb.interface.class\" int=\"%d\">\n", a->usb_class);
-			printf("    <match key=\"usb.interface.subclass\" int=\"%d\">\n", a->usb_subclass);
-			printf("     <match key=\"usb.interface.protocol\" int=\"%d\">\n", a->usb_protocol);
-			printf("      <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+			printf("    <match key=\"usb.interface.class\" int=\"%d\">\n", a->usb_class);
+			printf("     <match key=\"usb.interface.subclass\" int=\"%d\">\n", a->usb_subclass);
+			printf("      <match key=\"usb.interface.protocol\" int=\"%d\">\n", a->usb_protocol);
+			printf("       <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+			printf("      </match>\n");
 			printf("     </match>\n");
 			printf("    </match>\n");
-			printf("   </match>\n");
 		}
 #endif
 	}
@@ -1137,12 +1227,205 @@ fdi_device_camera_func (const func_params_t *params,
 static int
 fdi_device_end_func (const func_params_t *params, void *data)
 {
+	printf("   </match>\n");
 	printf("  </match>\n");
 	printf(" </device>\n");
 	printf("</deviceinfo>\n");
 	return 0;
 }
 
+/* HTML output */
+struct html_comment {
+	char *name;
+	char *comment;
+};
+struct html_data {
+	int nrofcomments;
+	struct html_comment *comments;
+};
+
+static int
+html_begin_func (const func_params_t *params, void **data) {
+	FILE *f;
+	char buf[512];
+	struct html_data *hd;
+
+	printf("<!-- This part was generated by %s - - html -->\n",
+	       "libgphoto2 " ARGV0);
+	print_version_comment(stdout, "    | ", "\n", "<!--+\n", "    +-->\n");
+	printf("Number of supported cameras and media players: %d\n",params->number_of_cameras);
+	printf("<table border=1>\n");
+	printf("<tr>\n");
+	printf("   <th>Camera Model</th>\n");
+	printf("   <th>Additional Abilities</th>\n");
+	printf("   <th>Comments</th>\n");
+	printf("</tr>\n");
+
+	*data = hd = malloc(sizeof(*hd));
+	hd->nrofcomments = 0;
+	hd->comments = NULL;
+
+	f = fopen("comments.txt","r");
+	if (!f)
+		return 0;
+	while (fgets(buf,sizeof(buf),f)) {
+		char *s = strchr(buf,';');
+		if (!s)
+			continue;
+		*s = '\0';
+		if (hd->nrofcomments) {
+			hd->comments = realloc(hd->comments, (hd->nrofcomments+1)*sizeof(hd->comments[0]));
+		} else {
+			hd->comments = malloc(sizeof(hd->comments[0]));
+		}
+		hd->comments[hd->nrofcomments].name = strdup(buf);
+		hd->comments[hd->nrofcomments].comment = strdup(s+1);
+		hd->nrofcomments++;
+	}
+	fclose (f);
+	return 0;
+}
+
+static char*
+escape_html(const char *str) {
+	const char *s;
+	char *newstr, *ns;
+	int inc = 0;
+
+	s = str;
+	do {
+		s = strchr(s,'&');
+		if (s) {
+			inc+=strlen("&amp;");
+			s++;
+		}
+	} while (s);
+	/* FIXME: if we ever get a camera with <> or so, add escape code here */
+	newstr = malloc(strlen(str)+1+inc);
+	s = str; ns = newstr;
+	do {
+		char *x;
+		x = strchr(s,'&');
+		if (x) {
+			memcpy (ns, s, x-s);
+			ns += x-s;
+			memcpy (ns, "&amp;", strlen("&amp;"));
+			ns += strlen("&amp;");
+			s = x+1;
+		} else {
+			strcpy (ns, s);
+			break;
+		}
+	} while (1);
+	return newstr;
+}
+
+static int
+html_camera_func (
+	const func_params_t *params,
+	const int i,
+	const int total,
+	const CameraAbilities *a,
+	void *data
+) {
+	char *m;
+	int j;
+	CameraOperation		op = a->operations;
+	struct html_data	*hd = data;
+
+	if (a->device_type != GP_DEVICE_STILL_CAMERA)
+		return 0;
+
+	printf ("<tr>\n");
+	m = escape_html (a->model);
+	printf (" <td>%s</td>", m); free (m);
+	printf (" <td>");
+	if (!op) printf ("&nbsp;");
+	if (op & GP_OPERATION_CAPTURE_IMAGE) {
+		printf ("Image Capture");
+		op &= ~GP_OPERATION_CAPTURE_IMAGE;
+		if (op) printf (", ");
+	}
+	if (op & GP_OPERATION_TRIGGER_CAPTURE) {
+		printf ("Trigger Capture");
+		op &= ~GP_OPERATION_TRIGGER_CAPTURE;
+		if (op) printf (", ");
+	}
+	if (op & GP_OPERATION_CAPTURE_PREVIEW) {
+		printf ("Liveview");
+		op &= ~GP_OPERATION_CAPTURE_PREVIEW;
+		if (op) printf (", ");
+	}
+	if (op & GP_OPERATION_CONFIG) {
+		printf ("Configuration");
+		op &= ~GP_OPERATION_CONFIG;
+		if (op) printf (", ");
+	}
+	if (op) {
+		printf ("Other Ops %x", op);
+	}
+	printf(" </td>");
+	printf(" <td>");
+	switch (a->status) {
+	case GP_DRIVER_STATUS_PRODUCTION: break;
+	case GP_DRIVER_STATUS_TESTING: printf("Testing (Beta)"); break;
+	case GP_DRIVER_STATUS_EXPERIMENTAL: printf("Experimental"); break;
+	case GP_DRIVER_STATUS_DEPRECATED: printf("Deprecated"); break;
+	}
+	/* read comments */
+	for (j=0;j<hd->nrofcomments;j++) {
+		if (!strcmp(a->model,hd->comments[j].name)) {
+			printf("%s",hd->comments[j].comment);
+			break;
+		}
+	}
+	if (j == hd->nrofcomments) printf(" &nbsp;");
+	printf(" </td>");
+	printf("</tr>\n");
+	return 0;
+}
+
+static int html_middle_func (
+	const func_params_t *params,
+	void **data
+) {
+	printf("</table><p>\n");
+	printf("Media Players that are supported by both libmtp and libgphoto2:<p/>\n");
+	printf("<table border=1>\n");
+	printf("<tr>\n");
+	printf("   <th>Media Player Model</th>\n");
+	printf("</tr>\n");
+	return 0;
+}
+
+static int
+html_camera2_func (
+	const func_params_t *params,
+	const int i,
+	const int total,
+	const CameraAbilities *a,
+	void *data
+) {
+	char *m;
+
+	if (a->device_type != GP_DEVICE_AUDIO_PLAYER)
+		return 0;
+
+	m = escape_html (a->model);
+	printf ("<tr>\n");
+	printf (" <td>%s</td>", m); free (m);
+	printf("</tr>\n");
+	return 0;
+}
+
+
+static int html_end_func (
+	const func_params_t *params,
+	void *data
+) {
+	printf("</table>\n");
+	return 0;
+}
 
 /* time zero for debug log time stamps */
 struct timeval glob_tv_zero = { 0, 0 };
@@ -1153,8 +1436,8 @@ debug_func (GPLogLevel level, const char *domain, const char *str,
 {
 	struct timeval tv;
 	gettimeofday (&tv,NULL);
-	fprintf (stderr, "%li.%06li %s(%i): %s\n", 
-		 (long) (tv.tv_sec - glob_tv_zero.tv_sec), 
+	fprintf (stderr, "%li.%06li %s(%i): %s\n",
+		 (long) (tv.tv_sec - glob_tv_zero.tv_sec),
 		 (1000000L + tv.tv_usec - glob_tv_zero.tv_usec) % 1000000L,
 		 domain, level, str);
 }
@@ -1174,8 +1457,8 @@ typedef struct {
 
 
 static int
-iterate_camera_list (const int add_comments, 
-		     const output_format_t *format, 
+iterate_camera_list (const int add_comments,
+		     const output_format_t *format,
 		     string_array_p argv)
 {
 	int number_of_cameras;
@@ -1293,19 +1576,40 @@ static const output_format_t formats[] = {
 	 "        If you give a script parameter, the mode, owner, group parameters will be ignored.\n"
 	 "        For mode \"136\" put it into /lib/udev/rules.d/40-libgphoto2.rules;\n"
 	 "        you can still use mode/owner/group, but the preferred mode of operation\n"
-	 "        is to use udev-extras for dynamic access permissions.\n",
+	 "        is to use udev-extras for dynamic access permissions.\n"
+	"	  Available versions of the rule generator: 0.98, 136, 175, 201.\n",
 	 "[script <PATH_TO_SCRIPT>|version <version>|mode <mode>|owner <owner>|group <group>]*",
-	 udev_begin_func, 
+	 udev_begin_func,
 	 udev_camera_func,
 	 udev_middle_func,
 	 udev_camera_func2,
 	 udev_end_func
 	},
+	{"hwdb",
+	 "hardware database file",
+	 "Put it into /usr/lib/udev/hwdb.d/20-gphoto.conf.\n",
+	 NULL,
+	 hwdb_begin_func,
+	 hwdb_camera_func,
+	 NULL,
+	 NULL,
+	 empty_end_func
+	},
+	{"html",
+	 "HTML table file for gphoto.org website",
+	 "Paste it into /proj/libgphoto2/support.php",
+	 NULL,
+	 html_begin_func,
+	 html_camera_func,
+	 html_middle_func,
+	 html_camera2_func,
+	 html_end_func
+	},
 	{"idlist",
 	 "list of IDs and names",
 	 "grep for an ID to find the device name",
 	 NULL,
-	 empty_begin_func, 
+	 empty_begin_func,
 	 idlist_camera_func,
 	 NULL,
 	 NULL,
@@ -1323,8 +1627,7 @@ static const output_format_t formats[] = {
 	 ddb_end_func
 	},
 #endif
-	{NULL, NULL, NULL, NULL, 
-	 NULL, NULL, NULL}
+	{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL}
 };
 
 
@@ -1337,12 +1640,12 @@ static int print_format_list(const output_format_t *formats)
 	printf("List of output formats:\n");
 	for (i=0; formats[i].name != NULL; ++i) {
 		if (formats[i].paramdescr != NULL) {
-			printf("    %s %s\n        %s\n", 
+			printf("    %s %s\n        %s\n",
 			       formats[i].name,
 			       formats[i].paramdescr,
 			       formats[i].descr);
 		} else {
-			printf("    %s\n        %s\n", 
+			printf("    %s\n        %s\n",
 			       formats[i].name, formats[i].descr);
 		}
 		if (formats[i].help != NULL) {
@@ -1418,14 +1721,14 @@ int main(int argc, char *argv[])
 		}
 		format_index++;
 	}
-	if ((formats[format_index].name == NULL) || 
+	if ((formats[format_index].name == NULL) ||
 	    (strcmp(formats[format_index].name, format_name) != 0)) {
 		return print_help();
 	}
 
 	/* copy remaining arguments */
-	for (j=i; 
-	     (j<argc) && ((j-i)<(int)((sizeof(fmt_argv)/sizeof(fmt_argv[0]))-1)); 
+	for (j=i;
+	     (j<argc) && ((j-i)<(int)((sizeof(fmt_argv)/sizeof(fmt_argv[0]))-1));
 	     j++) {
 		fmt_argv[j-i] = argv[j];
 	}
